<?php
/**
 * Admin Configuration File
 *
 * This file sets many global variables that define how the back-office/admin
 * interface works. Feel free to change any of these, but note that there is no
 * way to go back to the default value without editing again. If you do not make
 * a note of and forget the default value then you are on your own.
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package config
 */
if(!defined('QP_CONFIG_LOADED')) die('fail');

/**
 * Allow Cross Posting
 *
 * Possible values:
 * - -1 if you don't want to use categories at all
 * - 0 if you want users to post to a single category only
 * - 1 if you want to be able to cross-post among multiple categories
 * - 2 if you want to be able to cross-post among multiple blogs/categories
 * - 3 if you want to be able to change main cat among blogs
 * Note: #3 moves the posts from one blog to another, so use with caution.
 *
 * @global integer $allow_cross_posting
 */
$allow_cross_posting = 1;

/**
 * Balance Tags
 *
 * Automatically balance unmatched tags in posts and comments. Choose
 * the formatting options for your posts: 0 to disable, 1 to enable
 *
 * @global bool $balance_tags
 */
$balance_tags = 1;

/**
 * Antispam on Message Form
 *
 * Default of 1 messages sent via the message form will be checked against
 * the antispam blacklist. Set it to 0 if you like spam I guess.
 *
 * @global bool $antispam_on_message_form
 */
$antispam_on_message_form = 1;

/**
 * Cookie Expired
 *
 * Expired-time, used to erase comment meta data cookies. Default is 24 hours ago.
 *
 * @global integer $cookie_expired
 */
$cookie_expired = time() - 86400;

/**
 * Cookies Expire
 *
 * Expiration for comment meta data cookies, in seconds. Set this to 0 if you wish
 * to use non-permanent cookies (erased when browser is closed). Default is one
 * year from now.
 *
 * @global integer $cookie_expires
 */
$cookie_expires = time() + 31536000;

/**
 * Cron Timeout Delay
 *
 * Seconds after which a scheduled task is considered to be timed out, default is
 * 30 minutes.
 *
 * @global integer $cron_timeout_delay
 */
$cron_timeout_delay = 1800;

/**
 * Dashboard Items Count
 *
 * This controls how many items by section (comments, drafts, recent edits)
 * are displayed on the dashboard page.
 *
 * @global integer $dash_disp_num
 */
$dash_disp_num = 5;

/**
 * Display debugging information?
 *
 * Possible values:
 * - 0 = no
 * - 1 = yes
 * - 2 = yes and potentially die() to display debug info (needed before redirects,
 *   e-g message_send.php)
 *
 * @global integer $debug
 */
$debug = 0;

/**
 * OB Handler Debug
 *
 * Display elements that are different on each request (Page processing time, ...)
 * Set this to true to prevent displaying minor changing elements (like time) in
 * order not to have artificial content changes.
 *
 * @global bool $debug_obhandler
 */
$debug_obhandler = false;

/**
 * Display debugging information to visitors?
 *
 * $debug always applies to anyone logged in. Set this to 1 if you want debug info
 * displayed when you are not logged in (for troubleshooting what a visitor sees).
 * Default is 0. possible values: 0 = no, 1 = yes
 *
 * @global bool $debug_visitors
 */
$debug_visitors = 0;

/**
 * XMLRPC logging
 *
 * Set this to 1 to log XMLRPC calls received by this server (into /qp_srvc/xmlrpc.log).
 *
 * @global bool $debug_xmlrpc_logging
 */
$debug_xmlrpc_logging = 0;

/**
 * Demo mode
 *
 * Default is false, if set to true: Do not allow update of files in the file manager,
 * do not allow changes to the 'admin' or 'demouser' or 'demoblogger' or 'demospecial'
 * accounts/groups, blog media directories can only be configured to be inside of
 * {@link $media_path}.
 *
 * @global bool $demo_mode
 */
$demo_mode = false;

/**
 * Forbidden Upload Extensions
 *
 * File extensions that the admin will not be able to enable via the admin interface
 * (Global settings => files).
 *
 * @global array $forbidden_upload_exts
 */
$forbidden_upload_exts = array(
	'cgi',
	'exe',
	'htaccess',
	'htpasswd',
	'php',
	'php3',
	'php4',
	'php5',
	'php6',
	'phtml',
	'pl',
	'vbs'
);

/**
 * Force Regex FileName
 *
 * The admin can configure the regular expression for valid file names in the Settings
 * interface. However if this is set to non empty, the admin will not be able to
 * customize these values.
 *
 * @global string $force_regexp_filename
 */
$force_regexp_filename = '';

/**
 * Force Regex DirectoryName
 *
 * The admin can configure the regular expression for valid dir names in the Settings
 * interface. However if this is set to non empty, the admin will not be able to
 * customize these values.
 *
 * @global string $force_regexp_dirname
 */
$force_regexp_dirname = '';

/**
 * URL of the Home link
 *
 * The "home" button on the top right of the navbar is the only use of this. By
 * default this is the base url, and unless you do a complex installation there
 * is no need to change it.
 *
 * @global string $home_url
 */
$home_url = $app_baseurl;

/**
 * Instance Name
 *
 * MUST BE A SINGLE WORD! NO SPACES!! Short name of this system (will be used for
 * cookies and notification emails). Change this only if you install multiple instances
 * of this application on the same website. WARNING: don't play with this or you'll
 * have tons of cookies sent away and your readers will surely complain ... if they
 * aren't so mad they never come back!
 *
 * You can change the {@link $notify_from notification email address} in {@link _advanced.php}.
 *
 * @todo generate a random instance name at install and have it saved in the global
 * params in the DB
 *
 * @global string $instance_name
 */
$instance_name = 'quamplures';

/**
 * Lock user's ID and nickname
 *
 * Setting this to 'true' will mean a user can not change their login ID or nickname.
 * They will still be able to change their "known as" - just not their nickname
 * or login :) Useful if you need continuity over time for all your users (for example
 * when this app is the gateway to a forum or chatroom or other 3rd party with a
 * login requirement).
 *
 * @global bool $lock_id_and_nick
 */
$lock_id_and_nick = false;

/**
 * Minimum Comment Interval
 *
 * This is the minimum interval in seconds between consecutive comments from the
 * same IP. Increasing this can slow down spam attacks, and piss off real visitors
 * if too high.
 *
 * @global integer $minimum_comment_interval
 */
$minimum_comment_interval = 30;

/**
 * Locked post type IDs
 *
 * These post types can't be edited or deleted in the post type editor. They're
 * needed by certain features, so don't remove any IDs from this array.
 *
 * This is legacy code that needs to be reverse-engineered, completely figured out,
 * and either fixed or killed. As-is this is crap - EdB.
 *
 * @global array $posttypes_locked_IDs
 */
$posttypes_locked_IDs = array( 1000, 1500, 1520, 1530, 1570, 2000 );

/**
 * Reserved post type IDs
 *
 * These post types are reserved for future use and can't be edited or deleted in
 * the post type editor. It also is not possible to select them when creating a new
 * post. Do not remove any IDs from this array.
 *
 * This is legacy code that needs to be reverse-engineered, completely figured out,
 * and either fixed or killed. As-is this is crap - EdB.
 *
 * @global array $posttypes_reserved_IDs
 */
$posttypes_reserved_IDs = array( 3000, 4000, 5000 );

/**
 * Public Access to Media
 *
 * Do you want to allow public access to the media dir? WARNING: If you set this
 * to false this application will use /qp_srvc/getfile.php as a stub to access files,
 * and getfile.php will check the User permisssion to view files. HOWEVER this will
 * not prevent users from hitting directly into the media folder with their web
 * browser. You still need to restrict access to the media folder from your webserver.
 *
 * @global bool $public_access_to_media
 */
$public_access_to_media = true;

/**
 * Require Name & Email
 *
 * Default of 1 means a comment can not be left without the commenter providing
 * a name and email address. Set this to 0 to allow anonymous comments (without
 * name & email address).
 *
 * @global bool $require_name_email
 */
$require_name_email = 1;

/**
 * Upload Max (Kb)
 *
 * Admins can configure max file upload size, but they won't be able to set it higher
 * than this "max max" value.
 *
 * @global integer $upload_maxmaxkb
 */
$upload_maxmaxkb = 10000;

/**
 * Tags Dash Fix
 *
 * Enables a workaround (for an old b2evolution bug) to allow accessing posts with
 * URL titles ending with a dash. In b2evolution v2.4.5 new tag URLs were introduced:
 * You could choose to have tag URLs ending with a dash. This lead to problems with
 * post URL titles accidentially ending with a dash (today, URL titles cannot end
 * with a dash anymore): Instead of displaying the post, the post title was handled
 * as a tag name. When this setting is enabled, all tag names which are exactly 40
 * chars long and end with a dash are handled in the following way: Try to find a
 * post with the given tag name as the URL title. If there is a matching post, display
 * it; otherwise, display the normal tag page.
 *
 * Note: If you use a 39 chars-long tag name, have an URL title which is the same
 * as the tag *but* additionally has a dash at the end and you use the dash as a
 * tag URL "marker", you won't be able to access either the post or the tag page,
 * depending on the value of this setting. And no, that doesn't make sense to me
 * either :/
 *
 * @global bool $tags_dash_fix
 */
$tags_dash_fix = 0;

/**
 * XHTML use Strict
 *
 * Set this to true if you want to enforce XHTML strict validation
 *
 * @global bool $xhtml_use_strict
 */
$xhtml_use_strict = false;

/**
 * XHTML validate comments
 *
 * Do we want to use XHTML validation for comments?
 *
 * @global bool $xhtml_validate_comments
 */
$xhtml_validate_comments = true;

?>
