<?php
/**
 * Advanced Settings File
 *
 * Please NOTE: You should not comment variables out to prevent URL overrides.
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package config
 */
if(!defined('QP_CONFIG_LOADED')) die('fail');

/**
 * No Translation, does nothing
 *
 * Nevertheless, the string will be extracted by the gettext tools.
 */
function NT_( $string )
{
	return $string;
}

// Most of the time you'll want to see all errors, including notices
if( ! defined( 'E_DEPRECATED' ) )
{
	error_reporting( E_ALL );
}
else
{
	// Hopefully temporary fix for PHP >= v5.3 (Assigning the return value of new
	// by reference is deprecated) Tblue> This doesn't work properly on PHP 5.3...
	// For some reason, you have to set error_reporting to the desired value in php.ini
	// - error_reporting() and even ini_set() fail.
	error_reporting( E_ALL & ~E_DEPRECATED );
}

// To help debugging severe errors, you'll probably want PHP to display
// the errors on screen. In this case, uncomment the following line:
# ini_set( 'display_errors', 'on' );

// If you get blank pages, or if image resizing work all the time, PHP may
// be crashing because it doesn't have enough memory. The default is 128 MB
// in PHP > 5.2 but your host might have set a lower limit for some reason.
// Try uncommenting the next line and see if your troubles go away:
# ini_set( 'memory_limit', '128M' );

// Thumbnail size definitions (label => type, width, height, quality)
$thumbnail_sizes = array(
	'fit-800' => array( 'fit', 800, 1600, 80 ),
	'fit-760' => array( 'fit', 760, 1520, 80 ),
	'fit-720' => array( 'fit', 720, 1440, 80 ),
	'fit-680' => array( 'fit', 680, 1360, 80 ),
	'fit-640' => array( 'fit', 640, 1280, 80 ),
	'fit-600' => array( 'fit', 600, 1200, 80 ),
	'fit-560' => array( 'fit', 560, 1120, 85 ),
	'fit-520' => array( 'fit', 520, 1040, 85 ),
	'fit-480' => array( 'fit', 480, 960, 85 ),
	'fit-440' => array( 'fit', 440, 880, 85 ),
	'fit-400' => array( 'fit', 400, 800, 85 ),
	'fit-360' => array( 'fit', 360, 720, 85 ),
	'fit-320' => array( 'fit', 320, 640, 85 ),
	'fit-280' => array( 'fit', 280, 560, 90 ),
	'fit-240' => array( 'fit', 240, 480, 90 ),
	'fit-200' => array( 'fit', 200, 400, 90 ),
	'fit-160' => array( 'fit', 160, 320, 90 ),
	'fit-120' => array( 'fit', 120, 240, 90 ),
	'fit-80x160' => array( 'fit', 80, 160, 90 ),
	'fit-80x80' => array( 'fit', 80, 80, 90 ),
	'crop-80x80' => array( 'crop', 80, 80, 85 ),
	'crop-64x64' => array( 'crop', 64, 64, 85 ),
	'crop-48x48' => array( 'crop', 48, 48, 85 ),
	'crop-32x32' => array( 'crop', 32, 32, 85 ),
	'crop-15x15' => array( 'crop', 15, 15, 85 ),
);

// Get hostname out of baseurl: YOU SHOULD NOT EDIT THIS unless you know what you're doing
if( preg_match( '#^(https?://(.+?)(:(.+?))?)(/.*)$#', $app_baseurl, $matches ) )
{
	$app_baseurlroot = $matches[1]; // no ending slash!
	$basehost = $matches[2];
	$baseport = $matches[4];
	$basesubpath = $matches[5];
}
else
{
	die( 'Your baseurl ('.$app_baseurl.') set in _main_config.php seems invalid. You probably missed the "http://" prefix or the trailing slash.' );
}

/**
 * Base Domain
 *
 * By default we try to extract it automagically from $basehost (itself extracted
 * from $app_baseurl), but you may need to adjust this manually. And no I don't
 * know why or when :/
 * @global string $basedomain
 */
$basedomain = preg_replace( '/^( .* \. )? (.+? \. .+? )$/xi', '$2', $basehost );

/**
 * Default email address notifications
 *
 * Default email address for sending notifications of things like comments or trackbacks
 * or user registrations. Set a custom address like this:
 * <code>$notify_from = 'quamplures@your_server.com';</code>
 * Alternatively you can use this automated address generation (which removes "www." from
 * the beginning of $basehost):
 * <code>$notify_from = $instance_name.'@'.preg_replace( '/^www\./i', '', $basehost );</code>
 * @global string $notify_from
 */
$notify_from = $instance_name.'@'.preg_replace( '/^www\./i', '', $basehost );

// ** DB options **
// Show MySQL errors? (default: true). This is recommended on production environments.
$app_db_config['show_errors'] = true;
// Halt on MySQL errors? (default: true) Setting this to false is not recommended.
$app_db_config['halt_on_error'] = true;
// CREATE TABLE options. DO NOT USE unless you know what you're doing!
// For most options, we want to work on a table by table basis.
$app_db_config['table_options'] = ''; // Low ranking MySQL hosting compatibility Default
// Use transactions in DB? You need to use InnoDB in order to enable this.
$app_db_config['use_transactions'] = true;

/**
 * This is the path that will be associated to cookies.
 *
 * That means cookies set by this QP install won't be seen outside of this path
 * on the domain below.
 * @global string $cookie_path
 */
$cookie_path = preg_replace( '#https?://[^/]+#', '', $app_baseurl );

/**
 * Cookie domain
 *
 * That means cookies set by this application install won't be seen outside of this
 * domain.
 *
 * We'll take {@link $basehost} by default (the leading dot includes subdomains),
 * but when there's no dot in it, at least Firefox will not set the cookie. The
 * best example for having no dot in the host name is 'localhost', but it's the
 * case for host names in an intranet also.
 *
 * Note: ".domain.com" cookies will be sent to sub.domain.com too. But, see {@link 
 * http://www.faqs.org/rfcs/rfc2965}: "If multiple cookies satisfy the criteria
 * above, they are ordered in the Cookie header such that those with more specific
 * Path attributes precede those with less specific. Ordering with respect to other
 * attributes (e.g., Domain) is unspecified."
 * @global string $cookie_domain
 */
if( strpos($basehost, '.') === false )
{
	// localhost or windows machine name
	$cookie_domain = '';
}
elseif( preg_match( '~^[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+$~i', $basehost ) )
{
	// Use the basehost as it is
	$cookie_domain = $basehost;
}
else
{
	$cookie_domain = preg_replace( '/^(www\. )? (.+)$/xi', '.$2', $basehost );
	// When hosting multiple domains (not just subdomains) on a single instance of QP,
	// you may want to try this:
	// $cookie_domain = '.'.$_SERVER['HTTP_HOST'];
	// or this: -- Have a cookie domain of 2 levels only, base on current basehost.
	// $cookie_domain = preg_replace( '/^( .* \. )? (.+? \. .+? )$/xi', '.$2', $basehost );
	// fp> pb with domains like .co.uk !?
}

// The following remember the comment meta data for non registered users:
$cookie_name = 'cookie'.$instance_name.'name';
$cookie_email = 'cookie'.$instance_name.'email';
$cookie_url = 'cookie'.$instance_name.'url';
// The following handles the session:
$cookie_session = 'cookie'.$instance_name.'session';

// Location of Sub-directories. You should only move these around if you really
// need to, and you probably don't, and if you do and it all works consider yourself
// lucky. You should keep everything as sub-directories of the $app_baseurl base
// folder, which is set in _main_config.php. NOTE: All paths must have a trailing
// slash! NOTE: Seriously: mess with these at your own risk!

// This folder needs to by accessible by PHP only.
$conf_subdir = 'qp_config/'; // relative to base
$conf_path = str_replace( '\\', '/', dirname(__FILE__) ).'/';

/**
 * Base Path
 *
 * Made [i]nsensitive to case because of Windows URL oddities.
 * @global string $basepath
 */
$basepath = preg_replace( '#/'.$conf_subdir.'$#i', '', $conf_path ).'/';

// This folder may or may not need to be accessible by PHP and/or through HTTP.
// Exact requirements depend on $public_access_to_media
$media_subdir = 'media/';
$media_path = $basepath.$media_subdir;
$media_url = $app_baseurl.$media_subdir;

// This folder must be writable by PHP
$cache_subdir = 'qp_cache/';
$cache_path = $basepath.$cache_subdir;

// This folder needs to by accessible by PHP only
$inc_subdir = 'qp_inc/'; // relative to base
$inc_path = $basepath.$inc_subdir;
$inc_path_misc = $inc_path.'_misc/';

// This folder can be deleted after installation or upgrading is complete
$install_subdir = 'qp_install/'; // relative to base

// This folder needs to by accessible by PHP and may need to be accessible through HTTP
// Exact requirements depend on future uses like localized icons
$locales_subdir = 'qp_locales/'; // relative to base
$locales_path = $basepath.$locales_subdir;

// This folder needs to by accessible by PHP and should be accessible through HTTP
// Exact requirements depend on installed plugins
$plugins_subdir = 'qp_plugins/'; // relative to base
$plugins_path = $basepath.$plugins_subdir;
$plugins_url = $app_baseurl.$plugins_subdir;

// This folder needs to by accessible through HTTP
$rsc_subdir = 'qp_rsc/'; // relative to base
$rsc_path = $basepath.$rsc_subdir;
$rsc_url = $app_baseurl.$rsc_subdir;

// This folder needs to by accessible through HTTP
$srvc_subdir = 'qp_srvc/'; // relative to base
$srvc_path = $basepath.$srvc_subdir;
$srvc_url = $app_baseurl.$srvc_subdir;
$srvc_url_sensitive = $srvc_url;

$templates_subdir = 'qp_templates/'; // relative to base
$templates_path = $basepath.$templates_subdir;
$templates_url = $app_baseurl.$templates_subdir;

// This folder needs to by accessible by PHP and through HTTP
$admintemplates_subdir = 'qp_view_admin/'; // Subdirectory relative to ADMIN
$admintemplates_path = $basepath.$admintemplates_subdir;
$admintemplates_url = $app_baseurl.$admintemplates_subdir;

/**
 * Dispatcher
 *
 * Location of the admin interface dispatcher
 * @global string $dispatcher
 */
$dispatcher = 'admin.php'; // relative to base

$admin_url = $app_baseurl.$dispatcher;

?>
