<?php
/**
 * This file loads and initializes the blog to be displayed
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package core
 */
if(!defined('QP_CONFIG_LOADED')) die('fail');

/**
 * Initialize everything
 */
require_once dirname(__FILE__).'/_main.inc.php';

load_funcs( 'templates/_template.funcs.php' );

$Timer->start( '_blog_main.inc' );

// blog ID. This is a little bit special. In most cases $blog should be set by a
// stub file and the param() call below will just check that it's an integer. Note
// we do NOT memorize the param as we don't want it in regenerate_url() calls. Whenever
// we do, index.php will already have called param() with memorize=true In some cases
// $blog will not have been set before and it will be set with the param() call below.
// Currently, this only happens with the old /qp_srvc/ RSS stubs.
param( 'blog', 'integer', '', false );

// Getting current blog info
$BlogCache = & get_Cache( 'BlogCache' );
$Blog = & $BlogCache->get_by_ID( $blog, false, false );
if( empty( $Blog ) )
{
	// error & exit
	require $templates_path.'_404_blog_not_found.main.php';
}

// Init $disp
param( 'disp', 'string', 'posts', true );
$disp_detail = '';

// Locale / Charset for the Blog - activate matching locale
locale_activate( $Blog->get( 'locale' ) );

// Re-Init charset handling, in case current_charset has changed
if( init_charsets( $current_charset ) )
{
	// Reload Blog(s) (for encoding of name, tagline etc)
	$BlogCache->clear();
	$Blog = & $BlogCache->get_by_ID( $blog );
}

// Extra path info decoding. This will translate extra path into 'regular' params.
// Decoding should try to work like this:
// - baseurl/blog-urlname/junk/.../junk/post-title -> points to a single post (no ending slash)
// - baseurl/blog-urlname/junk/.../junk/p142 -> points to a single post
// - baseurl/blog-urlname/2006/ -> points to a yearly archive because of ending slash + 4 digits
// - baseurl/blog-urlname/2006/12/ -> points to a monthly archive
// - baseurl/blog-urlname/2006/12/31/ -> points to a daily archive
// - baseurl/blog-urlname/2006/w53/ -> points to a weekly archive
// - baseurl/blog-urlname/junk/.../junk/cat-urlname/ -> points to a single category (because of ending slash)
// Note: category names cannot be named like this [a-z][0-9]+
if( ! isset( $resolve_extra_path ) )
{
	$resolve_extra_path = true;
}

if( $resolve_extra_path )
{
	// Check and Remove blog base URI from ReqPath
	$blog_baseuri = substr( $Blog->gen_baseurl(), strlen( $Blog->get( 'baseurlroot' ) ) );

	// Remove trailer
	$blog_baseuri_regexp = preg_replace( '(\.php[0-9]?)?/?$', '', $blog_baseuri );
	// Read possibilities in order to get a broad match
	$blog_baseuri_regexp = '^'.preg_quote( $blog_baseuri_regexp ).'(\.php[0-9]?)?/(.+)$';

	if( preg_match( $blog_baseuri_regexp, $ReqPath, $matches ) )
	{
		// We have extra path info
		$path_string = $matches[2];
		// Replace encoded ";" and ":" with regular chars (used for tags)
		$path_string = str_replace(
			array( '%3b', '%3B', '%3a', '%3A' ),
			array( ';', ';', ':', ':' ),
			$path_string );

		// Slice the path
		$path_elements = preg_split( '~/~', $path_string, 20, PREG_SPLIT_NO_EMPTY );

		if( isset( $path_elements[0] )
			&& ( $path_elements[0] == $Blog->stub
				|| $path_elements[0] == $Blog->urlname
				|| $path_elements[0] == 'index.php' ) )
		{
			// Ignore stub files, blog url names, and index.php
			array_shift( $path_elements );
		}

		if( isset( $path_elements[0] )
			&& ( substr_count( $path_elements[0], '.' ) != 0 )
			&& is_file( $basepath.$path_elements[0] ) )
		{
			// Ignore filenames that exist in root
			array_shift( $path_elements );
		}

		// Do we still have extra path info to decode?
		if( count( $path_elements ) )
		{
			// Is this a tag ("prefix-only" mode)?
			if( $Blog->get_setting( 'tag_links' ) == 'prefix-only'
				&& count( $path_elements ) == 2
				&& $path_elements[0] == $Blog->get_setting( 'tag_prefix' )
				&& isset( $path_elements[1] ) )
			{
				$tag = strip_tags( urldecode( $path_elements[1] ) );
				// # of posts per page for tag page
				if( ! $posts = $Blog->get_setting( 'tag_posts_per_page' ) )
				{
					// use blog default
					$posts = $Blog->get_setting( 'posts_per_page' );
				}
			}
			else
			{
				// Does the pathinfo end with a / or a ; ?
				$last_char = substr( $path_string, -1 );
				$last_part = $path_elements[count( $path_elements )-1];
				$last_len = strlen( $last_part );
				if( ( $last_char == '-' && ( ! $tags_dash_fix || $last_len != 40 ) ) || $last_char == ':'|| $last_char == ';' )
				{
					// - : or ; -> We'll consider this to be a tag page
					$tag = substr( $last_part, 0, -1 );
					$tag = urldecode( $tag );
					$tag = strip_tags( $tag ); // security

					// # of posts per page:
					if( ! $posts = $Blog->get_setting( 'tag_posts_per_page' ) )
					{
						// use blog default
						$posts = $Blog->get_setting( 'posts_per_page' );
					}
				}
				elseif( ( $tags_dash_fix && $last_char == '-' && $last_len == 40 ) || $last_char != '/' )
				{
					// NO ENDING SLASH or ends with a dash, is 40 chars long and $tags_dash_fix is true
					// -> We'll consider this to be a reference to a post.

					// Set a lot of defaults as if we had received a complex URL
					$m = '';
					$more = 1; // Display the extended entries' text
					$c = 1; // Display comments
					$tb = 1; // Display trackbacks

					if( preg_match( '#^p([0-9]+)$#', $last_part, $req_post ) )
					{
						// The last param is of the form p000
						$p = $req_post[1]; // Post to display
					}
					else
					{
						// Last param is a string, we'll consider this to be a post urltitle
						$title = $last_part;
					}
				}
				else
				{
					// ENDING SLASH -> we are looking for a daterange OR a category
					if( preg_match( '|^w?[0-9]+$|', $last_part ) )
					{
						// Last part is a number or a "week" number
						$i=0;
						if( isset( $path_elements[$i] ) )
						{
							if( is_numeric( $path_elements[$i] ) )
							{
								// We'll consider this to be the year
								$m = $path_elements[$i++];

								// Also use the prefered posts per page for archives (may be NULL,
								// in which case the blog default will be used later on)
								if( ! $posts = $Blog->get_setting( 'archive_posts_per_page' ) )
								{
									// use blog default
									$posts = $Blog->get_setting( 'posts_per_page' );
								}

								if( isset( $path_elements[$i] ) && is_numeric( $path_elements[$i] ) )
								{
									// We'll consider this to be the month
									$m .= $path_elements[$i++];

									if( isset( $path_elements[$i] ) && is_numeric( $path_elements[$i] ) )
									{
										// We'll consider this to be the day
										$m .= $path_elements[$i++];
									}
								}
								elseif( isset( $path_elements[$i] ) && substr( $path_elements[$i], 0, 1 ) == 'w' )
								{
									// We consider this a week number
									$w = substr( $path_elements[$i], 1, 2 );
								}
							}
							else
							{
								// We did not get a number/year...
								$disp = '404';
								$disp_detail = '404-malformed_url-missing_year';
							}
						}
					}
					elseif( preg_match( '|^[A-Za-z0-9\-_]+$|', $last_part ) ) // UNDERSCORES for catching OLD URLS!!!
					{
						// We are pointing to a category
						$CategoryCache = & get_Cache( 'CategoryCache' );
						$Category = & $CategoryCache->get_by_urlname( $last_part, false );
						if( empty( $Category ) )
						{
							// We could not match a category; consider this to be a post title with a
							// misplaced trailing slash. That happens when upgrading from WP for example.
							$title = $last_part; // Will be sought later
							$already_looked_into_cats = true;
						}
						else
						{
							// We could match a category from the extra path
							$cat = $Category->ID;
							// Also use the prefered posts per page for a cat
							if( ! $posts = $Blog->get_setting( 'category_posts_per_page' ) )
							{
								// use blog default
								$posts = $Blog->get_setting( 'posts_per_page' );
							}
						}
					}
					else
					{
						// We did not get anything we can decode...
						$disp = '404';
						$disp_detail = '404-malformed_url-bad_char';
					}
				}
			}
		}
	}
}

// Query params ... Note: if the params have been set by the extra-path-info
// above, param() will not touch them.

// Specific post number to display
param( 'p', 'integer', '', true );
// urtitle of post to display
param( 'title', 'string', '', true );
// Do we allow redirection to canonical URL? (allows to force a 'single post' URL for commenting)
param( 'redir', 'string', 'yes', false );
// Is this preview ?
param( 'preview', 'integer', 0, true );
// Deprecated but might still be used by spambots
param( 'stats', 'integer', 0 );
// category ID
param( 'cat', 'integer', '', true );

// In case these were not set by the stub
if( ! isset( $timestamp_min ) ) $timestamp_min = '';
if( ! isset( $timestamp_max ) ) $timestamp_max = '';

// Get specific Item if requested
if( ! empty( $p ) || ! empty( $title ) )
{
	// We are going to display a single post
	// Make sure the single post we're requesting (still) exists
	$ItemCache = & get_Cache( 'ItemCache' );
	if( ! empty( $p ) )
	{
		// Get from post ID
		$Item = & $ItemCache->get_by_ID( $p, false );
	}
	else
	{
		// Get from post title
		$orig_title = $title;
		$title = preg_replace( '/[^A-Za-z0-9_]/', '-', $title );
		$Item = & $ItemCache->get_by_urltitle( $title, false );
	}
	if( empty( $Item ) )
	{
		// Post doesn't exist!
		// fp> TODO: ->viewing_allowed() for draft, private, protected and deprecated...
		$title_fallback = false;
		$tag_fallback = ( $tags_dash_fix && substr( $orig_title, -1 ) == '-' && strlen( $orig_title ) == 40 );

		if( ! $tag_fallback && ! empty( $title ) && empty( $already_looked_into_cats ) )
		{
			// Let's try to fall back to a category...
			$CategoryCache = & get_Cache( 'CategoryCache' );
			$Category = & $CategoryCache->get_by_urlname( $title, false );
			if( ! empty( $Category ) )
			{
				// We could match a category from the extra path
				$cat = $Category->ID;
				$title_fallback = true;
				$title = NULL;
				// Also use the prefered posts per page for a cat
				if( ! $posts = $Blog->get_setting( 'category_posts_per_page' ) )
				{
					// use blog default
					$posts = $Blog->get_setting( 'posts_per_page' );
				}
			}
		}

		if( ! empty( $title ) )
		{
			// Let's try to fall back to a tag...
			if( $tag_fallback )
			{
				$title = substr( $orig_title, 0, -1 );
			}
			if( $Blog->get_tag_post_count( $title ) )
			{
				// We could match a tag from the extra path
				$tag = $title;
				$title_fallback = true;
				$title = NULL;
			}
		}

		if( ! $title_fallback )
		{
			// We were not able to fallback to anything meaningful
			$disp = '404';
			$disp_detail = '404-post_not_found';
		}
	}
}
elseif( ! empty( $cat ) )
{
	// We are pointing to a category (by ID)
	$CategoryCache = & get_Cache( 'CategoryCache' );
	$Category = & $CategoryCache->get_by_ID( $cat, false );

	if( ! $Category )
	{
		// Category not found, bail out
		$disp = '404';
		$disp_detail = '404-category-not-found';
	}
}

// "Clean up" the request
// Make sure that:
// 1) disp is set to "single" if single post requested
// 2) URL is canonical if:
// - some content was requested in a weird/deprecated way
// - or if content identifiers have changed
if( $stats || $disp == 'stats' )
{
	// This used to be a spamfest, error & exit
	require $templates_path.'_410_stats_gone.main.php';
}
elseif( ! empty( $preview ) )
{
	// Preview
	$disp = 'single';
	// Consider this as an admin hit!
	$Hit->referer_type = 'admin';
}
elseif( $disp == 'posts' && ! empty( $Item ) )
{
	// We are going to display a single post
	if( $Item->ptyp_ID == 1000 )
	{
		// this is a page-type
		$disp = 'page';
	}
	elseif( in_array( $Item->ptyp_ID, array( 1500, 1520, 1530, 1570 ) ) )
	{
		// these are intro-types
		$disp = 'intro';
	}
	else
	{
		// must be good old permalink single post
		$disp = 'single';
	}
}

// DETERMINE WHICH TEMPLATE TO USE FOR DISPLAY
// Check if a temporary template has been requested (used for RSS syndication for example).
// This will be handled like any other template. tempskin is also checked for b2evolution
// backward compatibility.

if( param( 'viewmode', 'string', param( 'tempskin', 'string' ), true ) !== '' )
{
	$template = $viewmode;
}

if( isset( $template ) )
{
	// A template has been requested by folder_name (url or stub)
	// Check validity of requested template name
	if( preg_match( '~([^-A-Za-z0-9._]|\.\.)~', $template ) )
	{
		debug_die( 'The requested template name is invalid.' );
	}

	$TemplateCache = & get_cache( 'TemplateCache' );
	$Template = & $TemplateCache->new_obj( NULL, $template );

	if( $Template->type == 'feed' )
	{
		// Check if we actually allow the display of the feed; last chance to revert
		// to 404 displayed in default template. Note: Templates with the type "feed"
		// can always be accessed, even when they're not installed.
		if( $Blog->get_setting( 'feed_content' ) == 'none' )
		{
			// We don't want to provide feeds; revert to 404!
			unset( $template );
			unset( $Template );
			$disp = '404';
			$disp_detail = '404-feeds-disabled';
		}
	}
	elseif( template_exists( $template ) && ! template_installed( $template ) )
	{
		// The requested template is not a feed and exists, but isn't installed
		debug_die( sprintf( T_( 'The template [%s] is not installed on this system.' ), htmlspecialchars( $template ) ) );
	}
	else if( ! empty( $viewmode ) )
	{
		// By definition, we want to see the temporary template (if we don't use feedburner...)
		$redir = 'no';
	}
}

// Note: if $template is set to '', then we want to do a "no template" display
if( ! isset( $template ) && ! empty( $Blog->template_ID ) )
{
	// Use default template from the database
	$TemplateCache = & get_cache( 'TemplateCache' );
	$Template = & $TemplateCache->get_by_ID( $Blog->template_ID );
	$template = $Template->folder;
}

// we have to make this fool proof with extra checking ... if someone deletes a template
if( ! empty( $template ) && ! template_exists( $template ) )
{
	// We want to use a template, but it doesn't exist!
	$err_msg = sprintf( T_('The template [%s] set for blog [%s] does not exist. It must be properly set in the <a %s>blog properties</a> or properly overriden in a stub file.'),
		htmlspecialchars( $template ),
		$Blog->dget( 'shortname' ),
		'href="'.$admin_url.'?ctrl=blog_settings&amp;tab=template&amp;blog='.$Blog->ID.'"' );
	debug_die( $err_msg );
}

$Timer->pause( '_blog_main.inc');

// READY TO DISPLAY :)
// At this point $template holds the name of the template we want to use, or ''
// for no template ... which probably doesn't work at all so test it and fix it.

// Trigger plugin event
// fp> TODO: please doc with example of what this can be used for
$Plugins->trigger_event( 'BeforeBlogDisplay', array(
	'template' => $template
) );

if( ! empty( $template ) )
{
	// We want to display with a template now

	// Instanciate PageCache
	$PageCache = new PageCache( $Blog );
	// Check for cached content & Start caching if needed Note: there are some redirects
	// inside the templates themselves for canonical URLs, If we have a cache hit, the
	// redirect won't take place until the cache expires -- probably ok. If we start
	// collecting and a redirect happens, the collecting will just be lost and that's what we want.
	if( ! $PageCache->check() )
	{
		// Cache miss, we have to generate
		if( $template_provided_by_plugin = template_provided_by_plugin( $template ) )
		{
			$Plugins->call_method( $template_provided_by_plugin, 'DisplayTemplate', $tmp_params = array( 'template' => $template ) );
		}
		else
		{
			// Path for the current template
			$ads_current_template_path = $templates_path.$template.'/';

			$disp_handlers = array(
					'404' => '404_not_found.main.php',
					'arcdir' => 'arcdir.main.php',
					'catdir' => 'catdir.main.php',
					'comments' => 'comments.main.php',
					'credits' => 'credits.main.php',
					'feedback-popup' => 'feedback_popup.main.php',
					'intro' => 'intro.main.php',
					'mediaidx' => 'mediaidx.main.php',
					'msgform' => 'msgform.main.php',
					'page' => 'page.main.php',
					'posts' => 'posts.main.php',
					'profile' => 'profile.main.php',
					'single' => 'single.main.php',
					'subs' => 'subs.main.php',
					// All others will default to index.main.php
				);

			if( ! empty( $disp_handlers[$disp] ) )
			{
				if( file_exists( $disp_handler = $ads_current_template_path.$disp_handlers[$disp] ) )
				{
					/**
					 * The template has a customized page handler for this display
					 */
					require $disp_handler;
				}
				else
				{
					/**
					 * Use the default handler from the templates dir
					 */
					require $ads_current_template_path.'index.main.php';
				}
			}
			else
			{
				/**
				 * Use the default handler from the templates dir
				 */
				require $ads_current_template_path.'index.main.php';
			}
		}

		// Save collected cached data if needed
		$PageCache->end_collect();
	}

	// We probably don't want to return to the caller if we have displayed a template...
	// That is useful if the caller implements a custom display but we still use templates for RSS/ Atom etc..
	exit(0);
}
else
{
	// We don't use a template. Hopefully the caller will do some displaying.
	// Set a few vars with default values, just in case...
	$ads_current_template_path = $srvc_path;

	// We'll just return to the caller now... (if we have not used a template, the 
	// caller should do the display after this)
}

?>
