<?php
/**
 * This is the install file for the core modules
 *
 * $schema_queries gets updated through {@link db_delta()} which generates the
 * queries needed to get to this scheme. Please see {@link db_delta()} for things
 * to take care of.
 *
 * T_collections (collections table) and T_item_colls (items->collections table)
 * are not currently in use by core and might never be! The thinking was to provide
 * a way to 'bind' items together similar to categories, but not limited to a blog
 * and not required. Items with images attached might be a 'collection' called a
 * photo album for example.
 *
 * T_user_cyberspace (user cyberspace table) and T_user_realworld (users realworld
 * table) are not currently in use by core and might never be! The thinking was
 * we would provide core code that plugins could 'activate' so that multiple plugins
 * wouldn't have to ask for the same data about each user. If any plugin asks for
 * facebook page or city, then the next plugin wouldn't have to because the info
 * is already stored ... assuming each user provided the info for the first.
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package core
 */
if(!defined('QP_CONFIG_LOADED')) die('fail');

global $db_storage_charset;

/**
 * The database scheme
 *
 * This gets updated through {@link db_delta()} which generates the queries needed
 * to get to this scheme. Please see {@link db_delta()} for things to take care of.
 */
$schema_queries = array(
	'T_antispam' => array( 'Creating table for antispam',
		"CREATE TABLE T_antispam (
			aspm_ID bigint(11) NOT NULL auto_increment,
			aspm_string VARCHAR(80) NOT NULL,
			aspm_source ENUM('core','comment','entered','referer','shared') NOT NULL default 'core',
			aspm_shortened TINYINT(1) NOT NULL default 0,
			aspm_added_on DATETIME NOT NULL DEFAULT '2011-12-20 00:00:00',
			aspm_lastseen DATETIME NOT NULL DEFAULT '2011-12-20 00:00:00',
			aspm_comments INT(11) unsigned NOT NULL default '0',
			aspm_messages INT(11) unsigned NOT NULL default '0',
			aspm_referers INT(11) unsigned NOT NULL default '0',
			PRIMARY KEY aspm_ID ( aspm_ID ),
			UNIQUE aspm_string ( aspm_string )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

	'T_blogs' => array( 'Creating table for blogs',
		"CREATE TABLE T_blogs (
			blog_ID INT(11) unsigned NOT NULL auto_increment,
			blog_shortname VARCHAR(255) NULL default '',
			blog_name VARCHAR(255) NOT NULL default '',
			blog_owner_user_ID INT(11) unsigned NOT NULL default 1,
			blog_advanced_perms TINYINT(1) NOT NULL default 0,
			blog_tagline VARCHAR(250) NULL default '',
			blog_description VARCHAR(250) NULL default '',
			blog_longdesc TEXT NULL DEFAULT NULL,
			blog_locale VARCHAR(20) NOT NULL DEFAULT 'en-US',
			blog_access_type VARCHAR(10) NOT NULL DEFAULT 'index.php',
			blog_siteurl VARCHAR(120) NOT NULL default '',
			blog_urlname VARCHAR(255) NOT NULL DEFAULT 'urlname',
			blog_notes TEXT NULL DEFAULT NULL,
			blog_keywords tinytext,
			blog_allowcomments VARCHAR(20) NOT NULL default 'post_by_post',
			blog_allowtrackbacks TINYINT(1) NOT NULL default 0,
			blog_allowblogcss TINYINT(1) NOT NULL default 1,
			blog_allowusercss TINYINT(1) NOT NULL default 1,
			blog_template_ID INT(10) UNSIGNED NOT NULL DEFAULT 1,
			blog_template_type ENUM('xhtml','html5','feed','other') NOT NULL default 'xhtml',
			blog_in_bloglist TINYINT(1) NOT NULL DEFAULT 1,
			blog_media_location ENUM('default','subdir','custom','none') DEFAULT 'default' NOT NULL,
			blog_media_subdir VARCHAR(255) NULL,
			blog_media_fullpath VARCHAR(255) NULL,
			blog_media_url VARCHAR(255) NULL,
			blog_UID VARCHAR(20),
			PRIMARY KEY blog_ID ( blog_ID ),
			UNIQUE KEY blog_urlname ( blog_urlname )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

	'T_blog_groups' => array( 'Creating blog->group permissions table',
		"CREATE TABLE T_blog_groups (
			bloggroup_blog_ID INT(11) unsigned NOT NULL default 0,
			bloggroup_group_ID INT(11) unsigned NOT NULL default 0,
			bloggroup_ismember TINYINT NOT NULL default 0,
			bloggroup_perm_poststatuses set('published','deprecated','protected','private','draft','redirected') NOT NULL default '',
			bloggroup_perm_edit ENUM('no','own','lt','le','all','redirected') NOT NULL default 'no',
			bloggroup_perm_delpost TINYINT NOT NULL default 0,
			bloggroup_perm_comments TINYINT NOT NULL default 0,
			bloggroup_perm_cats TINYINT NOT NULL default 0,
			bloggroup_perm_properties TINYINT NOT NULL default 0,
			bloggroup_perm_admin TINYINT NOT NULL default 0,
			bloggroup_perm_media_upload TINYINT NOT NULL default 0,
			bloggroup_perm_media_browse TINYINT NOT NULL default 0,
			bloggroup_perm_media_change TINYINT NOT NULL default 0,
			PRIMARY KEY bloggroup_pk ( bloggroup_blog_ID, bloggroup_group_ID )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

	'T_blog_settings' => array( 'Creating blog settings table',
		"CREATE TABLE T_blog_settings (
			cset_blog_ID INT(11) UNSIGNED NOT NULL,
			cset_name VARCHAR(30) NOT NULL,
			cset_value VARCHAR(255) NULL,
			PRIMARY KEY ( cset_blog_ID, cset_name )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

	'T_blog_users' => array( 'Creating blog->user permissions table',
		"CREATE TABLE T_blog_users (
			bloguser_blog_ID INT(11) unsigned NOT NULL default 0,
			bloguser_user_ID INT(11) unsigned NOT NULL default 0,
			bloguser_ismember TINYINT NOT NULL default 0,
			bloguser_perm_poststatuses set('published','deprecated','protected','private','draft','redirected') NOT NULL default '',
			bloguser_perm_edit ENUM('no','own','lt','le','all','redirected') NOT NULL default 'no',
			bloguser_perm_delpost TINYINT NOT NULL default 0,
			bloguser_perm_comments TINYINT NOT NULL default 0,
			bloguser_perm_cats TINYINT NOT NULL default 0,
			bloguser_perm_properties TINYINT NOT NULL default 0,
			bloguser_perm_admin TINYINT NOT NULL default 0,
			bloguser_perm_media_upload TINYINT NOT NULL default 0,
			bloguser_perm_media_browse TINYINT NOT NULL default 0,
			bloguser_perm_media_change TINYINT NOT NULL default 0,
			PRIMARY KEY bloguser_pk ( bloguser_blog_ID, bloguser_user_ID )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

	'T_categories' => array( 'Creating categories table',
		"CREATE TABLE T_categories (
			cat_ID INT(10) unsigned NOT NULL auto_increment,
			cat_parent_ID INT(10) unsigned NULL,
			cat_name VARCHAR(255) NOT NULL,
			cat_urlname VARCHAR(255) NOT NULL,
			cat_blog_ID INT(10) unsigned NOT NULL default 1,
			cat_description VARCHAR(255) NULL DEFAULT NULL,
			cat_order INT(11) NULL DEFAULT NULL,
			PRIMARY KEY cat_ID ( cat_ID ),
			UNIQUE cat_urlname( cat_urlname ),
			KEY cat_blog_ID ( cat_blog_ID ),
			KEY cat_parent_ID ( cat_parent_ID ),
			KEY cat_order ( cat_order )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

	'T_collections' => array( 'Creating collections table',
		"CREATE TABLE T_collections (
			coll_ID INT(10) unsigned NOT NULL auto_increment,
			coll_parent_ID INT(10) unsigned NULL,
			coll_name VARCHAR(255) NOT NULL,
			coll_urlname VARCHAR(255) NOT NULL,
			coll_blog_ID INT(10) unsigned NOT NULL default 1,
			coll_description VARCHAR(255) NULL DEFAULT NULL,
			coll_order INT(11) NULL DEFAULT NULL,
			PRIMARY KEY coll_ID ( coll_ID ),
			UNIQUE coll_urlname ( coll_urlname ),
			KEY coll_blog_ID ( coll_blog_ID ),
			KEY coll_parent_ID ( coll_parent_ID ),
			KEY coll_order( coll_order )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

	'T_comments' => array( 'Creating comments table',
		"CREATE TABLE T_comments (
			comment_ID INT(11) unsigned NOT NULL auto_increment,
			comment_post_ID INT(11) unsigned NOT NULL default '0',
			comment_type ENUM('comment','linkback','trackback') NOT NULL default 'comment',
			comment_status ENUM('published','deprecated','protected','private','draft','redirected') DEFAULT 'published' NOT NULL,
			comment_author_ID INT unsigned NULL default NULL,
			comment_author VARCHAR(100) NULL,
			comment_author_email VARCHAR(255) NULL,
			comment_author_url VARCHAR(255) NULL,
			comment_author_IP VARCHAR(23) NOT NULL default '',
			comment_date DATETIME NOT NULL DEFAULT '2000-01-01 00:00:00',
			comment_content text NOT NULL,
			comment_rating TINYINT(1) NULL DEFAULT NULL,
			comment_featured TINYINT(1) NOT NULL DEFAULT 0,
			comment_nofollow TINYINT(1) NOT NULL DEFAULT 1,
			comment_karma INT(11) NOT NULL DEFAULT 0,
			comment_spam_karma TINYINT NULL,
			comment_allow_msgform TINYINT NOT NULL DEFAULT 0,
			PRIMARY KEY comment_ID ( comment_ID ),
			KEY comment_post_ID ( comment_post_ID ),
			KEY comment_date ( comment_date ),
			KEY comment_type ( comment_type )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

	'T_cron_log' => array( 'Creating cron log table',
		"CREATE TABLE T_cron_log (
			clog_ctsk_ID INT(10) unsigned not null,
			clog_realstart_datetime DATETIME not null DEFAULT '2000-01-01 00:00:00',
			clog_realstop_datetime DATETIME,
			clog_status ENUM('started','finished','error','timeout') not null default 'started',
			clog_messages text,
			PRIMARY KEY ( clog_ctsk_ID )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

	'T_cron_task' => array( 'Creating cron tasks table',
		"CREATE TABLE T_cron_task (
			ctsk_ID INT(10) unsigned not null AUTO_INCREMENT,
			ctsk_start_datetime DATETIME not null DEFAULT '2000-01-01 00:00:00',
			ctsk_repeat_after INT(10) unsigned,
			ctsk_name VARCHAR(50) not null,
			ctsk_controller VARCHAR(50) not null,
			ctsk_params text,
			PRIMARY KEY ( ctsk_ID )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

	'T_files' => array( 'Creating files table',
		"CREATE TABLE T_files (
			file_ID INT(11) unsigned not null AUTO_INCREMENT,
			file_root_type ENUM('absolute','user','blog','shared','templates') not null default 'absolute',
			file_root_ID INT(11) unsigned not null default 0,
			file_path VARCHAR(255) not null default '',
			file_title VARCHAR(255),
			file_alt VARCHAR(255),
			file_desc text,
			PRIMARY KEY ( file_ID ),
			unique file ( file_root_type, file_root_ID, file_path )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

	'T_filetypes' => array( 'Creating file types table',
		"CREATE TABLE T_filetypes (
			ftyp_ID INT(11) unsigned NOT NULL auto_increment,
			ftyp_extensions VARCHAR(30) NOT NULL,
			ftyp_name VARCHAR(30) NOT NULL,
			ftyp_mimetype VARCHAR(50) NOT NULL,
			ftyp_icon VARCHAR(20) default NULL,
			ftyp_viewtype VARCHAR(10) NOT NULL,
			ftyp_allowed TINYINT(1) NOT NULL default 0,
			PRIMARY KEY ( ftyp_ID )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

	'T_groups' => array( 'Creating groups table',
		"CREATE TABLE T_groups (
			grp_ID INT(11) NOT NULL auto_increment,
			grp_name VARCHAR(50) NOT NULL default '',
			grp_perm_admin ENUM('none','hidden','visible') NOT NULL default 'visible',
			grp_perm_blogs ENUM('user','viewall','editall') NOT NULL default 'user',
			grp_perm_bypass_antispam TINYINT(1) NOT NULL DEFAULT 0,
			grp_perm_xhtmlvalidation VARCHAR(10) NOT NULL default 'always',
			grp_perm_xhtmlvalidation_xmlrpc VARCHAR(10) NOT NULL default 'always',
			grp_perm_xhtml_css_tweaks TINYINT(1) NOT NULL DEFAULT 0,
			grp_perm_xhtml_iframes TINYINT(1) NOT NULL DEFAULT 0,
			grp_perm_xhtml_javascript TINYINT(1) NOT NULL DEFAULT 0,
			grp_perm_xhtml_objects TINYINT(1) NOT NULL DEFAULT 0,
			grp_perm_stats ENUM('none','user','view','edit') NOT NULL default 'none',
			grp_perm_spamblacklist ENUM('none','view','edit') NOT NULL default 'none',
			grp_perm_options ENUM('none','view','edit') NOT NULL default 'none',
			grp_perm_users ENUM('none','view','edit') NOT NULL default 'none',
			grp_perm_templates TINYINT NOT NULL DEFAULT 0,
			grp_perm_files ENUM('none','view','add','edit','all') NOT NULL default 'none',
			PRIMARY KEY grp_ID ( grp_ID )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

	'T_items' => array( 'Creating items table',
		"CREATE TABLE T_items (
			post_ID INT(11) unsigned NOT NULL auto_increment,
			post_parent_ID INT(11) unsigned NULL,
			post_creator_user_ID INT(11) unsigned NOT NULL,
			post_lastedit_user_ID INT(11) unsigned NULL,
			post_assigned_user_ID INT(11) unsigned NULL,
			post_datestart DATETIME NOT NULL DEFAULT '2000-01-01 00:00:00',
			post_datedeadline DATETIME NULL,
			post_datecreated DATETIME NULL,
			post_datemodified DATETIME NOT NULL DEFAULT '2000-01-01 00:00:00',
			post_status ENUM('published','deprecated','protected','private','draft','redirected') NOT NULL default 'published',
			post_pst_ID INT(11) unsigned NULL,
			post_ptyp_ID INT(10) unsigned NOT NULL DEFAULT 1,
			post_locale VARCHAR(20) NOT NULL DEFAULT 'en-EU',
			post_content MEDIUMTEXT NULL,
			post_excerpt TEXT NULL DEFAULT NULL,
			post_title text NOT NULL,
			post_urltitle VARCHAR(210) NULL DEFAULT NULL,
			post_titletag VARCHAR(255) NULL DEFAULT NULL,
			post_metadesc VARCHAR(255) NULL DEFAULT NULL,
			post_metakeywords VARCHAR(255) NULL DEFAULT NULL,
			post_url VARCHAR(255) NULL DEFAULT NULL,
			post_main_cat_ID INT(11) unsigned NOT NULL,
			post_notifications_status ENUM('noreq','todo','started','finished') NOT NULL DEFAULT 'noreq',
			post_notifications_ctsk_ID INT(10) unsigned NULL DEFAULT NULL,
			post_views INT(11) UNSIGNED NOT NULL DEFAULT 0,
			post_wordcount INT(11) default NULL,
			post_comment_status ENUM('disabled', 'open', 'closed') NOT NULL DEFAULT 'open',
			post_commentsexpire DATETIME DEFAULT NULL,
			post_renderers TEXT NOT NULL,
			post_priority INT(11) unsigned null COMMENT 'Task priority in workflow',
			post_featured TINYINT(1) NOT NULL DEFAULT 0,
			post_attachments TINYINT(1) NOT NULL DEFAULT 0,
			post_order DOUBLE NULL,
			post_editor_code VARCHAR(32) NULL COMMENT 'Plugin code of the editor used to edit this post',
			PRIMARY KEY post_ID( post_ID ),
			UNIQUE post_urltitle( post_urltitle ),
			INDEX post_datestart( post_datestart ),
			INDEX post_main_cat_ID( post_main_cat_ID ),
			INDEX post_creator_user_ID( post_creator_user_ID ),
			INDEX post_status( post_status ),
			INDEX post_parent_ID( post_parent_ID ),
			INDEX post_assigned_user_ID( post_assigned_user_ID ),
			INDEX post_ptyp_ID( post_ptyp_ID ),
			INDEX post_pst_ID( post_pst_ID ),
			INDEX post_order( post_order )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

	'T_item_cats' => array( 'Creating items->categories table',
		"CREATE TABLE T_item_cats (
			postcat_post_ID INT(11) unsigned NOT NULL,
			postcat_cat_ID INT(11) unsigned NOT NULL,
			PRIMARY KEY postcat_pk ( postcat_post_ID, postcat_cat_ID ),
			UNIQUE catpost ( postcat_cat_ID, postcat_post_ID )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

	'T_item_colls' => array( 'Creating items->collections table',
		"CREATE TABLE T_item_colls(
			itemcoll_post_ID INT(11) unsigned NOT NULL,
			itemcoll_coll_ID INT(11) unsigned NOT NULL,
			PRIMARY KEY itemcoll_pk ( itemcoll_post_ID, itemcoll_coll_ID ),
			UNIQUE collitem( itemcoll_coll_ID, itemcoll_post_ID )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

	'T_item_prerendering' => array( 'Creating item prerendering table',
		"CREATE TABLE T_item_prerendering(
			itpr_itm_ID INT(11) UNSIGNED NOT NULL,
			itpr_format ENUM('htmlbody','entityencoded','xml','text') NOT NULL,
			itpr_renderers TEXT NOT NULL,
			itpr_content_prerendered MEDIUMTEXT NULL,
			itpr_datemodified TIMESTAMP NOT NULL,
			PRIMARY KEY ( itpr_itm_ID, itpr_format )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

	'T_item_status' => array( 'Creating item status table',
		"CREATE TABLE T_item_status (
			pst_ID INT(11) unsigned not null AUTO_INCREMENT,
			pst_name VARCHAR(30) not null,
			PRIMARY KEY ( pst_ID )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

	'T_item_tags' => array( 'Creating items->tags table',
		"CREATE TABLE T_item_tags (
			itag_itm_ID INT(11) unsigned NOT NULL,
			itag_tag_ID INT(11) unsigned NOT NULL,
			PRIMARY KEY ( itag_itm_ID, itag_tag_ID ),
			UNIQUE tagitem ( itag_tag_ID, itag_itm_ID )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

	'T_item_types' => array( 'Creating item types table',
		"CREATE TABLE T_item_types (
			ptyp_ID INT(11) unsigned not null AUTO_INCREMENT,
			ptyp_name VARCHAR(30) not null,
			PRIMARY KEY ( ptyp_ID )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

	'T_item_versions' => array( 'Creating item versions table',
		"CREATE TABLE T_item_versions (
			iver_itm_ID INT UNSIGNED NOT NULL,
			iver_edit_user_ID INT UNSIGNED NULL,
			iver_edit_datetime DATETIME NOT NULL,
			iver_status ENUM('published','deprecated','protected','private','draft','redirected') NULL,
			iver_title TEXT NULL DEFAULT NULL,
			iver_content MEDIUMTEXT NULL,
			INDEX iver_itm_ID ( iver_itm_ID )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

	'T_links' => array( 'Creating links table',
		"CREATE TABLE T_links (
			link_ID INT(11) unsigned not null AUTO_INCREMENT,
			link_datecreated DATETIME not null DEFAULT '2000-01-01 00:00:00',
			link_datemodified DATETIME not null DEFAULT '2000-01-01 00:00:00',
			link_creator_user_ID INT(11) unsigned not null,
			link_lastedit_user_ID INT(11) unsigned not null,
			link_itm_ID INT(11) unsigned NOT NULL,
			link_dest_itm_ID INT(11) unsigned NULL,
			link_file_ID INT(11) unsigned NULL,
			link_ltype_ID INT(11) unsigned NOT NULL default 1,
			link_external_url VARCHAR(255) NULL,
			link_title TEXT NULL DEFAULT NULL,
			PRIMARY KEY ( link_ID ),
			INDEX link_itm_ID( link_itm_ID ),
			INDEX link_dest_itm_ID ( link_dest_itm_ID ),
			INDEX link_file_ID ( link_file_ID )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

	'T_locales' => array( 'Creating locales table',
		"CREATE TABLE T_locales (
			loc_locale VARCHAR(20) NOT NULL default '',
			loc_charset VARCHAR(15) NOT NULL default 'iso-8859-1',
			loc_datefmt VARCHAR(20) NOT NULL default 'y-m-d',
			loc_timefmt VARCHAR(20) NOT NULL default 'H:i:s',
			loc_startofweek TINYINT UNSIGNED NOT NULL DEFAULT 1,
			loc_name VARCHAR(40) NOT NULL default '',
			loc_messages VARCHAR(20) NOT NULL default '',
			loc_priority TINYINT(4) UNSIGNED NOT NULL default '0',
			loc_enabled TINYINT(4) NOT NULL default '1',
			PRIMARY KEY loc_locale ( loc_locale )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset COMMENT='saves available locales'
		" ),

	'T_plugins' => array( 'Creating plugins table',
		"CREATE TABLE T_plugins (
			plug_ID INT(11) UNSIGNED NOT NULL auto_increment,
			plug_priority TINYINT NOT NULL default 50,
			plug_classname VARCHAR(40) NOT NULL default '',
			plug_code VARCHAR(32) NULL,
			plug_apply_rendering ENUM('stealth','always','opt-out','opt-in','lazy','never') NOT NULL DEFAULT 'never',
			plug_version VARCHAR(42) NOT NULL default '0',
			plug_name VARCHAR(255) NULL default NULL,
			plug_shortdesc VARCHAR(255) NULL default NULL,
			plug_status ENUM('enabled','disabled','needs_config','broken') NOT NULL,
			plug_spam_weight TINYINT UNSIGNED NOT NULL DEFAULT 1,
			PRIMARY KEY ( plug_ID ),
			UNIQUE plug_code( plug_code ),
			INDEX plug_status( plug_status )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

	'T_plugin_events' => array( 'Creating plugin events table',
		"CREATE TABLE T_plugin_events (
			pevt_plug_ID INT(11) UNSIGNED NOT NULL,
			pevt_event VARCHAR(40) NOT NULL,
			pevt_enabled TINYINT NOT NULL DEFAULT 1,
			PRIMARY KEY ( pevt_plug_ID, pevt_event )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

	'T_plugin_settings' => array( 'Creating plugin settings table',
		"CREATE TABLE T_plugin_settings (
			pset_plug_ID INT(11) UNSIGNED NOT NULL,
			pset_name VARCHAR(30) NOT NULL,
			pset_value TEXT NULL DEFAULT NULL,
			PRIMARY KEY ( pset_plug_ID, pset_name )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

	'T_plugin_usersettings' => array( 'Creating plugin user settings table',
		"CREATE TABLE T_plugin_usersettings (
			puset_plug_ID INT(11) UNSIGNED NOT NULL,
			puset_user_ID INT(11) UNSIGNED NOT NULL,
			puset_name VARCHAR(30) NOT NULL,
			puset_value TEXT NULL DEFAULT NULL,
			PRIMARY KEY ( puset_plug_ID, puset_user_ID, puset_name )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

	'T_settings' => array( 'Creating settings table',
		"CREATE TABLE T_settings (
			set_name VARCHAR(30) NOT NULL,
			set_value VARCHAR(255) NULL,
			PRIMARY KEY ( set_name )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

	'T_subscriptions' => array( 'Creating subscriptions table',
		"CREATE TABLE T_subscriptions (
			sub_blog_ID INT(11) unsigned not null,
			sub_user_ID INT(11) unsigned not null,
			sub_items TINYINT(1) not null,
			sub_comments TINYINT(1) not null,
			PRIMARY KEY ( sub_blog_ID, sub_user_ID )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

	'T_tags' => array( 'Creating tags table',
		"CREATE TABLE T_tags (
			tag_ID INT(11) unsigned not null AUTO_INCREMENT,
			tag_name VARCHAR(50) not null,
			PRIMARY KEY ( tag_ID ),
			UNIQUE tag_name( tag_name )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

	'T_templates' => array( 'Creating templates table',
		"CREATE TABLE T_templates (
			template_ID INT(10) unsigned NOT NULL auto_increment,
			template_name VARCHAR(32) NOT NULL,
			template_type ENUM('xhtml','html5','feed','other') NOT NULL default 'xhtml',
			template_folder VARCHAR(32) NOT NULL,
			PRIMARY KEY template_ID ( template_ID ),
			UNIQUE template_folder( template_folder ),
			KEY template_name( template_name )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

	'T_template_containers' => array( 'Creating template containers table',
		"CREATE TABLE T_template_containers (
			sco_template_ID INT(10) unsigned NOT NULL,
			sco_name VARCHAR(40) NOT NULL,
			sco_order TINYINT NOT NULL default 0,
			PRIMARY KEY ( sco_template_ID, sco_name )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

	'T_users' => array( 'Creating users table',
		"CREATE TABLE T_users (
			user_ID INT(11) unsigned NOT NULL auto_increment,
			user_login VARCHAR(20) NOT NULL,
			user_pass CHAR(40) NOT NULL,
			user_firstname VARCHAR(50) NULL,
			user_lastname VARCHAR(50) NULL,
			user_nickname VARCHAR(50) NULL,
			user_email VARCHAR(255) NOT NULL,
			user_url VARCHAR(255) NULL,
			user_ip VARCHAR(15) NULL,
			user_domain VARCHAR(200) NULL,
			user_browser VARCHAR(200) NULL,
			dateYMDhour DATETIME NOT NULL DEFAULT '2000-01-01 00:00:00',
			user_level int unsigned DEFAULT 0 NOT NULL,
			user_locale VARCHAR(20) DEFAULT 'en-EU' NOT NULL,
			user_idmode VARCHAR(20) NOT NULL DEFAULT 'login',
			user_allow_msgform TINYINT NOT NULL DEFAULT '1',
			user_notify TINYINT(1) NOT NULL default 1,
			user_showonline TINYINT(1) NOT NULL default 1,
			user_grp_ID INT(4) NOT NULL default 1,
			user_validated TINYINT(1) NOT NULL DEFAULT 0,
			user_avatar_ID INT(10) NULL,
			user_biography TEXT NULL DEFAULT NULL,
			PRIMARY KEY user_ID ( user_ID ),
			UNIQUE user_login ( user_login ),
			KEY user_grp_ID ( user_grp_ID )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

	'T_user_cyberspace' => array( 'Creating user cyberspace table',
		"CREATE TABLE T_user_cyberspace (
			user_ID INT(11) unsigned NOT NULL DEFAULT 0,
			user_diggurl VARCHAR(255) NULL,
			user_facebook VARCHAR(255) NULL,
			user_flickr VARCHAR(255) NULL,
			user_linkdin VARCHAR(255) NULL,
			user_myspace VARCHAR(255) NULL,
			user_stumble VARCHAR(255) NULL,
			user_twitter VARCHAR(255) NULL,
			user_youtube VARCHAR(255) NULL,
			user_gplus VARCHAR(255) NULL,
			user_aolaim VARCHAR(255) NULL,
			user_icqid VARCHAR(255) NULL,
			user_msnim VARCHAR(255) NULL,
			user_skype VARCHAR(255) NULL,
			user_yahooim VARCHAR(255) NULL,
			PRIMARY KEY user_ID ( user_ID )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

	'T_user_realworld' => array( 'Creating users realworld table',
		"CREATE TABLE T_user_realworld (
			user_ID INT(11) unsigned NOT NULL DEFAULT 0,
			user_address_1 VARCHAR(255) NULL,
			user_address_2 VARCHAR(255) NULL,
			user_city VARCHAR(255) NULL,
			user_county VARCHAR(255) NULL,
			user_state VARCHAR(255) NULL,
			user_province VARCHAR(255) NULL,
			user_postal_code VARCHAR(255) NULL,
			user_nation VARCHAR(255) NULL,
			user_wallphone VARCHAR(50) NULL,
			user_cellphone VARCHAR(50) NULL,
			user_homefax VARCHAR(50) NULL,
			user_workphone VARCHAR(50) NULL,
			user_workfax VARCHAR(50) NULL,
			user_birthday VARCHAR(50) NULL,
			user_gender VARCHAR(50) NULL,
			PRIMARY KEY user_ID( user_ID )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

	'T_user_settings' => array( 'Creating user settings table',
		"CREATE TABLE T_user_settings (
			uset_user_ID INT(11) UNSIGNED NOT NULL,
			uset_name VARCHAR(30) NOT NULL,
			uset_value VARCHAR(255) NULL,
			PRIMARY KEY ( uset_user_ID, uset_name )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

	'T_widgets' => array( 'Creating widgets table',
		"CREATE TABLE T_widgets (
			wi_ID INT(10) UNSIGNED auto_increment,
			wi_widget_blog_ID INT(11) UNSIGNED NOT NULL,
			wi_sco_name VARCHAR(40) NOT NULL,
			wi_order INT(10) NOT NULL,
			wi_enabled TINYINT(1) NOT NULL DEFAULT 1,
			wi_type ENUM('widget','plugin') NOT NULL DEFAULT 'widget',
			wi_code VARCHAR(32) NOT NULL,
			wi_params TEXT NULL DEFAULT NULL,
			PRIMARY KEY ( wi_ID ),
			UNIQUE wi_order( wi_widget_blog_ID, wi_sco_name, wi_order )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ),

);

?>
