<?php
/**
 * This file implements the PageCache class
 *
 * This caches HTML pages genereated by the app.
 *
 * @todo (1111) vars type 'foo' need useful/valid descriptions
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package core
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * PageCache class
 *
 * @package core
 */
class PageCache
{
	/**
	 * How old can a cached object get before we consider it outdated
	 * @var foo
	 */
	var $max_age_seconds = 300;

	/**
	 * After how many bytes should we output sth live while collecting cache content
	 * @var foo
	 */
	var $output_chunk_size = 2000;

	/**
	 * By default we consider caching not to be enabled
	 * @var foo
	 */
	var $is_enabled = false;

	/**
	 * FIX THIS
	 * @var foo
	 */
	var $ads_blogcache_path;
	/**
	 * Filename of cache for current page
	 * @var foo
	 */
	var $cache_filepath;
	/**
	 * Progressively caching the content of the current page
	 * @var foo
	 */
	var $cached_page_content = '';
	/**
	 * Are we currently recording cache contents for this page?
	 * @var foo
	 */
	var $is_collecting = false;

	/**
	 * Constructor
	 *
	 * @param Blog to use, can be NULL
	 */
	function PageCache( $Blog = NULL )
	{
		global $cache_path;
		global $Settings;

		if( is_null( $Blog ) )
		{
			// Cache for "other" "generic" "special" pages
			$this->ads_blogcache_path = $cache_path.'general/';

			if( ! $Settings->get( 'general_cache_enabled' ) )
			{
				// We do NOT want caching for this blog
				// debuglog-add removed
			}
			else
			{
				$this->is_enabled = true;
			}
		}
		else
		{
			// Cache for a specific Blog
			// We need to set this even if cache is not enabled (yet) bc it's used for creating
			$this->ads_blogcache_path = $cache_path.'c'.$Blog->ID.'/';

			if( ! $Blog->get_setting( 'cache_enabled' ) )
			{
				// We do NOT want caching for this blog
				// debuglog-add removed
			}
			else
			{
				$this->is_enabled = true;
			}
		}
	}


	/**
	 * Get path to file for current URL
	 *
	 * @todo (0000) fp> We may need to add some keys like the locale or the charset, I'm not sure.
	 */
	function get_af_filecache_path()
	{
		global $ReqURI;
		global $ReqHost;

		// We want the cache for the current URL
		if( empty( $this->cache_filepath ) )
		{
			$ReqAbsUrl = $ReqHost.$ReqURI;

			$this->cache_filepath = $this->gen_filecache_path( $ReqAbsUrl );
		}

		return $this->cache_filepath;
	}


	/**
	 * Generate path for caching $url
	 *
	 * @param string URL
	 * @return string
	 */
	function gen_filecache_path( $url )
	{
		// fp> is this the fastest way to hash this into something not too obvious to guess?
		$url_hash = md5( $url );
		return $this->ads_blogcache_path.$url_hash.'.page';
	}


	/**
	 * Invalidate a particular page from the cache
	 *
	 * @param URL of the page to be invalidated
	 */
	function invalidate( $url )
	{
		// What would be the cache file for the current URL?
		$af_cache_file = $this->gen_filecache_path( $url );
		@unlink( $af_cache_file );
	}


	/**
	 * create cache
	 *
	 * @return boolean true if cache has been successfully created
	 */
	function cache_create()
	{
		// Create by using the filemanager's default chmod.
		// @todo (0000) we may not want to make these publicly readable
		if( ! mkdir_r( $this->ads_blogcache_path, NULL ) )
		{
			return false;
		}

		// Clear contents of folder, if any
		cleardir_r( $this->ads_blogcache_path );

		return true;
	}


	/**
	 * Delete all cache files
	 */
	function cache_delete()
	{
		rmdir_r( $this->ads_blogcache_path );
	}


	/**
	 * Check if cache contents are available, otherwise start collecting output to be cached
	 *
	 * @return true if we found and have echoed content from the cache
	 */
	function check()
	{
		global $disp;
		global $Messages;

		if( ! $this->is_enabled )
		{
			// We do NOT want caching for this page
			return false;
		}

		if( $disp == '404' )
		{
			// We do NOT want caching for 404 pages (illimited possibilities!)
			return false;
		}

		if( is_logged_in() )
		{
			// We do NOT want caching when a user is logged in (private data)
			return false;
		}

		if( $Messages->count('all') )
		{
			// There are some messages do be displayed. That means the user has done some action.
			// We do want to display those messages.
			// There may also be more... like a "comment pending review" etc...
			// DO NOT CACHE and do not present a cached page.
			return false;
		}

		if( $this->retrieve() )
		{
			// We could retrieve
			return true;
		}

		$this->is_collecting = true;
		ob_start( array( & $this, 'output_handler'), $this->output_chunk_size );
		return false;
	}


	/**
	 * Retrieve and output cache for current URL
	 *
	 * @return boolean true if we could retrieve
	 */
	function retrieve()
	{
		global $ReqHost;
		global $ReqURI;
		global $servertimenow;

		// What would be the cache file for the current URL?
		$af_cache_file = $this->get_af_filecache_path();


		/*
		// fstat() is interesting because it gives the last access time... use that for purging...
		if( $fh = @fopen( $af_cache_file, 'r', false ) )
		{
			$fstat = fstat( $fh );
			pre_dump( $fstat );
			fclose( $fh );
		}
		*/

		$lines = @file( $af_cache_file, false );
		// fp> note we are using empty() so that we detect both the case where there
		// is no file and the case wher ethe file might have ended up empty because
		// PHP crashed while writing to it or sth like that...
		if( ! empty( $lines ) )
		{
			// We have data in the cache!
			// Check that the format of the file if OK
			$sep = trim( $lines[2] );
			unset( $lines[2] );
			if( $sep != '' )
			{
				return false;
			}

			// Retrieved cached URL
			$retrieved_url = trim( $lines[0] );
			unset( $lines[0] );
			if( $retrieved_url != $ReqHost.$ReqURI )
			{
				return false;
			}

			// timestamp of cache generation
			$retrieved_ts = trim( $lines[1] );
			unset( $lines[1] );
			$cache_age = $servertimenow-$retrieved_ts;
			if( $cache_age > $this->max_age_seconds )
			{
				// Cache has expired
				return false;
			}

			// Go through headers
			$i = 2;
			while( $headerline = trim( $lines[++$i] ) )
			{
				header( $headerline );
				unset( $lines[$i] );
			}
			unset( $lines[$i] );

			// SEND CONTENT!
			echo implode('',$lines);

			return true;
		}
	}


	/**
	 * This is called every x bytes to provide real time output
	 */
	function output_handler( $buffer )
	{
		$this->cached_page_content .= $buffer;
		return $buffer;
	}


	/**
	 * We are going to output personal data and we want to abort collecting the data for the cache
	 */
	function abort_collect()
	{
		if( ! $this->is_collecting )
		{
			// We are not collecting anyway
			return;
		}

		ob_end_flush();
		// We are no longer collecting...
		$this->is_collecting = false;
	}


	/**
	 * End collecting output to be cached
	 */
	function end_collect()
	{
		if( ! $this->is_collecting )
		{
			// We are not collecting
			return;
		}

		ob_end_flush();
		// What would be the cache file for the current URL?
		$af_cache_file = $this->get_af_filecache_path();

		// fp> 'x' mode should either give an exclusive write lock or fail
		// @todo (0000) fp> this here should be ok, but it would be even better with
		// locking the file when we start collecting cache
		if( ! $fh = @fopen( $af_cache_file.'.tmp', 'x', false ) )
		{
			// debuglog-add removed
		}
		else
		{
			global $content_type_header;
			global $ReqHost;
			global $ReqURI;
			global $servertimenow;

			// Put the URL of the page we are caching into the cache. You can never be
			// to paranoid! People can change their domain names, folder structures, etc...
			// AND you cannot trust the hash to give a different file name in 100.00% of
			// the cases! Serving a page for a different URL would be REEEEEALLLY BAAAAAAD!
			$file_head = $ReqHost.$ReqURI."\n";

			// Put the time of the page generation into the file (btw this is the time of when we started this script)
			$file_head .= $servertimenow."\n";
			$file_head .= "\n";

			// We need to write the content type!
			if( ! empty( $content_type_header ) )
			{
				$file_head .= $content_type_header."\n";
			}

			$file_head .= "\n";

			fwrite( $fh, $file_head.$this->cached_page_content );
			fclose( $fh );

			// Now atomically replace old cache with new cache (at least on Linux)
			if( ! @rename( $af_cache_file.'.tmp', $af_cache_file ) )
			{
				// Rename failed, we are probably on windows PHP <= 5.2.5... http://bugs.php.net/bug.php?id=44805
				// we have to split this:
				// Kill cache
				unlink( $af_cache_file );
				// Now, some other process might start to try caching (and will likely give
				// up since the .tmp file already exists)
				if( ! @rename( $af_cache_file.'.tmp', $af_cache_file ) )
				{
					// Hide errors bc another PHP process could have beaten us to writing a new file there
					// Anyways, we still could not rename, let's drop the .tmp file
					unlink( $af_cache_file.'.tmp' );
				}
			}
		}
	}

}

?>
