<?php
/**
 * This file implements the IconLegend base class
 *
 * Use {@link get_IconLegend()} to get the instance.
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package core
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * IconLegend base class
 *
 * @package core
 */
class IconLegend
{
	/**
	 * List of used icon names
	 * @var array
	 */
	var $icons = array();

	/**
	 * Add an icon with his legend to the icons array
	 *
	 * @param string name of the icon
	 */
	function add_icon( $icon )
	{
		if( ! in_array( $icon, $this->icons ) )
		{
			$this->icons[] = $icon;
		}
	}


	/**
	 * Display the icon legend
	 */
	function display_legend()
	{
		if( empty( $this->icons ) )
		{
			return;
		}

		// There are some icons to display
		echo '<div id="icon_legend">'.T_('Legend').': ';
		// Loop on all map array of filenames for icons to display icons list in the same order
		foreach( $this->icons as $icon )
		{
			$icon_info = get_icon_info( $icon );
			if( ! $icon_info )
			{
				continue;
			}
			echo '<span class="legend_element">'.get_icon( $icon ).' ';
			if( isset( $icon_info['legend'] ) )
			{
				// Icon has a legend
				echo $icon_info['legend'].' ';
			}
			else
			{
				// Icon has no legend so we use the alt
				echo $icon_info['alt'].' ';
			}
			echo '</span>';
		}
		echo '</div>';
	}


	/**
	 * Reset icons array
	 */
	function reset()
	{
		$this->icons[] = array();
	}
}

?>
