<?php
/**
 * This file initializes everything BUT the blog!
 *
 * It is useful when you want to do very customized templates! It is also
 * called by more complete initializers.
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package core
 */
if(!defined('QP_CONFIG_LOADED')) die('fail');

if( $maintenance_mode )
{
	header('HTTP/1.0 503 Service Unavailable');
	echo '<h1>503 Service Unavailable</h1>';
	die( 'The site is temporarily down for maintenance.' );
}

// Prevent double loading since require_once won't work in all situations on windows
// when some subfolders have caps :( (Check it out on static page generation)
if( defined( 'QP_MAIN_INIT' ) )
{
	return;
}
define( 'QP_MAIN_INIT', true );

// Security check for older PHP versions
// Contributed by counterpoint / MAMBO team
// TODO: dh> this makes sense AFAICS. Please review.
// fp> yeah I don't know about that. If you ask me, i'm not taking chances.
// if( ini_get( 'register_globals' ) )
{
	$protects = array( '_REQUEST', '_GET', '_POST', '_COOKIE', '_FILES', '_SERVER', '_ENV', 'GLOBALS', '_SESSION' );
	foreach( $protects as $protect )
	{
		if( in_array( $protect, array_keys( $_REQUEST ) )
			|| in_array( $protect, array_keys( $_GET ) )
			|| in_array( $protect, array_keys( $_POST ) )
			|| in_array( $protect, array_keys( $_COOKIE ) )
			|| in_array( $protect, array_keys( $_FILES ) ) )
		{
			require_once $inc_path.'/_core/_misc.funcs.php';
			bad_request_die( 'Unacceptable params.' );
		}
	}
}

/**
 * Class loader
 */
require_once $inc_path.'_core/_class.funcs.php';

/**
 * Miscellaneous functions
 */
require_once $inc_path.'/_core/_misc.funcs.php';

// Debug message log for debugging only (initialized here)
if( $debug )
{
	$Debuglog = new Log( 'note' );
}
else
{
	$Debuglog = new Log_noop( 'note' );
}

/**
 * Info & error message log for end user (initialized here)
 *
 * @global Log $Messages
 */
$Messages = new Log( 'error' );

/**
 * Start timer
 *
 * @global Timer $Timer
 */
$Timer = new Timer('total');

$Timer->resume( '_main.inc' );

/**
 * Sets various arrays and vars, also $app_name!
 *
 * Needed before the error messages.
 */
require_once dirname(__FILE__).'/_vars.inc.php';

if( ! $app_config_is_done )
{
	// base config is not done!
	$error_message = 'Base configuration is not done! (see /qp_config/_main_config.php)';
}
elseif( ! isset( $locales[$default_locale] ) )
{
	$error_message = 'The default locale '.var_export( $default_locale, true ).' does not exist! (see /qp_config/_locales.php)';
}
if( isset( $error_message ) )
{
	/**
	 * error & exit
	 */
	require dirname(__FILE__).'/../qp_view_admin/conf_error.main.php';
}

/**
 * Connect to DB
 */
require_once dirname(__FILE__).'/_connect_db.inc.php';

/**
 * Interface to general settings
 *
 * Keep this below the creation of the {@link $DB DB object}, because it checks for the
 * correct db_version and catches "table does not exist" errors, providing a link to the
 * install script.
 *
 * @global GeneralSettings $Settings
 */
$Settings = new GeneralSettings();

/**
 * Interface to user settings
 *
 * @global UserSettings $UserSettings
 */
$UserSettings = new UserSettings();

/**
 * Absolute Unix timestamp for server
 *
 * @global integer $servertimenow
 */
$servertimenow = time();

$time_difference = $Settings->get( 'time_difference' );

/**
 * Corrected Unix timestamp to match server timezone
 *
 * @global integer $localtimenow
 */
$localtimenow = $servertimenow + $time_difference;

// The Hit class
// fp> The following constructor requires these right now
load_funcs( '_core/_param.funcs.php' );
load_funcs( '_core/_url.funcs.php' );

// Locale selection
// We need to do this as early as possible in order to set DB connection charset below
// fp> that does not explain why it needs to be here!! Why do we need to set the Db
// charset HERE? BEFORE WHAT? sam2kb> ideally we should set the right DB charset at
// the time when we connect to the database. The reason is until we do it all data
// pulled out from DB is in wrong encoding. I put the code here because it depends on
// _param.funcs, so if move the _param.funcs higher we can also move this code right
// under _connect_db
locale_overwritefromDB();
$default_locale = locale_from_httpaccept(); // set default locale by autodetect

if( ( $locale_from_get = param( 'locale', 'string', NULL, true ) ) )
{
	if( $locale_from_get != $default_locale )
	{
		if( isset( $locales[$locale_from_get] ) )
		{
			$default_locale = $locale_from_get;
		}
		else
		{
			$locale_from_get = false;
		}
	}

}

// Activate default locale
locale_activate( $default_locale );

// Set encoding for MySQL connection
$DB->set_connection_charset( $current_charset );

/**
 * @global Hit The Hit object
 */
$Hit = new Hit(); // This may INSERT a basedomain and a useragent but NOT the HIT itself!

/**
 * The Session object
 *
 * It has to be instantiated before the "SessionLoaded" hook.
 *
 * @todo (0000) This needs the same "SET NAMES" MySQL-setup as with Session::dbsave() -
 * see the "TODO" with unserialize() in Session::Session()
 * @global Session
 */
$Session = new Session(); // IF this can't pull asesion from the DB it will always INSERT a new one!

// Handle saving the HIT and updating the SESSION at the end of the page
register_shutdown_function( 'shutdown' );

/**
 * @global Plugins The Plugin management object
 */
$Plugins = new Plugins();

// NOTE: it might be faster (though more bandwidth intensive) to spit cached pages
// (CachePageContent event) than to look into blocking the request (SessionLoaded event).
$Plugins->trigger_event( 'SessionLoaded' );

// Trigger a page content caching plugin. This would either return the
// cached content here or start output buffering.
if( $Session->get( 'core.no_CachePageContent' ) )
{
	// The event is disabled for this request
	$Session->delete('core.no_CachePageContent');
}
elseif( ( $get_return = $Plugins->trigger_event_first_true( 'CachePageContent' ) ) // Plugin responded to the event
	&& ( isset( $get_return['data'] ) ) ) // cached content returned
{
	echo $get_return['data'];
	// Note: we should not use debug_info() here, because the plugin has probably sent a Content-Length header.
	exit(0);
}

// Includes
$Timer->resume( '_main.inc:requires' );
load_funcs( 'blogs/model/_blog.funcs.php' );
load_funcs( 'categories/model/_category.funcs.php' );
load_funcs( 'items/model/_item.funcs.php' );
load_funcs( 'users/model/_user.funcs.php' );
load_funcs( '_core/_template.funcs.php' );
load_funcs( 'comments/model/_comment.funcs.php' );
load_funcs( 'items/model/_item.funcs.php' );
load_funcs( '_core/ui/forms/_form.funcs.php' );
$Timer->pause( '_main.inc:requires' );

if( ! isset( $login_required ) )
{
	$login_required = false;
}

$login = NULL;
$pass = NULL;
$pass_md5 = NULL;

if( isset( $_POST['login'] ) && isset( $_POST['pwd'] ) )
{
	// Trying to log in with a POST
	$login = $_POST['login'];
	$pass = $_POST['pwd'];
	unset( $_POST['pwd'] ); // password will be hashed below
}
elseif( isset( $_GET['login'] ) )
{
	// Trying to log in with a GET; we might only provide a user here.
	$login = $_GET['login'];
	$pass = isset( $_GET['pwd'] ) ? $_GET['pwd'] : '';
	unset( $_GET['pwd'] ); // password will be hashed below
}

// either 'login' (normal) or 'redirect_to_backoffice' may be set here.
// This also helps to display the login form again, if either login or pass were empty.
$login_action = param_arrayindex( 'login_action' );

$UserCache = & get_Cache( 'UserCache' );

if( ! empty( $login_action ) || (! empty( $login ) && ! empty( $pass ) ) )
{
	// User is trying to login right now
	header_nocache();
	// Note: login and password cannot include '<' !
	$login = strtolower( strip_tags( remove_magic_quotes( $login ) ) );
	$pass = strip_tags( remove_magic_quotes( $pass ) );
	$pass_md5 = md5( $pass );

	// Handle javascript-hashed password: If possible, the login form will hash the
	// entered password with a salt that changes everytime.
	param('pwd_salt', 'string', ''); // just for comparison with the one from Session
	$pwd_salt_sess = $Session->get( 'core.pwd_salt' );

	$transmit_hashed_password = (bool)$Settings->get( 'js_passwd_hashing' )
		&& !(bool)$Plugins->trigger_event_first_true('LoginAttemptNeedsRawPassword');
	if( $transmit_hashed_password )
	{
		param( 'pwd_hashed', 'string', '' );
	}
	else
	{
		// at least one plugin requests the password un-hashed
		$pwd_hashed = '';
	}

	$pass_ok = false;
	// Trigger Plugin event, which could create the user, according to another database
	if( $Plugins->trigger_event( 'LoginAttempt', array(
		'login' => & $login,
		'pass' => & $pass,
		'pass_md5' => & $pass_md5,
		'pass_salt' => & $pwd_salt_sess,
		'pass_hashed' => & $pwd_hashed,
		'pass_ok' => & $pass_ok ) )
	)
	{
		// clear the UserCache, if a plugin has been called - it may have changed user(s)
		$UserCache->clear();
	}

	if( $Messages->count( 'login_error' ) )
	{
		// A plugin has thrown a login error; do nothing, the error will get displayed in the login form.
		// @todo (0000) dh> make sure that the user gets logged out?! (a
		// Plugin might have logged him in and another one thrown an error)
	}
	else
	{
		// Check login and password
		// Make sure that we can load the user:
		$User = & $UserCache->get_by_login( $login );

		if( $User && ! $pass_ok )
		{
			// check the password, if no plugin has said "it's ok"
			if( ! empty( $pwd_hashed ) )
			{
				// password hashed by JavaScript
				if( empty( $pwd_salt_sess ) )
				{
					// no salt stored in session: either cookie problem or the user had already tried logging in (from another window for example)
					if( ( $pos = strpos( $pass, '_hashed_' ) ) && substr( $pass, $pos+8 ) == $Session->ID )
					{
						// session ID matches, no cookie problem
						$Messages->add( T_('The login window has expired. Please try again.'), 'login_error' );
					}
					else
					{
						// more general error:
						$Messages->add( T_('Either you have not enabled cookies or this login window has expired.'), 'login_error' );
					}
				}
				elseif( $pwd_salt != $pwd_salt_sess )
				{
					// submitted salt differs from the one stored in the session
					$Messages->add( T_('The login window has expired. Please try again.'), 'login_error' );
				}
				else
				{
					// compare the password, using the salt stored in the Session:
					$pass_ok = sha1( $User->pass.$pwd_salt ) == $pwd_hashed;
					$Session->delete('core.pwd_salt');
				}
			}
			else
			{
				$pass_ok = ( $User->pass == $pass_md5 );
			}
		}
	}

	if( $pass_ok )
	{
		// Login succeeded, set cookies
		// set the user from the login that succeeded
		$current_User = & $UserCache->get_by_login( $login );
		// save the user for later hits
		$Session->set_User( $current_User );
	}
	elseif( ! $Messages->count( 'login_error' ) )
	{
		// if there's no login_error message yet, add the default one:
		// This will cause the login screen to "popup" (again)
		$Messages->add( T_('Wrong login/password.'), 'login_error' );
	}

}
elseif( $Session->has_User() /* logged in */
	&& /* No login param given or the same as current user: */
	( empty( $login ) || ( ( $tmp_User = & $UserCache->get_by_ID( $Session->user_ID ) ) && $login == $tmp_User->login ) ) )
{ /* if the session has a user assigned to it:
	 * User was not trying to log in, but he was already logged in:
	 */
	// get the user ID from the session and set up the user again
	$current_User = & $UserCache->get_by_ID( $Session->user_ID );
}
else
{
	// The Session has no user or $login is given (and differs from current user), allow alternate authentication through Plugin:
	if( ( $event_return = $Plugins->trigger_event_first_true( 'AlternateAuthentication' ) )
		&& $Session->has_User() # the plugin should have attached the user to $Session
	)
	{
		$current_User = & $UserCache->get_by_ID( $Session->user_ID );
	}
	elseif( $login_required )
	{
		// User was not logged in at all, but login is required
		$Messages->add( T_('You must log in!'), 'login_error' );
	}
}
unset( $pass );

// Check if the user needs to be validated, but is not yet:
// @todo (0000) dh> this block prevents registration, if you are logged in already, but not
// validated! (e.g. when registered as "foo", you cannot register as "bar" until you logout
// (but there's no link in sight) or validate the "foo" account)
if( ! empty( $current_User )
		&& ! $current_User->validated
		&& $Settings->get( 'newusers_mustvalidate' ) // same check as in login.php
		&& param('action', 'string', '') != 'logout' ) // fp> TODO: non validated users should be automatically logged out
{
	if( $action != 'req_validatemail' && $action != 'validatemail' )
	{
		// we're not in that action already:
		$action = 'req_validatemail'; // for login.php
		$Messages->add( T_('You must validate your email address before you can log in.'), 'login_error' );
	}
}
else
{
	// Trigger plugin event that allows the plugins to re-act on the login event
	if( empty( $current_User ) )
	{
		$Plugins->trigger_event( 'AfterLoginAnonymousUser', array() );
	}
	else
	{
		$Plugins->trigger_event( 'AfterLoginRegisteredUser', array() );

		if( ! empty( $login_action ) )
		{
			// We're coming from the Login form and need to redirect to the requested page
			if( $login_action == 'redirect_to_backoffice' )
			{
				// user pressed the "Log into backoffice!" button
				$redirect_to = $admin_url;
			}
			else
			{
				param( 'redirect_to', 'string', $app_baseurl );
			}

			header_redirect( $redirect_to );
			exit(0);
		}
	}
}

// If there are "login_error" messages, they trigger the login form at the end of this file.

// Login procedure ...

// User locale selection
if( is_logged_in() && $current_User->get( 'locale' ) != $current_locale && ! $locale_from_get )
{
	// User locale selection: change locale to users preference
	// @todo (0000) this should get done before instantiating $current_User, because we already use T_() there...
	locale_activate( $current_User->get( 'locale' ) );
	if( $current_locale == $current_User->get( 'locale' ) )
	{
		$default_locale = $current_locale;
	}
}

// Init charset handling
// @todo (0000) dh> anything translated before this call might have encoding issues (e.g. the login form!)
init_charsets( $current_charset );

// Display login errors (and form). This uses $io_charset, so it's at the end.

if( $Messages->count( 'login_error' ) )
{
	require $srvc_path.'login.php';
	exit(0);
}

$Timer->pause( '_main.inc');

if( file_exists( $conf_path.'hacks.php' ) )
{
	$Timer->resume( 'hacks.php' );
	/**
	 * Load hacks file if it exists
	 */
	include_once $conf_path.'hacks.php';
	$Timer->pause( 'hacks.php' );
}

?>
