<?php
/**
 * This file sets various general purpose arrays and global variables for use in the app
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package core
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * Do we want robots to index this page? -- Will be use to produce meta robots tag
 * @global boolean or NULL to ignore
 */
$robots_index = NULL;

/**
 * Do we want robots to follow links on this page? -- Will be use to produce meta robots tag
 * @global boolean or NULL to ignore
 */
$robots_follow = NULL;

/**
 * @global boolean Are we running on Command Line Interface instead of a web request?
 */
$is_cli = empty( $_SERVER['SERVER_SOFTWARE'] ) ? true : false;

$is_web = ! $is_cli;

$content_type_header = NULL;

/**
 * @global integer ID of featured post that is being displayed (will become an array()
 * in the future) -- needed so we can filter it out of normal post flow
 */
$featured_displayed_item_ID = NULL;

// Initialize some variables for template functions
$required_js = array();
$required_css = array();
$headlines = array();

// Investigation for following code by Isaac - http://isaacschlueter.com/
if( isset( $_SERVER['REQUEST_URI'] ) && ! empty( $_SERVER['REQUEST_URI'] ) )
{
	// Warning: on some IIS installs it it set but empty!
	$ReqURI = $_SERVER['REQUEST_URI'];

	// Build requested Path without query string
	$pos = strpos( $ReqURI, '?' );
	if( false !== $pos )
	{
		$ReqPath = substr( $ReqURI, 0, $pos  );
	}
	else
	{
		$ReqPath = $ReqURI;
	}
}
elseif( isset( $_SERVER['URL'] ) )
{
	// ISAPI
	$ReqPath = $_SERVER['URL'];
	$ReqURI = isset( $_SERVER['QUERY_STRING'] ) && ! empty( $_SERVER['QUERY_STRING'] ) ? ( $ReqPath.'?'.$_SERVER['QUERY_STRING'] ) : $ReqPath;
}
elseif( isset( $_SERVER['PATH_INFO'] ) )
{
	// CGI/FastCGI
	if( isset( $_SERVER['SCRIPT_NAME'] ) )
	{
		if( $_SERVER['SCRIPT_NAME'] == $_SERVER['PATH_INFO'] )
		{
			// both the same so just use one of them, this happens on a windoze 2003 box
			$ReqPath = $_SERVER['PATH_INFO'];
		}
		else
		{
			$ReqPath = $_SERVER['SCRIPT_NAME'].$_SERVER['PATH_INFO'];
		}
	}
	else
	{
		// does this happen??
		$ReqPath = $_SERVER['PATH_INFO'];
	}
	$ReqURI = isset( $_SERVER['QUERY_STRING'] ) && ! empty( $_SERVER['QUERY_STRING'] ) ? ( $ReqPath.'?'.$_SERVER['QUERY_STRING'] ) : $ReqPath;
}
elseif( isset( $_SERVER['ORIG_PATH_INFO'] ) )
{
	// Tomcat 5.5.x with Herbelin PHP servlet and PHP 5.1
	$ReqPath = $_SERVER['ORIG_PATH_INFO'];
	$ReqURI = isset( $_SERVER['QUERY_STRING'] ) && ! empty( $_SERVER['QUERY_STRING'] ) ? ( $ReqPath.'?'.$_SERVER['QUERY_STRING'] ) : $ReqPath;
}
elseif( isset( $_SERVER['SCRIPT_NAME']) )
{
	// Some Odd Win2k Stuff
	$ReqPath = $_SERVER['SCRIPT_NAME'];
	$ReqURI = isset( $_SERVER['QUERY_STRING'] ) && ! empty( $_SERVER['QUERY_STRING'] ) ? ( $ReqPath.'?'.$_SERVER['QUERY_STRING'] ) : $ReqPath;
}
elseif( isset( $_SERVER['PHP_SELF']) )
{
	// The Old Stand-By
	$ReqPath = $_SERVER['PHP_SELF'];
	$ReqURI = isset( $_SERVER['QUERY_STRING'] ) && ! empty( $_SERVER['QUERY_STRING'] ) ? ( $ReqPath.'?'.$_SERVER['QUERY_STRING'] ) : $ReqPath;
}
else
{
	$ReqPath = false;
	$ReqURI = false;
	?>
	<p class="error">Warning: $ReqPath could not be set. Probably an odd IIS problem.</p>
	<p>Go to your <a href="<?php echo $app_baseurl.$install_subdir ?>phpinfo.php">phpinfo page</a>, look for occurences of <code><?php
	// take the baseurlroot out..
	echo preg_replace('#^'.$app_baseurlroot.'#', '', $app_baseurl.$install_subdir )
	?>phpinfo.php</code> and copy all lines containing this to the <a href="http://forums.quamplures.net">forum</a>. Also specify what webserver
	you're running on.
	<br />(If you have deleted your install folder then you have to upload it again before doing this).</p>
	<?php
}

if( $is_web )
{
	// Full requested Host (including protocol)
	// Note: on IIS you can receive 'off' in the HTTPS field!! :[
	$ReqHost = ( ( isset( $_SERVER['HTTPS'] ) && ( $_SERVER['HTTPS'] != 'off' ) ) ? 'https://' : 'http://' ).$_SERVER['HTTP_HOST'];
	// on which page are we ?
	$pagenow = explode( '/', $_SERVER['PHP_SELF'] );
	$pagenow = trim( $pagenow[( sizeof( $pagenow ) - 1 )] );
	$pagenow = explode( '?', $pagenow );
	$pagenow = $pagenow[0];
}

// the weekdays and the months..
$weekday[0] = NT_('Sunday');
$weekday[1] = NT_('Monday');
$weekday[2] = NT_('Tuesday');
$weekday[3] = NT_('Wednesday');
$weekday[4] = NT_('Thursday');
$weekday[5] = NT_('Friday');
$weekday[6] = NT_('Saturday');

// the weekdays short form (typically 3 letters)
// TRANS: abbrev. for Sunday
$weekday_abbrev[0] = NT_('Sun');
// TRANS: abbrev. for Monday
$weekday_abbrev[1] = NT_('Mon');
// TRANS: abbrev. for Tuesday
$weekday_abbrev[2] = NT_('Tue');
// TRANS: abbrev. for Wednesday
$weekday_abbrev[3] = NT_('Wed');
// TRANS: abbrev. for Thursday
$weekday_abbrev[4] = NT_('Thu');
// TRANS: abbrev. for Friday
$weekday_abbrev[5] = NT_('Fri');
// TRANS: abbrev. for Saturday
$weekday_abbrev[6] = NT_('Sat');

// the weekdays even shorter form (typically 1 letter)
// TRANS: abbrev. for Sunday
$weekday_letter[0] = NT_(' S ');
// TRANS: abbrev. for Monday
$weekday_letter[1] = NT_(' M ');
// TRANS: abbrev. for Tuesday
$weekday_letter[2] = NT_(' T ');
// TRANS: abbrev. for Wednesday
$weekday_letter[3] = NT_(' W ');
// TRANS: abbrev. for Thursday
$weekday_letter[4] = NT_(' T  ');
// TRANS: abbrev. for Friday
$weekday_letter[5] = NT_(' F ');
// TRANS: abbrev. for Saturday
$weekday_letter[6] = NT_(' S  ');

// the months
$month['00'] = '\?\?'; // This can happen when importing junk dates from WordPress
$month['01'] = NT_('January');
$month['02'] = NT_('February');
$month['03'] = NT_('March');
$month['04'] = NT_('April');
// TRANS: space at the end only to differentiate from short form. You don't need to keep it in the translation.
$month['05'] = NT_('May ');
$month['06'] = NT_('June');
$month['07'] = NT_('July');
$month['08'] = NT_('August');
$month['09'] = NT_('September');
$month['10'] = NT_('October');
$month['11'] = NT_('November');
$month['12'] = NT_('December');

// the months short form (typically 3 letters)
// TRANS: abbrev. for January
$month_abbrev['01'] = NT_('Jan');
// TRANS: abbrev. for February
$month_abbrev['02'] = NT_('Feb');
// TRANS: abbrev. for March
$month_abbrev['03'] = NT_('Mar');
// TRANS: abbrev. for April
$month_abbrev['04'] = NT_('Apr');
// TRANS: abbrev. for May
$month_abbrev['05'] = NT_('May');
// TRANS: abbrev. for June
$month_abbrev['06'] = NT_('Jun');
// TRANS: abbrev. for July
$month_abbrev['07'] = NT_('Jul');
// TRANS: abbrev. for August
$month_abbrev['08'] = NT_('Aug');
// TRANS: abbrev. for September
$month_abbrev['09'] = NT_('Sep');
// TRANS: abbrev. for October
$month_abbrev['10'] = NT_('Oct');
// TRANS: abbrev. for November
$month_abbrev['11'] = NT_('Nov');
// TRANS: abbrev. for December
$month_abbrev['12'] = NT_('Dec');

// the post statuses
$post_statuses = array (
	'published' => NT_('Published'),
	'deprecated' => NT_('Deprecated'),
	'redirected' => NT_('Redirected'),
	'protected' => NT_('Protected'),
	'private' => NT_('Private'),
	'draft' => NT_('Draft'),
);

/**
 * Number of view counts increased on this page
 * @global integer
 */
$view_counts_on_this_page = 0;

?>
