<?php
/**
 * This file implements the UI controller for the antispam management
 *
 * @todo (1111) what is the current value in action "recheck"? No central list
 * means anything added purged prior to adding. Possibly need it with distributed
 * antispam, but I can't see why we would have it now.
 * @todo (1111) if "recheck" doesn't go away then replace the text with the "?" icon
 * @todo (1111) move the "allow" check mark to the actions column
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package antispam
 */
if(!defined('QP_MAIN_INIT')) die('fail');

global $AdminUI;

echo '<h2>'.T_('Antispam blacklist').'</h2>';

// Check permission
if( $current_User->check_perm( 'spamblacklist', 'edit' ) )
{
	global $keyword;

	// Add keyword form, this is when you select "tools->antispam"
	$Form = new Form( NULL, 'antispam_add', 'post', 'compact' );
	$Form->begin_form( 'fform', T_('Add a banned keyword') );
	$Form->hidden_ctrl();
	$Form->hidden( 'action', 'ban' );
	$Form->text( 'keyword', $keyword, 50, T_('Keyword/phrase to ban'), '', 80 );
	$Form->hidden( 'aspm_source', 'entered' ); // means manually entered
	$Form->end_form( array( array( 'submit', 'submit', T_('Check & ban...'), 'SaveButton' ) ) );
}

echo '<p class="center">'.T_('Any URL containing one of the following keywords will be banned from posts, comments and logs.');
if( $current_User->check_perm( 'spamblacklist', 'edit' ) )
{
	echo '<br />'.T_( 'If a keyword restricts legitimate domains, click on the green tick to stop banning with this keyword.');
}
echo '</p>';

// Query antispam blacklist
$keywords = param( 'keywords', 'string', '', true );
$where_clause = '';
if( ! empty( $keywords ) )
{
	$kw_array = split( ' ', $keywords );
	foreach( $kw_array as $kw )
	{
		$where_clause .= 'aspm_string LIKE "%'.$DB->escape( $kw ).'%" AND ';
	}
}

$sql = 'SELECT aspm_ID, aspm_string, aspm_source, aspm_lastseen, aspm_comments, aspm_messages, aspm_referers,
	(aspm_comments+aspm_messages+aspm_referers) AS aspm_blocked FROM T_antispam WHERE '.$where_clause.' 1';

// Create result set
$Results = new Results( $sql, 'antispam_' );
$Results->title = T_('Banned keywords blacklist');

$Results->filter_area = array(
	'callback' => 'filter_antispam',
	'url_ignore' => 'results_antispam_page,keywords',
	'presets' => array( 'all' => array( T_('All keywords'), '?ctrl=antispam' ) )
);

// Column definition: keyword column
$Results->cols[] = array(
	'th' => T_('Keyword'),
	'order' => 'aspm_string',
	'td' => '%htmlspecialchars(#aspm_string#)%',
);

// Column definition: source column
$Results->cols[] = array(
	'th' => T_('Source'),
	'order' => 'aspm_source',
	'td_class' => 'center',
	'td' => '%antispam_source2({row})%',
);

// Column definition: last seen column
$Results->cols[] = array(
	'th' => T_('Last Seen'),
	'order' => 'aspm_lastseen',
	'default_dir' => 'D',
	'td_class' => 'timestamp',
	'td' => '%mysql2localedate( #aspm_lastseen# )%<br /><span class="small">(%mysql_date_diff( #aspm_lastseen# )%)</span>',
);

// Column definition: blocked column
$Results->cols[] = array(
	'th' => T_('Blocked'),
	'order' => 'aspm_blocked',
	'td_class' => 'shrinkwrap',
	'td' => '$aspm_blocked$ ($aspm_comments$,$aspm_messages$,$aspm_referers$)',
);

// Check if we need to display more
if( $current_User->check_perm( 'spamblacklist', 'edit' ) ) // User can edit, spamlist: add controls to output columns
{
	// Add CHECK to 1st column
	$Results->cols[0]['td'] = action_icon( TS_('Allow keyword back (Remove it from the blacklist)'), 'allowback',
		'?ctrl=antispam&amp;action=remove&amp;hit_ID=$aspm_ID$' ).$Results->cols[0]['td'];

	// Column definition: actions column
	$Results->cols[] = array(
		'th' => T_('Actions'),
		'td' => '%antispam_actions({row})%',
	);
}

// Display results
$Results->display();

/**
 * Add a column for actions
 */
function antispam_actions( & $row )
{
	$output = '';
	return $output.'[<a href="'.regenerate_url( 'action,keyword', 'action=ban&amp;keyword='
		.rawurlencode( $row->aspm_string )).'" title="'.
		T_('Check hit-logs and comments for this keyword!').'">'.
		T_('Re-check').'</a>]';
}


/**
 * Set "source" column strings
 */
function antispam_source2( & $row )
{
	static $aspm_sources = NULL;
	if( $aspm_sources === NULL )
	{
		// the antispam sources
		$aspm_sources = array(
			'core' => T_('Core'),
			'entered' => T_('Entered'),
			'comment' => T_('Comment'),
			'referer' => T_('Referer'),
			'shared' => T_('Shared'),
		);
	}
	return $aspm_sources[$row->aspm_source];
}


/**
 * Callback to add filters on top of the result set
 *
 * @param Form
 */
function filter_antispam( & $Form )
{
	$Form->text( 'keywords', get_param( 'keywords' ), 20, T_('Keywords'), T_('Separate with space'), 50 );
}

?>
