<?php
/**
 * $ctrl_mappings: 'blog_settings' || Blog settings -> {{blog}} -> all except widgets
 *
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package blogs
 */
if(!defined('QP_MAIN_INIT')) die('fail');

param_action( 'edit' );
param( 'tab', 'string', 'general', true );

// Check permissions on requested blog and autoselect an appropriate blog if necessary.
// This will prevent a fat error when switching tabs and you have restricted perms on blog properties.
if( $selected = autoselect_blog( 'blog_properties', 'edit' ) ) // Includes perm check
{
	// We have a blog to work on
	if( set_working_blog( $selected ) ) // set $blog & memorize in user prefs
	{
		// Selected a new blog
		$BlogCache = & get_Cache( 'BlogCache' );
		$Blog = & $BlogCache->get_by_ID( $blog );
	}

	$edited_Blog = & $Blog;
}
else
{
	// We could not find a blog we have edit perms on... Note: we may still have
	// permission to edit categories!!
	// redirect to blog list
	header_redirect( '?ctrl=blogs' );
	// EXITED
	$Messages->add( T_('Sorry, you have no permission to edit blog properties.'), 'error' );
	$action = 'nil';
	$tab = '';
}

// Needed when generating static page for example
memorize_param( 'blog', 'integer', -1 );

if( $tab == 'template' || $tab == 'template_settings' )
{
	$TemplateCache = & get_Cache( 'TemplateCache' );
	$edited_Template = & $TemplateCache->get_by_ID( $Blog->template_ID );
}

if( ( $tab == 'perm' || $tab == 'permgroup' ) && ( empty( $blog ) || ! $Blog->advanced_perms ) )
{
	// We're trying to access advanced perms but they're disabled!
	$tab = 'features'; // the screen where you can enable advanced perms
	if( $action == 'update' )
	{
		// make sure we don't update anything here
		$action = 'edit';
	}
}

// Perform action
switch( $action )
{
	case 'edit':
	case 'filter1':
	case 'filter2': // Edit blog form (depending on tab)
	// Check permissions
	$current_User->check_perm( 'blog_properties', 'edit', true, $blog );
	break;

	case 'update': // Update DB
	// Check permissions
	$current_User->check_perm( 'blog_properties', 'edit', true, $blog );

	switch( $tab )
	{
		case 'general':
		if( $edited_Blog->load_from_Request( array() ) )
		{
			// Commit update to the DB
			$edited_Blog->dbupdate();
			$Messages->add( T_('The blog settings have been updated'), 'success' );
		}
		break;

		case 'urls':
		if( $edited_Blog->load_from_Request( array( 'urls' ) ) )
		{
			// Commit update to the DB
			$edited_Blog->dbupdate();
			$Messages->add( T_('The URL settings have been updated'), 'success' );
		}
		break;

		case 'features':
		$old_cache_status = $edited_Blog->get_setting( 'cache_enabled' );
		if( $edited_Blog->load_from_Request( array( 'features', 'pings', 'cache' ) ) )
		{
			// Commit update to the DB
			$new_cache_status = $edited_Blog->get_setting( 'cache_enabled' );
			$PageCache = new PageCache( $edited_Blog );
			if( $old_cache_status == false && $new_cache_status == true )
			{
				// Caching has been turned ON
				if( $PageCache->cache_create() )
				{
					$Messages->add( T_('Caching has been enabled for this blog.'), 'success' );
				}
				else
				{
					$Messages->add( T_('Caching could not be enabled for this blog. Check /cache/ folder file permissions.'), 'error' );
					$edited_Blog->set_setting( 'cache_enabled', 0 );
				}
			}
			elseif( $old_cache_status == true && $new_cache_status == false )
			{
				// Caching has been turned OFF
				$PageCache->cache_delete();
				$Messages->add( T_('Caching has been disabled for this blog. All cache contents have been purged.'), 'note' );
			}

			$edited_Blog->dbupdate();
			$Messages->add( T_('The blog settings have been updated'), 'success' );
		}
		break;

		case 'template':
		if( $edited_Blog->load_from_Request( array() ) )
		{
			// Commit update to the DB
			$edited_Blog->dbupdate();
			$Messages->add( T_('The blog template has been changed.')
				.' <a href="'.$admin_url.'?ctrl=blog_settings&amp;tab=template&amp;blog='.$edited_Blog->ID.'">'.T_('Edit...').'</a>', 'success' );
			header_redirect( $edited_Blog->gen_blogurl() );
		}
		break;

		case 'template_settings': // Update params/Settings
		$edited_Template->load_params_from_Request();
		if( ! param_errors_detected() )
		{
			// Update settings
			$edited_Template->dbupdate_settings();
			$Messages->add( T_('Template settings have been updated'), 'success' );
		}
		$tab = 'template';
		break;

		case 'plugin_settings': // Update Plugin params/Settings
		load_funcs( 'plugins/_plugin.funcs.php' );
		$Plugins->restart();
		while( $loop_Plugin = & $Plugins->get_next() )
		{
			$pluginsettings = $loop_Plugin->GetDefaultBlogSettings( $tmp_params = array('for_editing' => true) );
			if( empty( $pluginsettings ) )
			{
				continue;
			}
			// Loop through settings for this plugin
			foreach( $pluginsettings as $set_name => $set_meta )
			{
				autoform_set_param_from_request( $set_name, $set_meta, $loop_Plugin, 'BlogSettings', $Blog );
			}
		}
		if( ! param_errors_detected() )
		{
			// Update settings
			$Blog->dbupdate();
			$Messages->add( T_('Plugin settings have been updated'), 'success' );
		}
		break;

		case 'perm':
		blog_update_perms( $blog, 'user' );
		$Messages->add( T_('The blog permissions have been updated'), 'success' );
		break;

		case 'permgroup':
		blog_update_perms( $blog, 'group' );
		$Messages->add( T_('The blog permissions have been updated'), 'success' );
		break;
	}

	break;
}

$AdminUI->set_path( 'blogs', $tab );

// Display page header, menus & messages
$AdminUI->set_blog_list_params( 'blog_properties', 'edit', array(
	'ctrl' => 'blog_settings', 'tab' => $tab, 'action' => 'edit' ), T_('List'), '?ctrl=blogs&amp;blog=0' );
// Display <html><head>...</head> section (should be done early if actions do not redirect)
$AdminUI->disp_html_head();
// Display title, menu, messages, etc... (messages MUST be displayed AFTER the actions)
$AdminUI->disp_body_top();
// Begin payload block
$AdminUI->disp_payload_begin();

// Display VIEW
switch( $AdminUI->get_path(1) )
{
	case 'general':
	$next_action = 'update';
	$AdminUI->disp_view( 'blogs/views/_blog_general.form.php' );
	break;

	case 'features':
	$AdminUI->disp_view( 'blogs/views/_blog_features.form.php' );
	break;

	case 'template':
	$AdminUI->disp_view( 'templates/views/_template_settings.form.php' );
	$AdminUI->disp_view( 'templates/views/_template_select.view.php' );
	break;

	case 'plugin_settings':
	$AdminUI->disp_view( 'blogs/views/_blog_plugin_settings.form.php' );
	break;

	case 'urls':
	$AdminUI->disp_view( 'blogs/views/_blog_urls.form.php' );
	break;

	case 'perm':
	$AdminUI->disp_view( 'blogs/views/_blog_user_perm.form.php' );
	break;

	case 'permgroup':
	$AdminUI->disp_view( 'blogs/views/_blog_group_perm.form.php' );
	break;
}

// End payload block
$AdminUI->disp_payload_end();
// Display body bottom, debug info, close </html>
$AdminUI->disp_global_footer();

?>
