<?php
/**
 * $ctrl_mappings: 'blogs' || Blog settings -> List
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package blogs
 */
if(!defined('QP_MAIN_INIT')) die('fail');

$AdminUI->set_path( 'blogs' );
param_action( 'list' );

if( $action != 'new' && $action != 'new-seltemplate' && $action != 'new-name' && $action != 'list' && $action != 'create' )
{
	if( valid_blog_requested() )
	{
		$edited_Blog = & $Blog;
	}
	else
	{
		$action = 'list';
	}
}
else
{
	// We are not working on a specific blog (yet) -- prevent highlighting one in the list
	set_working_blog( 0 );
}

// Perform action
switch( $action )
{
	case 'new': // New blog
	// Check permissions
	$current_User->check_perm( 'blogs', 'create', true );
	$AdminUI->append_path_level( 'new', array( 'text' => T_('New') ) );
	break;

	case 'new-seltemplate': // New blog
	// Check permissions
	$current_User->check_perm( 'blogs', 'create', true );
	param( 'kind', 'string', true );
	$AdminUI->append_path_level( 'new', array( 'text' => sprintf( /* TRANS: %s can be "Photoblog", "Group blog" or "Standard blog" */ T_('New %s'), Blog::kind_name( $kind ) ) ) );
	break;

	case 'new-name': // New blog by kind
	// Check permissions
	$current_User->check_perm( 'blogs', 'create', true );
	$edited_Blog = new Blog( NULL );
	param( 'kind', 'string', true );
	$edited_Blog->init_by_kind( $kind );
	param( 'template_ID', 'integer', true );
	$AdminUI->append_path_level( 'new', array( 'text' => sprintf( T_('New %s'), Blog::kind_name( $kind ) ) ) );
	break;

	case 'create': // Insert into DB
	// Check permissions
	$current_User->check_perm( 'blogs', 'create', true );
	$edited_Blog = new Blog( NULL );
	param( 'kind', 'string', true );
	$edited_Blog->init_by_kind( $kind );
	param( 'template_ID', 'integer', true );
	$edited_Blog->set( 'template_ID', $template_ID );
	if( $edited_Blog->load_from_Request( array() ) )
	{
		$DB->begin();
		// DB INSERT
		$edited_Blog->dbinsert();
		$Messages->add( T_('The new blog has been created.'), 'success' );
		// Create a stub file using a canned format for now, due to there
		// is no option to say what stub you want during blog creation
		$source_file = $basepath.'stubfile.template.php';
		$file_loaded = @file( $source_file );
		if( empty( $file_loaded ) )
		{
			// we don't have the stubfile template? Not good :(
			$Messages->add( sprintf( T_('Could not load stubfile template [%s]. Did you delete it?'), $source_file ), 'info' );
			$use_new_stub = false;
		}
		else
		{
			// okay good, continue
			$file_imploded = implode( '', $file_loaded );
			$stub_filename = $edited_Blog->urlname.'.php';
			$new_file = $basepath.$stub_filename;
			$f = @fopen( $new_file, 'w' );
			if( $f == false )
			{
				// We could not fopen a new filename? cheap host :(
				$Messages->add( sprintf( T_('Could not open a new stub file [%s]. Lame host?'), $new_file ), 'info' );
				$use_new_stub = false;
			}
			else
			{
				// okay good, continue
				$new_stub = str_replace( '$blog = NNN;', '$blog = '.$edited_Blog->ID.';', $file_imploded );
				fwrite( $f, $new_stub );
				fclose( $f );
				$use_new_stub = true;
			}
		}
		if( $use_new_stub && ( ! is_file( $basepath.$stub_filename ) ) )
		{
			// Is it possible to fopen but not fwrite? If so we end up here :(
			$Messages->add( sprintf( T_('Could not write the new stub file [%s]. Lame host?'), $new_stub ), 'info' );
			$use_new_stub = false;
		}
		if( $use_new_stub && is_file( $basepath.$stub_filename ) )
		{
			// Stub file exists and is waiting :)
			$DB->query( 'UPDATE T_blogs SET blog_access_type = "relative", blog_siteurl = "'.$stub_filename.'" WHERE blog_ID = '.$edited_Blog->ID );
			$Messages->add( sprintf(T_('The new blog has been associated with the stub file &laquo;%s&raquo;.'), $stub_filename ), 'success' );
		}
		else
		{
			$Messages->add( sprintf(T_('Blog access will be default (index.php?blog=%s).'), $edited_Blog->ID ), 'info' );
		}
		// Set default user permissions for this blog (All permissions for the current user)
		// Proceed insertions
		$DB->query( "INSERT INTO T_blog_users( bloguser_blog_ID, bloguser_user_ID, bloguser_ismember,
			bloguser_perm_poststatuses, bloguser_perm_delpost, bloguser_perm_comments, bloguser_perm_cats,
			bloguser_perm_properties, bloguser_perm_media_upload, bloguser_perm_media_browse,
			bloguser_perm_media_change ) VALUES ( $edited_Blog->ID, $current_User->ID, 1,
			'published,protected,private,draft,deprecated', 1, 1, 1, 1, 1, 1, 1 )" );
		// Create default category
		$edited_Category = new Category( NULL, $edited_Blog->ID );
		$edited_Category->set( 'name', T_('Uncategorized') );
		$edited_Category->set( 'urlname', 'main' );
		$edited_Category->dbinsert( false );
		$Messages->add( T_('A default category has been created for this blog.'), 'success' );
		// ADD DEFAULT WIDGETS
		$templateCache = get_Cache( 'TemplateCache' );
		$_template = $templateCache->get_by_ID( $template_ID );
		$_template->CreateDefaultWidgets( $edited_Blog );
		$DB->commit();
		// Commit changes in cache
		$BlogCache = & get_Cache( 'BlogCache' );
		$BlogCache->add( $edited_Blog );
		// We want to highlight the edited object on next list display
		// $Session->set( 'fadeout_array', array( 'blog_ID' => array( $edited_Blog->ID ) ) );
		header_redirect( $dispatcher.'?ctrl=blog_settings&tab=features&blog='.$edited_Blog->ID ); // will save $Messages into Session
	}
	break;

	case 'delete': // Delete a blog from DB
	// Check permissions
	$current_User->check_perm( 'blog_properties', 'edit', true, $blog );
	if( param( 'confirm', 'integer', 0 ) )
	{
		// confirmed
		// Delete from DB
		$msg = sprintf( T_('Blog &laquo;%s&raquo; deleted.'), $edited_Blog->dget( 'name' ) );
		$edited_Blog->dbdelete();
		$Messages->add( $msg, 'success' );
		$BlogCache->remove_by_ID( $blog );
		unset( $edited_Blog );
		unset( $Blog );
		forget_param( 'blog' );
		set_working_blog( 0 );
		$UserSettings->delete( 'selected_blog' ); // Needed or subsequent pages may try to access the delete blog
		$UserSettings->dbupdate();
		$action = 'list';
	}
	break;
}

// Display page header, menus & messages
if( strpos( $action, 'new' ) === false )
{
	// Not creating a new blog
	// fp> TODO: fall back to ctrl=cats when no perm for blog_properties
	$AdminUI->set_blog_list_params( 'blog_properties', 'edit', array( 'ctrl' => 'blog_settings', 'tab' => 'general' ), T_('List'), '?ctrl=blogs&amp;blog=0' );
}

// Display <html><head>...</head> section (should be done early if actions do not redirect)
$AdminUI->disp_html_head();
// Display title, menu, messages, etc... (messages MUST be displayed AFTER the actions)
$AdminUI->disp_body_top();

switch( $action )
{
	case 'new':
	// DIRTY HACK :/ replacing an even worse hack...
	$AdminUI->displayed_sub_begin = 1;

	// Begin payload block
	$AdminUI->disp_payload_begin();
	$AdminUI->disp_view( 'blogs/views/_blog_sel_type.view.php' );
	$AdminUI->disp_payload_end();
	break;

	case 'new-seltemplate':
	// DIRTY HACK :/ replacing an even worse hack...
	$AdminUI->displayed_sub_begin = 1;

	// Begin payload block
	$AdminUI->disp_payload_begin();
	$AdminUI->disp_view( 'templates/views/_template_select_create.view.php' );
	$AdminUI->disp_payload_end();
	break;

	case 'new-name':
	case 'create': // in case of validation error
	// DIRTY HACK :/ replacing an even worse hack...
	$AdminUI->displayed_sub_begin = 1;

	// Begin payload block
	$AdminUI->disp_payload_begin();
	// "New blog" form
	echo '<h2>'.sprintf( T_('New %s'), Blog::kind_name( $kind ) ).':</h2>';
	$next_action = 'create';
	$AdminUI->disp_view( 'blogs/views/_blog_general.form.php' );
	$AdminUI->disp_payload_end();
	break;

	case 'delete': // Delete a blog from DB
	// Not confirmed
	?>
	<div class="panelinfo">
	<h3><?php printf( T_('Delete blog [%s]?'), $edited_Blog->dget( 'name' ) )?></h3>
	<p><?php echo T_('Deleting this blog will also delete all its categories, posts and comments!') ?></p>
	<p><?php echo T_('THIS CANNOT BE UNDONE!') ?></p>
	<p>
	<?php
	$Form = new Form( NULL, '', 'get', 'none' );
	$Form->begin_form( 'inline' );
	$Form->hidden_ctrl();
	$Form->hidden( 'action', 'delete' );
	$Form->hidden( 'blog', $edited_Blog->ID );
	$Form->hidden( 'confirm', 1 );
	$Form->submit( array( '', T_('I am sure!'), 'DeleteButton' ) );
	$Form->end_form();
	$Form->begin_form( 'inline' );
	$Form->hidden_ctrl();
	$Form->hidden( 'blog', 0 );
	$Form->submit( array( '', T_('CANCEL'), 'CancelButton' ) );
	$Form->end_form();
	?>
	</p>
	</div>
	<?php
	break;

	default: // List the blogs
	$AdminUI->displayed_sub_begin = 1; // DIRTY HACK :/ replacing an even worse hack...
	// Begin payload block
	$AdminUI->disp_payload_begin();
	// Display VIEW
	$AdminUI->disp_view( 'blogs/views/_blog_list.view.php' );
	$AdminUI->disp_payload_end();
}

// Display body bottom, debug info, close </html>
$AdminUI->disp_global_footer();

?>
