<?php
/**
 * This file implements the BlogSettings class
 *
 * This handles blog_ID/name/value triplets for blogs.
 *
 * @todo (1111) IMPORTANT! make $_defaults be admin-configurable
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package blogs
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * BlogSettings class
 *
 * @package blogs
 */
class BlogSettings extends AbstractSettings
{
	/**
	 * The default settings to use when a setting is not defined in the database
	 * @var array
	 */
	var $_defaults = array(
		// Blog settings -> General -> "Content / Posts"
		'orderby' => 'datestart',
		'orderdir' => 'DESC',
		'what_to_show' => 'posts', // 'posts', 'days'
		'posts_per_page' => '5', // * also on URLs subtab *
		'archive_mode' => 'monthly', // 'monthly', 'weekly', 'daily', 'postbypost'
		// Blog settings -> General -> "Description"
		'blog_footer_text' => '&copy;$year$ by $owner$',
		'single_item_footer_text' => '',
		// Blog settings -> General -> "Meta data"
		'xml_item_footer_text' => '<p><small><a href="$item_perm_url$" title="$item_title$">$item_title$</a></small></p>',
		// Blog settings -> General -> "Aggregation"
		'aggregate_blog_IDs' => '',
		// Blog settings -> Features -> "Post options"
		'require_title' => 'optional', // 'required', 'optional', 'none'
		// Blog settings -> Features -> "Feedback options"
		'allow_rating' => 'never',
		'new_feedback_status' => 'draft', // 'draft', 'published', 'deprecated'
		// Blog settings -> Features -> "RSS/Atom feeds"
		'feed_content' => 'normal',
		'posts_per_feed' => '8',
		// Blog settings -> Features -> "Multiple authors"
		'use_workflow' => 0,
		// Blog settings -> Features -> "Subscriptions"
		'allow_subscriptions' => 0,
		'send_subs_what' => 'excerpt', // 'none', 'excerpt', 'stop_more', 'full_item'
		// Blog settings -> Features -> "After each new post..."
		'ping_plugins' => '', // ping plugin codes, separated by comma
		// Blog settings -> Features -> "External Feeds"
		'atom_redirect' => '',
		'rss2_redirect' => '',
		// Blog settings -> Features -> "Contact Form"
		'contact_id' => 1,
		'contact_title_before' => 'Send a message',
		'contact_msg_before' => '',
		'contact_subject' => '',
		'contact_redirect' => '',
		'contact_title_after' => 'Message sent',
		'contact_msg_after' => '',
		// Blog settings -> Features -> "Caching [Admin]"
		'cache_enabled' => 0,
		// Blog settings -> Features -> "Credits Page [Admin]"
		'credits_page' => 'credits',
		'credit_sections' => 'template,plugins,contribs', // sections shown on credits page
		// Blog settings -> Categories -> "Categories for blog"
		'default_cat_ID' => NULL,
		// Blog settings -> URLs -> "Main page / post list"
		'default_noindex' => '0',
		'canonical_homepage' => 1,
		'relcanonical_homepage' => 1,
		'paged_noindex' => '1',
		'paged_nofollowto' => '0',
		'title_link_type' => 'permalink',
		'main_content' => 'normal',
		'permalinks' => 'single', // 'single', 'archive', 'subcat'
		// Blog settings -> URLs -> "Single post pages / "Permalink" pages"
		'single_links' => 'short', // 'param_num', 'param_title', 'short', 'y', 'ym', 'ymd', 'subcat', 'chapters'
		'canonical_item_urls' => 1,
		'relcanonical_item_urls' => 1,
		'excerpts_meta_description' => '0',
		// Blog settings -> URLs -> "Category pages"
		'category_links' => 'cats', // 'param_num', 'subcat', 'cats'
		'category_prefix' => '',
		'canonical_cat_urls' => 1,
		'relcanonical_cat_urls' => 1,
		'category_noindex' => '1',
		'category_content' => 'excerpt',
		'category_posts_per_page' => NULL,
		'posts_per_page' => '5', // * also on General subtab *
		'catdir_noindex' => '1',
		'categories_meta_description' => '0',
		// Blog settings -> URLs -> "Tag pages"
		'tag_links' => 'colon', // 'param', 'colon', 'semicolon' -- fp> we want this changed to prefix only for new blogs only
		'tag_prefix' => '', // fp> fp> we want this changed to prefix only for new blogs only
		'tag_rel_attrib' => 1, // rel="tag" attribute for tag links (http://microformats.org/wiki/rel-tag) -- valid only in prefix-only mode
		'canonical_tag_urls' => 1,
		'relcanonical_tag_urls' => 1,
		'tag_noindex' => '1',
		'tag_content' => 'excerpt',
		'tag_posts_per_page' => NULL,
		// Blog settings -> URLs -> ""By date" archives"
		'archive_links' => 'extrapath', // param, extrapath
		'canonical_archive_urls' => 1,
		'relcanonical_archive_urls' => 1,
		'archive_noindex' => '1',
		'archive_nofollowto' => '0',
		'archive_content' => 'excerpt',
		'archive_posts_per_page' => '100',
		'arcdir_noindex' => '1',
		// Blog settings -> URLs -> "Other filtered pages"
		'filtered_noindex' => '1',
		'filtered_content' => 'excerpt',
		// Blog settings -> URLs -> "Other pages"
		'feedback-popup_noindex' => '1',
		'msgform_noindex' => '1',
		'special_noindex' => '1',
		'404_response' => '404',
	);

	/**
	 * Constructor
	 */
	function BlogSettings()
	{
		parent::AbstractSettings( 'T_blog_settings', array( 'cset_blog_ID', 'cset_name' ), 'cset_value', 1 );
	}


	/**
	 * Loads the settings
	 *
	 * Not meant to be called directly, but gets called when needed.
	 *
	 * @param string First column key
	 * @param string Second column key
	 * @return boolean
	 */
	function _load( $blog_ID, $arg )
	{
		if( empty( $blog_ID ) )
		{
			return false;
		}
		return parent::_load( $blog_ID, $arg );
	}

}

?>
