<?php
/**
 * This file implements the UI view for the blogs list on blog management screens
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package blogs
 */
if(!defined('QP_MAIN_INIT')) die('fail');

global $current_User;
global $dispatcher;
global $Settings;

$sql = 'SELECT T_blogs.*, user_login FROM T_blogs INNER JOIN T_users ON blog_owner_user_ID = user_ID';

if( ! $current_User->check_perm( 'blogs', 'view' ) )
{
	// We do not have perm to view all blogs... we need to restrict to those we're a member of
	$sql .= " LEFT JOIN T_blog_users ON (blog_advanced_perms <> 0
		AND blog_ID = bloguser_blog_ID
		AND bloguser_user_ID = {$current_User->ID} )
		LEFT JOIN T_blog_groups ON (blog_advanced_perms <> 0
		AND blog_ID = bloggroup_blog_ID
		AND bloggroup_group_ID = {$current_User->group_ID} )
		WHERE blog_owner_user_ID = {$current_User->ID}
		OR bloguser_ismember <> 0
		OR bloggroup_ismember <> 0";
	$no_results = T_('Sorry, you have no permission to edit/view any blog\'s properties.');
}
else
{
	$no_results = T_('No blog has been created yet!');
}

// Create result set
$Results = new Results( $sql, 'blog_' );
$Results->Cache = & get_Cache( 'BlogCache' );
$Results->title = T_('Blog list');
$Results->no_results_text = $no_results;

if( $current_User->check_perm( 'blogs', 'create' ) )
{
	$Results->global_icon( T_('New blog...'), 'new', url_add_param( $dispatcher, 'ctrl=blogs&amp;action=new' ), T_('New blog...') );
}

$Results->cols[] = array(
	'th' => T_('ID'),
	'order' => 'blog_ID',
	'th_class' => 'shrinkwrap',
	'td_class' => 'shrinkwrap',
	'td' => '$blog_ID$',
);

$Results->cols[] = array(
	'th' => T_('Name'),
	'order' => 'blog_shortname',
	'td' => '<strong>%disp_blog_name( #blog_shortname#, #blog_ID# )%</strong>',
);

$Results->cols[] = array(
	'th' => T_('Full Name'),
	'order' => 'blog_name',
	'td' => '%strmaxlen( #blog_name#, 40 )%',
);

$Results->cols[] = array(
	'th' => T_('Owner'),
	'order' => 'user_login',
	'td' => '$user_login$',
);

$Results->cols[] = array(
	'th' => T_('Blog URL'),
	'td' => '<a href="@get(\'url\')@">@get(\'url\')@</a>',
);

$Results->cols[] = array(
	'th' => T_('Locale'),
	'order' => 'blog_locale',
	'th_class' => 'shrinkwrap',
	'td_class' => 'shrinkwrap',
	'td' => '%locale_flag( #blog_locale# )%',
);

$Results->cols[] = array(
	'th' => T_('Actions'),
	'th_class' => 'shrinkwrap',
	'td_class' => 'shrinkwrap',
	'td' => '%disp_actions( #blog_ID# )%',
);

$Results->display( NULL, 'session' );


/**
 * dummy docblock
 */
function disp_actions( $curr_blog_ID )
{
	global $current_User;

	$r = '';

	if( $current_User->check_perm( 'blog_properties', 'edit', false, $curr_blog_ID ) )
	{
		$r .= action_icon( T_('Edit properties...'), 'properties', regenerate_url( 'ctrl', 'ctrl=blog_settings&amp;blog='.$curr_blog_ID ) );
	}

	if( $current_User->check_perm( 'blog_cats', '', false, $curr_blog_ID ) )
	{
		$r .= action_icon( T_('Edit categories...'), 'edit', regenerate_url( 'ctrl', 'ctrl=cats&amp;blog='.$curr_blog_ID ) );
	}

	if( $current_User->check_perm( 'blog_properties', 'edit', false, $curr_blog_ID ) )
	{
		$r .= action_icon( T_('Delete this blog...'), 'delete', regenerate_url( 'ctrl', 'ctrl=blogs&amp;action=delete&amp;blog='.$curr_blog_ID ) );
	}

	if( empty( $r ) )
	{
		$r = '&nbsp;'; // for IE
	}

	return $r;
}


/**
 * dummy docblock
 */
function disp_blog_name( $blog_name, $blog_ID )
{
	global $ctrl;
	global $current_User;

	if( $ctrl == 'dashboard' )
	{
		// Dashboard
		$edit_url = regenerate_url( 'ctrl', 'ctrl=dashboard&amp;blog='.$blog_ID );
		$r = '<a href="'.$edit_url.'">';
		$r .= $blog_name;
		$r .= '</a>';
	}
	elseif( $current_User->check_perm( 'blog_properties', 'edit', false, $blog_ID ) )
	{
		// Blog setting & can edit
		$edit_url = regenerate_url( 'ctrl', 'ctrl=blog_settings&amp;blog='.$blog_ID );
		$r = '<a href="'.$edit_url.'" title="'.T_('Edit properties...').'">';
		$r .= $blog_name;
		$r .= '</a>';
	}
	else
	{
		$r = $blog_name;
	}
	return $r;
}

?>
