<?php
/**
 * This file implements the Category class
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package categories
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * Category Class
 *
 * @package categories
 */
class Category extends GenericCategory
{
	/**
	 * Blog ID number
	 * @var integer
	 */
	var $blog_ID;
	/**
	 * The blog this Category belongs to
	 * @var object
	 */
	var $Blog;
	/**
	 * Unique string used in URLs for each category
	 * @var string
	 */
	var $urlname;
	/**
	 * Description of a category (not used?)
	 * @var string
	 */
	var $description;
	/**
	 * Order number of the category, if Manual Order is enabled
	 * @var integer
	 */
	var $order;
	/**
	 * ID of the category's parent cat, NULL if this is a parent
	 * @var integer
	 */
	var $parent_Category;

	/**
	 * Constructor
	 *
	 * @param table Database row
	 * @param integer|NULL subset to use for new object
	 */
	function Category( $db_row = NULL, $subset_ID = NULL )
	{
		// Call parent constructor
		parent::GenericCategory( 'T_categories', 'cat_', 'cat_ID', $db_row );

		if( is_null( $db_row ) )
		{
			// We are creating an object here
			$this->set( 'blog_ID', $subset_ID );
		}
		else
		{
			// Wa are loading an object
			$this->blog_ID = $db_row->cat_blog_ID;
			$this->urlname = $db_row->cat_urlname;
			$this->description = $db_row->cat_description;
			$this->order = $db_row->cat_order;
		}
	}


	/**
	 * Load data from Request form fields
	 *
	 * @return boolean true if loaded data seems valid.
	 */
	function load_from_request()
	{
		global $DB;
		global $Settings;

		parent::load_from_Request();

		// Check url name
		param( 'cat_urlname', 'string' );
		$this->set_from_Request( 'urlname' );

		// Check description
		param( 'cat_description', 'string' );
		$this->set_from_Request( 'description' );

		if( $Settings->get( 'category_order ') == 'manual' )
		{
			// Manual ordering
			param( 'cat_order', 'integer' );
			$this->set_from_Request( 'order' );
		}

		return ! param_errors_detected();
	}


	/**
	 * dummy docblock
	 */
	function & get_parent_Category()
	{
		if( ! isset( $this->parent_Category ) )
		{
			// Not resoleved yet!
			if( empty( $this->parent_ID ) )
			{
				$this->parent_Category = NULL;
			}
			else
			{
				$CategoryCache = & get_Cache( 'CategoryCache' );
				$this->parent_Category = & $CategoryCache->get_by_ID( $this->parent_ID );
			}
		}
		return $this->parent_Category;
	}


	/**
	 * Get URL path (made of URL names) back to the root
	 */
	function get_url_path()
	{
		$r = $this->urlname.'/';
		$parent_Category = & $this->get_parent_Category();
		if( ! is_null( $parent_Category ) )
		{
			// Recurse
			$r = $parent_Category->get_url_path().$r;
		}
		return $r;
	}


	/**
	 * Generate the URL to access the category
	 *
	 * @param string|NULL 'param_num', 'subcat', 'chapters'
	 * @param string|NULL url to use
	 * @param integer category page to link to, default:1
	 * @param integer|NULL number of posts per page (used for param_num only)
	 * @param string glue between url params
	 */
	function get_permanent_url( $link_type = NULL, $blogurl = NULL, $paged = 1, $category_posts_per_page = NULL, $glue = '&amp;' )
	{
		global $DB, $cacheweekly, $Settings;

		if( empty( $link_type ) )
		{
			// Use default from settings
			$this->get_Blog();
			$link_type = $this->Blog->get_setting( 'category_links' );
		}

		if( empty( $blogurl ) )
		{
			$this->get_Blog();
			$blogurl = $this->Blog->gen_blogurl();
		}

		switch( $link_type )
		{
			case 'param_num':
			$r = url_add_param( $blogurl, 'cat='.$this->ID, $glue );
			if( empty( $category_posts_per_page ) )
			{
				// Use default from Blog
				$this->get_Blog();
				$category_posts_per_page = $this->Blog->get_setting( 'category_posts_per_page' );
			}
			if( ! empty( $category_posts_per_page ) && $category_posts_per_page != $this->Blog->get_setting( 'posts_per_page' ) )
			{
				// We want a specific post per page count
				$r = url_add_param( $r, 'posts='.$category_posts_per_page, $glue );
			}
			break;

			case 'subcat':
			$this->get_Blog();
			$category_prefix = $this->Blog->get_setting( 'category_prefix' );
			if( ! empty( $category_prefix ) )
			{
				$r = url_add_tail( $blogurl, '/'.$category_prefix.'/'.$this->urlname.'/' );
			}
			else
			{
				$r = url_add_tail( $blogurl, '/'.$this->urlname.'/' );
			}
			break;

			case 'cats':
			default:
			$this->get_Blog();
			$category_prefix = $this->Blog->get_setting( 'category_prefix' );
			if( ! empty( $category_prefix ) )
			{
				$r = url_add_tail( $blogurl, '/'.$category_prefix.'/'.$this->get_url_path() );
			}
			else
			{
				$r = url_add_tail( $blogurl, '/'.$this->get_url_path() );
			}
			break;
		}

		if( $paged > 1 )
		{
			$r = url_add_param( $r, 'paged='.$paged, $glue );
		}

		return $r;
	}


	/**
	 * Get the Blog object for the Category
	 *
	 * @return Blog
	 */
	function & get_Blog()
	{
		if( is_null( $this->Blog ) )
		{
			$this->load_Blog();
		}
		return $this->Blog;
	}


	/**
	 * Load the Blog object for the Category, without returning it
	 */
	function load_Blog()
	{
		if( is_null( $this->Blog ) )
		{
			$BlogCache = & get_Cache( 'BlogCache' );
			$this->Blog = & $BlogCache->get_by_ID( $this->blog_ID );
		}
	}


	/**
	 * Insert object into DB based on previously recorded changes
	 *
	 * @param boolean Whether to notify the user when the url title has been changed.
	 * @return boolean true on success
	 */
	function dbinsert( $urlname_verbose_change = true )
	{
		global $DB;

		if( $this->ID != 0 ) debug_die( 'Existing object cannot be inserted!' );

		$DB->begin();
		// validate url title / slug
		$this->set( 'urlname', urltitle_validate( $this->urlname, $this->name, $this->ID, false,
			$this->dbprefix.'urlname', $this->dbIDname, $this->dbtablename, $urlname_verbose_change ) );
		$r = parent::dbinsert();
		$DB->commit();

		return $r;
	}


	/**
	 * Update the DB based on previously recorded changes
	 *
	 * @return boolean true on success
	 */
	function dbupdate()
	{
		global $DB;

		$DB->begin();
		// validate url title / slug
		if( empty( $this->urlname ) || isset( $this->dbchanges['cat_urlname'] ) )
		{
			// Url title has changed or is empty
			$this->set( 'urlname', urltitle_validate( $this->urlname, $this->name, $this->ID, false, $this->dbprefix.'urlname', $this->dbIDname, $this->dbtablename ) );
		}
		$r = parent::dbupdate();
		$DB->commit();

		return $r;
	}

}

?>
