<?php
/**
 * This file implements Category handling functions
 *
 * @todo (0000) implement CategoryCache based on LinkCache
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package categories
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * Get blog ID for a given cat
 *
 * This halts on error.
 * @return integer
 */
function get_catblog( $cat_ID )
{
	$CategoryCache = & get_Cache( 'CategoryCache' );
	$Category = $CategoryCache->get_by_ID( $cat_ID );

	return $Category->blog_ID;
}


/**
 * Load cache for category definitions
 *
 * Warning: this loads all categories for ALL blogs
 */
function cat_load_cache()
{
	global $DB;
	global $cache_categories;
	global $Settings;
	global $Timer;
	global $timestamp_max;
	global $timestamp_min;

	if( isset( $cache_categories ) )
	{
		return;
	}

	$Timer->resume( 'cat_load_cache' );

	$sql = 'SELECT cat_ID, cat_parent_ID, cat_name, cat_blog_ID FROM T_categories';
	if( $Settings->get( 'category_order' ) == 'manual' )
	{
		$sql .= ' ORDER BY cat_order'; // Manual order
	}
	else
	{
		$sql .= ' ORDER BY cat_name'; // Alphabetic order
	}

	foreach( $DB->get_results( $sql, ARRAY_A, 'loading CAT cache' ) as $myrow )
	{
		$this_cat['cat_name'] = $myrow['cat_name'];
		$this_cat['cat_blog_ID'] = $myrow['cat_blog_ID'];
		$this_cat['cat_parent_ID'] = $myrow['cat_parent_ID'];
		$this_cat['cat_postcount'] = 0; // Will be increased later
		$this_cat['cat_children'] = array();
		$cache_categories[$myrow['cat_ID']] = $this_cat;
	}

	// Reveal children
	if( ! empty( $cache_categories ) )
	{
		foreach( $cache_categories as $icat_ID => $i_cat )
		{
			$cat_parent_ID = $i_cat['cat_parent_ID'];
			if( $cat_parent_ID )
			{
				if( isset( $cache_categories[$cat_parent_ID] ) )
				{
					// If the parent exists!
					$cache_categories[$cat_parent_ID]['cat_children'][] = $icat_ID;
				}
				else
				{
					echo( "Category #$icat_ID is orphan of non existant parent #$cat_parent_ID!<br />" );
				}
			}
		}
	}

	$Timer->pause( 'cat_load_cache' );
}


/**
 * Get category associations with given post
 */
function postcats_get_byID( $post_ID )
{
	global $DB;

	$sql = "SELECT postcat_cat_ID FROM T_item_cats WHERE postcat_post_ID = $post_ID ORDER BY postcat_cat_ID";
	return $DB->get_col( $sql );
}


/**
 * Taking a recursive walk in the category park...
 *
 * @param array PHP requires this stupid cloning of the cache_categories array in
 * order to be able to perform foreach on it
 * @param integer
 * @param integer
 * @param string|array Callback for first category
 * @param string|array Callback before each category
 * @param string|array Callback after each category
 * @param string|array Callback after last category
 * @param integer Caller nesting level, just to keep track of how far we go :)
 * @return string
 */
function cat_children(
	$ccats,
	$blog_ID,
	$parent_ID,
	$callback_before_first,
	$callback_before_each,
	$callback_after_each,
	$callback_after_last,
	$level = 0,
	$root_call = true )
{
	static $total_count = 0;

	$r = '';

	if( empty( $ccats ) )
	{
		// this can happen if there are no cats
		return '';
	}

	if( $root_call )
	{
		// Init
		$total_count = 0;
	}

	$child_count = 0;
	foreach( $ccats as $icat_ID => $i_cat )
	{
		if( empty( $icat_ID )
			|| ! ( /* @todo (0000) check ( $blog_ID == 0 ) || */ ( $i_cat['cat_blog_ID'] == $blog_ID ) )
			|| ! ( $i_cat['cat_parent_ID'] == $parent_ID ) )
		{
			// this cat is not in the blog and or is not a child of the parent
			continue;
		}

		// this cat is in the blog and is a child of the parent...
		$total_count++;

		// "before first"
		if( $child_count++ == 0 )
		{
			// this is the first child
			if( is_array( $callback_before_first ) )
			{
				// object callback
				$r .= $callback_before_first[0]->{$callback_before_first[1]}( $parent_ID, $level, $total_count, 1 );
			}
			else
				$r .= $callback_before_first( $parent_ID, $level, $total_count, 1 );
		}

		// "before each"
		if( is_array( $callback_before_each ) )
		{
			// object callback
			$r2 = $callback_before_each[0]->{$callback_before_each[1]}( $icat_ID, $level, $total_count, $child_count );
		}
		else
			$r2 = $callback_before_each( $icat_ID, $level, $total_count, $child_count );
		if( $r2 === true )
		{
			// callback function has requested that we stop recursing for this branch
			continue;
		}
		$r .= $r2;

		// Recursion
		$r .= cat_children( $ccats, $blog_ID, $icat_ID, $callback_before_first, $callback_before_each, $callback_after_each, $callback_after_last, $level+1, false );

		// "after each"
		if( is_array( $callback_after_each ) )
		{
			// object callback
			$r .= $callback_after_each[0]->{$callback_after_each[1]}( $icat_ID, $level, $total_count, $child_count );
		}
		else
		{
			$r .= $callback_after_each( $icat_ID, $level, $total_count, $child_count );
		}
	}
	if( $child_count )
	{
		// There have been children
		if( is_array( $callback_after_last ) )
			$r .= $callback_after_last[0]->{$callback_after_last[1]}( $parent_ID, $level, $total_count, $child_count );
		else
			$r .= $callback_after_last( $parent_ID, $level, $total_count, $child_count );
	}

	return $r;
}


/**
 * Does a given blog have categories?
 *
 * @param integer Blog ID
 * @return boolean
 */
function blog_has_cats( $blog_ID )
{
	global $cache_categories;

	cat_load_cache( 'none' );

	if( count( $cache_categories ) ) foreach( $cache_categories as $icat_ID => $i_cat )
	{
		if( $icat_ID && $i_cat['cat_blog_ID'] == $blog_ID )
		{
			return true; // this cat is in the blog
		}
	}

	return false;
}


/**
 * Compiles the cat array from $cat
 *
 * Compiles the cat array from $cat (recursive + optional modifiers) and $catsel[]
 * (non recursive).
 *
 * @param string
 * @param array
 * @param array by ref, will be modified
 * @param string by ref, will be modified
 * @param integer blog number to restrict to
 */
function compile_cat_array( $cat, $catsel, & $cat_array, & $cat_modifier, $restrict_to_blog = 0 )
{
	global $cache_categories;

	$cat_array = array();
	$cat_modifier = '';

	// Check for cat string (which will be handled recursively)
	if( $cat != 'all' && ! empty( $cat ) )
	{
		// specified a category string
		$cat_modifier = substr( $cat, 0, 1 );
		if( ( $cat_modifier == '*' )
			|| ( $cat_modifier == '-' ) )
		{
			$cat = substr( $cat, 1 );
		}
		else
		{
			$cat_modifier = '';
		}

		if( strlen( $cat ) )
		{
			// There are some values to explode...
			$req_cat_array = explode(',', $cat);

			// Getting required sub-categories
			// and add everything to cat array
			// START RECURSIVE CAT LIST
			cat_load_cache(); // make sure the caches are loaded
			foreach( $req_cat_array as $cat_ID )
			{
				// run recursively through the cats
				if( ! in_array( $cat_ID, $cat_array ) )
				{
					// Not already in list
					$cat_array[] = $cat_ID;
					cat_children( $cache_categories, $restrict_to_blog, $cat_ID, 'cat_req_dummy', 'cat_req',
						'cat_req_dummy', 'cat_req_dummy', 1 );
				}
			}
		}
	}

	// Add explicit selections
	if( ! empty( $catsel ))
	{
		$cat_array = array_merge( $cat_array, $catsel );
		$cat_array = array_unique( $cat_array );
	}

}


/**
 * Callback used in compile_cat_array()
 */
function cat_req( $parent_cat_ID, $level )
{
	global $cat_array;

	if( ! in_array( $parent_cat_ID, $cat_array ) )
	{
		$cat_array[] = $parent_cat_ID; // Not already visited
	}
	else
	{
		return -1; // STOP going through that branch
	}
}

/**
 * Callback used in compile_cat_array()
 */
function cat_req_dummy()
{
}

?>
