<?php
/**
 * This file implements the CategoryCache class
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package categories
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * CategoryCache class
 *
 * @package categories
 */
class CategoryCache extends GenericCategoryCache
{
	/**
	 * Lazy filled index of urlnames
	 * @var array
	 */
	var $urlname_index = array();

	/**
	 * Constructor
	 */
	function CategoryCache()
	{
		global $Settings;

		if( $Settings->get( 'category_order' ) == 'manual' )
		{
			$order_by = 'cat_order'; // Manual order
		}
		else
		{
			$order_by = 'cat_name'; // Alphabetic order
		}
		// Call parent constructor
		parent::GenericCategoryCache( 'Category', false, 'T_categories', 'cat_', 'cat_ID', 'cat_name', 'blog_ID', $order_by );
	}


	/**
	 * Empty/reset the cache
	 */
	function clear()
	{
		$this->urlname_index = array();
		parent::clear();
	}


	/**
	 * Get an object from cache by ID
	 *
	 * Load the cache if necessary (all at once if allowed).
	 *
	 * @param integer ID of object to load
	 * @param boolean true if function should die on error
	 * @param boolean true if function should die on empty/null
	 * @param integer|NULL NULL for all subsets
	 * @return Category Reference on cached object or false.
	 */
	function & get_by_ID( $req_ID, $halt_on_error = true, $halt_on_empty = true, $subset_ID = NULL )
	{
		global $DB;

		if( empty( $req_ID ) )
		{
			if( $halt_on_empty )
			{
				debug_die( "Requested Category from $this->dbtablename without ID!" );
			}
			$r = NULL;
			return $r;
		}

		if( ! empty( $this->cache[$req_ID] ) )
		{
			// Already in cache
			return $this->cache[$req_ID];
		}
		elseif( ! $this->all_loaded )
		{
			// Not in cache, but not everything is loaded yet
			if( $this->load_all || is_null( $subset_ID ) )
			{
				// It's ok to just load everything
				$this->load_all();
			}
			else
			{
				// Load just the requested object
				// Note: $req_ID MUST be an unsigned integer. This is how DataObject works.
				$sql = "SELECT * FROM T_categories WHERE cat_ID = $req_ID AND cat_blog_ID = ".$subset_ID;
				if( $row = $DB->get_row( $sql, OBJECT, 0, 'CategoryCache::get_by_ID()' ) )
				{
					if( ! $this->instantiate( $row ) )
					{
						// debuglog-add removed
					}
				}
				else
				{
					// debuglog-add removed
				}
			}
		}

		if( empty( $this->cache[$req_ID] ) )
		{
			// Requested object does not exist
			if( $halt_on_error ) debug_die( "Requested $this->objtype does not exist!" );
			$r = false;
			return $r;
		}

		return $this->cache[$req_ID];
	}


	/**
	 * Get an object from cache by urlname
	 *
	 * Load the cache if necessary (all at once if allowed).
	 *
	 * @param string ID of object to load
	 * @param boolean true if function should die on error
	 * @return reference on cached object
	 */
	function & get_by_urlname( $req_urlname, $halt_on_error = true )
	{
		global $DB;

		if( ! isset( $this->urlname_index[$req_urlname] ) )
		{
			// not yet in cache, Load just the requested object
			$sql = "SELECT * FROM $this->dbtablename WHERE cat_urlname = ".$DB->quote( $req_urlname );
			$row = $DB->get_row( $sql );
			if( empty( $row ) )
			{
				// Requested object does not exist
				if( $halt_on_error ) debug_die( "Requested $this->objtype does not exist!" );
				// put into index
				$this->urlname_index[$req_urlname] = false;
				return $this->urlname_index[$req_urlname];
			}

			$this->instantiate( $row );
			// put into index
			$this->urlname_index[$req_urlname] = & $this->cache[$row->cat_ID];
		}

		return $this->urlname_index[$req_urlname];
	}


	/**
	 * Load a keyed subset of the cache
	 *
	 * @param integer|NULL NULL for all subsets
	 */
	function load_subset( $subset_ID )
	{
		global $DB;
		global $Settings;

		if( $this->all_loaded || isset( $this->loaded_subsets[$subset_ID] ) )
		{
			// Already loaded
			return false;
		}

		// fp> TODO: This kills other subsets. BAD if we want to handle multiple subsets independently
		$this->clear( true );

		$sql = 'SELECT * FROM T_categories WHERE cat_blog_ID = '.$subset_ID;

		if( $Settings->get( 'category_order' ) == 'manual' )
		{
			$sql .= ' ORDER BY cat_order'; // Manual order
		}
		else
		{
			$sql .= ' ORDER BY cat_name'; // Alphabetic order
		}

		foreach( $DB->get_results( $sql, OBJECT, 'Loading categories('.$subset_ID.') into cache' ) as $row )
		{
			// Instantiate a custom object
			$this->instantiate( $row );
		}

		$this->loaded_subsets[$subset_ID] = true;

		return true;
	}


	/**
	 * Move a category and its descendants to a different blog
	 *
	 * @todo (0000) [TODO_HERE] find out what's freakin wrong
	 * @param integer
	 * @param integer
	 * @param integer
	 */
	function move_Category_subtree( $cat_ID, $src_blog_ID, $dest_blog_ID )
	{
		global $DB;

		// Make sure children have been revealed for specific subset
		$this->reveal_children( $src_blog_ID );

		// [TODO_HERE] fp> We get the Category AFTER reveal_children, because something
		// is wrong with reveal_children or get_by_ID. I don't know what right now,
		// but if we get Category earlier, we'll be stuck with an old copy of it that
		// does NOT have the children
		$Category = $this->get_by_ID( $cat_ID );

		// Get $cats_to_move
		$cats_to_move = array();
		$this->recurse_move_subtree( $Category, $cats_to_move );

		$DB->begin();
		// Move to root
		if( $parent_Category = $Category->get_parent_Category() )
		{
			// Was not already at root, cut it and move it
			$Category->set( 'parent_ID', NULL );
			$Category->dbupdate();
		}
		// Move categories to new Blog
		$sql = 'UPDATE T_categories
			SET cat_blog_ID = '.$dest_blog_ID.'
			WHERE cat_blog_ID = '.$src_blog_ID /* extra security */ .'
			AND cat_ID IN ('.implode( ',', $cats_to_move ).')';
		$DB->query( $sql );
		$DB->commit();

		// Now the cache is badly screwed. Reseting it is fair enough, because this won't happen very often.
		$this->clear();
	}


	/**
	 * Support function for move_Category_subtree
	 *
	 * @param Category
	 * @param array
	 */
	function recurse_move_subtree( & $Category, & $list_array )
	{
		// Add this to the list
		$list_array[] = $Category->ID;
		foreach( $Category->children as $child_Category )
		{
			$this->recurse_move_subtree( $child_Category, $list_array );
		}
	}


	/**
	 * Instanciate a new object within this cache
	 *
	 * @param object|NULL
	 * @param integer|NULL subset to use for new object
	 */
	function & new_obj( $row = NULL, $subset_ID = NULL )
	{
		// Instantiate a custom object
		$Category = new Category( $row, $subset_ID ); // Copy
		return $Category;
	}

}

?>
