<?php
/**
 * This file implements the recursive category list
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package categories
 */
if(!defined('QP_MAIN_INIT')) die('fail');

global $Blog;
global $current_default_cat_ID;
global $dispatcher;
global $GenericCategoryCache;
global $line_class;
global $permission_to_edit;
global $result_fadeout;
global $Settings;
global $subset_ID;

$current_default_cat_ID = $Blog->get_setting( 'default_cat_ID' );

$line_class = 'odd';

$callbacks = array(
	'line' => 'cat_line',
	'no_children' => 'cat_no_children',
	'before_level' => 'cat_before_level',
	'after_level' => 'cat_after_level'
);

// Display generic categories
$Table = new Table();
$Table->title = sprintf( T_('Categories for blog: %s'), $Blog->get_maxlen_name( 50 ) );
$Table->global_icon( T_('Create a new category...'), 'new', regenerate_url( 'action,'.$GenericCategoryCache->dbIDname, 'action=new' ), T_('New category').' &raquo;' );

$Table->cols[] = array(
	'th' => T_('ID'),
);

$Table->cols[] = array(
	'th' => T_('Default'),
	'th_class' => 'shrinkwrap',
);

$Table->cols[] = array(
	'th' => T_('Name'),
);

$Table->cols[] = array(
	'th' => T_('URL "slug"'),
);
// are we manually ordering categories?
if( $Settings->get( 'category_order' ) == 'manual' )
{
	$Table->cols[] = array(
		'th' => T_('Order'),
		'th_class' => 'shrinkwrap',
	);
}
// do we display action column?
if( $permission_to_edit )
{
	$Table->cols[] = array(
		'th' => T_('Actions'),
	);
}

$Table->display_init( NULL, $result_fadeout );

// add an id for jquery to hook into
$Table->params['head_title'] = str_replace( '<table', '<table id="chapter_list"', $Table->params['head_title'] );

$Table->display_list_start();
$Table->display_head();
$Table->display_body_start();
echo $GenericCategoryCache->recurse( $callbacks, $subset_ID );
$Table->display_body_end();
$Table->display_list_end();

// a couple of notes about what settings you got and where to change them
echo '<p class="note">'.sprintf( T_('<strong>Note:</strong> Moving categories across blogs is currently %s in the %sglobal settings%s.'),
	$Settings->get( 'allow_moving_cats' ) == '0' ? T_('disabled ') : T_('enabled'), '<a href="'.$dispatcher.'?ctrl=settings#categories">', '</a>' ).'</p>';

echo '<p class="note">'.sprintf( T_('<strong>Note:</strong> Ordering of categories is currently set to %s in the %sglobal settings%s.'),
	$Settings->get( 'category_order' ) == 'manual' ? /* TRANS: Manual here = "by hand" */ T_('Manual ') : T_('Alphabetical'), '<a href="'.$dispatcher.'?ctrl=settings#categories">', '</a>' ).'</p>';


/**
 * Generate category line when it has children
 *
 * @param Category generic category we want to display
 * @param int level of the category in the recursive tree
 * @return string HTML
 */
function cat_line( $Category, $level )
{
	global $current_default_cat_ID;
	global $current_User;
	global $GenericCategoryCache;
	global $line_class;
	global $permission_to_edit;
	global $result_fadeout;
	global $Settings;

	$line_class = $line_class == 'even' ? 'odd' : 'even';

	$r = '<tr id="tr-'.$Category->ID.'"class="'.$line_class.' chapter_parent_'.( $Category->parent_ID ? $Category->parent_ID : '0' ).
		// Fadeout?
		( isset( $result_fadeout[$GenericCategoryCache->dbIDname] ) && in_array( $Category->ID, $result_fadeout[$GenericCategoryCache->dbIDname] ) ? ' fadeout-ffff00': '' ).'">
		<td class="firstcol shrinkwrap">'.$Category->ID.'</td>';

	$makedef_url = regenerate_url( 'action,cat_ID', 'cat_ID='.$Category->ID.'&amp;action=make_default' );
	$makedef_title = T_('Click to make this the default category');

	if( $current_default_cat_ID == $Category->ID )
	{
		$makedef_icon = 'enabled';
	}
	else
	{
		$makedef_icon = 'disabled';
	}
	$r .= '<td class="center"><a href="'.$makedef_url.'" title="'.$makedef_title.'">'.get_icon( $makedef_icon, 'imgtag' ).'</a></td>';

	if( $permission_to_edit )
	{
		// We have permission permission to edit
		$edit_url = regenerate_url( 'action,cat_ID', 'cat_ID='.$Category->ID.'&amp;action=edit' );
		$r .= '<td><strong style="padding-left: '.( $level ).'em;"><a href="'.$edit_url.'" title="'.T_('Edit...').'">'.$Category->dget( 'name' ).'</a></strong></td>';
	}
	else
	{
		$r .= '<td><strong style="padding-left: '.( $level ).'em;">'.$Category->dget( 'name' ).'</strong></td>';
	}

	$r .= '<td>'.$Category->dget( 'urlname' ).'</td>';

	if( $Settings->get( 'category_order' ) == 'manual' )
	{
		$r .= '<td class="center">'.$Category->dget( 'order' ).'</td>';
	}

	$r .= '<td class="lastcol shrinkwrap">';
	if( $permission_to_edit )
	{
		// We have permission permission to edit, so display action column
		$r .= '<a href="'.$makedef_url.'" title="'.$makedef_title.'">'.get_icon( 'activate', 'imgtag' ).'</a>';
		$r .= action_icon( T_('Edit...'), 'edit', $edit_url );
		if( $Settings->get( 'allow_moving_cats' ) )
		{
			// If moving cats between blogs is allowed
			$r .= action_icon( T_('Move to a different blog...'), 'move', regenerate_url( 'action,cat_ID', 'cat_ID='.$Category->ID.'&amp;action=move' ), T_('Move') );
		}
		$r .= action_icon( T_('New...'), 'new', regenerate_url( 'action,cat_ID,cat_parent_ID', 'cat_parent_ID='.$Category->ID.'&amp;action=new' ) )
			.action_icon( T_('Delete...'), 'delete', regenerate_url( 'action,cat_ID', 'cat_ID='.$Category->ID.'&amp;action=delete' ) );
	}
	$r .= '</td></tr>';

	return $r;
}


/**
 * Generate code when exiting from a level
 *
 * @param int level of the category in the recursive tree
 * @return string HTML
 */
function cat_after_level( $level )
{
	return '';
}


/**
 * Generate code when entering a new level
 *
 * @param int level of the category in the recursive tree
 * @return string HTML
 */
function cat_before_level( $level )
{
	return '';
}


/**
 * Generate category line when it has no children
 *
 * @param Category generic category we want to display
 * @param int level of the category in the recursive tree
 * @return string HTML
 */
function cat_no_children( $Category, $level )
{
	return '';
}

?>
