<?php
/**
 * This file implements Comment handling functions
 *
 * @todo (0000) implement CommentCache based on LinkCache
 * @todo (0000) generic_ctp_number(): check this in a multiblog page...
 * @todo (0000) generic_ctp_number(): This should support visibility, at least in
 * the default front office (_feedback.php), there should only the number of visible
 * comments/trackbacks get used ({@link Item::feedback_link()}).
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package comments
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * Generic comments/trackbacks counting
 *
 * @param integer
 * @param string what to count
 */
function generic_ctp_number( $post_id, $mode = 'comments', $status = 'published' )
{
	global $cache_ctp_number;
	global $DB;
	global $debug;
	global $postdata;
	global $postIDarray;
	global $postIDlist;
	global $preview;

	if( $preview )
	{
		// no comments in preview mode
		return 0;
	}

	// Make sure cache is loaded for current display list
	if( ! isset( $cache_ctp_number ) )
	{
		if( ! empty( $postIDlist ) )
		{
			// This can happen when displaying a featured post of something that's not in the MainList
			foreach( $postIDarray as $tmp_post_id)
			{
				// Initializes each post to nocount!
				$cache_ctp_number[$tmp_post_id] = array(
					'comments' => array( 'published' => 0, 'draft' => 0, 'deprecated' => 0, 'total' => 0 ),
					'trackbacks' => array( 'published' => 0, 'draft' => 0, 'deprecated' => 0, 'total' => 0 ),
					'feedbacks' => array( 'published' => 0, 'draft' => 0, 'deprecated' => 0, 'total' => 0 )
				);
			}

			$query = 'SELECT comment_post_ID, comment_type, comment_status,
				COUNT(*) AS type_count
				FROM T_comments
				WHERE comment_post_ID IN ('.$postIDlist.')
				GROUP BY comment_post_ID, comment_type, comment_status';

			foreach( $DB->get_results( $query ) as $row )
			{
				// detail by status, tyep and post
				$cache_ctp_number[$row->comment_post_ID][$row->comment_type.'s'][$row->comment_status] = $row->type_count;
				// Total for type on post
				$cache_ctp_number[$row->comment_post_ID][$row->comment_type.'s']['total'] += $row->type_count;
				// Total for status on post
				$cache_ctp_number[$row->comment_post_ID]['feedbacks'][$row->comment_status] += $row->type_count;
				// Total for post
				$cache_ctp_number[$row->comment_post_ID]['feedbacks']['total'] += $row->type_count;
			}
		}
	}

	if( ! isset( $cache_ctp_number[$post_id] ) ) // this should be extremely rare...
	{
		// Initializes post to nocount!
		$cache_ctp_number[intval( $post_id )] = array(
			'comments' => array( 'published' => 0, 'draft' => 0, 'deprecated' => 0, 'total' => 0 ),
			'trackbacks' => array( 'published' => 0, 'draft' => 0, 'deprecated' => 0, 'total' => 0 ),
			'feedbacks' => array( 'published' => 0, 'draft' => 0, 'deprecated' => 0, 'total' => 0 )
		);

		$query = 'SELECT comment_post_ID, comment_type, comment_status,
			COUNT(*) AS type_count
			FROM T_comments
			WHERE comment_post_ID = '.intval( $post_id ).'
			GROUP BY comment_post_ID, comment_type, comment_status';

		foreach( $DB->get_results( $query ) as $row )
		{
			// detail by status, tyep and post
			$cache_ctp_number[$row->comment_post_ID][$row->comment_type.'s'][$row->comment_status] = $row->type_count;
			// Total for type on post
			$cache_ctp_number[$row->comment_post_ID][$row->comment_type.'s']['total'] += $row->type_count;
			// Total for status on post
			$cache_ctp_number[$row->comment_post_ID]['feedbacks'][$row->comment_status] += $row->type_count;
			// Total for post
			$cache_ctp_number[$row->comment_post_ID]['feedbacks']['total'] += $row->type_count;
		}
	}

	if( ( $mode != 'comments' ) && ( $mode != 'trackbacks' ) )
	{
		$mode = 'feedbacks';
	}

	if( ( $status != 'published' ) && ( $status != 'draft' ) && ( $status != 'deprecated' ) )
	{
		$status = 'total';
	}

	return $cache_ctp_number[$post_id][$mode][$status];
}


/**
 * Get a Comment by ID
 *
 * Exits if the requested comment does not exist!
 *
 * @param integer
 * @return Comment
 */
function Comment_get_by_ID( $comment_ID )
{
	global $cache_Comments;
	global $DB;

	if( empty( $cache_Comments[$comment_ID] ) )
	{
		// Load this entry into cache
		$query = "SELECT *
			FROM T_comments
			WHERE comment_ID = $comment_ID";
		if( $row = $DB->get_row( $query, ARRAY_A ) )
		{
			$cache_Comments[$comment_ID] = new Comment( $row ); // COPY !
		}
	}

	if( empty( $cache_Comments[$comment_ID] ) )
	{
		die('Requested comment does not exist!');
	}

	return $cache_Comments[$comment_ID];
}


/**
 * comments_number(-)
 *
 * @deprecated deprecated by {@link Item::feedback_link()} NOT deprecated, this
 * is used by qp_inc/items/views/_item_list_full.view.php in v0.0.0 (EdB)
 */
function comments_number( $zero='#', $one='#', $more='#', $post_ID = NULL )
{
	global $id;

	if( $zero == '#' )
	{
		$zero = T_('Leave a comment');
	}
	if( $one == '#' )
	{
		$one = T_('1 comment');
	}
	if( $more == '#' )
	{
		$more = T_('%d comments');
	}

	// original hack by dodo@regretless.com
	if( empty( $post_ID ) )
	{
		$post_ID = $id;
	}
	$number = generic_ctp_number( $post_ID, 'comments' );
	if( $number == 0 )
	{
		$blah = $zero;
	}
	elseif( $number == 1 )
	{
		$blah = $one;
	}
	elseif( $number > 1 )
	{
		$n = $number;
		$more = str_replace('%d', $n, $more);
		$blah = $more;
	}
	echo $blah;
}

?>
