<?php
/**
 * This file implements cron (scheduled tasks) handling functions
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package cron
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * Log a message from cron
 *
 * @param string Message
 * @param integer Level of importance. The higher the more important. (if $quiet
 * (number of "-q" params passed to cron_exec.php) is higher than this, the message
 * gets skipped)
 */
function cron_log( $message, $level = 0 )
{
	global $is_web;
	global $quiet;

	if( $quiet > $level )
	{
		return;
	}

	if( $is_web )
	{
		echo '<p>'.$message.'</p>';
	}
	else
	{
		echo "\n".$message."\n";
	}
}


/**
 * Call a cron job
 *
 * @param string Name of the job
 * @param string Params for the job
 */
function call_job( $job_name, $job_params = array() )
{
	global $DB;
	global $inc_path;
	global $Plugins;
	global $result_message;
	global $result_status;
	global $time_difference;
	global $timestop;

	$result_message = NULL;
	$result_status = 'error';

	if( preg_match( '~^plugin_(\d+)_(.*)$~', $job_name, $match ) )
	{
		// Cron job provided by a plugin
		if( ! is_object( $Plugins ) )
		{
			$Plugins = new Plugins();
		}

		$Plugin = & $Plugins->get_by_ID( $match[1] );
		if( ! $Plugin )
		{
			$result_message = sprintf( T_('Plugin for controller [%s] could not get instantiated.'), $job_name );
			cron_log( $result_message, 2 );
			return;
		}

		// call the plugin to handle the job
		$tmp_params = array( 'ctrl' => $match[2], 'params' => $job_params );
		$sub_r = $Plugins->call_method( $Plugin->ID, 'ExecCronJob', $tmp_params );
		$error_code = (int)$sub_r['code'];
		$result_message = $sub_r['message'];
	}
	else
	{
		$controller = $inc_path.$job_name;
		if( ! is_file( $controller ) )
		{
			$result_message = sprintf( T_('Controller [%s] does not exist.'), $job_name );
			cron_log( $result_message, 2 );
			return;
		}

		// include the job file and run it
		$error_code = require $controller;
	}

	if( $error_code != 1 )
	{
		// We got an error :(
		$result_status = 'error';
		$result_message = sprintf( T_('[Error code: %s ] '), $error_code ).$result_message;
		$cron_log_level = 2;
	}
	else
	{
		$result_status = 'finished';
		$cron_log_level = 1;
	}

	$timestop = time() + $time_difference;
	cron_log( 'Task finished at '.date( 'H:i:s', $timestop ).' with status: '.$result_status
		."\nMessage: $result_message", $cron_log_level );
}

?>
