<?php
/**
 * This file implements the Cronjob class
 *
 * Manages a single cron job as registered in the DB.
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package cron
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * Cronjob class
 *
 * @package cron
 */
class Cronjob extends DataObject
{
	/**
	 * Start date/time for the task
	 * @todo needs useful description
	 * @var string
	 */
	var $start_datetime;
	/**
	 * Frequency of the task (seconds?)
	 * @var integer
	 */
	var $repeat_after = NULL;
	/**
	 * Task name
	 * @var string
	 */
	var $name;
	/**
	 * Controller
	 * @var string
	 */
	var $controller;
	/**
	 * Params
	 * @var array
	 */
	var $params;

	/**
	 * Constructor
	 *
	 * @param table Database row
	 */
	function Cronjob( $db_row = NULL )
	{
		// Call parent constructor
		parent::DataObject( 'T_cron_task', 'ctsk_', 'ctsk_ID', '', '', '', '' );

		if( $db_row != NULL )
		{
			// Loading an object from DB
			$this->ID = $db_row->ctsk_ID;
			$this->start_datetime = $db_row->ctsk_start_datetime;
			$this->repeat_after = $db_row->ctsk_repeat_after;
			$this->name = $db_row->ctsk_name;
			$this->controller = $db_row->ctsk_controller;
			$this->params = $db_row->ctsk_params;
		}
		else
		{
			// New object
		}
	}


	/**
	 * Set param value
	 *
	 * By default, all values will be considered strings
	 *
	 * @param string parameter name
	 * @param mixed parameter value
	 * @param boolean true to set to NULL if empty value
	 * @return boolean true, if a value has been set; false if it has not changed
	 */
	function set( $parname, $parvalue, $make_null = false )
	{
		switch( $parname )
		{
			case 'params':
			return $this->set_param( 'params', 'string', serialize( $parvalue ), false );

			case 'name':
			return $this->set_param( $parname, 'string', substr( $parvalue, 0, 50 ), false );
		}

		return $this->set_param( $parname, 'string', $parvalue, $make_null );
	}


	/**
	 * Get a member param by its name
	 *
	 * @param mixed Name of parameter
	 * @return mixed Value of parameter
	 */
	function get( $parname )
	{
		switch( $parname )
		{
			case 'params':
			return unserialize( $this->params );
		}

		return parent::get( $parname );
	}

}

?>
