<?php
/**
 * This file implements the UI view for the general settings
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package cron
 */
if(!defined('QP_MAIN_INIT')) die('fail');

global $ctst_error;
global $ctst_finished;
global $ctst_pending;
global $ctst_started;
global $ctst_timeout;
global $srvc_url;

// Get filters
if( ! $ctst_pending && ! $ctst_started && ! $ctst_timeout && ! $ctst_error && ! $ctst_finished )
{
	// Set default status filters
	$ctst_error = 1;
	$ctst_finished = 0;
	$ctst_pending = 1;
	$ctst_started = 1;
	$ctst_timeout = 1;
}

// Create result set
$SQL = new SQL();
$SQL->SELECT( 'ctsk_ID, ctsk_start_datetime, ctsk_name, ctsk_repeat_after, IFNULL( clog_status, "pending" ) as status' );
$SQL->FROM( 'T_cron_task LEFT JOIN T_cron_log ON ctsk_ID = clog_ctsk_ID' );
if( $ctst_pending )
{
	$SQL->WHERE_or( 'clog_status IS NULL' );
}
if( $ctst_started )
{
	$SQL->WHERE_or( 'clog_status = "started"' );
}
if( $ctst_timeout )
{
	$SQL->WHERE_or( 'clog_status = "timeout"' );
}
if( $ctst_error )
{
	$SQL->WHERE_or( 'clog_status = "error"' );
}
if( $ctst_finished )
{
	$SQL->WHERE_or( 'clog_status = "finished"' );
}
$SQL->ORDER_BY( '*, ctsk_ID' );

$Results = new Results( $SQL->get(), 'crontab_', '-A' );
$Results->title = T_('Scheduled jobs');
$Results->global_icon( T_('Refresh'), 'refresh', regenerate_url(), T_('Refresh') );
if( $current_User->check_perm( 'options', 'edit', false, NULL ) )
{
	// Permission to edit settings
	$Results->global_icon( T_('Create a new scheduled job...'), 'new', regenerate_url( 'action,cjob_ID', 'action=new' ), T_('New job').' &raquo;' );
}

$Results->filter_area = array(
	'callback' => 'filter_crontab',
	'url_ignore' => 'results_crontab_page,ctst_pending,ctst_started,ctst_timeout,ctst_error,ctst_finished', // ignore epage param and checkboxes
	'presets' => array(
		'schedule' => array( T_('Schedule'), '?ctrl=crontab&amp;ctst_pending=1&amp;ctst_started=1&amp;ctst_timeout=1&amp;ctst_error=1' ),
		'attention' => array( T_('Attention'), '?ctrl=crontab&amp;ctst_timeout=1&amp;ctst_error=1' ),
		'all' => array( T_('All'), '?ctrl=crontab&amp;ctst_pending=1&amp;ctst_started=1&amp;ctst_timeout=1&amp;ctst_error=1&amp;ctst_finished=1' ),
	)
);

$Results->cols[] = array(
	'th' => T_('ID'),
	'order' => 'ctsk_ID',
	'th_class' => 'shrinkwrap',
	'td_class' => 'shrinkwrap',
	'td' => '$ctsk_ID$'
);

$Results->cols[] = array(
	'th' => T_('Planned at'),
	'order' => 'ctsk_start_datetime',
	'td_class' => 'shrinkwrap',
	'td' => '$ctsk_start_datetime$',
);

$Results->cols[] = array(
	'th' => T_('Name'),
	'order' => 'ctsk_name',
	'td' => '<a href="%regenerate_url(\'action,cjob_ID\',\'action=view&amp;cjob_ID=$ctsk_ID$\')%">$ctsk_name$</a>',
);

$Results->cols[] = array(
	'th' => T_('Status'),
	'order' => 'status',
	'td_class' => 'shrinkwrap cron_$status$',
	'td' => '$status$',
);

$Results->cols[] = array(
	'th' => T_('Repeat'),
	'order' => 'ctsk_repeat_after',
	'td_class' => 'shrinkwrap',
	'td' => '$ctsk_repeat_after$',
);

$Results->cols[] = array(
	'th' => T_('Actions'),
	'td_class' => 'shrinkwrap',
	'td' => '%crontab_actions( #ctsk_ID#, #status# )%',
);

// Display results
$Results->display();

echo '<p>[<a href="'.$srvc_url.'cron_exec.php" onclick="return pop_up_window( \''.$srvc_url.'cron_exec.php\', \'evo_cron\' )" target="evo_cron">'.T_('Execute pending jobs in a popup window now!').'</a>]</p>';

/**
 * dummy docblock
 */
function crontab_actions( $ctsk_ID, $status )
{
	global $current_User;
	global $srvc_url;

	$col = '';
	if( $status != 'started' && $current_User->check_perm( 'options', 'edit', false, NULL ) )
	{
		// User can edit options
		$col = action_icon( T_('Delete this job!'), 'delete', regenerate_url( 'action', 'ctsk_ID='.$ctsk_ID.'&amp;action=delete' ) );
	}
	return $col;
}


/**
 * Callback to add filters on top of the result set
 *
 * @param Form
 */
function filter_crontab( & $Form )
{
	global $ctst_error;
	global $ctst_finished;
	global $ctst_pending;
	global $ctst_started;
	global $ctst_timeout;

	$Form->checkbox( 'ctst_pending', $ctst_pending, T_('Pending') );
	$Form->checkbox( 'ctst_started', $ctst_started, T_('Started') );
	$Form->checkbox( 'ctst_timeout', $ctst_timeout, T_('Timed out') );
	$Form->checkbox( 'ctst_error', $ctst_error, T_('Error') );
	$Form->checkbox( 'ctst_finished', $ctst_finished, T_('Finished') );
}

?>
