<?php
/**
 * $ctrl_mappings: 'fileset' || Global settings -> Files
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package files
 */
if(!defined('QP_MAIN_INIT')) die('fail');

// Check minimum permission
$current_User->check_perm( 'options', 'view', true );

$AdminUI->set_path( 'options', 'files' );

param( 'action', 'string' );

switch( $action )
{
	case 'update':
	// Check permission
	$current_User->check_perm( 'options', 'edit', true );
	param( 'submit', 'array', array() );
	if( isset( $submit['restore_defaults'] ) )
	{
		$Settings->delete_array( array(
			'fm_enable_roots_blog',
			'fm_blog_limit_files',
			'fm_blog_limit_bytes',
			'fm_enable_roots_user',
			'fm_user_limit_files',
			'fm_user_limit_bytes',
			'fm_enable_roots_shared',
			'fm_shared_limit_files',
			'fm_shared_limit_bytes',
			'fm_enable_roots_templates',
			'fm_templates_limit_files',
			'fm_templates_limit_bytes',
			'fm_enable_create_dir',
			'fm_default_chmod_dir',
			'fm_enable_create_file',
			'fm_default_chmod_file',
			'upload_enabled',
			'upload_maxkb',
			'regexp_filename',
		) );
		if( $Settings->dbupdate() )
		{
			$Messages->add( T_('Restored default values.'), 'success' );
		}
		else
		{
			$Messages->add( T_('Settings have not changed.'), 'note' );
		}
	}
	else
	{
		// "Accessible file roots" settings
		// ... blog folders
		param( 'fm_enable_roots_blog', 'integer', 0 );
		$Settings->set( 'fm_enable_roots_blog', $fm_enable_roots_blog );
		param_check_number( 'fm_blog_limit_files', sprintf( T_('"%s file limit" must be numbers only'), 'blog' ) );
		$Settings->set( 'fm_blog_limit_files', $fm_blog_limit_files );
		param_check_number( 'fm_blog_limit_bytes', sprintf( T_('"%s dir size" must be numbers only'), 'blog' ) );
		$Settings->set( 'fm_blog_limit_bytes', $fm_blog_limit_bytes );
		// ... user folders
		param( 'fm_enable_roots_user', 'integer', 0 );
		$Settings->set( 'fm_enable_roots_user', $fm_enable_roots_user );
		param_check_number( 'fm_user_limit_files', sprintf( T_('"%s file limit" must be numbers only'), 'user' ) );
		$Settings->set( 'fm_user_limit_files', $fm_user_limit_files );
		param_check_number( 'fm_user_limit_bytes', sprintf( T_('"%s dir size" must be numbers only'), 'user' ) );
		$Settings->set( 'fm_user_limit_bytes', $fm_user_limit_bytes );
		// ... shared folders
		param( 'fm_enable_roots_shared', 'integer', 0 );
		$Settings->set( 'fm_enable_roots_shared', $fm_enable_roots_shared );
		param_check_number( 'fm_shared_limit_files', sprintf( T_('"%s file limit" must be numbers only'), 'shared' ) );
		$Settings->set( 'fm_shared_limit_files', $fm_shared_limit_files );
		param_check_number( 'fm_shared_limit_bytes', sprintf( T_('"%s dir size" must be numbers only'), 'shared' ) );
		$Settings->set( 'fm_shared_limit_bytes', $fm_shared_limit_bytes );
		// ... template folders
		param( 'fm_enable_roots_templates', 'integer', 0 );
		$Settings->set( 'fm_enable_roots_templates', $fm_enable_roots_templates );
		param_check_number( 'fm_templates_limit_files', sprintf( T_('"%s file limit" must be numbers only'), 'template' ) );
		$Settings->set( 'fm_templates_limit_files', $fm_templates_limit_files );
		param_check_number( 'fm_templates_limit_bytes', sprintf( T_('"%s dir size" must be numbers only'), 'template' ) );
		$Settings->set( 'fm_templates_limit_bytes', $fm_templates_limit_bytes );

		// "File creation options" settings
		param( 'fm_enable_create_dir', 'integer', 0 );
		$Settings->set( 'fm_enable_create_dir', $fm_enable_create_dir );
		param( 'fm_enable_create_file', 'integer', 0 );
		$Settings->set( 'fm_enable_create_file', $fm_enable_create_file );
		param( 'upload_enabled', 'integer', 0 );
		$Settings->set( 'upload_enabled', $upload_enabled );
		param_integer_range( 'upload_maxkb', 1, $upload_maxmaxkb, T_('Maximum allowed filesize must be between %d and %d KB.') );
		$Settings->set( 'upload_maxkb', $upload_maxkb ); // upload limit

		// "File creation options" settings
		// ... default dir CHMOD
		if( param( 'fm_default_chmod_dir', 'string', NULL ) !== NULL )
		{
			if( ! preg_match('~^[0-7]{3}$~', $fm_default_chmod_dir) )
			{
				param_error('fm_default_chmod_dir', T_('Invalid CHMOD value. Use 3 digits.'));
			}
			$Settings->set( 'fm_default_chmod_dir', $fm_default_chmod_dir );
		}
		// ... default files CHMOD
		if( param( 'fm_default_chmod_file', 'string', NULL ) !== NULL )
		{
			if( ! preg_match('~^[0-7]{3}$~', $fm_default_chmod_file) )
			{
				param_error('fm_default_chmod_file', T_('Invalid CHMOD value. Use 3 digits.'));
			}
			$Settings->set( 'fm_default_chmod_file', $fm_default_chmod_file );
		}
		// ... valid folder and file name
		param( 'regexp_dirname', 'string', '' );
		if( param_check_isregexp( 'regexp_dirname', T_('Valid dirname pattern is not a regular expression!') ) )
		{
			$Settings->set( 'regexp_dirname', $regexp_dirname );
		}
		param( 'regexp_filename', 'string', '' );
		if( param_check_isregexp( 'regexp_filename', T_('Valid filename pattern is not a regular expression!') ) )
		{
			$Settings->set( 'regexp_filename', $regexp_filename );
		}

		// wrap it up, ship it out
		if( ! $Messages->count('error') )
		{
			if( $Settings->dbupdate() )
			{
				$Messages->add( T_('File settings updated.'), 'success' );
			}
			else
			{
				$Messages->add( T_('Settings have not changed.'), 'note' );
			}
		}
	}

	break;
}

// Display <html><head>...</head> section (should be done early if actions do not redirect)
$AdminUI->disp_html_head();
// Display title, menu, messages, etc... (messages MUST be displayed AFTER the actions)
$AdminUI->disp_body_top();
// Begin payload block
$AdminUI->disp_payload_begin();
// Display VIEW:
$AdminUI->disp_view( 'files/views/_file_settings.form.php' );
// End payload block
$AdminUI->disp_payload_end();
// Display body bottom, debug info, close </html>
$AdminUI->disp_global_footer();

?>
