<?php
/**
 * This file implements the FileCache class
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package files
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * FileCache class
 *
 * @package files
 */
class FileCache extends DataObjectCache
{
	/**
	 * Cache for 'root_type:root_in_type_ID:relative_path' -> File object reference
	 * @var array
	 */
	var $cache_root_and_path = array();

	/**
	 * Constructor
	 */
	function FileCache()
	{
		// Call parent constructor
		parent::DataObjectCache( 'File', false, 'T_files', 'file_', 'file_ID' );
	}


	/**
	 * Instantiate a DataObject from a table row and then cache it
	 *
	 * @param Object Database row
	 * @return Object
	 */
	function & instantiate( & $db_row )
	{
		// Get ID of the object we'ere preparing to instantiate...
		$obj_ID = $db_row->{$this->dbIDname};

		if( ! empty( $obj_ID ) )
		{
			// If the object ID is valid
			if( ! isset( $this->cache[$obj_ID] ) )
			{
				// Instantiate a File object for this line
				$current_File = new File( $db_row->file_root_type, $db_row->file_root_ID, $db_row->file_path ); // COPY!
				// Flow meta data into File object
				$current_File->load_meta( false, $db_row );
				$this->add( $current_File );
			}
			else
			{
				// Already cached
				$current_File = & $this->cache[$obj_ID];
				// Flow meta data into File object
				$current_File->load_meta( false, $db_row );
			}
		}

		return $this->cache[$obj_ID];
	}


	/**
	 * Get by Root and Path
	 *
	 * Creates an object of the {@link File} class, while providing caching and making
	 * sure that only one reference to a file exists.
	 *
	 * @param string Root type: 'user', 'blog', 'shared', 'template' or 'absolute'
	 * @param integer ID of the user, the blog or template the file belongs to...
	 * @param string Subpath for this file/folder, relative the associated root, including
	 * trailing slash (if directory)
	 * @param boolean check for meta data?
	 * @return File a {@link File} object
	 */
	function & get_by_root_and_path( $root_type, $root_in_type_ID, $rel_path, $load_meta = false )
	{
		global $cache_File;

		if( is_windows() )
		{
			$rel_path = strtolower(str_replace( '\\', '/', $rel_path ));
		}

		// Generate cache key for this file
		$cacheindex = $root_type.':'.$root_in_type_ID.':'.$rel_path;

		if( isset( $this->cache_root_and_path[$cacheindex] ) )
		{
			$File = & $this->cache_root_and_path[$cacheindex];
			if( $load_meta )
			{
				// Make sure meta is loaded
				$File->load_meta();
			}
		}
		else
		{
			// Not in cache
			$File = new File( $root_type, $root_in_type_ID, $rel_path, $load_meta ); // COPY !!
			$this->cache_root_and_path[$cacheindex] = & $File;
		}

		return $File;
	}

}

?>
