<?php
/**
 * This file implements the FileRoot base class
 *
 * A FileRoot describes a directory available for media file storage, under access permission.
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package files
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * FileRoot base class
 *
 * @package files
 */
class FileRoot
{
	/**
	 * Type: 'user', 'blog', 'shared', 'template' or 'absolute'
	 *
	 * Note: Absolute isn't used and was going to be deprecated in b2evo, but
	 * maybe QP will be smart and use it?
	 *
	 * @var string
	 */
	var $type;
	/**
	 * ID of user, blog or template
	 * @var string
	 */
	var $in_type_ID;
	/**
	 * Unique Root ID constructed from type and in_type_ID
	 * @var string
	 */
	var $ID;
	/**
	 * Name of the root
	 * @var string
	 */
	var $name;
	/**
	 * Absolute path, ending with slash
	 * @var string
	 */
	var $ads_path;
	/**
	 * Absolute URL, ending with slash
	 * @var string
	 */
	var $ads_url;

	/**
	 * Constructor
	 *
	 * Will fail if non existent User or Blog is requested. But specific access permissions
	 * on (threfore existence of) this User or Blog should have been tested before anyway.
	 *
	 * @param string Root type: 'user', 'blog', 'shared', 'template' or 'absolute'
	 * @param integer ID of the user, the blog or the template the file belongs to...
	 * @param boolean Create the directory, if it does not exist yet?
	 */
	function FileRoot( $root_type, $root_in_type_ID, $create = true )
	{
		global $current_User;
		global $Messages;
		global $Settings;
		global $templates_path;
		global $templates_url;

		// Store type
		$this->type = $root_type;
		// Store ID in type
		$this->in_type_ID = $root_in_type_ID;
		// Generate unique ID
		$this->ID = FileRoot::gen_ID( $root_type, $root_in_type_ID );

		switch( $root_type )
		{
			case 'user':
			$UserCache = & get_Cache( 'UserCache' );
			$User = & $UserCache->get_by_ID( $root_in_type_ID );
			$this->name = $User->get( 'preferredname' );
			$this->ads_path = $User->get_media_dir( $create );
			$this->ads_url = $User->get_media_url();
			return;

			case 'blog':
			$BlogCache = & get_Cache( 'BlogCache' );
			$Blog = & $BlogCache->get_by_ID( $root_in_type_ID );
			$this->name = $Blog->get( 'shortname' );
			$this->ads_path = $Blog->get_media_dir( $create );
			$this->ads_url = $Blog->get_media_url();
			return;

			case 'shared':
			global $media_path;
			global $media_url;

			// fp> TODO: handle multiple shared directories
			$rds_shared_subdir = 'shared/global/';
			$ads_shared_dir = $media_path.$rds_shared_subdir;
			if( ! $Settings->get( 'fm_enable_roots_shared' ) )
			{
				// Templates root is disabled
				// debuglog-add removed
			}
			elseif( ! is_dir( $ads_shared_dir ) )
			{
				$Messages->add( sprintf( T_('The directory &laquo;%s&raquo; does not exist.'), $rds_shared_subdir ), 'error' );
			}
			else
			{
				$this->name = T_('Shared');
				$this->ads_path = $ads_shared_dir;
				$this->ads_url = $media_url.'shared/global/';
			}
			return;

			case 'templates':
			// fp> some stuff here should go out of here... but I don't know where to put it yet. I'll see after the Template refactoring.
			if( ! $Settings->get( 'fm_enable_roots_templates' ) )
			{
				// Templates root is disabled
				// debuglog-add removed
			}
			elseif( ! $current_User->check_perm( 'templates' ) )
			{
				// No perm to access templates
				// debuglog-add removed
			}
			else
			{
				$this->name = T_('Templates');
				$this->ads_path = $templates_path;
				$this->ads_url = $templates_url;
			}
			return;
		}

		debug_die( "Root_type=$root_type not supported" );
	}


	/**
	 * dummy docblock
	 */
	function get_typegroupname()
	{
		switch( $this->type )
		{
			case 'user':
			return NT_('User roots');

			case 'blog':
			return NT_('Blog roots');

			default:
			return NT_('Special roots');
		}
	}

	/**
	 * dummy docblock
	 */
	function gen_ID( $root_type, $root_in_type_ID )
	{
		switch( $root_type )
		{
			case 'user':
			case 'blog':
			case 'shared':
			case 'templates':
			return $root_type.'_'.$root_in_type_ID;
		}
		debug_die( "Root_type=$root_type not supported" );
	}

}

?>
