<?php
/**
 * This file implements the FileRootCache base class
 *
 * This class provides info about File Roots. These are root directories available
 * for media file storage, under access permission.
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package files
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * FileRootCache base class
 *
 * @package files
 */
class FileRootCache
{
	/**
	 * Internal cache
	 * @todo needs valid description
	 * @var array
	 */
	var $cache = array();

	/**
	 * Get an array of ALL available Fileroots (not just the cached ones)
	 *
	 * @todo fp> it would probably make sense to refactor this as the constructor for the file roots
	 * and initialize the whole cache at construction time
	 * @return array of FileRoots (key being the FileRoot's ID)
	 */
	function get_available_FileRoots()
	{
		global $current_User;

		$r = array();

		// The user's blog (if available) is the default/first one
		$user_FileRoot = & $this->get_by_type_and_ID( 'user', $current_User->ID, true );
		if( $user_FileRoot )
		{
			// We got a user media dir
			$r[$user_FileRoot->ID] = & $user_FileRoot;
		}

		// Blog media dirs
		$BlogCache = & get_Cache( 'BlogCache' );
		$bloglist = $BlogCache->load_user_blogs( 'blog_media_browse', $current_User->ID );
		foreach( $bloglist as $blog_ID )
		{
			if( $Root = & $this->get_by_type_and_ID( 'blog', $blog_ID, true ) )
			{
				$r[$Root->ID] = & $Root;
			}
		}

		// Shared root
		$shared_FileRoot = & $this->get_by_type_and_ID( 'shared', 0, true );
		if( $shared_FileRoot )
		{
			// We got a shared dir
			$r[$shared_FileRoot->ID] = & $shared_FileRoot;
		}

		// Templates root
		$templates_FileRoot = & $this->get_by_type_and_ID( 'templates', 0, false );
		if( $templates_FileRoot )
		{
			// We got a templates dir
			$r[$templates_FileRoot->ID] = & $templates_FileRoot;
		}
		return $r;
	}


	/**
	 * Get a FileRoot (cached) by ID
	 *
	 * @uses FileRootCache::get_by_type_and_ID()
	 * @param string ID of the FileRoot (e.g. 'user_X' or 'blog_X')
	 * @param boolean Create the directory, if it does not exist yet?
	 * @return FileRoot|false FileRoot on success, false on failure (ads_path is false).
	 */
	function & get_by_ID( $id, $create = false )
	{
		$part = explode( '_', $id );
		$root_type = $part[0];
		$root_in_type_ID = $part[1];
		return $this->get_by_type_and_ID( $root_type, $root_in_type_ID, $create );
	}


	/**
	 * Get a FileRoot (cached)
	 *
	 * @param string Root type: 'user', 'blog', 'shared', 'template', or 'absolute'
	 * @param integer ID of the user, blog or template the file belongs to...
	 * @param boolean Create the directory, if it does not exist yet?
	 * @return FileRoot|false FileRoot on success, false on failure (ads_path is false).
	 */
	function & get_by_type_and_ID( $root_type, $root_in_type_ID, $create = false )
	{
		$root_ID = FileRoot::gen_ID( $root_type, $root_in_type_ID );

		if( ! isset( $this->cache[$root_ID] ) )
		{
			// Not in Cache, let's instantiate
			$Root = new FileRoot( $root_type, $root_in_type_ID, $create );
			if( empty( $Root->ads_path ) )
			{
				$Root = false;
			}
			$this->cache[$root_ID] = & $Root;
		}

		return $this->cache[$root_ID];
	}

}

?>
