<?php
/**
 * This file implements the Filetype class
 *
 * @todo (1111) vars type 'foo' need useful/valid descriptions
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package files
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * Filetype Class
 *
 * @package files
 */
class Filetype extends DataObject
{
	/**
	 * Extensions
	 * @var foo
	 */
	var $extensions = '';
	/**
	 * Name
	 * @var foo
	 */
	var $name = '';
	/**
	 * Mime Type
	 * @var foo
	 */
	var $mimetype = '';
	/**
	 * Icon
	 * @var foo
	 */
	var $icon = '';
	/**
	 * View Type
	 * @var foo
	 */
	var $viewtype = '';
	/**
	 * Allowed
	 * @var foo
	 */
	var $allowed = '';

	/**
	 * Constructor
	 *
	 * @param table Database row
	 */
	function Filetype( $db_row = NULL )
	{
		// Call parent constructor
		parent::DataObject( 'T_filetypes', 'ftyp_', 'ftyp_ID' );

		$this->delete_restrictions = array();
		$this->delete_cascades = array();

		if( $db_row != NULL )
		{
			$this->ID = $db_row->ftyp_ID;
			$this->extensions = $db_row->ftyp_extensions;
			$this->name = $db_row->ftyp_name;
			$this->mimetype = $db_row->ftyp_mimetype;
			$this->icon = $db_row->ftyp_icon;
			$this->viewtype = $db_row->ftyp_viewtype;
			$this->allowed = $db_row->ftyp_allowed;
		}
		else
		{
			// Create a new filetype
			$this->set( 'viewtype', 'browser' );
		}
	}


	/**
	 * Load data from Request form fields
	 *
	 * @return boolean true if loaded data seems valid.
	 */
	function load_from_Request()
	{
		global $forbidden_upload_exts;

		// Extensions
		if( param_string_not_empty( 'ftyp_extensions', T_('Please enter file extensions separated by space.') ) )
		{
			// Check if estensions has a valid format
			$GLOBALS['ftyp_extensions'] = strtolower( trim( $GLOBALS['ftyp_extensions'] ) );
			$reg_exp = '/^[a-z0-9]+( [a-z0-9]+)*$/';
			if( ! preg_match( $reg_exp, $GLOBALS['ftyp_extensions'], $res ) )
			{
				// Extension has an invalid format
				param_error( 'ftyp_extensions', T_( 'Invalid file extensions format.' ) );
			}
		}
		$this->set_from_Request( 'extensions' );

		// Name
		param_string_not_empty( 'ftyp_name', T_('Please enter a name.') );
		$this->set_from_Request( 'name' );
		// Mime type
		param_string_not_empty( 'ftyp_mimetype', T_('Please enter a mime type.') );
		$this->set_from_Request( 'mimetype' );
		// Icon for the mime type
		if( param( 'ftyp_icon', 'string', '' ) )
		{
			param_check_filename( 'ftyp_icon', T_('Please enter a file name.') );
		}
		$this->set_from_Request( 'icon' );
		// View type
		param( 'ftyp_viewtype', 'string' );
		$this->set_from_Request( 'viewtype' );
		// Allowed to upload theses extensions
		param( 'ftyp_allowed', 'integer', '0' );
		if( $GLOBALS['ftyp_allowed'] )
		{
			// Check if the extension is in the array of the not allowed extensions (_advanced.php)
			$not_allowed = false;
			$extensions = explode ( ' ', $GLOBALS['ftyp_extensions'] );
			foreach( $extensions as $extension )
			{
				if( in_array( $extension, $forbidden_upload_exts ) )
				{
					$not_allowed = true;
					continue;
				}
			}
			if( $not_allowed )
			{
				// this extension is not allowed
				$GLOBALS['ftyp_allowed'] = 0;
			}
		}
		$this->set_from_Request( 'allowed' );

		return ! param_errors_detected();
	}


	/**
	 * Set param value
	 *
	 * By default, all values will be considered strings
	 *
	 * @param string parameter name
	 * @param mixed parameter value
	 */
	function set( $parname, $parvalue )
	{
		switch( $parname )
		{
			case 'extensions':
			case 'name':
			case 'mimetype':
			case 'icon':
			case 'viewtype':
			case 'allowed':
			default:
			$this->set_param( $parname, 'string', $parvalue );
		}
	}

	/**
	 * Return the img html code of the icon
	 *
	 * @todo (1111) this function needs to provide width and height attributes
	 * @return string
	 */
	function get_icon()
	{
		global $rsc_url;

		$icon = $this->icon;
		if( empty( $icon ) )
		{
			// use default icon
			$icon = 'default.png';
		}

		return '<img src="'.$rsc_url.'icons/fileicons/'.$icon.'" alt="" title="'.$this->dget( 'name', 'htmlattr' ).'" class="middle" />';
	}

}

?>
