<?php
/**
 * This file implements the FiletypeCache class
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package files
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * FiletypeCache Class
 *
 * @package files
 */
class FiletypeCache extends DataObjectCache
{
	/**
	 * Extension Cache
	 * @todo needs valid description
	 * @var array
	 */
	var $extension_cache = array();

	/**
	 * Constructor
	 */
	function FiletypeCache()
	{
		// Call parent constructor
		parent::DataObjectCache( 'Filetype', true, 'T_filetypes', 'ftyp_', 'ftyp_ID', 'ftyp_extensions' );
	}


	/**
	 * Add a dataobject to the cache
	 */
	function add( & $Obj )
	{
		if( empty( $Obj->ID ) )
		{
			return false;
		}
		if( isset( $this->cache[$Obj->ID] ) )
		{
			return false;
		}

		// If the object is valid and not already cached
		$this->cache[$Obj->ID] = & $Obj;

		// cache all extensions
		$extensions = explode( ' ', $Obj->extensions );
		foreach( $extensions as $extension )
		{
			$this->extension_cache[$extension] = $Obj;
		}

		return true;
	}


	/**
	 * Get an object from cache by extensions ID
	 *
	 * Load the cache if necessary (all at once if allowed).
	 *
	 * @param string Extension string of object to load
	 * @param boolean true if function should die on error
	 * @param boolean true if function should die on empty/null
	 * @return reference on cached object
	 */
	function & get_by_extension( $req_ID, $halt_on_error = true, $halt_on_empty = true )
	{
		global $DB;

		if( empty( $req_ID ) )
		{
			if( $halt_on_empty )
			{
				debug_die( "Requested $this->objtype from $this->dbtablename without ID!" );
			}
			$r = NULL;
			return $r;
		}

		$this->load_all();

		if( empty( $this->extension_cache[$req_ID] ) )
		{
			// Requested object does not exist
			if( $halt_on_error )
			{
				debug_die( "Requested $this->objtype does not exist!" );
			}
			$r = false;
			return $r;
		}

		return $this->extension_cache[$req_ID];
	}

}

?>
