<?php
/**
 * This file implements ther UI controler for element list editor management
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package core
 */
if(!defined('QP_MAIN_INIT')) die('fail');

param( 'action', 'string', 'list' );

// Init fadeout result array
$result_fadeout = array();

if( param( $GenericElementCache->dbIDname, 'integer', NULL, true, false, false ) )
{
	if( ( $edited_GenericElement = & $GenericElementCache->get_by_ID( ${$GenericElementCache->dbIDname}, false ) ) === false )
	{
		// We could not find the element to edit
		unset( $edited_GenericElement );
		$Messages->head = T_('Cannot edit element!');
		$Messages->add( T_('Requested element does not exist any longer.'), 'error' );
		$action = 'nil';
	}
}

// Check locked elements
if( ! empty( $locked_IDs )
	&& in_array( $action, array( 'edit', 'update', 'delete' ) )
	&& in_array( ${$GenericElementCache->dbIDname}, $locked_IDs ) )
{
	$Messages->add( T_('This element is locked and cannot be edited!') );
	$action = 'list';
}

// Perform action
switch( $action )
{
	// New element...
	case 'new':
	if( isset( $perm_name ) )
	{
		// We need to Check permission
		$current_User->check_perm( $perm_name, $perm_level, true );
	}
	$edited_GenericElement = & $GenericElementCache->new_obj();
	break;

	// Edit element form...
	case 'edit': 
	// Make sure we got an ID
	param( $GenericElementCache->dbIDname, 'integer', true );
	if( isset( $perm_name ) )
	{
		// We need to Check permission
		$current_User->check_perm( $perm_name, $perm_level, true );
	}
	// Get the page number we come from
	$previous_page = param( 'results'.$GenericElementCache->dbprefix.'page', 'integer', 1, true );
	break;

	// Insert new element...
	case 'create':
	if( isset( $perm_name ) )
	{
		// We need to Check permission:
		$current_User->check_perm( $perm_name, $perm_level, true );
	}
	$edited_GenericElement = & $GenericElementCache->new_obj();
	// load data from request
	if( $edited_GenericElement->load_from_Request() )
	{
		// We could load data from form without errors, Insert in DB
		if( $edited_GenericElement->dbinsert() !== false )
		{
			$Messages->add( T_('New element created.'), 'success' );
			// Add the ID of the new element to the result fadeout
			$result_fadeout[$GenericElementCache->dbIDname][] = $edited_GenericElement->ID;
			$action = 'list';
		}
	}
	break;

	case 'update':
	// Make sure we got an ID
	param( $GenericElementCache->dbIDname, 'integer', true );

	if( isset( $perm_name ) )
	{
		// We need to Check permission
		$current_User->check_perm( $perm_name, $perm_level, true );
	}
	// LOAD FORM DATA
	if( $edited_GenericElement->load_from_Request() )
	{
		// We could load data from form without errors, Update in DB
		if( $edited_GenericElement->dbupdate() !== false )
		{
			$Messages->add( T_('Element updated.'), 'success' );
			// Add the ID of the updated element to the result fadeout
			$result_fadeout[$GenericElementCache->dbIDname][] = $edited_GenericElement->ID;
			unset( $edited_GenericElement );
			forget_param( $GenericElementCache->dbIDname );
			$action = 'list';
		}
	}
	else
	{
		// Get the page number we come from
		$previous_page = param( 'results'.$GenericElementCache->dbprefix.'page', 'integer', 1, true );
	}
	break;

	case 'copy':
	// Duplicate an element by prefilling create form
	param( $GenericElementCache->dbIDname, 'integer', true );
	if( isset( $perm_name ) )
	{
		// We need to Check permission
		$current_User->check_perm( $perm_name, $perm_level, true );
	}
	$new_Element = $edited_GenericElement; // COPY
	$new_Element->ID = 0;
	$edited_GenericElement = & $new_Element;
	$AdminUI->append_to_titlearea( T_('Copy element...') );
	break;

	// Move up the element order...
	case 'move_up':
	param( $GenericElementCache->dbIDname, 'integer', true );
	if( isset( $perm_name ) )
	{
		// We need to Check permission
		$current_User->check_perm( $perm_name, $perm_level, true );
	}
	$GenericElementCache->move_up_by_ID( $edited_GenericElement->ID );
	break;

	// Move down the element order...
	case 'move_down':
	param( $GenericElementCache->dbIDname, 'integer', true );
	if( isset( $perm_name ) )
	{
		// We need to Check permission
		$current_User->check_perm( $perm_name, $perm_level, true );
	}
	$GenericElementCache->move_down_by_ID( $edited_GenericElement->ID );
	break;

	// Delete entry...
	case 'delete':
	param( $GenericElementCache->dbIDname, 'integer', true );
	if( isset( $perm_name ) )
	{
		// We need to Check permission
		$current_User->check_perm( $perm_name, $perm_level, true );
	}
	// Set restrictions for element
	$edited_GenericElement->delete_restrictions = $delete_restrictions;
	if( param( 'confirm', 'integer', 0 ) )
	{
		// confirmed, Delete from DB
		$msg = sprintf( T_('Element &laquo;%s&raquo; deleted.'), $edited_GenericElement->dget( 'name' ) );
		$GenericElementCache->dbdelete_by_ID( $edited_GenericElement->ID );
		unset( $edited_GenericElement );
		forget_param( $GenericElementCache->dbIDname );
		$Messages->add( $msg, 'success' );
		$action = 'list';
	}
	else
	{
		// not confirmed, Check for restrictions
		// Get the page number we come from
		$previous_page = param( 'results_'.$GenericElementCache->dbprefix.'page', 'integer', 1, true );
		if( ! $edited_GenericElement->check_delete( sprintf( T_('Cannot delete element &laquo;%s&raquo;'), $edited_GenericElement->dget( 'name' ) ) ) )
		{
			// There are restrictions
			$action = 'edit';
		}
	}
	break;

	// The list is sorted by the order column now...
	case 'sort_by_order':
	$Results->order = '--A';
	set_param( 'results_'.$GenericElementCache->dbprefix.'order', '--A' );
	$action = 'list';
	break;
}

// Display <html><head>...</head> section (should be done early if actions do not redirect)
$AdminUI->disp_html_head();
// Display title, menu, messages, etc... (messages MUST be displayed AFTER the actions)
$AdminUI->disp_body_top();

// Display payload
switch( $action )
{
	case 'nil':
	// Do nothing
	break;

	case 'new':
	case 'copy':
	case 'create':
	case 'edit':
	case 'update':
	case 'delete':
	// Begin payload block
	$AdminUI->disp_payload_begin();
	if( $action == 'delete' )
	{
		// We need to ask for confirmation
		$edited_GenericElement->confirm_delete(
			sprintf( T_('Delete element &laquo;%s&raquo;?'), $edited_GenericElement->dget( 'name' ) ),
			$action, get_memorized( 'action' )
		);
	}
	if( $form_below_list )
	{
		// Display List VIEW before form VIEW
		if( ! empty( $list_view ) )
		{
			$AdminUI->disp_view( $list_view );
		}
		else
		{
			$AdminUI->disp_view( 'generic/views/_generic_list.inc.php' );
		}
	}
	// Display form
	$edited_GenericElement->disp_form();
	// End payload block
	$AdminUI->disp_payload_end();
	break;

	case 'list':
	default:
	// Begin payload block
	$AdminUI->disp_payload_begin();
	// Display VIEW
	if( ! empty( $list_view ) )
	{
		$AdminUI->disp_view( $list_view );
	}
	else
	{
		$AdminUI->disp_view( 'generic/views/_generic_list.inc.php' );
	}
	if( $form_below_list )
	{
		// Display form after list
		$action = 'new';
		$edited_GenericElement = & $GenericElementCache->new_obj();
		$edited_GenericElement->disp_form();
	}
	// End payload block
	$AdminUI->disp_payload_end();
	break;
}

// Display body bottom, debug info, close </html>
$AdminUI->disp_global_footer();

?>
