<?php
/**
 * This file implements the generic recursive list editor
 *
 * It uses <code>$AdminUI->get_path(1).'.php'</code> to link back to the ID of
 * the entry. If that causes problems later, we'd probably need to set a global
 * like $listeditor_url.
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package core
 */
if(!defined('QP_MAIN_INIT')) die('fail');

param( 'action', 'string', 'list' );

if( param( $GenericCategoryCache->dbIDname, 'integer', NULL, true, false, false ) )
{
	if( ( $edited_GenericCategory = & $GenericCategoryCache->get_by_ID( ${ $GenericCategoryCache->dbIDname }, false, true, $subset_ID ) ) === false )
	{
		// We could not find the element to edit
		unset( $edited_GenericCategory );
		$Messages->head = T_('Cannot edit element!');
		$Messages->add( T_('Requested element does not exist any longer.'), 'error' );
		$action = 'nil';
	}
}

if( ! is_null( param( $GenericCategoryCache->dbprefix.'parent_ID', 'integer', NULL ) ) )
{
	if( ( $edited_parent_GenericElement = & $GenericCategoryCache->get_by_ID( ${$GenericCategoryCache->dbprefix.'parent_ID'}, false, true, $subset_ID ) ) === false )
	{
		// Parent generic category doesn't exist any longer
		unset( $GenericCategoryCache->dbIDname );
		$Messages->head = T_('Cannot edit element!');
		$Messages->add( T_('Requested element does not exist any longer.'), 'error' );
		$action = 'nil';
	}
}

// Init fadeout result array of IDs
$result_fadeout = array();

// Check locked elements
if( ! empty( $locked_IDs )
	&& in_array( $action, array( 'edit', 'update', 'delete' ) )
	&& in_array( $$GenericCategoryCache->dbIDname, $locked_IDs ) )
{
	$Messages->add( T_('This element is locked and cannot be edited!') );
	$action = 'list';
}

// Perform action
switch( $action )
{
	// New action
	case 'new':
	if( ! $permission_to_edit )
	{
		debug_die( 'No permission to edit' );
	}
	$edited_GenericCategory = & $GenericCategoryCache->new_obj( NULL, $subset_ID );
	if( isset( $edited_parent_GenericElement ) )
	{
		$edited_GenericCategory->parent_ID = $edited_parent_GenericElement->ID;
		$edited_GenericCategory->parent_name = $edited_parent_GenericElement->name;
	}
	else
	{
		$edited_GenericCategory->parent_name = T_('Root');
	}
	break;

	// Edit element form...
	case 'edit':
	// Make sure we got an ID
	param( $GenericCategoryCache->dbIDname, 'integer', true );
	if( ! $permission_to_edit )
	{
		debug_die( 'No permission to edit' );
	}
	// Get the page number we come from
	$previous_page = param( 'results'.$GenericCategoryCache->dbprefix.'page', 'integer', 1, true );
	break;

	// Insert new element...
	case 'create':
	if( ! $permission_to_edit )
	{
		debug_die( 'No permission to edit' );
	}
	$edited_GenericCategory = & $GenericCategoryCache->new_obj( NULL, $subset_ID );
	// load data from request
	if( $edited_GenericCategory->load_from_Request() )
	{
		// We could load data from form without errors, insert in DB
		if( $edited_GenericCategory->dbinsert() !== false )
		{
			$Messages->add( T_('New element created.'), 'success' );
			// Add the ID of the new element to the result fadeout
			$result_fadeout[$edited_GenericCategory->dbIDname][] = $edited_GenericCategory->ID;
			$action = 'list';
		}
	}
	break;

	// Make sure we got an ID
	case 'update':
	param( $GenericCategoryCache->dbIDname, 'integer', true );
	if( ! $permission_to_edit )
	{
		debug_die( 'No permission to edit' );
	}
	// LOAD FORM DATA
	if( $edited_GenericCategory->load_from_Request() )
	{
		// We could load data from form without errors, update in DB
		if( $edited_GenericCategory->dbupdate() !== false )
		{
			$Messages->add( T_('Element updated.'), 'success' );
			// Add the ID of the updated element to the result fadeout
			$result_fadeout[$edited_GenericCategory->dbIDname][] = $edited_GenericCategory->ID;
		}
		$action = 'list';
	}
	else
	{
		// Get the page number we come from
		$previous_page = param( 'results'.$GenericCategoryCache->dbprefix.'page', 'integer', 1, true );
	}
	break;

	// Delete entry
	case 'delete':
	param( $GenericCategoryCache->dbIDname, 'integer', true );
	if( ! $permission_to_edit )
	{
		debug_die( 'No permission to edit' );
	}
	// Set restrictions for element
	$edited_GenericCategory->delete_restrictions = $delete_restrictions;

	if( param( 'confirm', 'integer', 0 ) )
	{
		// confirmed, Delete from DB
		$msg = sprintf( T_('Element &laquo;%s&raquo; deleted.'), $edited_GenericCategory->dget( 'name' ) );
		$GenericCategoryCache->dbdelete_by_ID( $edited_GenericCategory->ID );
		unset( $edited_GenericCategory );
		forget_param( $GenericCategoryCache->dbIDname );
		$Messages->add( $msg, 'success' );
		$action = 'list';
	}
	else
	{
		// not confirmed, Check for restrictions, get the page number we come from
		$previous_page = param( 'results_'.$GenericCategoryCache->dbprefix.'page', 'integer', 1, true );
		if( ! $edited_GenericCategory->check_delete( sprintf( T_('Cannot delete element &laquo;%s&raquo;'), $edited_GenericCategory->dget( 'name' ) ) ) )
		{
			// There are restrictions
			$action = 'edit';
		}
	}
	break;
}

// Display <html><head>...</head> section (should be done early if actions do not redirect)
$AdminUI->disp_html_head();
// Display title, menu, messages, etc... (messages MUST be displayed AFTER the actions)
$AdminUI->disp_body_top();

// Display payload
switch( $action )
{
	case 'nil':
	// Do nothing
	break;

	case 'new':
	case 'copy':
	case 'create':
	case 'edit':
	case 'update':
	case 'delete':
	// Begin payload block
	$AdminUI->disp_payload_begin();
	if( $action == 'delete' )
	{
		// We need to ask for confirmation
		$edited_GenericCategory->confirm_delete(
			sprintf( T_('Delete element &laquo;%s&raquo;?'), $edited_GenericCategory->dget( 'name' ) ),
			$action, get_memorized( 'action' )
		);
	}
	if( $form_below_list )
	{
		// Display list VIEW before form view
		if( ! empty( $list_view_path ) )
		{
			$AdminUI->disp_view( $list_view_path );
		}
		else
		{
			$AdminUI->disp_view( 'generic/_generic_recursive_list.inc.php' );
		}
	}
	// Display category edit form
	if( ! empty( $edit_view_path ) )
	{
		$AdminUI->disp_view( $edit_view_path );
	}
	else
	{
		$AdminUI->disp_view( 'generic/_generic_category.form.php' );
	}
	// End payload block
	$AdminUI->disp_payload_end();
	break;

	case 'list':
	default:
	// Begin payload block
	$AdminUI->disp_payload_begin();
	// Display list VIEW
	if( ! empty( $list_view_path ) )
	{
		$AdminUI->disp_view( $list_view_path );
	}
	else
	{
		$AdminUI->disp_view( 'generic/_generic_recursive_list.inc.php' );
	}
	// End payload block
	$AdminUI->disp_payload_end();
	break;
}

// Fadeout javascript
echo '<script type="text/javascript" src="'.$rsc_url.'js/fadeout.js"></script>';
echo '<script type="text/javascript">addEvent( window, "load", Fat.fade_all, false);</script>';

// End payload block
$AdminUI->disp_payload_end();
// Display body bottom, debug info, close </html>
$AdminUI->disp_global_footer();

?>
