<?php
/**
 * This file implements the GenericCategoryCache class
 *
 * This is the object handling genreric category lists.
 *
 * @todo (1111) vars type 'foo' need useful/valid descriptions
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package core
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * GenericCategoryCache class
 *
 * @package core
 */
class GenericCategoryCache extends GenericCache
{
	/**
	 * Subset Cache
	 * @var array
	 */
	var $subset_cache = array();
	/**
	 * Which property of the objects defines the subset
	 * @var foo
	 */
	var $subset_property;
	/**
	 * Which subsets have been loaded
	 * @var array
	 */
	var $loaded_subsets = array();
	/**
	 * These are the level 0 categories (which have no parent)
	 * @var array
	 */
	var $root_cats = array();
	/**
	 * These are the level 0 categories (which have no parent) for each subset
	 * @var array
	 */
	var $subset_root_cats = array();
	/**
	 * Have the children been revealed for all subsets yet?
	 * @var boolean
	 */
	var $revealed_all_children = false;
	/**
	 * Have the children been revealed for all subsets yet?
	 * @var array
	 */
	var $revealed_subsets = array();

	/**
	 * Constructor
	 */
	function GenericCategoryCache( $objtype, $load_all, $tablename, $prefix = '', $dbIDname = 'ID', $name_field = NULL, $subset_property = NULL, $order_by = '' )
	{
		// Call parent constructor
		parent::GenericCache( $objtype, $load_all, $tablename, $prefix, $dbIDname, $name_field, $order_by );
		$this->subset_property = $subset_property;
	}


	/**
	 * Empty/reset the cache
	 */
	function clear()
	{
		$this->subset_cache = array();
		$this->loaded_subsets = array();
		$this->root_cats = array();
		$this->subset_root_cats = array();
		$this->revealed_all_children = false;
		$this->revealed_subsets = array();
		parent::clear();
	}


	/**
	 * Add a dataobject to the cache
	 */
	function add( & $Obj )
	{
		if( parent::add( $Obj ) )
		{
			// Successfully added
			if( ! empty( $this->subset_property ) )
			{
				// Also add to subset cache
				$this->subset_cache[$Obj->{$this->subset_property}][$Obj->ID] = & $Obj;
			}
			return true;
		}
		return false;
	}


	/**
	 * Reveal children
	 *
	 * After this each Category will have an array pointing to its direct children
	 *
	 * @param integer|NULL NULL for all subsets
	 */
	function reveal_children( $subset_ID = NULL )
	{
		global $Timer;

		if( $this->revealed_all_children )
		{
			// ALL Children have already been revealed: (can happen even if we require a subset *now*)
			return;
		}

		if( empty( $this->subset_property ) )
		{
			// We are not handling subsets
			echo 'Revealing all children -- this is not yet handling all edge cases that the subset version can handle';
			// Make sure everything has been loaded
			$this->load_all();

			// Reveal children
			if( ! empty( $this->cache ) )
			{
				// There are loaded categories, so loop on all loaded categories to set their children list if it has
				foreach( $this->cache as $cat_ID => $GenericCategory )
				{
					if( ! is_null( $GenericCategory->parent_ID ) )
					{
						// This category has a parent, so add it to its parent children list
						$this->cache[$GenericCategory->parent_ID]->add_children( $this->cache[$cat_ID] );
					}
					else
					{
						// This category has no parent, so add it to the parent categories list
						$this->root_cats[] = & $this->cache[$cat_ID];
					}
				}
			}

			$this->revealed_all_children = true;
		}
		else
		{
			// We are handling subsets
			if( is_null( $subset_ID ) )
			{
				// No specific subset requested, we are going to reveal all subsets
				// Make sure everything has been loaded
				$this->load_all();
				echo 'REVEALING ALL SUBSETS in a row. Is this needed?';

				foreach( $this->subset_cache as $subset_ID => $dummy )
				{
					$this->reveal_children( $subset_ID );
				}
				$this->revealed_all_children = true;
			}
			else
			{
				// We're interested in a specific subset
				if( ! empty( $this->revealed_subsets[$subset_ID] ) )
				{
					// Children have already been revealed
					return;
				}

				$Timer->resume('reveal_children');

				// Make sure the requested subset has been loaded
				$this->load_subset( $subset_ID );

				// Reveal children
				if( ! empty( $this->subset_cache[$subset_ID] ) )
				{
					// There are loaded categories
					// Now check that each category has a path to the root
					foreach( $this->subset_cache[$subset_ID] as $cat_ID => $dummy )
					{
						// "as" would give a freakin copy of the object :(((
						$this->check_path_to_root( $subset_ID, $cat_ID );
					}

					// loop on all loaded categories to set their children list if it has some
					foreach( $this->subset_cache[$subset_ID] as $cat_ID => $dummy )
					{
						// "as" would give a freakin copy of the object :(((
						$GenericCategory = & $this->subset_cache[$subset_ID][$cat_ID];
						if( ! is_null( $GenericCategory->parent_ID ) )
						{
							// This category has a parent, so add it to its parent children list
							$this->cache[$GenericCategory->parent_ID]->add_children( $this->cache[$cat_ID] );
						}
						else
						{
							// This category has no parent, so add it to the parent categories list
							$this->root_cats[] = & $GenericCategory; // $this->cache[$cat_ID];
							$this->subset_root_cats[$this->cache[$cat_ID]->{$this->subset_property}][] = & $GenericCategory; // $this->cache[$cat_ID];
						}
					}

					$Timer->pause('reveal_children');
				}

				// Children have been revealed
				$this->revealed_subsets[$subset_ID] = true;
			}
		}
	}


	/**
	 * Support function for reveal_children()
	 *
	 * @param integer
	 * @param integer
	 * @return true if root parent
	 */
	function check_path_to_root( $subset_ID, $cat_ID, $path_array = array() )
	{
		$padding = str_pad( '', count( $path_array ), '*' );
		$GenericCategory = & $this->subset_cache[$subset_ID][$cat_ID];
		if( is_null( $GenericCategory->parent_ID ) )
		{
			// Found root parent
			return true;
		}

		// This is not the last parent ... Record path
		$path_array[] = $GenericCategory->ID;

		if( in_array( $GenericCategory->parent_ID, $path_array ) )
		{
			// We are about to enter an infinite loop
			$GenericCategory->parent_ID = NULL;
			return false;
		}
		elseif( ! isset( $this->cache[$GenericCategory->parent_ID] ) )
		{
			$GenericCategory->parent_ID = NULL;
			return false;
		}

		// Recurse!
		return $this->check_path_to_root( $subset_ID, $GenericCategory->parent_ID, $path_array );
	}

	/**
	 * Return recursive display of loaded categories
	 * @param array callback funtions (to format the display)
	 * @param integer|NULL NULL for all subsets
	 * @param array categories list to display
	 * @param int depth of categories list
	 * @return string recursive list of all loaded categories
	 */
	function recurse( $callbacks, $subset_ID = NULL, $cat_array = NULL, $level = 0 )
	{
		// Make sure children have been revealed for specific subset
		$this->reveal_children( $subset_ID );

		if( is_null( $cat_array ) )
		{
			// Get all parent categories
			if( is_null( $subset_ID ) )
			{
				$cat_array = $this->root_cats;
			}
			elseif( isset( $this->subset_root_cats[$subset_ID] ) )
			{
				// We have root cats for the requested subset
				$cat_array = $this->subset_root_cats[$subset_ID];
			}
			else
			{
				$cat_array = array();
			}
		}

		$r = '';

		if( is_array( $callbacks['before_level'] ) )
		{
			// object callback
			$r .= $callbacks['before_level'][0]->{$callbacks['before_level'][1]}( $level ); // <ul>
		}
		else
		{
			$r .= $callbacks['before_level']( $level ); // <ul>
		}

		foreach( $cat_array as $cat )
		{
			if( is_array( $callbacks['line'] ) )
			{
				// object callback
				// <li> Category - or - <tr><td>Category</td></tr> ...
				$r .= $callbacks['line'][0]->{$callbacks['line'][1]}( $cat, $level );
			}
			else
			{
				// <li> Category - or - <tr><td>Category</td></tr> ...
				$r .= $callbacks['line']( $cat, $level );
			}

			if( ! empty( $cat->children ) )
			{
				// Add children categories
				$r .= $this->recurse( $callbacks, $subset_ID, $cat->children, $level+1 );
			}
			elseif( is_array( $callbacks['no_children'] ) )
			{
				// object callback
				$r .= $callbacks['no_children'][0]->{$callbacks['no_children'][1]}( $cat, $level ); // </li>
			}
			else
			{
				$r .= $callbacks['no_children']( $cat, $level ); // </li>
			}
		}

		if( is_array( $callbacks['after_level'] ) )
		{
			// object callback
			$r .= $callbacks['after_level'][0]->{$callbacks['after_level'][1]}( $level ); // </ul>
		}
		else
		{
			$r .= $callbacks['after_level']( $level ); // </ul>
		}

		return $r;
	}


	/**
	 * Return recursive select options list of all loaded categories
	 *
	 * @param integer selected category in the select input
	 * @param integer NULL for all subsets
	 * @param boolean Include the root element?
	 * @param array GenercCategory objects to display (will recurse from those starting points)
	 * @param integer depth of categories list
	 * @param array IDs of categories to exclude (their children will be ignored to)
	 * @return string select options list of all loaded categories
	 */
	function recurse_select( $selected = NULL, $subset_ID = NULL, $include_root = false,
		$Cat_array = NULL, $level = 0, $exclude_array = array() )
	{
		// Make sure children have been revealed for specific subset
		$this->reveal_children( $subset_ID );

		if( is_null( $Cat_array ) )
		{
			// Get all parent categories
			$Cat_array = $this->root_cats;
		}

		$r = '';

		if( $include_root )
		{
			$r .= '<option value="">'.T_('Root').'</option>';
			$level++;
		}

		foreach( $Cat_array as $GenericCategory )
		{
			if( in_array( $GenericCategory->ID, $exclude_array ) )
			{
				// We want to exclude that cat
				continue;
			}

			// Set category indentation in the select
			$indent = '';
			for( $i = 0; $i < $level; $i++ )
			{
				$indent .='&nbsp;&nbsp;';
			}
			// Set category option
			$r .= '<option value="'.$GenericCategory->ID.'" ';
			if( $GenericCategory->ID == $selected ) $r .= ' selected="selected"';
			$r .= ' >'.$indent.$GenericCategory->name.'</option>';

			if( ! empty( $GenericCategory->children ) )
			{
				// Add children categories
				$r .= $this->recurse_select( $selected, $subset_ID, false, $GenericCategory->children, $level+1, $exclude_array );
			}
		}

		return $r;
	}

}

?>
