<?php
/**
 * This file implements the GenericOrdered class
 *
 * Generic Ordered of users with specific permissions.
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package core
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * GenericOrdered class
 *
 * @package core
 */
class GenericOrdered extends GenericElement
{
	/**
	 * Order object
	 * @var integer
	 */
	var $order;

	/**
	 * Constructor
	 *
	 * @param string Name of table in database
	 * @param string Prefix of fields in the table
	 * @param string Name of the ID field (including prefix)
	 * @param object DB row
	 */
	function GenericOrdered( $tablename, $prefix = '', $dbIDname = 'ID', $db_row = NULL )
	{
		// Call parent constructor
		parent::GenericElement( $tablename, $prefix, $dbIDname, $db_row );

		if( $db_row != NULL )
		{
			$this->order = $db_row->{$prefix.'order'};
		}

	}


	/**
	 * Set param value
	 *
	 * By default, all values will be considered strings
	 * @param string parameter name
	 * @param mixed parameter value
	 */
	function set( $parname, $parvalue )
	{
		switch( $parname )
		{
			case 'order':
			$this->set_param( $parname, 'integer', $parvalue );
			break;

			case 'name':
			default:
			$this->set_param( $parname, 'string', $parvalue );
		}
	}


	/**
	 * Insert object into DB based on previously recorded changes
	 */
	function dbinsert()
	{
		global $DB;

		$DB->begin();
		if( $max_order = $DB->get_var( 'SELECT MAX('.$this->dbprefix.'order) FROM '.$this->dbtablename ) )
		{
			// The new element order must be the lastest
			$max_order++;
		}
		else
		{
			// There are no elements in the database yet, so this order is set to 1
			$max_order = 1;
		}
		// Set Object order
		$this->set( 'order', $max_order );
		parent::dbinsert();
		$DB->commit();
	}

}

?>
