<?php
/**
 * This file implements the ItemCache class
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package items
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * ItemCache class
 *
 * @package items
 */
class ItemCache extends DataObjectCache
{
	/**
	 * Lazy filled index of url titles
	 * @var array
	 */
	var $urltitle_index = array();

	/**
	 * Constructor
	 *
	 * @param string object type of elements in Cache
	 * @param string Name of the DB table
	 * @param string Prefix of fields in the table
	 * @param string Name of the ID field (including prefix)
	 */
	function ItemCache( $objType = 'Item', $dbtablename = 'T_items', $dbprefix = 'post_', $dbIDname = 'post_ID' )
	{
		// Call parent constructor
		parent::DataObjectCache( $objType, false, $dbtablename, $dbprefix, $dbIDname );
	}


	/**
	 * Get an object from cache by its urltitle
	 *
	 * Load into cache if necessary
	 *
	 * @param string stub of object to load
	 * @param boolean false if you want to return false on error
	 */
	function & get_by_urltitle( $req_urltitle, $halt_on_error = true )
	{
		global $DB;

		if( ! isset( $this->urltitle_index[$req_urltitle] ) )
		{
			// not yet in cache
			// Load just the requested object
			$sql = "SELECT *
			FROM $this->dbtablename
			WHERE post_urltitle = ".$DB->quote( $req_urltitle );
			$row = $DB->get_row( $sql );
			if( empty( $row ) )
			{
				// Requested object does not exist
				if( $halt_on_error )
				{
					debug_die( "Requested $this->objtype does not exist!" );
				}
				// put into index
				$this->urltitle_index[$req_urltitle] = false;
				return $this->urltitle_index[$req_urltitle];
			}
			$this->instantiate( $row );
			// put into index
			$this->urltitle_index[$req_urltitle] = & $this->cache[$row->post_ID];
		}
		return $this->urltitle_index[$req_urltitle];
	}


	/**
	 * Load a list of item referenced by their urltitle into the cache
	 *
	 * @param array of urltitles of Items to load
	 */
	function load_urltitle_array( $req_array )
	{
		global $DB;

		$req_list = "'".implode( "','", $req_array)."'";
		$sql = "SELECT * FROM $this->dbtablename WHERE post_urltitle IN ( $req_list )";
		$dbIDname = $this->dbIDname;
		$objtype = $this->objtype;
		foreach( $DB->get_results( $sql ) as $row )
		{
			$this->cache[$row->$dbIDname] = new $objtype( $row ); // COPY!
			// $obj = $this->cache[$row->$dbIDname];
			// $obj->disp( 'name' );

			// put into index
			$this->urltitle_index[$row->post_urltitle] = & $this->cache[$row->$dbIDname];
		}

		// Set cache for non found objects
		foreach( $req_array as $urltitle )
		{
			if( ! isset( $this->urltitle_index[$urltitle] ) )
			{
				// not yet in cache
				$this->urltitle_index[$urltitle] = false; // Remember it doesn't exist in DB either
			}
		}
	}

}

?>
