<?php
/**
 * This file implements the ItemTypeCache class
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package items
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * ItemTypeCache class
 *
 * @package items
 */
class ItemTypeCache extends DataObjectCache
{
	/**
	 * Item type cache for each blog
	 * @var array
	 */
	var $col_cache = array();
	/**
	 * Default item type for each blog
	 * @var array
	 */
	var $col_default = array();

	/**
	 * Constructor
	 *
	 * @param table Database row
	 */
	function ItemTypeCache()
	{
		// Call parent constructor
		parent::DataObjectCache( 'ItemType', true, 'T_item_types', 'ptyp_', 'ptyp_ID', 'ptyp_name', 'ptyp_ID' );
	}


	/**
	 * Load a list of item types for a given blog and store them into the blog cache
	 *
	 * Note: object will also get stored into the global cache.
	 */
	function load_col( $col_ID )
	{
		global $DB;

		$rows = $DB->get_results( 'SELECT *
			FROM T_item_types
			INNER JOIN T_ityp_col ON ityp_ID = itco_ityp_ID
			WHERE itco_col_ID = '.$col_ID.'
			ORDER BY ityp_name' );

		foreach( $rows as $row )
		{
			// Instantiate the item type to the global cache and add it to the blogs cache
			$this->col_cache[$col_ID][$row->ityp_ID] = & $this->instantiate( $row );
			if( $row->itco_coldefault <> 0 )
			{
				// Item type is selected by default, so update the default item types blog
				$this->col_default[$col_ID] = $row->ityp_ID;
			}
		}
	}


	/**
	 * Return the default item type ID for a given blog
	 *
	 * @param integer blog ID
	 */
	function get_col_default_type_ID( $col_ID )
	{
		if( ! isset( $this->col_default[$col_ID] ) )
		{
			// blog is not in cache yet
			$this->load_col( $col_ID );
		}
		return $this->col_default[$col_ID];
	}


	/**
	 * Returns form option list with cache contents restricted to a blog
	 *
	 * Load the item types blog cache if necessary
	 *
	 * @param integer selected ID
	 * @param integer blog ID
	 * @return string
	 */
	function get_option_list_by_col_ID( $default, $col_ID )
	{
		if( ! isset( $this->col_cache[$col_ID] ) )
		{
			// blog cache for this blog ID is not set yet, so we load all item types in blog cache for this blog
			$this->load_col( $col_ID );
		}

		if( empty( $default ) )
		{
			// No default param, so we set it to the blog item type by default if exist else to 0
			$default = isset( $this->col_default[$col_ID] ) ? $this->col_default[$col_ID] : 0 ;
		}

		$r = '';
		// Loop on all item types from the blog cache
		foreach( $this->col_cache[$col_ID] as $loop_Obj )
		{
			$r .= '<option value="'.$loop_Obj->ID.'"';
			if( $loop_Obj->ID == $default )
			{
				$r .= ' selected="selected"';
			}
			$r .= '>';
			$r .= format_to_output( $loop_Obj->name, 'htmlbody' );
			$r .= '</option>'."\n";
		}
		return $r;
	}

	/**
	 * Returns a form option list which only contains unreserved post types
	 *
	 * @see $posttypes_reserved_IDs
	 * @param integer The selected ID.
	 * @param boolean Provide a choice for "none" with ID ''
	 * @param string Callback method name.
	 * @return string
	 */
	function get_option_list_unreserved_only( $default = 0, $allow_none = false, $method = 'get_name' )
	{
		global $posttypes_reserved_IDs;

		return $this->get_option_list( $default, $allow_none, $method, $posttypes_reserved_IDs );
	}

	/**
	 * For use by Universal Item List widget
	 */
	function get_option_array()
	{
		global $posttypes_reserved_IDs;

		return parent::get_option_array( 'get_name', $posttypes_reserved_IDs );
	}

}

?>
