<?php
/**
 * This file implements the LinkCache class
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package items
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * LinkCache class
 *
 * @package items
 */
class LinkCache extends DataObjectCache
{
	/**
	 * Cache for item -> array of object references
	 * @var array
	 */
	var $cache_item = array();

	/**
	 * Constructor
	 */
	function LinkCache()
	{
		// Call parent constructor
		parent::DataObjectCache( 'Link', false, 'T_links', 'link_', 'link_ID' );
	}


	/**
	 * Add a dataobject to the cache
	 */
	function add( & $Obj )
	{
		if( isset( $Obj->ID ) && $Obj->ID != 0 )
		{
			// If the object wasn't already cached and is valid
			$this->cache[$Obj->ID] = & $Obj;
			// Also cache indexed by Item ID
			$this->cache_item[$Obj->Item->ID][$Obj->ID] = & $Obj;
			return true;
		}
		return false;
	}


	/**
	 * Returns links for a given Item
	 *
	 * Loads if necessary
	 *
	 * @param integer item ID to load links for
	 * @return array of refs to Link objects
	 */
	function & get_by_item_ID( $item_ID )
	{
		// Make sure links are loaded
		$this->load_by_item_ID( $item_ID );
		return $this->cache_item[$item_ID];
	}


	/**
	 * Load links for a given Item
	 *
	 * Optimization: If the Item happens to be in the current MainList, Links for
	 * the whole MainList will be cached.
	 *
	 * @todo cache Link targets before letting the Link constructor handle it
	 * @param integer item ID to load links for
	 */
	function load_by_item_ID( $item_ID )
	{
		global $DB;
		global $ItemList;
		global $MainList;

		if( isset( $this->cache_item[$item_ID] ) )
		{
			return false;
		}

		// Check if this Item is part of the MainList
		if( $MainList || $ItemList )
		{
			$prefetch_IDs = array();
			if( $MainList )
			{
				$prefetch_IDs = array_merge( $prefetch_IDs, $MainList->get_page_ID_array() );
			}
			if( $ItemList )
			{
				$prefetch_IDs = array_merge( $prefetch_IDs, $ItemList->get_page_ID_array() );
			}
			$this->load_by_item_list( $prefetch_IDs );
		}
		else
		{
			// NO it isn't, load Links for this single Item
			// Remember this special load
			$this->cache_item[$item_ID] = array();

			$sql = 'SELECT *
				FROM T_links
				WHERE link_itm_ID = '.$item_ID.'
				ORDER BY link_ltype_ID, link_dest_itm_ID, link_file_ID';
			foreach( $DB->get_results( $sql ) as $row )
			{
				// Cache each matching object
				$this->add( new Link( $row ) );
			}
		}

		return true;
	}


	/**
	 * Load links for a given Item list
	 *
	 * @todo cache Link targets before letting the Link constructor handle it
	 * @param array of of item IDs to load links for
	 */
	function load_by_item_list( $itemIDarray )
	{
		global $DB;

		$item_list = implode( ',', $itemIDarray );
		// For each item in list...
		foreach( $itemIDarray as $item_ID )
		{
			// Remember this special load
			$this->cache_item[$item_ID] = array();
		}

		foreach( $DB->get_results( 'SELECT * FROM T_links WHERE link_itm_ID IN ('.$item_list.')' ) as $row )
		{
			// Cache each matching object
			$this->add( new Link( $row ) );
		}

		return true;
	}

}

?>
