<?php
/**
 * $ctrl_mappings: 'locales' || Global settings -> Regional
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package locales
 */
if(!defined('QP_MAIN_INIT')) die('fail');

// Check minimum permission
$current_User->check_perm( 'options', 'view', true );

$AdminUI->set_path( 'options', 'regional' );

param( 'action', 'string' );
param( 'edit_locale', 'string' );
param( 'loc_transinfo', 'integer', 0 );

// Load all available locale defintions:
locales_load_available_defs();

if( in_array( $action, array( 'update', 'reset', 'updatelocale', 'createlocale', 'deletelocale', 'extract', 'prioup', 'priodown' ) ) )
{
	// We have an action to do...
	// Check permission
	$current_User->check_perm( 'options', 'edit', true );
	// clear settings cache
	$cache_settings = '';

	// switch between regional actions
	switch( $action )
	{
		// UPDATE regional settings
		case 'update':
		param( 'newdefault_locale', 'string', true);
		$Settings->set( 'default_locale', $newdefault_locale );

		param( 'newtime_difference', 'string', '' );
		$newtime_difference = trim( $newtime_difference );
		if( $newtime_difference == '' )
		{
			$newtime_difference = 0;
		}
		if( strpos( $newtime_difference, ':' ) !== false )
		{
			// hh:mm:ss format
			$ntd = explode(':', $newtime_difference);
			if( count( $ntd ) > 3 )
			{
				param_error( 'newtime_difference', T_('Invalid time format.') );
			}
			else
			{
				$newtime_difference = $ntd[0]*3600 + ( $ntd[1]*60 );
				if( count( $ntd ) == 3 )
				{
					// add seconds
					$newtime_difference += $ntd[2];
				}
			}
		}
		else
		{
			// just hours
			$newtime_difference = $newtime_difference*3600;
		}

		$Settings->set( 'time_difference', $newtime_difference );

		if( ! $Messages->count('error') )
		{
			locale_updateDB();
			$Settings->dbupdate();
			$Messages->add( T_('Regional settings updated.'), 'success' );
		}
		break;

		// CREATE/EDIT locale
		case 'updatelocale':
		case 'createlocale':
		param( 'newloc_locale', 'string', true );
		param( 'newloc_enabled', 'integer', 0);
		param( 'newloc_name', 'string', true);
		param( 'newloc_charset', 'string', true);
		param( 'newloc_datefmt', 'string', true);
		param( 'newloc_timefmt', 'string', true);
		param( 'newloc_startofweek', 'integer', true);
		param( 'newloc_priority', 'integer', 1);
		param( 'newloc_messages', 'string', true);

		if( $action == 'updatelocale' )
		{
			param( 'oldloc_locale', 'string', true);
			$query = "SELECT loc_locale FROM T_locales WHERE loc_locale = '$oldloc_locale'";
			if( $DB->get_var( $query ) )
			{
				// old locale exists in DB
				if( $oldloc_locale != $newloc_locale )
				{
					// locale key was renamed, we delete the old locale in DB and remember to create the new one
					$q = $DB->query( 'DELETE FROM T_locales WHERE loc_locale = "'.$oldloc_locale.'"' );
					if( $DB->rows_affected )
					{
						$Messages->add( sprintf(T_('Deleted settings for locale &laquo;%s&raquo; in database.'), $oldloc_locale), 'success' );
					}
				}
			}
			else
			{
				// old locale is not in DB yet. Insert it.
				$query = "INSERT INTO T_locales
					( loc_locale, loc_charset, loc_datefmt, loc_timefmt, loc_startofweek, loc_name, loc_messages, loc_priority, loc_enabled )
					VALUES ( '$oldloc_locale', '{$locales[$oldloc_locale]['charset']}', '{$locales[$oldloc_locale]['datefmt']}',
					'{$locales[$oldloc_locale]['timefmt']}', '{$locales[$oldloc_locale]['startofweek']}',
					'{$locales[$oldloc_locale]['name']}', '{$locales[$oldloc_locale]['messages']}',
					'{$locales[$oldloc_locale]['priority']}',";
				if( $oldloc_locale != $newloc_locale )
				{
					// disable old locale
					$query .= ' 0)';
					$Messages->add( sprintf(T_('Inserted (and disabled) locale &laquo;%s&raquo; into database.'), $oldloc_locale), 'success' );
				}
				else
				{
					// keep old state
					$query .= ' '.$locales[$oldloc_locale]['enabled'].')';
					$Messages->add( sprintf(T_('Inserted locale &laquo;%s&raquo; into database.'), $oldloc_locale), 'success' );
				}
				$q = $DB->query( $query );
			}
		}

		$query = 'REPLACE INTO T_locales
			( loc_locale, loc_charset, loc_datefmt, loc_timefmt, loc_startofweek, loc_name, loc_messages, loc_priority, loc_enabled )
			VALUES ( '.$DB->quote( $newloc_locale ).',
			'.$DB->quote( $newloc_charset ).',
			'.$DB->quote( $newloc_datefmt ).',
			'.$DB->quote( $newloc_timefmt ).',
			'.$DB->quote( $newloc_startofweek ).',
			'.$DB->quote( $newloc_name ).',
			'.$DB->quote( $newloc_messages ).',
			'.$DB->quote( $newloc_priority ).',
			'.$DB->quote( $newloc_enabled ).' )';
		$q = $DB->query( $query );
		$Messages->add( sprintf(T_('Saved locale &laquo;%s&raquo;.'), $newloc_locale), 'success' );

		// reload locales: an existing one could have been renamed (but we keep $app_charset, which may have changed)
		$old_app_charset = $app_charset;
		unset( $locales );
		/**
		 * dummy docblock
		 */
		include $conf_path.'_locales.php';
		if( file_exists( $conf_path.'_overrides_TEST.php' ) )
		{
			/**
			 * also overwrite settings again
			 */
			include $conf_path.'_overrides_TEST.php';
		}
		$app_charset = $old_app_charset;

		// Load all available locale defintions
		locales_load_available_defs();
		break;

		// RESET locales in DB
		case 'reset':
		// reload locales from files
		unset( $locales );
		include $conf_path.'_locales.php';
		if( file_exists( $conf_path.'_overrides_TEST.php' ) )
		{
			// also overwrite settings again
			include $conf_path.'_overrides_TEST.php';
		}

		// delete everything from locales table
		$q = $DB->query( 'DELETE FROM T_locales WHERE 1=1' );

		if( ! isset( $locales[$current_locale] ) )
		{
			// activate default locale
			locale_activate( $default_locale );
		}

		// reset default_locale
		$Settings->set( 'default_locale', $default_locale );
		$Settings->dbupdate();

		// Load all available locale defintions
		locales_load_available_defs();

		$Messages->add( T_('Locale definitions reset to defaults. (<code>/qp_config/_locales.php</code>)'), 'success' );
		break;

		// EXTRACT locale
		case 'extract':
		$AdminUI->append_to_titlearea( 'Extracting language file for '.$edit_locale.'...' );
		// Get $edit_locale.po file for source material
		$po_file = $locales_path.$locales[$edit_locale]['messages'].'.po';
		if( ! is_file( $po_file ) )
		{
			$Messages->add( sprintf( T_('File <code>%s</code> not found.'), rel_path_to_base( $po_file ) ), 'error' );
			break;
		}
		// create/open _global.php for $edit_locale
		$outfile = $locales_path.$locales[$edit_locale]['messages'].'/_global.php';
		if( ! is_writable( $outfile ) )
		{
			$Messages->add( sprintf( 'The file &laquo;%s&raquo; is not writable.', rel_path_to_base( $outfile ) ) );
			break;
		}

		$POFile = new POFile( $po_file );
		$POFile->read( true ); // adds info about sources to $Messages
		$charset = $POFile->write_trans_file( $outfile, $locales[$edit_locale]['messages'] );
		unset( $POFile );

		// translate plugins, start with a sorted list of all plugins
		$AvailablePlugins = new Plugins_admin_no_DB(); // do not load registered plugins/events from DB
		$AvailablePlugins->discover();
		$AvailablePlugins->sort( 'name' );

		while( $loop_Plugin = $AvailablePlugins->get_next() )
		{
			$okay_to_translate = true;
			$these_msgids = array();
			$pot_file = '';

			// first things first: do we even have a /po/ folder?
			if( ! is_dir( $plugins_path.$loop_Plugin->classname.'/po' ) )
			{
				$Messages->add( sprintf( 'The path for &laquo;%s&raquo; does not exist.', rel_path_to_base( $outfile ) ) );
				$okay_to_translate = false;
				continue;
			}

			// Get messages.$edit_locale.po file for source material for this plugin
			$plugin_po_file = $plugins_path.$loop_Plugin->classname.'/po/messages.'.$locales[$edit_locale]['messages'].'.po';
			if( ! is_file( $plugin_po_file ) )
			{
				// no translation for this plugin: get the pot file if we can
				$pot_file = $plugins_path.$loop_Plugin->classname.'/po/messages.pot';
				if( ! is_file( $pot_file ) )
				{
					// we have no pot file? give up
					$Messages->add( sprintf( 'The file &laquo;%s&raquo; does not exist.', rel_path_to_base( $pot_file ) ) );
					$okay_to_translate = false;
					continue;
				}
				$plugin_po_file = $po_file;
			}

			// create/open _global.php for $edit_locale for this plugin
			$outfile = $plugins_path.$loop_Plugin->classname.'/po/_global.'.$locales[$edit_locale]['messages'].'.php';
			if( ! is_file( $outfile ) )
			{
				if( $foo = fopen( $outfile, 'w' ) )
				{
					fclose( $foo );
					if( ! is_writable( $outfile ) )
					{
						$Messages->add( sprintf( 'The file &laquo;%s&raquo; is not writable.', rel_path_to_base( $outfile ) ) );
						$okay_to_translate = false;
					}
				}
				else
				{
					$Messages->add( sprintf( 'The file &laquo;%s&raquo; is not writable.', rel_path_to_base( $outfile ) ) );
					$okay_to_translate = false;
				}
			}

			if( $okay_to_translate )
			{
				// extract the translatable strings from the .pot file
				$lines = file( $pot_file );
				$lines[] = ''; // Adds a blank line at the end in order to ensure complete handling of the file
				$trans_these = array();
				foreach ( $lines as $line )
				{
					if( strpos( $line, 'Project-Id-Version' ) == 1 )
					{
						$foo = substr( $line, 1, -1 );
					}
					if( strpos( $line, 'Report-Msgid-Bugs-To' ) == 1 )
					{
						$foo .= substr( $line, 1, -1 );
					}
					if( strpos( $line, 'POT-Creation-Date' ) == 1 )
					{
						$foo .= substr( $line, 1, -1 );
					}
					if( strpos( $line, 'PO-Revision-Date' ) == 1 )
					{
						$foo .= substr( $line, 1, -1 );
					}
					if( strpos( $line, 'Last-Translator' ) == 1 )
					{
						$foo .= substr( $line, 1, -1 );
					}
					if( strpos( $line, 'Language-Team' ) == 1 )
					{
						$foo .= substr( $line, 1, -1 );
					}
					if( strpos( $line, 'MIME-Version' ) == 1 )
					{
						$foo .= substr( $line, 1, -1 );
					}
					if( strpos( $line, 'Content-Type' ) == 1 )
					{
						$foo .= substr( $line, 1, -1 );
					}
					if( strpos( $line, 'Content-Transfer-Encoding' ) == 1 )
					{
						$foo .= substr( $line, 1, -1 );
						$foo = str_replace( 'charset=CHARSET', 'charset='.$charset, $foo );
						$these_msgids[''] = array( 'trans' => $foo );
					}
					if( preg_match( '#^msgid "(.*)"#', $line, $matches ) )
					{
						// Encountered an original text
						$msgstr = $matches[1];
						if( trim( $msgstr ) != '' )
						{
							$trans_these[] = $msgstr;
						}
					}
				}

				// create a new POFile for comparison
				$Plugin_Po_File = new POFile( $plugin_po_file );
				$plugin_po_read = $Plugin_Po_File->read( true );

				// search for plugin strings in the core's translation file
				foreach( $trans_these as $trans_this )
				{
					if( array_key_exists( $trans_this, $plugin_po_read ) )
					{
						$these_msgids[$trans_this] = $plugin_po_read[$trans_this];
					}
				}

				$Plugin_Po_File->write_trans_file( $outfile, $locales[$edit_locale]['messages'], $these_msgids );
				unset( $Plugin_Po_File );
			}

			// free up memory
			flush();
			$AvailablePlugins->unregister( $loop_Plugin );
		}
		break;

		case 'deletelocale':
		// --- DELETE locale from DB
		if( $DB->query( 'DELETE FROM T_locales WHERE loc_locale = "'.$DB->escape( $edit_locale ).'"' ) )
		{
			$Messages->add( sprintf(T_('Deleted locale &laquo;%s&raquo; from database.'), $edit_locale), 'success' );
		}

		// reload locales
		unset( $locales );
		require $conf_path.'_locales.php';
		if( file_exists( $conf_path.'_overrides_TEST.php' ) )
		{
			// also overwrite settings again
			include $conf_path.'_overrides_TEST.php';
		}

		// Load all available locale defintions
		locales_load_available_defs();
		break;

		// SWITCH PRIORITIES
		case 'prioup':
		case 'priodown':
		$switchcond = '';
		if( $action == 'prioup' )
		{
			$switchcond = 'return ( $lval[\'priority\'] > $i && $lval[\'priority\'] < $locales[$edit_locale][\'priority\'] );';
			$i = -1;
		}
		elseif( $action == 'priodown' )
		{
			$switchcond = 'return ( $lval[\'priority\'] < $i && $lval[\'priority\'] > $locales[$edit_locale][\'priority\'] );';
			$i = 256;
		}

		if( ! empty( $switchcond ) )
		{
			// we want to switch priorities
			foreach( $locales as $lkey => $lval )
			{
				// find nearest priority
				if( eval( $switchcond ) )
				{
					// remember it
					$i = $lval['priority'];
					$lswitchwith = $lkey;
				}
			}
			if( $i > -1 && $i < 256 )
			{
				// switch
				#echo 'Switching prio '.$locales[$lswitchwith]['priority'].' with '.$locales[$lswitch]['priority'].'<br />';
				$locales[$lswitchwith]['priority'] = $locales[$edit_locale]['priority'];
				$locales[$edit_locale]['priority'] = $i;

				$query = "REPLACE INTO T_locales ( loc_locale, loc_charset, loc_datefmt, loc_timefmt, loc_name, loc_messages, loc_priority, loc_enabled )	VALUES
					( '$edit_locale', '{$locales[$edit_locale]['charset']}', '{$locales[$edit_locale]['datefmt']}', '{$locales[$edit_locale ]['timefmt']}', '{$locales[$edit_locale]['name']}', '{$locales[$edit_locale]['messages']}', '{$locales[$edit_locale]['priority']}', '{$locales[$edit_locale]['enabled']}'),
					( '$lswitchwith', '{$locales[$lswitchwith]['charset']}', '{$locales[$lswitchwith]['datefmt']}', '{$locales[$lswitchwith ]['timefmt']}', '{$locales[$lswitchwith]['name']}', '{$locales[$lswitchwith]['messages']}', '{$locales[$lswitchwith]['priority']}', '{$locales[$lswitchwith]['enabled']}')";
				$q = $DB->query( $query );

				$Messages->add( T_('Switched priorities.'), 'success' );
			}
		}
		break;
	}

	locale_overwritefromDB();
}

// Display <html><head>...</head> section (should be done early if actions do not redirect)
$AdminUI->disp_html_head();
// Display title, menu, messages, etc... (messages MUST be displayed AFTER the actions)
$AdminUI->disp_body_top();
// Begin payload block
$AdminUI->disp_payload_begin();
// Display VIEW
$AdminUI->disp_view( 'locales/_locale_settings.form.php' );
// End payload block
$AdminUI->disp_payload_end();
// Display body bottom, debug info, close </html>
$AdminUI->disp_global_footer();

?>
