<?php
/**
 * This file implements the PluginUserSettings class
 *
 * This file is to handle plugin/user/name/value "pairs".
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package plugins
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * PluginUserSettings class
 *
 * @package plugins
 */
class PluginUserSettings extends AbstractSettings
{
	/**
	 * Constructor
	 *
	 * @param integer plugin ID where these settings are for
	 */
	function PluginUserSettings( $plugin_ID )
	{
		parent::AbstractSettings( 'T_plugin_usersettings', array( 'puset_plug_ID', 'puset_user_ID', 'puset_name' ), 'puset_value', 1 );
		$this->plugin_ID = $plugin_ID;
	}


	/**
	 * Get a setting by name for the Plugin
	 *
	 * @param string The settings name.
	 * @param integer User ID (by default $current_User->ID will be used - make sure
	 * that it is available already in your event!)
	 * @return mixed|NULL|false False in case of error, NULL if not found, the value otherwise.
	 */
	function get( $setting, $user_ID = NULL )
	{
		global $current_User;

		if( ! isset( $user_ID ) )
		{
			if( ! isset( $current_User ) )
			{
				return false;
			}
			$user_ID = $current_User->ID;
		}
		return parent::get( $this->plugin_ID, $user_ID, $setting );
	}


	/**
	 * Set a Plugin setting
	 *
	 * Use {@link dbupdate()} to write it to the database.
	 *
	 * @param string The settings name.
	 * @param string The settings value.
	 * @param integer User ID (by default $current_User->ID will be used - make sure
	 * that it is available already in your event!)
	 * @return boolean true, if the value has been set, false if it has not changed.
	 */
	function set( $setting, $value, $user_ID = NULL )
	{
		if( ! isset( $user_ID ) )
		{
			global $current_User;

			if( ! isset( $current_User ) )
			{
				return false;
			}
			$user_ID = $current_User->ID;
		}
		return parent::set( $this->plugin_ID, $user_ID, $setting, $value );
	}


	/**
	 * Delete a setting
	 *
	 * Use {@link dbupdate()} to commit it to the database.
	 *
	 * @param string name of setting
	 * @param integer User ID (by default $current_User->ID will be used - make sure
	 * that it is available already in your event!)
	 */
	function delete( $setting, $user_ID = NULL )
	{
		global $current_User;

		if( ! isset( $user_ID ) )
		{
			if( ! isset( $current_User ) )
			{
				return false;
			}
			$user_ID = $current_User->ID;
		}
		return parent::delete( $this->plugin_ID, $user_ID, $setting );
	}

}

?>
