<?php
/**
 * This file implements the UI view for the plugin settings
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package plugins
 */
if(!defined('QP_MAIN_INIT')) die('fail');

global $admin_Plugins;
global $current_User;
global $edit_Plugin;

load_funcs( 'plugins/_plugin.funcs.php' );

$Results = new Results( 'SELECT plug_status, plug_ID, plug_priority, plug_code, plug_apply_rendering
	FROM T_plugins', 'plug_', '-A-' /* by name */, 0 /* no limit */ );
$Results->Cache = & $admin_Plugins;
$Results->title = T_('Installed plugins');

$Results->cols[] = array(
	'th' => T_('Info'),
	'td' => '% plugin_results_td_info( {Obj} ) %',
	'td_class' => 'center',
);

$Results->cols[] = array(
	'th' => T_('Plugin'),
	'order_objects_callback' => 'plugin_results_name_order_callback',
	'td' => '% plugin_results_td_name( {Obj} ) %',
);

if( count( $admin_Plugins->get_plugin_groups() ) )
{
	$Results->cols[] = array(
		'th' => T_('Group'),
		'order_objects_callback' => 'plugin_results_group_order_callback',
		'td' => '% {Obj}->group %',
	);
}

// PRIORITY TD
$Results->cols[] = array(
	'th' => T_('Priority'),
	'order' => 'plug_priority',
	'td' => '$plug_priority$',
	'td_class' => 'right',
);

$apply_rendering_values = $admin_Plugins->get_apply_rendering_values(true); // with descs

$Results->cols[] = array(
	'th' => T_('Apply'),
	'th_title' => T_('When should rendering apply?'),
	'order' => 'plug_apply_rendering',
	'td' => '%plugin_results_td_apply_rendering( \'$plug_apply_rendering$\' )%',
);

$Results->cols[] = array(
	'th' => T_('Description'),
	'td' => '% {Obj}->short_desc %',
	'order_objects_callback' => 'plugin_results_desc_order_callback',
);

$Results->cols[] = array(
	'th' => T_('Help'),
	'td_class' => 'nowrap right',
	'td' => '% plugin_results_td_help( {Obj} ) %',
);

if( $current_User->check_perm( 'options', 'edit', false ) )
{
	$Results->cols[] = array(
		'th' => T_('Actions'),
		'td' => '% plugin_results_td_actions( {Obj} ) %',
		'td_class' => 'shrinkwrap',
	);
}

// Action icons
if( $current_User->check_perm( 'options', 'edit' ) )
{
	// Display action link to reload plugins
	$Results->global_icon( T_('Reload events and codes for installed plugins.'), 'reload', regenerate_url( 'action', 'action=reload_plugins' ), T_('Reload plugins') );
}
$Results->global_icon( T_('Install new plugin...'), 'new', regenerate_url( 'action', 'action=list_available' ), T_('Install new') );

// if there happened something with a plugin_ID, apply fadeout to the row
$highlight_fadeout = empty( $edit_Plugin ) || ! is_object( $edit_Plugin ) /* may be error string */ ? array() : array( 'plug_ID' => array( $edit_Plugin->ID ) );

$Results->display( NULL, $highlight_fadeout );

unset( $Results ); // free memory

/**
 * PLUGIN DESCRIPTION TD
 */
function plugin_results_desc_order_callback( $a, $b, $order )
{
	$r = strcasecmp( $a->short_desc, $b->short_desc );
	if( $order == 'DESC' ) { $r = -$r; }
	return $r;
}


/**
 * PLUGIN GROUP TD
 */
function plugin_results_group_order_callback( $a, $b, $order )
{
	global $admin_Plugins;

	$r = $admin_Plugins->sort_Plugin_group( $a->ID, $b->ID );
	if( $order == 'DESC' ) { $r = -$r; }
	return $r;
}


/**
 * results name callback
 */
function plugin_results_name_order_callback( $a, $b, $order )
{
	$r = strcasecmp( $a->name, $b->name );
	if( $order == 'DESC' ) { $r = -$r; }
	return $r;
}


/**
 * ACTIONS TD
 */
function plugin_results_td_actions( $Plugin )
{
	global $dispatcher;

	$r = '';
	if( $Plugin->status == 'enabled' )
	{
		$r .= action_icon( T_('Disable the plugin!'), 'deactivate', $dispatcher.'?ctrl=plugins&amp;action=disable_plugin&amp;plugin_ID='.$Plugin->ID );
	}
	elseif( $Plugin->status != 'broken' )
	{
		$r .= action_icon( T_('Enable the plugin!'), 'activate', $dispatcher.'?ctrl=plugins&amp;action=enable_plugin&amp;plugin_ID='.$Plugin->ID );
	}
	$r .= $Plugin->get_edit_settings_link();
	$r .= action_icon( T_('Un-install this plugin!'), 'delete', $dispatcher.'?ctrl=plugins&amp;action=uninstall&amp;plugin_ID='.$Plugin->ID );
	return $r;
}


/**
 * APPLY RENDERING TD
 */
function plugin_results_td_apply_rendering( $apply_rendering )
{
	global $admin_Plugins;
	global $apply_rendering_values;

	return '<span title="'.format_to_output( $apply_rendering_values[$apply_rendering], 'htmlattr' ).'">'.$apply_rendering.'</span>';
}


/**
 * HELP TD
 */
function plugin_results_td_help( $Plugin )
{
	return $Plugin->get_help_link( '$help_url' ).' '.$Plugin->get_help_link( '$readme' );
}


/**
 * INFO TD
 */
function plugin_results_td_info( $Plugin )
{
	return action_icon( T_('Display info'), 'info', regenerate_url( 'action, plugin_class', 'action=info&amp;plugin_class='.$Plugin->classname ) );
}


/**
 * PLUGIN NAME TD
 */
function plugin_results_td_name( $Plugin )
{
	global $current_User;

	$r = '<strong>'.$Plugin->name.'</strong>';

	if( $current_User->check_perm( 'options', 'edit', false ) )
	{
		// Wrap in "edit settings" link
		$r = '<a href="'.regenerate_url( '', 'action=edit_settings&amp;plugin_ID='.$Plugin->ID ).'" title="'.T_('Edit plugin settings!').'">'.$r.'</a>';
	}
	return $r;
}

?>
