<?php
/**
 * This file implements the UI view for the available plugins
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package plugins
 */
if(!defined('QP_MAIN_INIT')) die('fail');

global $admin_Plugins;
global $current_User;
global $dispatcher;

$Table = new Table();
$Table->title = T_('Plugins available for installation');
$Table->global_icon( T_('Cancel install!'), 'close', regenerate_url(), T_('Cancel') );
$Table->cols = array(
	array( 'th' => T_('Plugin') ),
	array( 'th' => T_('Description') ),
	array( 'th' => T_('Version') ),
	array( 'th' => T_('Help'), 'td_class' => 'nowrap' ),
	array( 'th' => T_('Actions'), 'td_class' => 'nowrap' ),
);
$Table->display_init();
$Table->display_list_start();

// title / column headers
$Table->display_head();

// body start
$Table->display_body_start();
if( empty( $AvailablePlugins ) || ! is_a( $AvailablePlugins, 'Plugins_admin_no_DB' ) )
{
	// may have been instantiated for action 'info'
	$AvailablePlugins = new Plugins_admin_no_DB(); // do not load registered plugins/events from DB
	$AvailablePlugins->discover();
}

// Sort the plugins by group
$AvailablePlugins->sort( 'group' );
// Grouping
$current_group = false; // False so it does the header once
$number_of_groups = count( $AvailablePlugins->get_plugin_groups() );
while( $loop_Plugin = & $AvailablePlugins->get_next() )
{
	// exclude any plugin that has reached it's $number_of_installs limit
	list( $registrations, $seq_number ) = $admin_Plugins->count_regs( $loop_Plugin->classname, true );
	if( ! ( $current_User->check_perm( 'options', 'edit', false )
		&& ( ! isset( $loop_Plugin->number_of_installs )
		|| $registrations < $loop_Plugin->number_of_installs ) ) )
	{
		continue;
	}

	// create a plugin group heading
	if( $loop_Plugin->group !== $current_group && $number_of_groups )
	{
		// Reason why $current_group is false
		$current_group = $loop_Plugin->group;
		?>
		<tr class="group"><td colspan="5" class="first">
		<?php
		if( $current_group == '' || $current_group == 'Un-Grouped' )
		{
			echo T_('Un-Classified');
		}
		else
		{
			echo $current_group;
		}
		?>
		</td></tr>
		<?php
	}

	$Table->display_line_start();

	// the "Plugin" column
	$Table->display_col_start();
	?>
	<strong><a title="<?php echo T_('Display info') ?>" href="<?php echo regenerate_url( 'action, plugin_class', 'action=info&amp;plugin_class='.$loop_Plugin->classname) ?>">
	<?php echo format_to_output( $loop_Plugin->name ); ?></a></strong>
	<?php
	$Table->display_col_end();

	// the "Description" column
	$Table->display_col_start();
	echo format_to_output( $loop_Plugin->short_desc );
	$Table->display_col_end();

	// the "Version" column
	$Table->display_col_start();
	$clean_version = preg_replace( array( '~^(CVS\s+)?\$'.'Revision:\s*~i', '~\s*\$$~'), '', $loop_Plugin->version );
	echo format_to_output( $clean_version );
	$Table->display_col_end();

	// the "Help" column
	$Table->display_col_start();
	echo action_icon( T_('Display info'), 'info', regenerate_url( 'action,plugin_class', 'action=info&amp;plugin_class='.$loop_Plugin->classname ) );
	// Help icons, if available:
	$help_icons = array();
	if( $help_external = $loop_Plugin->get_help_link() )
	{
		$help_icons[] = $help_external;
	}
	if( $help_internal = $loop_Plugin->get_help_link( '$readme' ) )
	{
		$help_icons[] = $help_internal;
	}
	if( ! empty( $help_icons ) )
	{
		echo ' '.implode( ' ', $help_icons );
	}
	$Table->display_col_end();

	// the "Actions" column
	$Table->display_col_start();
	echo '[<a href="'.$dispatcher.'?ctrl=plugins&amp;action=install&amp;plugin='.rawurlencode( $loop_Plugin->classname ).'">'.
		T_('Install');
	if( $seq_number > 1 )
	{
		// This plugin is already installed
		echo ' #'.$seq_number;
	}
	echo '</a>]';
	$Table->display_col_end();

	$Table->display_line_end();

	flush();
	// free memory
	$AvailablePlugins->unregister( $loop_Plugin );
}

// body end
$Table->display_body_end();

$Table->display_list_end();

?>
