<?php
/**
 * This is the init file for the session module
 *
 * Aliases for table names: You should not need to change them. If you want
 * to have multiple installations in a single database you should change
 * {@link $app_db_tableprefix} in _main_config.php.
 *
 * Controller mappings: For each controller name, we associate a controller file
 * to be found in /qp_inc/. The advantage of this indirection is that it is easy
 * to reorganize the controllers into subdirectories by modules. It is also easy
 * to deactivate some controllers if you don't want to provide this functionality
 * on a given installation.
 *
 * Note: while the controller mappings might more or less follow the menu structure,
 * we do not merge the two tables since we could, at any time, decide to make a
 * template with a different menu structure. The controllers however would most
 * likely remain the same.
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package sessions
 */
if(!defined('QP_CONFIG_LOADED')) die('fail');

// Aliases for table names
$app_db_config['aliases']['T_basedomains'] = $app_db_tableprefix.'basedomains';
$app_db_config['aliases']['T_goals'] = $app_db_tableprefix.'goals';
$app_db_config['aliases']['T_goal_hits'] = $app_db_tableprefix.'goal_hits';
$app_db_config['aliases']['T_hitlog'] = $app_db_tableprefix.'hitlog';
$app_db_config['aliases']['T_keyphrases'] = $app_db_tableprefix.'keyphrases';
$app_db_config['aliases']['T_sessions'] = $app_db_tableprefix.'sessions';
$app_db_config['aliases']['T_useragents'] = $app_db_tableprefix.'useragents';

// Controller mappings
$ctrl_mappings['stats'] = 'sessions/stats.ctrl.php';
$ctrl_mappings['goals'] = 'sessions/goals.ctrl.php';

/**
 * sessions_Module class
 *
 * @package sessions
 */
class sessions_Module
{
	/**
	 * Build the navbar menu
	 */
	function build_navbar_menu()
	{
		/**
		 * @todo (1111) EdB> this function in this file is obsolete! Everything that used to be here is
		 * now handled by qp_inc/_core/__core.init.php. Reckon we ought to just chuck this one day?
		 */
	}


	/**
	 * Builds the 1st half of the menu
	 *
	 * This is the one with the most important features
	 */
	function build_menu_1()
	{
		global $AdminUI;
		global $blog;
		global $Blog;
		global $current_User;
		global $dispatcher;

		if( $current_User->check_perm( 'stats', 'list' ) )
		{
			// Permission to view stats for user's blogs
			$AdminUI->add_menu_entries(
					NULL, // root
					array(
						'stats' => array(
							'text' => T_('Stats'),
							'href' => $dispatcher.'?ctrl=stats&amp;tab=summary&amp;tab3=global',
							'entries' => array(
								'summary' => array(
									'text' => T_('Hit summary'),
									'href' => $dispatcher.'?ctrl=stats&amp;tab=summary&amp;tab3=global&amp;blog='.$blog,
									'entries' => array(
										'global' => array(
											'text' => T_('Global hits'),
											'href' => $dispatcher.'?ctrl=stats&amp;tab=summary&amp;tab3=global&amp;blog='.$blog ),
										'browser' => array(
											'text' => T_('Browser hits'),
											'href' => $dispatcher.'?ctrl=stats&amp;tab=summary&amp;tab3=browser&amp;blog='.$blog ),
										'robot' => array(
											'text' => T_('Robot hits'),
											'href' => $dispatcher.'?ctrl=stats&amp;tab=summary&amp;tab3=robot&amp;blog='.$blog ),
										'feed' => array(
											'text' => T_('RSS/Atom hits'),
											'href' => $dispatcher.'?ctrl=stats&amp;tab=summary&amp;tab3=feed&amp;blog='.$blog ),
										),
									),
								'refsearches' => array(
									'text' => T_('Search B-hits'),
									'href' => $dispatcher.'?ctrl=stats&amp;tab=refsearches&amp;tab3=hits&amp;blog='.$blog,
									'entries' => array(
										'hits' => array(
											'text' => T_('Search hits'),
											'href' => $dispatcher.'?ctrl=stats&amp;tab=refsearches&amp;tab3=hits&amp;blog='.$blog ),
										'keywords' => array(
											'text' => T_('Keywords'),
											'href' => $dispatcher.'?ctrl=stats&amp;tab=refsearches&amp;tab3=keywords&amp;blog='.$blog ),
										'topengines' => array(
											'text' => T_('Top engines'),
											'href' => $dispatcher.'?ctrl=stats&amp;tab=refsearches&amp;tab3=topengines&amp;blog='.$blog ),
										),
									 ),
								'referers' => array(
									'text' => T_('Referered B-hits'),
									'href' => $dispatcher.'?ctrl=stats&amp;tab=referers&amp;blog='.$blog ),
								'other' => array(
									'text' => T_('Direct B-hits'),
									'href' => $dispatcher.'?ctrl=stats&amp;tab=other&amp;blog='.$blog ),
								'useragents' => array(
									'text' => T_('User agents'),
									'href' => $dispatcher.'?ctrl=stats&amp;tab=useragents&amp;blog='.$blog ),
								'domains' => array(
									'text' => T_('Referring domains'),
									'href' => $dispatcher.'?ctrl=stats&amp;tab=domains&amp;blog='.$blog ),
							)
						),
					)
				);
		}

		if( $blog == 0 && $current_User->check_perm( 'stats', 'view' ) )
		{
			// Viewing aggregate + Permission to view stats for ALL blogs
			$AdminUI->add_menu_entries(
					'stats',
					array(
						'sessions' => array(
							'text' => T_('User sessions'),
							'href' => $dispatcher.'?ctrl=stats&amp;tab=sessions&amp;tab3=login&amp;blog=0',
							'entries' => array(
								'login' => array(
									'text' => T_('Users'),
									'href' => $dispatcher.'?ctrl=stats&amp;tab=sessions&amp;tab3=login&amp;blog=0'
									),
								'sessid' => array(
									'text' => T_('Sessions'),
									'href' => $dispatcher.'?ctrl=stats&amp;tab=sessions&amp;tab3=sessid&amp;blog=0'
									),
								'hits' => array(
									'text' => T_('Hits'),
									'href' => $dispatcher.'?ctrl=stats&amp;tab=sessions&amp;tab3=hits&amp;blog=0'
									),
								),
							),
						'goals' => array(
							'text' => T_('Goals'),
							'href' => $dispatcher.'?ctrl=goals',
							'entries' => array(
								'goals' => array(
									'text' => T_('Goals'),
									'href' => $dispatcher.'?ctrl=goals'
									),
								'hits' => array(
									'text' => T_('Goal hits'),
									'href' => $dispatcher.'?ctrl=stats&amp;tab=goals&amp;tab3=hits&amp;blog=0'
									),
								'stats' => array(
									'text' => T_('Stats'),
									'href' => $dispatcher.'?ctrl=goals&amp;tab3=stats'
									),
								),
							),
						)
				);
		}
	}


	/**
	 * Builds the 2nd half of the menu
	 */
	function build_menu_2()
	{
	}
}

$sessions_Module = new sessions_Module();

?>
