<?php
/**
 * This file implements the Hit class
 *
 * @todo (1111) vars type 'foo' need useful/valid descriptions
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package sessions
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * Hit class
 *
 * @package sessions
 */
class Hit
{
	/**
	 * Hit ID
	 *
	 * Stored in T_hitlog as hit_ID, gets set when {@link log()} is called and the hit is logged.
	 *
	 * @var integer
	 */
	var $ID;
	/**
	 * Referer Type
	 *
	 * Stored in T_hitlog as hit_referer_type, possible values: 'search'|'blacklist'|
	 * 'spam'|'referer'|'direct'|'self'|'admin'. By default this is not set to anything.
	 * Note: "spam" referers do not get logged.
	 *
	 * @var string
	 */
	var $referer_type;
	/**
	 * Referer
	 *
	 * Stored in T_hitlog as hit_referer, from either HTTP_REFERER or _SERVER or getenv().
	 * The stored value is truncated to 500 characters, which should be more than enough.
	 *
	 * @var string
	 */
	var $referer;
	/**
	 * Referer's domain ID
	 *
	 * Stored in T_hitlog as hit_referer_dom_ID, the ID of the referer's base domain in T_basedomains.
	 * @var integer
	 */
	var $referer_domain_ID;
	/**
	 * SERP rank
	 *
	 * Stored in T_hitlog as hit_serprank, If it is a search hit then the referer's SERP rank, else NULL
	 *
	 * @var integer
	 */
	var $_serprank = NULL;
	/**
	 * Remote Address
	 *
	 * Stored in T_hitlog as hit_remote_addr, the remote address (IP) of the hit.
	 *
	 * @var string
	 */
	var $IP;
	/**
	 * User Agent
	 *
	 * The user agent string, cut to 500 characters. Stored in T_hitlog as hit_agent_signature,
	 * the user agent string is truncated to 500 characters, which should be more than enough.
	 *
	 * @see Hit::get_user_agent()
	 * @var string
	 */
	var $user_agent;
	/**
	 * Type of Hit
	 *
	 * Stored in T_hitlog as hit_agent_type, the default setting ('unknown') is
	 * for hits that are not detected as 'rss'|'robot'|'browser'.
	 *
	 * @see Hit::detect_useragent()
	 * @var string
	 */
	var $agent_type;
	/**
	 * Platform
	 *
	 * Stored in T_hitlog as hit_agent_platform, the user agent platform (operating system really)
	 *
	 * @see Hit::detect_useragent()
	 * @var string
	 */
	var $agent_platform;
	/**
	 * Ignore this hit?
	 * @var boolean
	 */
	var $ignore = false;
	/**
	 * Is this a reload?
	 *
	 * This gets lazy-filled by {@link is_new_view()}.
	 *
	 * @var boolean
	 */
	var $_is_new_view;
	/**
	 * Is the hit already logged?
	 * @var boolean
	 */
	var $logged = false;
	/**
	 * The user agent name, eg "safari"
	 * @see Hit::get_agent_name()
	 * @var string
	 */
	var $agent_name;
	/**
	 * The user's remote host.
	 * Use {@link get_remote_host()} to access it (lazy filled).
	 * @var string
	 */
	var $_remoteHost;
	/**
	 * The ID of the user agent entry in T_useragents.
	 * @see Hit::get_agent_ID()
	 * @var integer
	 */
	var $agent_ID;

	/**
	 * Extracted from search referers:
	 */
	var $_extracted_keyphrase = false;

	/**
	 * FIX THIS
	 * @var foo
	 */
	var $_keyphrase = NULL;

	/**
	 * FIX THIS
	 * @var foo
	 */
	var $_extracted_serprank = false;


	/**
	 * Constructor
	 *
	 * This may INSERT a basedomain and a useragent but NOT the HIT itself!
	 */
	function Hit()
	{
		// Get the first IP in the list of REMOTE_ADDR and HTTP_X_FORWARDED_FOR
		$this->IP = get_ip_list( true );

		// Check the REFERER and determine referer_type
		// @todo (0000) dh> move this out of here, too, only if "antispam_block_spam_referers"
		// is true, do something about it (here or somewhere else, but early).
		$this->detect_referer(); // May EXIT if we are set up to block referer spam.
	}


	/**
	 * Detect admin page
	 *
	 * We are inside of admin, this supersedes 'direct' access.
	 * NOTE: this is not really a referer type but more a hit type
	 */
	function detect_admin_page()
	{
		if( is_admin_page() )
		{
			$this->referer_type = 'admin';
			return true;
		}
		return false;
	}


	/**
	 * Detect Referer (sic!)
	 *
	 * Due to potential non-thread safety with getenv() (fallback), we'd better do
	 * this early. referer_type: enum('search', 'blacklist', 'referer', 'direct');
	 * 'spam' gets used internally
	 */
	function detect_referer()
	{
		global $blackList;
		global $debug;
		global $HTTP_REFERER; // might be set by PHP
		global $search_engines;
		global $self_referer_list;
		global $Settings;
		global $templates_path;

		// detect the referer if possible
		if( isset( $HTTP_REFERER ) )
		{
			// Referer provided by PHP
			$this->referer = $HTTP_REFERER;
		}
		else
		{
			// use _SERVER if we can get something
			if( isset( $_SERVER['HTTP_REFERER'] ) )
			{
				$this->referer = $_SERVER['HTTP_REFERER'];
			}
			else
			{
				// Fallback method (not thread safe), this function does not work in ISAPI mode
				$this->referer = getenv('HTTP_REFERER');
			}
		}

		// still no referer?
		if( empty( $this->referer ) )
		{
			// This type may be superseeded and set to 'admin'
			if( ! $this->detect_admin_page() )
			{
				// Not an admin page, assume 'direct'
				$this->referer_type = 'direct';
			}
			return;
		}

		// Check self referer list (see $self_referer_list)
		// fplanque: we log these (again), because if we didn't we wouldn't detect
		// reloads on these ... and that would be a problem!
		foreach( $self_referer_list as $self_referer )
		{
			$pos = strpos( $this->referer, $self_referer );
			// If not starting within in the first 12 chars it's probably an url param as in &url=http://this_blog.com
			if( $pos !== false && $pos <= 12 && ! ( $debug && strpos( $this->referer, '/search.html' ) ) )
			{
				// This type may be superceded by admin page
				if( ! $this->detect_admin_page() )
				{
					// Not an admin page, must be self
					$this->referer_type = 'self';
				}
				return;
			}
		}

		// Check blacklist, see {@link $blackList} NOTE: This is NOT the antispam!!
		// fplanque: we log these (again), because if we didn't we woudln't detect
		// reloads on these ... and that would be a problem!
		foreach( $blackList as $lBlacklist )
		{
			$pos = strpos( $this->referer, $lBlacklist );
			// If not starting within in the first 12 chars it's probably an url param as in &url=http://this_blog.com
			if( $pos !== false && $pos <= 12 )
			{
				if( ! $this->detect_admin_page() )
				{
					// Not an admin page:
					$this->referer_type = 'blacklist';
				}
				return;
			}
		}

		// Check if the referer is valid and does not match the antispam blacklist:
		// NOTE: requests to admin pages should not arrive here, because they
		// should be matched above through $self_referer_list!
		if( $error = validate_url( $this->referer, 'commenting', true, 'referers' ) )
		{
			// This is most probably referer spam!!
			$this->referer_type = 'spam';
			// In order to preserve server resources, we're going to stop processing immediatly (no logging)!!
			if( $Settings->get( 'antispam_block_spam_referers' ) )
			{
				require $templates_path.'_403_referer_spam.main.php'; // error & exit
				exit(0); // just in case.
			}
			return;
		}

		// Is the referer a search engine?
		// Note: for debug simulation, you may need to add sth like $search_engines[] = '/credits.html'; into the conf
		foreach( $search_engines as $lSearchEngine )
		{
			if( stristr( $this->referer, $lSearchEngine ) ) // search simulation
			{
				$this->referer_type = 'search';
				return;
			}
		}

		$this->referer_type = 'referer';
	}


	/**
	 * Get referer_domain_ID (ID of the referer in T_basedomains).
	 *
	 * @return integer (may be NULL, but should never).
	 */
	function get_referer_domain_ID()
	{
		if( ! isset( $this->referer_domain_ID ) )
		{
			global $DB;

			// Check if we know the base domain
			$referer_basedomain = get_base_domain( $this->referer );
			if( $referer_basedomain )
			{
				// This referer has a base domain, check if we have met it before
				$hit_basedomain = $DB->get_row( 'SELECT dom_ID FROM T_basedomains WHERE dom_name = '.$DB->quote( $referer_basedomain ) );
				if( ! empty( $hit_basedomain->dom_ID ) )
				{
					// This basedomain has visited before
					$this->referer_domain_ID = $hit_basedomain->dom_ID;
					// fp> The blacklist handling that was here made no sense.
				}
				else
				{
					// This is the first time this base domain visits
					// The INSERT below can fail, probably if we get two simultaneous hits (seen in the demo logfiles)
					$DB->save_error_state();

					if( $DB->query( 'INSERT INTO T_basedomains( dom_name ) VALUES( '.$DB->quote( $referer_basedomain ).' )' ) )
					{
						// INSERTed ok
						$this->referer_domain_ID = $DB->insert_id;
					}
					else
					{
						// INSERT failed: see, try to select again (may become/stay NULL)
						$this->referer_domain_ID = $DB->get_var( 'SELECT dom_ID FROM T_basedomains WHERE dom_name = '.$DB->quote( $referer_basedomain ) );
					}

					$DB->restore_error_state();
				}
			}
		}
		return $this->referer_domain_ID;
	}


	/**
	 * Set {@link $user_agent} and detect the browser.
	 * This function also handles the relations with T_useragents and sets {@link $agent_type}.
	 */
	function detect_useragent()
	{
		if( isset( $this->agent_type ) )
		{
			// already detected
			return;
		}

		global $DB;
		global $user_agents;
		global $template; // to detect agent_type (gets set in /qp_srvc/atom.php for example)
		global $viewmode; // Temporary template, if requested.


		// Init is_* members.
		$this->is_lynx = false;
		$this->is_firefox = false;
		$this->is_gecko = false;
		$this->is_winIE = false;
		$this->is_macIE = false;
		$this->is_safari = false;
		$this->is_opera = false;
		$this->is_NS4 = false;

		$this->agent_type = 'unknown';
		$this->agent_name = '';
		$this->agent_platform = '';

		$user_agent = $this->get_user_agent();

		if( ! empty( $user_agent ) )
		{
			// detect browser
			if( strpos( $user_agent, 'Lynx' ) !== false )
			{
				$this->is_lynx = 1;
				$this->agent_name = 'lynx';
				$this->agent_type = 'browser';
			}
			elseif( strpos( $user_agent, 'Firefox/' ) !== false )
			{
				$this->is_firefox = 1;
				$this->agent_name = 'firefox';
				$this->agent_type = 'browser';
			}
			elseif( strpos( $user_agent, 'Gecko/' ) !== false ) // We don't want to see Safari as Gecko
			{
				$this->is_gecko = 1;
				$this->agent_name = 'gecko';
				$this->agent_type = 'browser';
			}
			elseif( strpos( $user_agent, 'MSIE' ) !== false && strpos( $user_agent, 'Win' ) !== false )
			{
				$this->is_winIE = 1;
				$this->agent_name = 'msie';
				$this->agent_type = 'browser';
			}
			elseif( strpos( $user_agent, 'MSIE' ) !== false && strpos( $user_agent, 'Mac' ) !== false )
			{
				$this->is_macIE = 1;
				$this->agent_name = 'msie';
				$this->agent_type = 'browser';
			}
			elseif( strpos( $user_agent, 'Safari/' ) !== false )
			{
				$this->is_safari = true;
				$this->agent_name = 'safari';
				$this->agent_type = 'browser';
			}
			elseif( strpos( $user_agent, 'Opera' ) !== false )
			{
				$this->is_opera = 1;
				$this->agent_name = 'opera';
				$this->agent_type = 'browser';
			}
			elseif( strpos( $user_agent, 'Nav' ) !== false || preg_match('/Mozilla\/4\./', $user_agent ) )
			{
				$this->is_NS4 = 1;
				$this->agent_name = 'nav4';
				$this->agent_type = 'browser';
			}

			if( strpos( $user_agent, 'Win' ) !== false )
			{
				$this->agent_platform = 'win';
			}
			elseif( strpos( $user_agent, 'Mac' ) !== false )
			{
				$this->agent_platform = 'mac';
			}
		}
		$this->is_IE = ( ( $this->is_macIE ) || ( $this->is_winIE ) );

		/*
		 * Detect requests for XML feeds by $template / $viewmode param.
		 *
		 * Use $template, if not empty (may be set in /qp_srvc/atom.php for example),
		 * otherwise $viewmode (and if that isn't set, $tempskin for b2evolution
		 * backward compatiblity).
		 *
		 * @todo (0000) fp> this is WEAK! Do we really need to know before going into
		 * the template? dh> not necessary, but only where ->agent_type gets used (logging).
		 * @todo (0000) It would be less "weak" to call Template::get_default_type(), but that
		 * would mean having to instantiate a Template object! Another possibility: Use PHP's
		 * magic __get() method. I think combining both approaches would be a good solution
		 * (perhaps we could simply check whether the template name begins with an underscore?
		 * That would avoid having to instantiate a Template object (hmm, isn't that done
		 * in qp_inc/_blog_main.inc.php anyway?).
		 */
		if( ! empty( $template ) )
		{
			$used_template = $template;
		}
		else
		{
			$used_template = param( 'viewmode', 'string', param( 'tempskin', 'string' ), true );
		}

		if( ! empty( $used_template ) && in_array( $used_template,
					array( '_atom', '_rdf', '_rss', '_rss2' ) ) )
		{
			$this->agent_type = 'rss';
		}
		else
		{
			// Lookup robots
			foreach( $user_agents as $lUserAgent )
			{
				if( ( $lUserAgent[0] == 'robot' ) && ( strstr( $this->user_agent, $lUserAgent[1] ) ) )
				{
					$this->agent_type = 'robot';
				}
			}
		}
	}


	/**
	 * Log a hit on a blog page / rss feed
	 *
	 * This function should be called at the end of the page, otherwise if the page
	 * is displaying previous hits, it may display the current one too. The hit will
	 * not be logged in special occasions, see {@link $ignore} and {@link is_good_hit()}.
	 * It will call {@link Hitlist::dbprune()} to do the automatic pruning of old
	 * hits in case of auto_prune_stats_mode == "page".
	 *
	 * @uses Log::should_be_logged()
	 * @uses Log::record_the_hit()
	 * @return boolean true if the hit gets logged; false if not
	 */
	function log()
	{
		global $Plugins;

		if( $this->logged )
		{
			// Already logged, don't log twice
			return false;
		}

		// Remember we have already attempted to log
		$this->logged = true;

		// Check if this hit should be logged
		if( ! $this->should_be_logged() )
		{
			return false;
		}

		if( $Plugins->trigger_event_first_true('AppendHitLog', array( 'Hit' => &$this ) ) )
		{
			// Plugin has handled recording
			return true;
		}

		// Check if this hit should STILL be logged after plugin call
		if( ! $this->should_be_logged() )
		{
			return false;
		}

		// Record the HIT now
		$this->record_the_hit();

		return true;
	}


	/**
	 * Tell if a HIT should be logged
	 * @return boolean
	 */
	function should_be_logged()
	{
		global $is_admin_page;
		global $Settings;

		if( $is_admin_page && ! $Settings->get( 'log_admin_hits' ) )
		{
			// We don't want to log admin hits
			return false;
		}

		if( ! $is_admin_page && ! $Settings->get( 'log_public_hits' ) )
		{
			// We don't want to log public hits
			return false;
		}

		if( $this->referer_type == 'spam' && ! $Settings->get( 'log_spam_hits' ) )
		{
			// We don't want to log referer spam hits
			return false;
		}

		return true;
	}


	/**
	 * This records the hit
	 *
	 * You should not call this directly, but {@link Hit::log()} instead! However,
	 * if a Plugin registers the {@link Plugin::AppendHitLog() AppendHitLog event},
	 * it could be necessary to call this as a shutdown function.
	 */
	function record_the_hit()
	{
		global $blog;
		global $Blog;
		global $DB;
		global $localtimenow;
		global $ReqURI;
		global $Session;

		if( ! empty( $Blog ) )
		{
			$blog_ID = $Blog->ID;
		}
		elseif( ! empty( $blog ) )
		{
			if( ! is_numeric( $blog ) )
			{
				// this can be anything given by URL param "blog"! (because it's called on shutdown)
				// see todo in param().
				$blog = NULL;
			}
			$blog_ID = $blog;
		}
		else
		{
			$blog_ID = NULL;
		}

		// VARCHAR(250) and likely to be longer
		$hit_uri = substr( $ReqURI, 0, 250 );
		// VARCHAR(250) and likely to be longer
		$hit_referer = substr( $this->referer, 0, 250 );

		// Extract the keyphrase from search referers
		$keyphrase = $this->get_keyphrase();

		$keyp_ID = NULL;
		if( ! empty( $keyphrase ) )
		{
			$DB->begin();

			$sql = 'SELECT keyp_ID FROM T_keyphrases WHERE keyp_phrase = '.$DB->quote( $keyphrase );
			$keyp_ID = $DB->get_var( $sql, 0, 0, 'Get keyphrase ID' );

			if( empty( $keyp_ID ) )
			{
				$sql = 'INSERT INTO T_keyphrases( keyp_phrase ) VALUES ('.$DB->quote( $keyphrase ).')';
				$DB->query( $sql, 'Add new keyphrase' );
				$keyp_ID = $DB->insert_id;
			}
		}

		// Extract the serprank from search referers
		$serprank = $this->get_serprank();

		// insert hit into DB table
		$sql = "INSERT INTO T_hitlog(
			hit_sess_ID, hit_datetime, hit_uri, hit_referer_type,
			hit_referer, hit_referer_dom_ID, hit_keyphrase_keyp_ID, hit_serprank, hit_blog_ID, hit_remote_addr, hit_agnt_signature, hit_agnt_type )
			VALUES( '".$Session->ID."', FROM_UNIXTIME(".$localtimenow."), '".$DB->escape( $hit_uri )."', '".$this->referer_type
			."', '".$DB->escape( $hit_referer )."', ".$DB->null( $this->get_referer_domain_ID() ).', '.$DB->null( $keyp_ID )
			.', '.$DB->null( $serprank ).', '.$DB->null( $blog_ID ).", '".$DB->escape( $this->IP )."', ".$DB->quote( $this->get_user_agent() ).", ".$DB->quote( $this->get_agent_type() ).'
		)';

		$DB->query( $sql, 'Record the hit' );

		if( ! empty( $keyphrase ) )
		{
			$DB->commit();
		}

		$this->ID = $DB->insert_id;
	}



	/**
	 * Get the keyphrase from the referer
	 */
	function get_keyphrase()
	{
		if( ! empty( $this->_extracted_keyphrase ) )
		{
			return $this->_keyphrase;
		}

		if( $this->referer_type == 'search' )
		{
			$this->_keyphrase = Hit::extract_keyphrase_from_referer( $this->referer );
		}

		$this->_extracted_keyphrase = true;

		return $this->_keyphrase;
	}


	/**
	 * Get the serprank from the referer
	 */
	function get_serprank()
	{
		if( ! empty( $this->_extracted_serprank ) )
		{
			return $this->_extracted_serprank;
		}

		if( $this->referer_type == 'search' )
		{
			$this->_serprank = Hit::extract_serprank_from_referer( $this->referer );
		}

		$this->_extracted_serprank = true;

		return $this->_serprank;
	}


	/**
	 * Get the User agent's signature
	 *
	 * @return string False, if not provided or empty, if it included tags.
	 */
	function get_user_agent()
	{
		if( ! isset( $this->user_agent ) )
		{
			global $HTTP_USER_AGENT; // might be set by PHP, give highest priority

			if( isset( $HTTP_USER_AGENT ) )
			{
				$this->user_agent = $HTTP_USER_AGENT;
			}
			elseif( isset( $_SERVER['HTTP_USER_AGENT'] ) )
			{
				$this->user_agent = $_SERVER['HTTP_USER_AGENT'];
			}
			else
			{
				$this->user_agent = false;
			}

			if( $this->user_agent != strip_tags( $this->user_agent ) )
			{
				// then they have tried something funky, putting HTML or PHP into the user agent
				$this->user_agent = '';
			}
			else
			{
				$this->user_agent = substr( $this->user_agent, 0, 250 );
			}
		}
		return $this->user_agent;
	}


	/**
	 * Get the User agent's name
	 *
	 * @return string
	 */
	function get_agent_name()
	{
		if( ! isset( $this->agent_name ) )
		{
			$this->detect_useragent();
		}
		return $this->agent_name;
	}


	/**
	 * Get the User agent's platform
	 *
	 * @return string
	 */
	function get_agent_platform()
	{
		if( ! isset( $this->agent_platform ) )
		{
			$this->detect_useragent();
		}
		return $this->agent_platform;
	}


	/**
	 * Get the User agent's type
	 *
	 * @return string
	 */
	function get_agent_type()
	{
		if( ! isset( $this->agent_type ) )
		{
			$this->detect_useragent();
		}
		return $this->agent_type;
	}



	/**
	 * Get the remote hostname
	 *
	 * @return string
	 */
	function get_remote_host( $allow_nslookup = false )
	{
		global $Timer;

		$Timer->resume( 'Hit::get_remote_host' );

		if( is_null( $this->_remoteHost ) )
		{
			if( isset( $_SERVER['REMOTE_HOST'] ) )
			{
				$this->_remoteHost = $_SERVER['REMOTE_HOST'];
			}
			elseif( $allow_nslookup )
			{
				// We allowed reverse DNS lookup. This can be terribly time consuming (4/5 seconds!)
				// when there is no reverse dns available! This is the case on many intranets
				// and many users' first time installs!!! Some people end up considering 
				// the application core is very slow just because of this line! This cannot
				// be enabled by default.
				$this->_remoteHost = @gethostbyaddr( $this->IP );
			}
			else
			{
				$this->_remoteHost = '';
			}
		}

		$Timer->pause( 'Hit::get_remote_host' );

		return $this->_remoteHost;
	}


	/**
	 * Determine if a hit is a new view (not reloaded or from a robot)
	 *
	 * 'Reloaded' means: visited before from the same user (in a session) or from
	 * same IP/user_agent in the last {@link $Settings reloadpage_timeout} seconds.
	 *
	 * This gets queried by the Item objects before incrementing its view count (if
	 * the Item gets viewed in total ({@link $dispmore})).
	 *
	 * @todo (0000) if this is only useful to display who's online or view counts,
	 * provide option to disable all those resource consuming gadgets. (Those gadgets
	 * should be plugins actually, and they should enable this query only if needed)
	 * blueyed>> Move functionality to Plugin (with a hook in Item::content())?!
	 * @return boolean
	 */
	function is_new_view()
	{
		if( $this->get_agent_type() == 'robot' )
		{
			// Robot requests are not considered as (new) views
			return false;
		}

		if( ! isset( $this->_is_new_view ) )
		{
			global $current_User;
			global $DB;
			global $localtimenow;
			global $ReqURI;
			global $Settings;

			// Restrict to current user if logged in
			if( ! empty( $current_User->ID ) )
			{
				// select by user ID: one user counts really just once. May be even faster
				// than the anonymous query below!?
				$sql = "
					SELECT hit_ID FROM T_hitlog INNER JOIN T_sessions ON hit_sess_ID = sess_ID
					WHERE sess_user_ID = ".$current_User->ID."
					AND hit_uri = '".$DB->escape( substr( $ReqURI, 0, 250 ) )."'
					LIMIT 1";
			}
			else
			{
				// select by remote_addr/agnt_signature
				$sql = "SELECT hit_ID
					FROM T_hitlog
					WHERE hit_datetime > '".date( 'Y-m-d H:i:s', $localtimenow - $Settings->get( 'reloadpage_timeout' ) )."'
					AND hit_remote_addr = ".$DB->quote( $this->IP )."
					AND hit_uri = '".$DB->escape( substr( $ReqURI, 0, 250 ) )."'
					AND hit_agnt_signature = ".$DB->quote( $this->get_user_agent() )."
					LIMIT 1";
			}
			if( $DB->get_var( $sql, 0, 0, 'Hit: Check for reload' ) )
			{
				$this->_is_new_view = false; // We don't want to log this hit again
			}
			else
			{
				$this->_is_new_view = true;
			}
		}

		return $this->_is_new_view;
	}


	/**
	 * Is this a browser reload (F5)?
	 *
	 * @return boolean true on reload, false if not.
	 */
	function is_browser_reload()
	{
		if( ( isset( $_SERVER['HTTP_CACHE_CONTROL'] ) && strpos( $_SERVER['HTTP_CACHE_CONTROL'], 'max-age=0' ) !== false )
			|| ( isset( $_SERVER['HTTP_PRAGMA'] ) && $_SERVER['HTTP_PRAGMA'] == 'no-cache' ) )
		{
			// Reload
			return true;
		}

		return false;
	}


	/**
	 * Extract the keyphrase from a search engine referer url
	 *
	 * Typically http://google.com?s=keyphraz returns keyphrase
	 *
	 * @param string referer
	 * @return string keyphrase
	 */
	function extract_keyphrase_from_referer( $ref )
	{
		global $app_charset;

		$kwout = '';
		if( ( $pos_question = strpos( $ref, '?' ) ) == false )
		{
			return NULL;
		}

		$known_search_params = array(
			'q',
			'as_q', // Google Advanced Search Query
			'as_epq', // Google Advanced Search Query
			'query',
			'search',
			's', // google.co.uk
			'p',
			'kw',
			'qs',
			'searchfor', // mysearch.myway.com
			'r',
			'rdata', // search.ke.voila.fr
			'string', // att.net
			'su', // suche.web.de
			'Gw', // scroogle.org
			'text', // yandex.ru
		);

		$ref_params = explode( '&', app_substr( $ref, $pos_question+1 ) );
		foreach( $ref_params as $ref_param )
		{
			$param_parts = explode( '=', $ref_param );
			if( ! empty( $param_parts[1] ) && in_array( $param_parts[0], $known_search_params ) )
			{
				// found "q" query parameter
				$q = trim( urldecode( $param_parts[1] ) );
				$q = convert_charset( $q, $app_charset );
				return $q;
			}
		}

		return NULL;
	}


	/**
	 * Extract the "serp rank" from a search engine referer url
	 *
	 * Typically http://google.com?s=keyphraz&start=18 returns 18
	 *
	 * @param string referer
	 * @return string keyphrase
	 */
	function extract_serprank_from_referer( $ref )
	{
		if( preg_match( '/[?&](start|cd)=([0-9]+)/i', $ref, $matches ) )
		{
			return $matches[2];
		}

		return NULL;
	}


	/**
	 * Is this Lynx?
	 *
	 * @return boolean
	 */
	function is_lynx()
	{
		if( ! isset( $this->is_lynx ) )
		{
			$this->detect_useragent();
		}
		return $this->is_lynx;
	}

	/**
	 * Is this Firefox?
	 *
	 * @return boolean
	 */
	function is_firefox()
	{
		if( ! isset( $this->is_firefox ) )
		{
			$this->detect_useragent();
		}
		return $this->is_firefox;
	}

	/**
	 * Is this Gecko?
	 *
	 * @return boolean
	 */
	function is_gecko()
	{
		if( ! isset( $this->is_gecko ) )
		{
			$this->detect_useragent();
		}
		return $this->is_gecko;
	}

	/**
	 * Is this WinIE?
	 *
	 * @return boolean
	 */
	function is_winIE()
	{
		if( ! isset( $this->is_winIE ) )
		{
			$this->detect_useragent();
		}
		return $this->is_winIE;
	}

	/**
	 * Is this MacIE?
	 *
	 * @return boolean
	 */
	function is_macIE()
	{
		if( ! isset( $this->is_macIE ) )
		{
			$this->detect_useragent();
		}
		return $this->is_macIE;
	}

	/**
	 * Is this Safari?
	 *
	 * @return boolean
	 */
	function is_safari()
	{
		if( ! isset( $this->is_safari ) )
		{
			$this->detect_useragent();
		}
		return $this->is_safari;
	}

	/**
	 * Is this Opera?
	 *
	 * @return boolean
	 */
	function is_opera()
	{
		if( ! isset( $this->is_opera ) )
		{
			$this->detect_useragent();
		}
		return $this->is_opera;
	}

	/**
	 * Is this Netscape4?
	 *
	 * @return boolean
	 */
	function is_NS4()
	{
		if( ! isset( $this->NS4 ) )
		{
			$this->detect_useragent();
		}
		return $this->is_NS4;
	}

}

?>
