<?php
/**
 * This file implements the Hitlist class
 *
 * A list of hits. Provides functions for maintaining and extraction of Hits.
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package sessions
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * Hitlist class
 *
 * @package sessions
 */
class Hitlist
{
	/**
	 * Delete a hit
	 *
	 * @param int ID to delete
	 * @return mixed Return value of {@link DB::query()}
	 */
	function delete( $hit_ID )
	{
		global $DB;

		return $DB->query( "DELETE FROM T_hitlog WHERE hit_ID = $hit_ID", 'Delete a hit' );
	}


	/**
	 * Delete all hits for a specific date
	 *
	 * @param int unix timestamp to delete hits for
	 * @return mixed Return value of {@link DB::query()}
	 */
	function prune( $date )
	{
		global $DB;

		$iso_date = date( 'Y-m-d', $date );
		$sql = "DELETE FROM T_hitlog WHERE DATE_FORMAT( hit_datetime,'%Y-%m-%d' ) = '$iso_date'";
		return $DB->query( $sql, 'Prune hits for a specific date' );
	}


	/**
	 * Change type for a hit
	 *
	 * @param int ID to change
	 * @param string new type, must be valid ENUM for hit_referer_type field
	 * @return mixed Return value of {@link DB::query()}
	 */
	function change_type( $hit_ID, $type )
	{
		global $DB;

		$sql = 'UPDATE T_hitlog SET hit_referer_type = '.$DB->quote( $type ).",
			hit_datetime = hit_datetime " /* prevent mySQL from updating timestamp */ ." WHERE hit_ID = $hit_ID";
		return $DB->query( $sql, 'Change type for a specific hit' );
	}


	/**
	 * Auto pruning of old stats
	 *
	 * It uses a general setting to store the day of the last prune, avoiding multiple
	 * prunes per day. fplanque>> Check: How much faster is this than DELETING right
	 * away with an INDEX on the date field?
	 *
	 * Note: we're using {@link $localtimenow} to log hits, so use this for pruning, too.
	 *
	 * NOTE: do not call this directly, but only in conjuction with auto_prune_stats_mode.
	 *
	 * @todo (3069) EdB> there is a check here for MySQL less than 4 that can go away now
	 * @return string Empty, if ok.
	 */
	function dbprune()
	{
		global $DB;
		global $localtimenow;
		global $Plugins;
		global $Settings;

		// Prune when $localtime is a NEW day (which will be the 1st request after midnight)
		$last_prune = $Settings->get( 'auto_prune_stats_done' );
		if( $last_prune >= date( 'Y-m-d', $localtimenow ) )
		{
			// Already pruned today
			return T_('Pruning has already been done today');
		}

		$time_prune_before = ( $localtimenow - ( $Settings->get( 'auto_prune_stats' ) * 86400 ) ); // 1 day = 86400 seconds

		$rows_affected = $DB->query( "DELETE FROM T_hitlog
			WHERE hit_datetime < '".date('Y-m-d', $time_prune_before)."'", 'Autopruning hit log' );

		// Prune sessions that have timed out and are older than auto_prune_stats
		$sess_prune_before = ( $localtimenow - $Settings->get( 'timeout_sessions' ) );
		$smaller_time = min( $sess_prune_before, $time_prune_before );
		// allow plugins to prune session based data
		$Plugins->trigger_event( 'BeforeSessionsDelete', $temp_array = array(
			'cutoff_timestamp' => $smaller_time
		) );

		$rows_affected = $DB->query( 'DELETE FROM T_sessions WHERE sess_lastseen < '.$DB->quote( date('Y-m-d H:i:s', $smaller_time ) ), 'Autoprune sessions' );
		$rows_affected = $DB->query( 'DELETE FROM T_cron_log WHERE clog_realstart_datetime < '.$DB->quote( date('Y-m-d H:i:s', $time_prune_before ) ), 'Autoprune cron log' );

		// Prune non-referrered basedomains (where the according hits got deleted).
		// BUT only those with unknown dom_type/dom_status, because otherwise this
		// info is useful when we get hit again.
		$mysql_ver = mysql_get_server_info();
		if( ( $pos = strpos( $mysql_ver, '.' ) ) && substr( $mysql_ver, 0, $pos ) >= '4' )
		{
			// MySQL server version >= 4 (required for multi-table deletes)
			$rows_affected = $DB->query( "DELETE T_basedomains
				FROM T_basedomains
				LEFT JOIN T_hitlog ON hit_referer_dom_ID = dom_ID
				WHERE hit_referer_dom_ID IS NULL
				AND dom_type = 'unknown'
				AND dom_status = 'unknown'" );
		}
		else
		{
			// two queries for MySQL < 4
			$ids = $DB->get_row( "SELECT dom_ID FROM T_basedomains LEFT JOIN T_hitlog ON hit_referer_dom_ID = dom_ID
				WHERE hit_referer_dom_ID IS NULL AND dom_type = 'unknown' AND dom_status = 'unknown'", ARRAY_N );

			if( ! empty( $ids ) )
			{
				$rows_affected = $DB->query( 'DELETE FROM T_basedomains WHERE dom_ID IN ( '.implode( ', ', $ids ).' )' );
			}

		}

		$Settings->set( 'auto_prune_stats_done', date('Y-m-d H:i:s', $localtimenow) ); // save exact datetime
		$Settings->dbupdate();

		return ''; /* ok */
	}

}

?>
