<?php
/**
 * This file implements the UI view for the Hit list.
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package sessions
 */
if(!defined('QP_MAIN_INIT')) die('fail');

global $admin_url;
global $blog;
global $rsc_url;
global $Session;

/**
 * View funcs
 */
require_once dirname(__FILE__).'/_stats_view.funcs.php';

$exclude = param( 'exclude', 'integer', 0, true );
$sess_ID = param( 'sess_ID', 'integer', NULL, true );
$remote_IP = param( 'remote_IP', 'string', NULL, true );

// Create result set
$sql = 'SELECT hit_ID, sess_ID, hit_datetime, hit_referer_type, hit_uri, hit_blog_ID, hit_referer, hit_remote_addr,
	user_login, hit_agnt_type, blog_shortname, dom_name, goal_name, keyp_phrase, hit_serprank
	FROM T_hitlog LEFT JOIN T_basedomains ON dom_ID = hit_referer_dom_ID
	LEFT JOIN T_keyphrases ON hit_keyphrase_keyp_ID = keyp_ID
	LEFT JOIN T_sessions ON hit_sess_ID = sess_ID
	LEFT JOIN T_blogs ON hit_blog_ID = blog_ID
	LEFT JOIN T_users ON sess_user_ID = user_ID
	LEFT JOIN T_goal_hits ON hit_ID = ghit_hit_ID
	LEFT JOIN T_goals ON ghit_goal_ID = goal_ID';
$count_sql = 'SELECT COUNT(hit_ID) FROM T_hitlog';

$operator = ( $exclude ? ' <> ' : ' = ' );

if( ! empty( $sess_ID ) )
{
	// We want to filter on the session ID
	$sql .= ' WHERE hit_sess_ID'.$operator.$sess_ID;
	$count_sql .= ' WHERE hit_sess_ID'.$operator.$sess_ID;
}
elseif( ! empty( $remote_IP ) )
{
	// We want to filter on the goal name
	$sql .= ' WHERE hit_remote_addr'.$operator.$DB->quote( $remote_IP );
	$count_sql .= ' WHERE hit_remote_addr'.$operator.$DB->quote( $remote_IP );
}

$Results = new Results( $sql, 'hits_', '--D', 20, $count_sql );

$Results->title = T_('Recent hits');

$Results->filter_area = array(
	'callback' => 'filter_hits',
	'url_ignore' => 'results_hits_page,exclude,sess_ID,remote_IP',
	'presets' => array(
		'all' => array( T_('All'), '?ctrl=stats&amp;tab=sessions&amp;tab3=hits&amp;blog=0' ),
		'all_but_curr' => array( T_('All but current session'), '?ctrl=stats&amp;tab=sessions&amp;tab3=hits&amp;blog=0&amp;sess_ID='.$Session->ID.'&amp;exclude=1' ),
	)
);

$Results->cols[] = array(
	'th' => T_('Session'),
	'order' => 'hit_sess_ID',
	'td_class' => 'right',
	'td' => '<a href="?ctrl=stats&amp;tab=sessions&amp;tab3=hits&amp;blog=0&amp;sess_ID=$sess_ID$">$sess_ID$</a>',
);

$Results->cols[] = array(
	'th' => T_('User'),
	'order' => 'user_login',
	'td' => '%stat_session_login( #user_login#, true )%',
);

$Results->cols[] = array(
	'th' => T_('Date Time'),
	'order' => 'hit_ID',
	'default_dir' => 'D',
	'td_class' => 'timestamp',
	'td' => '%mysql2localedatetime_spans( #hit_datetime#, "M-d" )%',
);

$Results->cols[] = array(
	'th' => T_('Type'),
	'order' => 'hit_referer_type',
	'td' => '$hit_referer_type$',
);

$Results->cols[] = array(
	'th' => T_('U.A.'),
	'order' => 'hit_agnt_type',
	'td' => '$hit_agnt_type$',
);

$Results->cols[] = array(
	'th' => T_('Referer'),
	'order' => 'dom_name',
	'td_class' => 'nowrap',
	'td' => '<a href="$hit_referer$">$dom_name$</a>',
);

// Keywords
$Results->cols[] = array(
	'th' => T_('Search keywords'),
	'order' => 'keyp_phrase',
	'td' => '%stats_search_keywords( #keyp_phrase#, 45 )%',
);

// Serp Rank
$Results->cols[] = array(
	'th' => T_('SR'),
	'order' => 'hit_serprank',
	'td_class' => 'center',
	'td' => '$hit_serprank$',
);

$Results->cols[] = array(
	'th' => T_('Goal'),
	'order' => 'goal_name',
	'default_dir' => 'D',
	'td' => '$goal_name$',
);

$Results->cols[] = array(
	'th' => T_('Blog'),
	'order' => 'hit_blog_ID',
	'td' => '$blog_shortname$',
);

// Requested URI (linked to blog's baseurlroot+URI)
$Results->cols[] = array(
	'th' => T_('Requested URI'),
	'order' => 'hit_uri',
	'td' => '%stats_format_req_URI( #hit_blog_ID#, #hit_uri# )%',
);

$Results->cols[] = array(
	'th' => T_('Remote IP'),
	'order' => 'hit_remote_addr',
	'td' => '<a href="?ctrl=stats&amp;tab=sessions&amp;tab3=hits&amp;blog=0&amp;remote_IP=$hit_remote_addr$">$hit_remote_addr$</a>',
);

// Display results
$Results->display();

/**
 * Callback to add filters on top of the result set
 *
 * @param Form
 */
function filter_hits( & $Form )
{
	$Form->checkbox_basic_input( 'exclude', get_param('exclude'), T_('Exclude').' &mdash; ' );
	$Form->text_input( 'sess_ID', get_param('sess_ID'), 15, T_('Session ID'), '', array( 'maxlength' => 20 ) );
	$Form->text_input( 'remote_IP', get_param('remote_IP'), 15, T_('Remote IP'), '', array( 'maxlength' => 23 ) );
}

?>
