<?php
/**
 * This file implements the UI view for the User Agents stats
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package sessions
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * view funcs
 */
require_once dirname(__FILE__).'/_stats_view.funcs.php';

global $admin_url;
global $agnt_browser;
global $agnt_robot;
global $agnt_rss;
global $agnt_unknown;
global $blog;
global $rsc_url;

// For the user agents list:
param( 'agnt_browser', 'integer', 0, true );
param( 'agnt_robot', 'integer', 0, true );
param( 'agnt_rss', 'integer', 0, true );
param( 'agnt_unknown', 'integer', 0, true );

if( !$agnt_browser && !$agnt_robot && !$agnt_rss && !$agnt_unknown )
{
	// Set default status filters
	$agnt_browser = 1;
	$agnt_robot = 1;
	$agnt_rss = 1;
	$agnt_unknown = 1;
}

echo '<h2>'.T_('User agents').'</h2>';

$selected_agnt_types = array();
if( $agnt_browser ) $selected_agnt_types[] = "'browser'";
if( $agnt_robot ) $selected_agnt_types[] = "'robot'";
if( $agnt_rss ) $selected_agnt_types[] = "'rss'";
if( $agnt_unknown ) $selected_agnt_types[] = "'unknown'";
$from = 'T_hitlog';

// Note: fp> oddly enough the query is faster with the where condition even if all apply
$where_clause = ' WHERE hit_agnt_type IN ('.implode( ',', $selected_agnt_types ).')';

if( ! empty( $blog ) )
{
	$from .= ' INNER JOIN T_blogs ON hit_blog_ID = blog_ID';
	$where_clause .= ' AND hit_blog_ID = '.$blog;
}

$total_hit_count = $DB->get_var( "SELECT COUNT(*) AS hit_count
	FROM $from ".$where_clause, 0, 0, 'Get total hit count - hits with an UA' );

// Create result set
$sql = "SELECT hit_agnt_signature, hit_agnt_type, COUNT( hit_ID ) AS hit_count
	FROM $from".$where_clause.' GROUP BY hit_agnt_signature ';

$count_sql = "SELECT COUNT( DISTINCT hit_agnt_signature ) FROM $from".$where_clause;

$Results = new Results( $sql, 'uagnt_', '--D', 20, $count_sql );

$Results->filter_area = array(
	'callback' => 'filter_useragents',
	'url_ignore' => 'results_uagnt_page,agnt_browser,agnt_robot,agnt_rss,agnt_unknown', // ignore page param and checkboxes
	'presets' => array(
		'browser' => array( T_('Browser'), '?ctrl=stats&amp;tab=useragents&amp;agnt_browser=1&amp;blog='.$blog ),
		'robot' => array( T_('Robots'), '?ctrl=stats&amp;tab=useragents&amp;agnt_robot=1&amp;blog='.$blog ),
		'rss' => array( T_('XML'), '?ctrl=stats&amp;tab=useragents&amp;agnt_rss=1&amp;blog='.$blog ),
		'unknown' => array( T_('Unknown'), '?ctrl=stats&amp;tab=useragents&amp;agnt_unknown=1&amp;blog='.$blog ),
		'all' => array( T_('All'), '?ctrl=stats&amp;tab=useragents&amp;agnt_browser=1&amp;agnt_robot=1&amp;agnt_rss=1&amp;agnt_unknown=1&amp;blog='.$blog ),
	)
);

$Results->title = T_('User agents');

$Results->cols[] = array(
 	'th' => T_('Agent signature'),
 	'order' => 'hit_agnt_signature',
 	'td' => 'hit_agnt_signature',
 	'total' => '<strong>'.T_('Global total').'</strong>',
);

$Results->cols[] = array(
 	'th' => T_('Agent type'),
 	'order' => 'hit_agnt_type',
 	'td' => 'hit_agnt_type',
 	'total' => '',
);

$Results->cols[] = array(
 	'th' => T_('Hit count'),
 	'order' => 'hit_count',
 	'td_class' => 'right',
 	'total_class' => 'right',
 	'td' => '$hit_count$',
 	'total' => $total_hit_count,
);

$Results->cols[] = array(
 	'th' => T_('Hit %'),
 	'order' => 'hit_count',
 	'td_class' => 'right',
 	'total_class' => 'right',
 	'td' => '%percentage( #hit_count#, '.$total_hit_count.' )%',
 	'total' => '%percentage( 100, 100 )%',
);

// Display results
$Results->display();

/**
 * Callback to add filters on top of the result set
 *
 * @param Form
 */
function filter_useragents( & $Form )
{
	global $agnt_browser;
	global $agnt_robot;
	global $agnt_rss;
	global $agnt_unknown;
	global $blog;

	$Form->checkbox( 'agnt_browser', $agnt_browser, T_('Browsers') );
	$Form->checkbox( 'agnt_robot', $agnt_robot, T_('Robots') );
	$Form->checkbox( 'agnt_rss', $agnt_rss, T_('XML readers') );
	$Form->checkbox( 'agnt_unknown', $agnt_unknown, T_('Unknown') );
}

?>
