<?php
/**
 * This file implements the GeneralSettings class
 *
 * This class handles Name/Value pairs.
 *
 * @todo (3048) EdB> this file is 1 of 2 that sets the default admin template (part of "var $_defaults")
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package settings
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * GeneralSettings class
 *
 * @package settings
 */
class GeneralSettings extends AbstractSettings
{
	/**
	 * The default settings to use when a setting is not given in the database
	 *
	 * The following are set and reset on Global settings -> general and have no defined defaults:
	 * - 'eblog_server_host'
	 * - 'eblog_username'
	 * - 'eblog_password'
	 * - 'eblog_novalidatecert',
	 * - 'eblog_body_terminator'
	 * - 'eblog_test_mode'
	 * - 'newusers_grp_ID'
	 * - 'default_blog_ID'
	 *
	 * @todo (1111) IMPORTANT! make this be configurable new user defaults
	 * @var array
	 */
	var $_defaults = array(
		// GLOBAL SETTINGS => GENERAL
		// "After each new post..." section
		'outbound_notifications_mode' => 'immediate', // 'off', 'immediate', 'cron'
		// "Blog by email" section
		'eblog_enabled' => 0,
		'eblog_method' => 'pop3',
		'eblog_server_port' => 110,
		'eblog_encrypt' => 'none',
		'eblog_default_category' => 1,
		'eblog_subject_prefix' => 'blog:',
		'eblog_add_imgtag' => 1,
		'AutoBR' => 0,
		'eblog_body_terminator' => '___',
		// "Categories" section
		'allow_moving_cats' => '0',
		'category_order' => 'alpha',
		// "Default user permissions" section
		'newusers_canregister' => '0',
		'newusers_level' => '1',
		// "Display options" section
		'default_admin_template' => 'earth',
		// "Email validation" section
		'newusers_mustvalidate' => '1',
		'newusers_revalidate_emailchg' => '0',
		// "Features" section
		'fm_enabled' => '1',
		'general_cache_enabled' => 0,
		'webhelp_enabled' => '0',
		// "Hit & session logging" section
		'log_public_hits' => '1',
		'log_admin_hits' => '0',
		'auto_prune_stats_mode' => 'page', // 'off', 'page', 'cron'
		'auto_prune_stats' => '15',
		// "Security options" section
		'user_minpwdlen' => '5',
		'js_passwd_hashing' => '1',
		// "Timeouts" section
		'timeout_sessions' => '1209600', // = 14 days
		'reloadpage_timeout' => '300',
		// GLOBAL SETTINGS -> FILES
		// "Accessible file roots" section
		'fm_enable_roots_blog' => '1',
		'fm_blog_limit_files' => '',
		'fm_blog_limit_bytes' => '',
		'fm_enable_roots_user' => '1',
		'fm_user_limit_files' => '',
		'fm_user_limit_bytes' => '',
		'fm_enable_roots_shared' => '0',
		'fm_shared_limit_files' => '',
		'fm_shared_limit_bytes' => '',
		'fm_enable_roots_templates' => '0',
		'fm_templates_limit_files' => '',
		'fm_templates_limit_bytes' => '',
		// "File creation options" section
		'fm_enable_create_dir' => '1',
		'fm_enable_create_file' => '1',
		'upload_enabled' => '1',
		'upload_maxkb' => '8192',
		// "Advanced options" section
		'fm_default_chmod_file' => '664',
		'fm_default_chmod_dir' => '775',
		// @todo (0000) accept (spaces and) special chars / do full testing on this
		'regexp_filename' => '^[a-zA-Z0-9\-_. ]+$',
		// @todo (0000) accept spaces and special chars / do full testing on this
		'regexp_dirname' => '^[a-zA-Z0-9\-_]+$',
		// not sorted by tab/subtab/section yet...
		'fm_showtypes' => '0',
		'fm_showfsperms' => '0',
		'log_spam_hits' => '0',
		'antispam_last_update' => '2000-01-01 00:00:00',
		'antispam_threshold_publish' => '-90',
		'antispam_threshold_delete' => '100', // do not delete by default!
		// By default, let spam referers go in silently (just don't log them). This is in case the blacklist is too paranoid (social media, etc.)
		'antispam_block_spam_referers' => '0',
		'antispam_report_to_central' => '1',
		'time_difference' => '0',
	);


	/**
	 * Constructor
	 *
	 * This loads the general settings and checks db_version. It will also turn off
	 * error-reporting/halting of the {@link $DB DB object} temporarily to present
	 * a more decent error message if tables do not exist yet. Because the {@link
	 * $DB DB object} itself creates a connection when it gets created. "Error selecting
	 * database" occurs before we can check for it here.
	 */
	function GeneralSettings()
	{
		global $app_db_version;
		global $DB;

		$save_DB_show_errors = $DB->show_errors;
		$save_DB_halt_on_error = $DB->halt_on_error;
		$DB->halt_on_error = false;
		$DB->show_errors = false;

		// Init through the abstract constructor. This should be the first DB connection
		parent::AbstractSettings( 'T_settings', array( 'set_name' ), 'set_value', 0 );

		// check DB version
		if( $this->get( 'db_version' ) != $app_db_version )
		{
			// Database is not up to date
			if( $DB->last_error )
			{
				$error_message = '<p>MySQL error:</p>'.$DB->last_error;
			}
			else
			{
				$error_message = '<p>Database schema is not up to date!</p>'
					.'<p>You have schema version &laquo;'.(integer)$this->get( 'db_version' ).'&raquo;, '
					.'but we would need &laquo;'.(integer)$app_db_version.'&raquo;.</p>';
			}
			global $admintemplates_path;

			/**
			 * Error & exit
			 */
			require $admintemplates_path.'conf_error.main.php';
		}

		$DB->halt_on_error = $save_DB_halt_on_error;
		$DB->show_errors = $save_DB_show_errors;
	}


	/**
	 * Get a 32-byte string that can be used as salt for public keys
	 *
	 * @return string
	 */
	function get_public_key_salt()
	{
		$public_key_salt = $this->get( 'public_key_salt' );
		if( empty( $public_key_salt ) )
		{
			$public_key_salt = generate_random_key(32);
			$this->set( 'public_key_salt', $public_key_salt );
			$this->dbupdate();
		}
		return $public_key_salt;
	}

}

?>
