<?php
/**
 * This file implements the Template class
 *
 * @todo (1111) vars here are missing a title - they only have an at-var tag :(
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package templates
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * Template class
 *
 * @package templates
 */
class Template extends DataObject
{
	/**
	 * Translations by locale
	 * @see Template::T_()
	 * @var array The translations keyed by locale, loaded through include() of _global.php
	 */
	var $_trans = array();
	/**
	 * Container List (lazy-filled)
	 * @var array
	 */
	var $container_list = NULL;
	/**
	 * Container Order
	 * @var array Template's ordering of containers, mostly for the Widgets page
	 */
	var $container_order = NULL;
	/**
	 * Template's Folder
	 * @var string The folder the template is in
	 */
	var $folder;
	/**
	 * Template's Name
	 * @var string Displayed name of template
	 */
	var $name;
	/**
	 * Application Credits (lazy filled by constructor)
	 * @var array These are displayed by the "credits" display handler.
	 */
	var $poweredby_linkback;
	/**
	 * Template's Type
	 * @var array Type of template (xhtml, html5, feed, other)
	 */
	var $type;
	/**
	 * Template's Version
	 * @var string use "date(yz)+1" on release day for the template
	 */
	var $version;

	/**
	 * Constructor
	 *
	 * @param table database row
	 * @param template folder name
	 */
	function Template( $db_row = NULL, $template_folder = NULL )
	{
		global $app_homepage;
		global $app_name;

		// Call parent constructor
		parent::DataObject( 'T_templates', 'template_', 'template_ID' );

		$this->delete_restrictions = array(
			array( 'table' => 'T_blogs', 'fk' => 'blog_template_ID', 'msg' => T_('%d blogs using this template') ),
		);

		$this->delete_cascades = array(
			array( 'table' => 'T_template_containers', 'fk' => 'sco_template_ID', 'msg' => T_('%d linked containers') ),
		);

		$this->poweredby_linkback = array(
			'pre' => T_('Powered by'),
			'text' => $app_name,
			'post' => sprintf( T_('%s is fully committed to making the most awesome open-source web engine available, focusing intensely on your visitor\'s experience.'), $app_name ),
			'url' => $app_homepage,
			'title' => T_('A multi-user multi-blog app made by users who care'),
		);

		if( is_null( $db_row ) )
		{
			// We are creating an object here
			$this->set( 'folder', $template_folder );
			$this->set( 'name', $this->name );
			$this->set( 'type', $this->type );
		}
		else
		{
			// Wa are loading an object
			$this->ID = $db_row->template_ID;
			$this->folder = $db_row->template_folder;
			$this->name = $db_row->template_name;
			$this->type = $db_row->template_type;
		}

	}


	/**
	 * Create default widgets for requested blog
	 *
	 * @todo (1111) Yabs> change this to param array
	 * @param object Blog $edited_Blog
	 */
	function CreateDefaultWidgets( & $edited_Blog )
	{
		global $DB;
		global $Messages;

		if( $default_widgets = $this->get_default_widgets( array( 'Blog' => & $edited_Blog ), $this->container_order ) )
		{
			// we have widgets
			foreach( $default_widgets as $container => $numeric_key )
			{
				$count = 0;
				foreach( $numeric_key as $numeric_widget )
				{
					foreach( $numeric_widget as $widget => $settings )
					{
						$settings = array_merge( array(
							'type' => 'widget',
							'params' => array(),
						), $settings );

						$DB->query( 'INSERT INTO T_widgets( wi_widget_blog_ID, wi_sco_name, wi_order, wi_type, wi_code, wi_params )
							VALUES( '.$edited_Blog->ID.', '.$DB->quote( $container ).', '.( ++$count ).',
								'.$DB->quote( $settings['type'] ).', '.$DB->quote( $widget ).',
								'.$DB->quote( serialize( $settings['params'] ) ).'
							)'
						);
					}
				}
			}
			$Messages->add( T_('Default widgets have been set-up for this blog.'), 'success' );
		}
	}


	/**
	 * Display a container
	 *
	 * <code>
	 * $params = array_merge( array(
	 *   'container_start' => '<div id="%sco_name%" class="container">',
	 *   'container_end' => '</div>',
	 * ), $params );
	 * </code>
	 *
	 * @uses ComponentWidget::display()
	 * @uses ComponentWidget::init_display()
	 * @param string Container name
	 * @param array Params for this container and container's widgets
	 */
	function container( $sco_name, $params = array() )
	{
		global $admin_url;
		global $Blog;
		global $rsc_url;

		$params = array_merge( array(
			'container_start' => '<div id="%sco_name%" class="container">',
			'container_end' => '</div>',
		), $params );

		$sco_dom_ID = preg_replace( '~[^a-z0-9]~i', '', ucwords( $sco_name ) );
		$params['container_start'] = str_replace( '%sco_name%', 'sco_'.$sco_dom_ID, $params['container_start'] );

		$EnabledWidgetCache = & get_Cache( 'EnabledWidgetCache' );
		$Widget_array = & $EnabledWidgetCache->get_by_blog_container( $Blog->ID, $sco_name );

		if( ! empty( $Widget_array ) )
		{
			echo $params['container_start']."\n";
			foreach( $Widget_array as $ComponentWidget )
			{
				// Let the Widget display itself (with contextual params)
				$ComponentWidget->display( $params );
			}
			echo $params['container_end']."\n";
		}
	}


	/**
	 * Compare discovered containers to database info
	 *
	 * Called when reloading a template
	 *
	 * @todo (3085) EdB> There seems to be a bug in here that leaves mention of a
	 * container that was removed when doing a reload.
	 */
	function db_compare_containers()
	{
		global $DB;

		// some templates have no containers
		if( $this->type == 'feed' || $this->type == 'other' )
		{
			return true;
		}

		// what does the database have right now?
		$db_container_list = $DB->get_col( 'SELECT sco_name FROM T_template_containers
			WHERE sco_template_ID = '.$this->ID.' ORDER BY sco_order, sco_name',
			0, 'get list of containers for comparison' );

		// if order and list are not the same length then the template failed
		if( count( $this->container_order ) != count( $this->container_list ) )
		{
			debug_die( 'Different number of ontainers discovered and those listed in this template\'s class file.' );
		}

		// if list and database are the same length then we changed either name(s) or order
		if( count( $this->container_list ) == count( $db_container_list ) )
		{
			// first check: name change
			$new_containers = array_values( array_diff( $this->container_list, $db_container_list ) );
			if( count( $new_containers ) )
			{
				// we have at least one new name, so find the missing name(s)
				$missing_containers = array_values( array_diff( $db_container_list, $this->container_list ) );
				// update the database with the name change(s)
				$ii = 0;
				foreach( $new_containers as $new_name )
				{
					$old_name = $missing_containers[$ii];
					$DB->query( "UPDATE T_template_containers SET sco_name = ".$DB->quote( $new_name )." WHERE sco_name = ".$DB->quote( $old_name )." AND sco_template_ID = ".$this->ID );
					// store the changes for the widgets table
					$widget_table_changes[$old_name] = $new_name;
					// "update" our $db_container_list
					foreach ( $db_container_list as $k => $v )
					{
						if( $v == $old_name )
						{
							$db_container_list[$k] = $new_name;
						}
					}
					$ii++;
				}
			}

			// second check: order change
			$changed_order = array_diff_assoc( $this->container_order, $db_container_list );
			if( count( $changed_order ) )
			{
				$ii = 0;
				// update the database with the new order instructions
				foreach( $changed_order as $name )
				{
					$new_order = array_search( $name, $this->container_order ) + 1;
					$DB->query( "UPDATE T_template_containers SET sco_order = ".$new_order." WHERE sco_name = ".$DB->quote( $name )." AND sco_template_ID = ".$this->ID );
					// "update" our $db_container_list
					$db_container_list = $this->container_order;
				}
			}
		}
		else
		{
			// they are not the same length; we added or removed containers
			$new_containers = array_diff( $this->container_list, $db_container_list );
			if( count( $new_containers ) )
			{
				// we have new containers so just do the update(s) and be happy
				foreach( $this->container_order as $order => $name )
				{
					$order++;
					$DB->query( "REPLACE INTO T_template_containers( sco_template_ID, sco_name, sco_order ) VALUES (".$this->ID.", ".$DB->quote( $name ).", ".$order." )" );
				}
				// "update" our $db_container_list
				$db_container_list = $this->container_order;
			}
			$missing_containers = array_diff( $db_container_list, $this->container_list );
			if( count( $missing_containers ) )
			{
				// we are missing containers so do the update(s) and prepare to clean up the widgets table
				foreach( $this->container_order as $order => $name )
				{
					$order++;
					$DB->query( "REPLACE INTO T_template_containers( sco_template_ID, sco_name, sco_order ) VALUES (".$this->ID.", ".$DB->quote( $name ).", ".$order." )" );
				}
				// delete the missing container
				foreach( $missing_containers as $gone )
				{
					$DB->query( "DELETE FROM T_template_containers WHERE sco_name = ".$DB->quote( $gone )." AND sco_template_ID = ".$this->ID );
				}
				// store the changes for the widgets table
				$widget_table_changes[0] = $missing_containers;
				// "update" our $db_container_list
				$db_container_list = $this->container_order;
			}
		}

		// nothing that affects widgets? we're done
		if( ! isset( $widget_table_changes ) )
		{
			return true;
		}
		else
		{
			if( isset( $widget_table_changes[0] ) )
			{
				// this is the container(s) that went away
				$removed = $widget_table_changes[0];
				unset( $widget_table_changes[0] );
			}
		}

		// no blogs to work on? we're done
		$blogs_affected = $DB->get_results( "SELECT blog_ID FROM T_blogs WHERE blog_template_ID = ".$this->ID );
		if( empty( $blogs_affected ) )
		{
			return true;
		}

		// time to fix the widgets table!
		foreach( $blogs_affected as $foo => $bar )
		{
			$blog_ID = $bar->blog_ID;
			if( isset( $removed ) )
			{
				// this is where we delete widgets against this blog
				foreach( $removed as $key => $gone )
				{
					$DB->query( "DELETE FROM T_widgets WHERE wi_sco_name = ".$DB->quote( $gone )." AND wi_widget_blog_ID = ".$blog_ID );
				}
			}
			// this is where we rename containers for widgets in this blog
			foreach( $widget_table_changes as $old_name => $new_name )
			{
				$DB->query( "UPDATE T_widgets SET wi_sco_name = ".$DB->quote( $new_name )." WHERE wi_sco_name = ".$DB->quote( $old_name )." AND wi_widget_blog_ID = ".$blog_ID );
			}

		}

		return true;
	}


	/**
	 * Save containers
	 *
	 * to be called by dbinsert / dbupdate
	 */
	function db_save_containers()
	{
		global $DB;

		// can't save nothing!
		if( empty( $this->container_list ) )
		{
			return false;
		}

		// get the template's container order info, if present
		if( is_null( $this->container_order ) )
		{
			debug_die( 'This template does not have $container_order defined.' );
		}
		$ii = 1;
		foreach( $this->container_order as $bar )
		{
			$containers_ordered[$bar] = $ii;
			$ii++;
		}

		// does discovery match what the template says?
		foreach( $containers_ordered as $k => $v )
		{
			if( ! in_array( $k, $this->container_list ) )
			{
				debug_die( 'Mismatch between containers discovered and those listed in this template\'s class file :: '.$k );
			}
		}

		// save the ordered containers
		$values = array();
		foreach( $containers_ordered as $name => $order )
		{
			$values[] = '( '.$this->ID.', '.$DB->quote( $name ).', '.$order.' )';
		}
		$DB->query( 'REPLACE INTO T_template_containers( sco_template_ID, sco_name, sco_order )
			VALUES '.implode( ',', $values ), 'Insert containers' );

		return true;
	}


	/**
	 * Insert object into DB based on previously recorded changes
	 *
	 * @return boolean true
	 */
	function dbinsert()
	{
		global $DB;

		$DB->begin();
		if( parent::dbinsert() )
		{
			// Template saved, also save containers
			$this->db_save_containers();
		}
		$DB->commit();
		return true;
	}


	/**
	 * Update the DB based on previously recorded changes
	 *
	 * @return boolean true
	 */
	function dbupdate()
	{
		global $DB;

		$DB->begin();
		if( parent::dbupdate() !== false )
		{
			// Template updated, also save containers
			$this->db_save_containers();
		}
		$DB->commit();
		return true;
	}


	/**
	 * Save template specific settings for current blog to DB
	 */
	function dbupdate_settings()
	{
		global $Blog;

		$Blog->dbupdate();
	}


	/**
	 * Discover containers included in template file
	 */
	function discover_containers()
	{
		global $Messages;
		global $templates_path;

		$this->container_list = array();

		if( ! $dir = @opendir( $templates_path.$this->folder ) )
		{
			$Messages->add( T_('Cannot open template directory.'), 'error' );
			return false;
		}

		while( ( $file = readdir( $dir ) ) !== false )
		{
			$rf_main_subpath = $this->folder.'/'.$file;
			$af_main_path = $templates_path.$rf_main_subpath;

			if( ! is_file( $af_main_path ) || ! preg_match( '\.php$', $file ) )
			{
				// Not a php template file
				continue;
			}

			if( ! is_readable( $af_main_path ) )
			{
				$Messages->add( sprintf( T_('Cannot read template file &laquo;%s&raquo;!'), $rf_main_subpath ), 'error' );
				continue;
			}

			$file_contents = @file_get_contents( $af_main_path );
			if( ! is_string( $file_contents ) )
			{
				$Messages->add( sprintf( T_('Cannot read template file &laquo;%s&raquo;!'), $rf_main_subpath ), 'error' );
				continue;
			}

			if( ! preg_match_all( '~ (\$Template->|template_)container\( .*? ((\' (.+?) \')|(" (.+?) ")) ~xmi', $file_contents, $matches ) )
			{
				// No containers in this file
				continue;
			}

			// Merge matches from the two regexp parts (due to regexp "|" )
			$container_list = array_merge( $matches[4], $matches[6] );

			$c = 0;
			foreach( $container_list as $container )
			{
				if( empty( $container ) )
				{
					// regexp empty match
					continue;
				}
				$c++;
				if( in_array( $container, $this->container_list ) )
				{
					// we already have that one
					continue;
				}
				$this->container_list[] = $container;
			}

			if( $c )
			{
				$Messages->add( sprintf( T_('%d containers have been found in the &laquo;%s&raquo; file.'), $c, $rf_main_subpath ), 'success' );
			}
		}

		if( empty( $this->container_list ) )
		{
			// Display a warning
			$Messages->add( T_('No containers found in this template!'), 'note' );
		}

		return true;
	}


	/**
	 * Display screenshot for template
	 *
	 * Also for templates not yet installed.
	 */
	function disp_screenshot( $template_folder, $template_name, $template_type, $function = NULL, $selected = false, $select_url = NULL, $function_url = NULL )
	{
		global $templates_path;
		global $templates_url;

		if( ! empty( $select_url ) )
		{
			$select_a_begin = '<a href="'.$select_url.'" title="'.T_('Select this template!').'">';
			$select_a_end = '</a>';
		}
		else
		{
			$select_a_begin = '<a href="'.$function_url.'" title="'.T_('Install NOW!').'">';
			$select_a_end = '</a>';
		}

		// outer screenshot box
		echo '<div class="screenshot';
		if( $template_type == 'html5' )
		{
			echo ' html5';
		}
		echo '">';

		// inner screenshot box
		echo '<div class="screenshot_placeholder';
		if( $selected )
		{
			echo ' current';
		}
		echo '">';

		if( file_exists( $templates_path.$template_folder.'/screenshot.png' ) )
		{
			echo $select_a_begin;
			echo '<img src="'.$templates_url.$template_folder.'/screenshot.png" width="240" height="180" alt="'.$template_folder.'" />';
			echo $select_a_end;
		}
		elseif( file_exists( $templates_path.$template_folder.'/screenshot.jpg' ) )
		{
			echo $select_a_begin;
			echo '<img src="'.$templates_url.$template_folder.'/screenshot.jpg" width="240" height="180" alt="'.$template_folder.'" />';
			echo $select_a_end;
		}
		elseif( file_exists( $templates_path.$template_folder.'/screenshot.gif' ) )
		{
			echo $select_a_begin;
			echo '<img src="'.$templates_url.$template_folder.'/screenshot.gif" width="240" height="180" alt="'.$template_folder.'" />';
			echo $select_a_end;
		}
		else
		{
			echo '<div class="screenshot_noshot">'.T_('No screenshot available for').'</div>';
			echo '<div class="screenshot_name">'.$select_a_begin.$template_folder.$select_a_end.'</div>';
		}
		echo '</div>';
		echo '<div class="legend">';
		if( ! empty( $function ) )
		{
			echo '<div class="actions">';
			switch( $function )
			{
				case 'install':
				echo '<a href="'.$function_url.'" title="'.T_('Install NOW!').'">';
				echo T_('Install NOW!').'</a>';
				break;

				case 'select':
				echo '<a href="'.$function_url.'" target="_blank" title="'.T_('Preview blog with this template in a new window').'">';
				echo T_('Preview').'</a>';
				break;
			}
			echo '</div>';
		}
		echo '<strong>'.$template_name.'</strong>';
		echo '</div>';
		echo '</div>';
	}


	/**
	 * Get ready for displaying the template
	 *
	 * This may register some CSS or JS...
	 */
	function display_init()
	{
		// override in specific templates...
	}


	/**
	 * Get Template's containers
	 *
	 * @return array
	 */
	function get_containers()
	{
		global $DB;

		if( is_null( $this->container_list ) )
		{
			$this->container_list = $DB->get_col( 'SELECT sco_name
				FROM T_template_containers
				WHERE sco_template_ID = '.$this->ID.'
				ORDER BY sco_order, sco_name',
				0, 'get list of containers for template' );
		}
		return $this->container_list;
	}


	/**
	 * Get default widgets for new blog
	 *
	 * The new blog's template can suggest/select a list of widgets. If it does not
	 * a default list is used. A template can suggest/select any widget by identifying
	 * the container, and each widget's code and type. The code can be 'widget' or
	 * 'plugin'. If 'plugin' and the plugin is not installed nothing bad happens. I
	 * think. A widget installed with only code and type will have default values for
	 * any parameters it may have.
	 *
	 * <code>
	 * $default_widgets = array(
	 *   'Header' => array(),
	 *   'Menu' => array(),
	 *   'Sidebar' => array(),
	 * );
	 * $default_widgets['Header'][] = array(
	 *   'title' => array(
	 *     'type' => 'widget',
	 *   ),
	 * );
	 * $default_widgets['Menu'][] = array(
	 *   'menu_link' => array(
	 *     'type' => 'widget',
	 *     'params' => array(
	 *       'link_type' => 'ownercontact',
	 *     ),
	 *   ),
	 * );
	 * $default_widgets['Sidebar'][] = array(
	 *   'calendar' => array(
	 *     'type' => 'plugin',
	 *   ),
	 * );
	 * </code>
	 *
	 * @return array widgets for each container
	 */
	function get_default_widgets( $params = array(), $containers = array() )
	{
		global $Blog;

		$params = array_merge( array(
			'Blog' => & $Blog,
		), $params );

		// build a list of containers from the list of containers
		$default_widgets = array();
		foreach( $containers as $container )
		{
			$default_widgets[$container] = array();
		}

		// Page Top widgets?
		if( in_array( 'Page Top', $containers ) )
		{
			$default_widgets['Page Top'][] = array(
				'bloglist' => array(
					'type' => 'widget',
					'params' => array( 'blog_list_type' => ( $params['Blog']->get( 'in_bloglist' ) ? 'public' : 'owner' ) ),
				),
			);
		}

		// Header widgets?
		if( in_array( 'Header', $containers ) )
		{
			$default_widgets['Header'][] = array(
				'title' => array(
					'type' => 'widget',
				),
			);
			$default_widgets['Header'][] = array(
				'tagline' => array(
					'type' => 'widget',
				),
			);
		}
		// Menu widgets?
		if( in_array( 'Menu', $containers ) )
		{
			$default_widgets['Menu'][] = array(
				'menu_link' => array(
					'type' => 'widget',
					'params' => array( 'link_type' => 'home' ),
				),
			);
			$default_widgets['Menu'][] = array(
				'page_list' => array(
					'type' => 'widget',
				),
			);
			$default_widgets['Menu'][] = array(
				'menu_link' => array(
					'type' => 'widget',
					'params' => array( 'link_type' => 'ownercontact' ),
				),
			);
			$default_widgets['Menu'][] = array(
				'menu_link' => array(
					'type' => 'widget',
					'params' => array( 'link_type' => 'login' ),
				),
			);
		}
		// Sidebar widgets?
		if( in_array( 'Sidebar', $containers ) )
		{
			$default_widgets['Sidebar'][] = array(
				'calendar' => array(
					'type' => 'plugin',
				),
			);
			$default_widgets['Sidebar'][] = array(
				'title' => array(
					'type' => 'widget',
				),
			);
			$default_widgets['Sidebar'][] = array(
				'longdesc' => array(
					'type' => 'widget',
				),
			);
			$default_widgets['Sidebar'][] = array(
				'common_links' => array(
					'type' => 'widget',
				),
			);
			$default_widgets['Sidebar'][] = array(
				'search_form' => array(
					'type' => 'widget',
				),
			);
			$default_widgets['Sidebar'][] = array(
				'category_list' => array(
					'type' => 'widget',
				),
			);
			$default_widgets['Sidebar'][] = array(
				'xml_feeds' => array(
					'type' => 'widget',
				),
			);
		}
		// 2nd Sidebar widgets?
		if( in_array( 'Sidebar 2', $containers ) )
		{
			$default_widgets['Sidebar 2'][] = array(
				'post_list' => array(
					'type' => 'widget',
				),
			);
			$default_widgets['Sidebar 2'][] = array(
				'comment_list' => array(
					'type' => 'widget',
				),
			);
			$default_widgets['Sidebar 2'][] = array(
				'linkblog' => array(
					'type' => 'widget',
					'params' => array( 'blog_ID' => 3 ),
				),
			);
		}
		// Footer widgets?
		if( in_array( 'Footer', $containers ) )
		{
			$default_widgets['Footer'][] = array(
				'user_tools' => array(
					'type' => 'widget',
				),
			);
		}

		return $default_widgets;
	}


	/**
	 * Get the template's customized name
	 */
	function get_name()
	{
		return $this->name;
	}


	/**
	 * Get definitions for editable params
	 *
	 * @todo this is destined to be overridden by derived Template classes
	 * @see Plugin::GetDefaultSettings()
	 * @param local params like 'for_editing' => true
	 */
	function get_param_definitions( $params )
	{
		$r = array();
		return $r;
	}


	/**
	 * Get a template-specific param value from current Blog
	 */
	function get_setting( $parname )
	{
		global $Blog;

		// Name of the setting in the blog settings
		$blog_setting_name = 'template'.$this->ID.'_'.$parname;

		$value = $Blog->get_setting( $blog_setting_name );

		if( ! is_null( $value ) )
		{
			// We have a value for this param
			return $value;
		}

		// Try default values
		$params = $this->get_param_definitions( NULL );
		if( isset( $params[$parname]['defaultvalue'] ) )
		{
			// We have a default value
			return $params[$parname]['defaultvalue'] ;
		}

		return NULL;
	}


	/**
	 * Get credits for linkback on credits page
	 *
	 * Each template can and should provide an array for credits for domains/authors
	 * who created the original design, inspired your design, or ported the template
	 * in the following format:
	 * <code>
	 * $template_linkbacks = array(
	 *   'designer' => array(
	 *     'pre' => T_('Design by'),
	 *     'text' => 'linked text',
	 *     'url' => 'http://yourdomain.tld/',
	 *     'title' => T_('insert creative link title'),
	 *     'post' => T_('A reasonably brief summary of the template. A sentence or two should be enough?'),
	 *   ),
	 * );
	 * return $template_linkbacks;
	 * </code>
	 *
	 * @return array The template's credits array
	 */
	function get_template_credits()
	{
		// These are just dummy calls to include the strings in core.
		// You do not have to include them in the overriding method.
		NT_('Design by');
		NT_('Inspired by');
		NT_('Ported by');
		NT_('License');

		return array();
	}


	/**
	 * Install current template to DB
	 *
	 * @return boolean True on success, false on failure.
	 */
	function install()
	{
		// look for containers in template file
		if( ! $this->discover_containers() )
		{
			// Error messages already added.
			return false;
		}

		// store the "changes" so we actually install the fucker
		$this->dbchange( 'template_folder', 'string', 'folder' );
		$this->dbchange( 'template_name', 'string', 'name' );
		$this->dbchange( 'template_type', 'string', 'type' );

		// insert new template into database
		$this->dbinsert();
		return true;
	}


	/**
	 * Load data from Request form fields
	 *
	 * @return boolean true if loaded data seems valid.
	 */
	function load_from_Request()
	{
		// Name
		param_string_not_empty( 'template_name', T_('Please enter a name.') );
		$this->set_from_Request( 'name' );

		// Template type
		param( 'template_type', 'string' );
		$this->set_from_Request( 'type' );

		return ! param_errors_detected();
	}


	/**
	 * Load params
	 */
	function load_params_from_Request()
	{
		load_funcs( 'plugins/_plugin.funcs.php' );

		// Loop through all widget params
		foreach( $this->get_param_definitions( array( 'for_editing' => true ) ) as $parname => $parmeta )
		{
			autoform_set_param_from_request( $parname, $parmeta, $this, 'Template' );
		}
	}


	/**
	 * Set a template specific param value for current Blog
	 *
	 * @param string parameter name
	 * @param mixed parameter value
	 */
	function set_setting( $parname, $parvalue )
	{
		global $Blog;

		// Name of the setting in the blog settings
		$blog_setting_name = 'template'.$this->ID.'_'.$parname;
		$Blog->set_setting( $blog_setting_name, $parvalue );
	}


	/**
	 * Translate a given string, in the Template's context
	 *
	 * This means that the translation is obtained from the Template's "po" folder.
	 * It uses the global/regular {@link T_()} function as a fallback.
	 *
	 * @uses T_()
	 * @param string The string (english), that should be translated
	 * @param string Requested locale ({@link $current_locale} gets used by default)
	 * @return string The translated string.
	 */
	function T_( $string, $req_locale = '' )
	{
		global $templates_path;

		if( ( $return = T_( $string, $req_locale, array(
				'ext_transarray' => & $this->_trans,
				'alt_basedir' => $templates_path.$this->folder.'/po/',
			) ) ) == $string )
		{
			// This template did not provide a translation - fallback to global T_()
			return T_( $string, $req_locale );
		}

		return $return;
	}


	/**
	 * Translate and escape single quotes
	 *
	 * This is to be used mainly for Javascript strings.
	 * @uses Template::T_()
	 * @param string String to translate
	 * @param string Locale to use
	 * @return string The translated and escaped string.
	 */
	function TS_( $string, $req_locale = '' )
	{
		return str_replace( "'", "\\'", $this->T_( $string, $req_locale ) );
	}

}

?>
