<?php
/**
 * This file implements login/logout handling functions
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package users
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * Log the user out
 */
function logout()
{
	global $current_User;
	global $Plugins;
	global $Session;

	$Plugins->trigger_event( 'Logout', array( 'User' => $current_User ) );

	// Reset all global variables
	// Note: unset is bugguy on globals
	$current_User = NULL; // NULL, as we do isset() on it in several places!

	$Session->logout();
}


/**
 * is_logged_in(-)
 */
function is_logged_in()
{
	global $current_User;

	return is_object( $current_User ) && ! empty( $current_User->ID );
}


/**
 * Check if a password is ok for a login
 *
 * @param string login
 * @param string password
 * @param boolean Is the password parameter already MD5()'ed?
 * @return boolean
 */
function user_pass_ok( $login, $pass, $pass_is_md5 = false )
{
	$UserCache = & get_Cache( 'UserCache' );
	$User = & $UserCache->get_by_login( $login );
	if( !$User )
	{
		return false;
	}

	return $User->check_password( $pass, $pass_is_md5 );
}


/**
 * Template tag: Output link to login
 *
 * @todo (2140) EdB> change to array of params
 */
function user_login_link( $before = '', $after = '', $link_text = '', $link_title = '#' )
{
	if( is_logged_in() ) return false;

	if( $link_text == '' ) $link_text = T_('Log in');
	if( $link_title == '#' ) $link_title = T_('Log in if you have an account...');

	$r = $before;
	$r .= '<a href="'.get_login_url().'" title="'.$link_title.'">';
	$r .= $link_text;
	$r .= '</a>';
	$r .= $after;

	echo $r;
}


/**
 * Get url to login
 *
 * @return string
 */
function get_login_url()
{
	global $srvc_url_sensitive;

	$redirect = '?redirect_to='.rawurlencode( url_rel_to_same_host(
		regenerate_url( '', '', '', '&' ), $srvc_url_sensitive ) );
	return $srvc_url_sensitive.'login.php'.$redirect;
}


/**
 * Get url to login
 *
 * @return string
 */
function get_register_url()
{
	global $srvc_url_sensitive;

	$redirect = '?redirect_to='.rawurlencode( url_rel_to_same_host(
		regenerate_url( '', '', '', '&' ), $srvc_url_sensitive ) );
	return $srvc_url_sensitive.'register.php'.$redirect;
}


/**
 * Template tag: Output a link to new user registration
 *
 * @todo (2140) EdB> change to array of params
 * @param string
 * @param string
 * @param string
 * @param boolean Display the link, if the user is already logged in? (this is
 * used by the login form)
 */
function user_register_link(
	$before = '',
	$after = '',
	$link_text = '',
	$link_title = '#',
	$disp_when_logged_in = false )
{
	echo get_user_register_link( $before, $after, $link_text, $link_title, $disp_when_logged_in );
}


/**
 * Template tag: Get a link to new user registration
 *
 * @param string
 * @param string
 * @param string
 * @param boolean Display the link, if the user is already logged in? (this is
 * used by the login form)
 * @param string Where to redirect
 * @return string
 */
function get_user_register_link(
	$before = '',
	$after = '',
	$link_text = '',
	$link_title = '#',
	$disp_when_logged_in = false,
	$redirect = null )
{
	global $Settings;
	global $srvc_url_sensitive;

	if( is_logged_in() && ! $disp_when_logged_in )
	{
		// Do not display, when already logged in
		return false;
	}

	if( ! $Settings->get( 'newusers_canregister' ) )
	{
		// We won't let him register
		return false;
	}

	if( $link_text == '' ) $link_text = T_('Register...');
	if( $link_title == '#' ) $link_title = T_('Register to open an account...');

	if( ! empty( $redirect ) || ( $redirect = regenerate_url( '', '', '', '&' ) ) )
	{
		$redirect = '?redirect_to='.rawurlencode( url_rel_to_same_host( $redirect, $srvc_url_sensitive ) );
	}

	$r = $before;
	$r .= '<a href="'.$srvc_url_sensitive.'register.php'.$redirect.'" title="'.$link_title.'">';
	$r .= $link_text;
	$r .= '</a>';
	$r .= $after;
	return $r;
}


/**
 * Template tag: Output a link to logout
 *
 * @todo (2140) EdB> change to array of params
 */
function user_logout_link( $before = '', $after = '', $link_text = '', $link_title = '#', $params = array() )
{
	echo get_user_logout_link( $before, $after, $link_text, $link_title, $params );
}


/**
 * Template tag: Get a link to logout
 *
 * @param string
 * @param string
 * @param string link text can include %s for current user login
 * @return string
 */
function get_user_logout_link( $before = '', $after = '', $link_text = '', $link_title = '#', $params = array() )
{
	global $current_User;

	if( ! is_logged_in() )
	{
		return false;
	}

	if( $link_text == '' ) $link_text = T_('Logout');
	if( $link_title == '#' ) $link_title = T_('Logout from your account');

	$r = $before;
	$r .= '<a href="'.get_user_logout_url().'"';
	$r .= get_field_attribs_as_string( $params, false );
	$r .= ' title="'.$link_title.'">';
	$r .= sprintf( $link_text, $current_User->login );
	$r .= '</a>';
	$r .= $after;
	return $r;
}


/**
 * Get the URL for the logout button
 *
 * @return string
 */
function get_user_logout_url()
{
	global $admin_url;
	global $app_baseurl;
	global $Blog;
	global $is_admin_page;
	global $srvc_url_sensitive;

	if( ! is_logged_in() )
	{
		return false;
	}

	if( $is_admin_page )
	{
		if( isset( $Blog ) )
		{
			// Go to the home page of the blog that was being edited
			$redirect_to = $Blog->get( 'url' );
		}
		else
		{
			// We were not editing a blog...
			// return to global home
			$redirect_to = url_rel_to_same_host( $app_baseurl, $srvc_url_sensitive);
			// Alternative: return to the login page (fp> a basic user would be pretty lost on that login page)
			// $redirect_to = url_rel_to_same_host( $admin_url, $srvc_url_sensitive );
		}
	}
	else
	{
		// Return to current blog page
		$redirect_to = url_rel_to_same_host(regenerate_url('','','','&'), $srvc_url_sensitive);
	}

	return $srvc_url_sensitive.'login.php?action=logout&amp;redirect_to='.rawurlencode( $redirect_to );
}


/**
 * Template tag: Display a link to the backoffice
 *
 * Usually provided in templates in order for newbies to find the admin interface
 * more easily...
 *
 * @todo (2140) EdB> change to array of params
 * @param string To be displayed before the link.
 * @param string To be displayed after the link.
 * @param string Text for the link.
 * @param string Title for the link.
 * @param string The page/controller to link to inside of {@link $admin_url}
 * @param string the returned string if the user should not see the admin link
 */
function user_admin_link( $before = '', $after = '', $link_text = '', $link_title = '#', $not_visible = '' )
{
	echo get_user_admin_link( $before, $after, $link_text, $link_title, $not_visible );
}


/**
 * Get a link to the backoffice
 *
 * Usually provided in templates in order for newbies to find the admin interface
 * more easily...
 *
 * @param string To be displayed before the link.
 * @param string To be displayed after the link.
 * @param string Text for the link.
 * @param string Title for the link.
 * @param string The returned string if the user should not see the admin link
 * @return string
 */
function get_user_admin_link( $before = '', $after = '', $link_text = '', $link_title = '#', $not_visible = '' )
{
	global $admin_url;
	global $blog;
	global $current_User;

	if( is_logged_in() && ! $current_User->check_perm( 'admin', 'visible' ) )
	{
		// If user should NOT see admin link
		return $not_visible;
	}

	if( $link_text == '' ) $link_text = T_('Admin');
	if( $link_title == '#' ) $link_title = T_('Go to the back-office...');
	// add the blog param to $page if it is not already in there

	if( ! empty( $blog ) )
	{
		$url = url_add_param( $admin_url, 'blog='.$blog );
	}
	else
	{
		$url = $admin_url;
	}

	$r = $before;
	$r .= '<a href="'.$url.'" title="'.$link_title.'">';
	$r .= $link_text;
	$r .= '</a>';
	$r .= $after;
	return $r;
}


/**
 * Template tag: Display a link to user profile
 *
 * @param string To be displayed before the link.
 * @param string To be displayed after the link.
 * @param string Text for the link.
 * @param string Title for the link.
 * @param integer The ID of the user we want a profile link for
 */
function user_profile_link( $before = '', $after = '', $link_text = '', $link_title = '#', $user_id = '#' )
{
	echo get_user_profile_link( $before, $after, $link_text, $link_title, $user_id );
}


/**
 * Get a link to user profile
 *
 * @param string To be displayed before the link.
 * @param string To be displayed after the link.
 * @param string Text for the link.
 * @param string Title for the link.
 * @param integer The ID of the user we want a profile for
 * @return string|false
 */
function get_user_profile_link( $before = '', $after = '', $link_text = '', $link_title = '#', $user_id = '#' )
{
	global $current_User;

	if( ! is_logged_in() )
	{
		return false;
	}

	if( $link_text == '' )
	{
		$link_text = T_('Profile');
	}
	else
	{
		$link_text = str_replace( '%s', $current_User->login, $link_text );
	}

	if( $link_title == '#' )
	{
		$link_title = T_('Edit your profile');
	}

	if( $user_id == '#' )
	{
		$user_id = $current_User->ID;
	}


	$r = $before
		.'<a href="'.get_user_profile_url( $user_id ).'" title="'.$link_title.'">'
		.sprintf( $link_text, $current_User->login )
		.'</a>'
		.$after;

	return $r;
}


/**
 * Get URL to edit user profile
 *
 * @param integer The ID of the user we want a profile for
 * @return string The link to profile, either in admin or public with redirect
 */
function get_user_profile_url( $user_id )
{
	global $admin_url;
	global $Blog;
	global $current_User;
	global $is_admin_page;
	global $ReqURI;

	if( $is_admin_page || empty( $Blog ) )
	{
		$url = $admin_url.'?ctrl=users&amp;user_ID='.$current_User->ID;
	}
	else
	{
		$url = url_add_param( $Blog->gen_blogurl(), 'disp=profile&amp;user_ID='.$user_id.'&amp;redirect_to='.rawurlencode( $ReqURI ) );
	}

	return $url;
}


/**
 * Template tag: Provide a link to subscription screen
 */
function user_subs_link( $before = '', $after = '', $link_text = '', $link_title = '#' )
{
	global $Blog;
	global $current_User;

	if( ! $url = get_user_subs_url() )
	{
		return false;
	}

	if( $link_text == '' ) $link_text = T_('Subscribe');
	if( $link_title == '#' ) $link_title = T_('Subscribe to email notifications');

	echo $before;
	echo '<a href="'.$url.'" title="', $link_title, '">';
	printf( $link_text, $current_User->login );
	echo '</a>';
	echo $after;
}

/**
 * Template tag: Provide url to subscription screen
 */
function get_user_subs_url()
{
	global $Blog;
	global $is_admin_page;

	if( ! is_logged_in() || $is_admin_page )
	{
		return false;
	}

	if( empty( $Blog ) || ! $Blog->get_setting( 'allow_subscriptions' ) )
	{
		return false;
	}

	return url_add_param( $Blog->gen_blogurl(), 'disp=subs&amp;redirect_to='.rawurlencode( url_rel_to_same_host(regenerate_url('','','','&'), $Blog->gen_blogurl())));
}


/**
 * Template tag: Display the user's preferred name
 *
 * Used in result lists.
 *
 * @param integer user ID
 */
function user_preferredname( $user_ID )
{
	$UserCache = & get_Cache( 'UserCache' );
	if( ! empty( $user_ID )
		&& ( $User = & $UserCache->get_by_ID( $user_ID ) ) )
	{
		$User->disp('preferredname');
	}
}


/**
 * Check profile parameters and add errors through {@link param_error()}.
 *
 * @param array associative array. Either array [$value, $input_name] or just
 * $value; $input_name gets used for associating it to a form fieldname
 * - 'login': check for non-empty
 * - 'nickname': check for non-empty
 * - 'icq': must be a number
 * - 'email': mandatory, must be well formed
 * - 'url': must be well formed, in allowed scheme, not blacklisted
 * - 'pass1' / 'pass2': passwords (twice), must be the same and not == login (if given)
 * - 'pass_required': false/true (default is true)
 * @param User|NULL A user to use for additional checks (password != login/nick).
 */
function profile_check_params( $params, $User = NULL )
{
	global $Messages;
	global $Settings;

	foreach( $params as $k => $v )
	{
		// normalize params
		if( $k != 'pass_required' && ! is_array( $v ) )
		{
			$params[$k] = array( $v, $k );
		}
	}

	// checking login has been typed
	if( isset( $params['login'] ) && empty( $params['login'][0] ) )
	{
		param_error( 'login', T_('Please enter a login.') );
	}

	// checking the nickname has been typed
	if( isset( $params['nickname'] ) && empty( $params['nickname'][0] ) )
	{
		param_error( $params['nickname'][1], T_('Please enter a nickname (can be the same as your login).') );
	}

	// checking e-mail address
	if( isset( $params['email'][0] ) )
	{
		if( empty( $params['email'][0] ) )
		{
			param_error( $params['email'][1], T_('Please enter an e-mail address.') );
		}
		elseif( ! is_email( $params['email'][0] ) )
		{
			param_error( $params['email'][1], T_('The email address is invalid.') );
		}
	}

	// Checking URL
	if( isset( $params['url'] ) )
	{
		if( $error = validate_url( $params['url'][0], 'commenting' ) )
		{
			param_error( $params['url'][1], T_('Supplied URL is invalid: ').$error );
		}
	}

	// Check passwords
	$pass_required = isset( $params['pass_required'] ) ? $params['pass_required'] : true;

	if( isset( $params['pass1'][0] ) && isset( $params['pass2'][0] ) )
	{
		if( $pass_required || ! empty( $params['pass1'][0] ) || ! empty( $params['pass2'][0] ) )
		{
			// Password is required or was given, checking it has been typed twice
			if( empty( $params['pass1'][0] ) || empty( $params['pass2'][0] ) )
			{
				param_error( $params['pass2'][1], T_('Please enter your password twice.') );
			}

			// checking it has been typed twice the same
			if( $params['pass1'][0] !== $params['pass2'][0] )
			{
				param_error( $params['pass1'][1], T_('You typed two different passwords.') );
			}
			elseif( strlen( $params['pass1'][0] ) < $Settings->get( 'user_minpwdlen' ) )
			{
				param_error( $params['pass1'][1], sprintf( T_('The minimum password length is %d characters.'), $Settings->get( 'user_minpwdlen' ) ) );
			}
			elseif( isset( $User ) && $params['pass1'][0] == $User->get( 'login' ) )
			{
				param_error( $params['pass1'][1], T_('The password must be different from your login.') );
			}
			elseif( isset( $User ) && $params['pass1'][0] == $User->get( 'nickname' ) )
			{
				param_error( $params['pass1'][1], T_('The password must be different from your nickname.') );
			}
		}
	}
}

?>
