<?php
/**
 * This file implements the UI view for the user properties
 *
 * @todo (1111) In the "Identity" section add something that checks to see if any plugin has
 * asked for either of the user avatar fields, and if so put then display those fields. This
 * way the form will submit those values to T_users along with all the other bits instead of
 * submitting them to T_user_settings like a plugin would want to do.
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package users
 */
if(!defined('QP_MAIN_INIT')) die('fail');

global $action;
global $AdminUI;
global $current_User;
global $edited_User;
global $lock_id_and_nick;
global $Plugins;
global $Settings;
global $UserSettings;
global $user_profile_only;

// Begin payload block
$this->disp_payload_begin();

$Form = new Form( NULL, 'user_checkchanges' );
if( ! $user_profile_only )
{
	$Form->global_icon( ( $action != 'view_user' ? T_('Cancel editing!') : T_('Close user profile!') ), 'close', regenerate_url( 'user_ID,action' ), T_('cancel') );
}
if( $edited_User->ID == 0 )
{
	// Creating new user
	$creating = true;
	$Form->begin_form( 'fform', T_('Create new user profile') );
}
else
{
	// Editing existing user
	$creating = false;
	$Form->begin_form( 'fform', T_('Profile for:').' '.$edited_User->dget( 'fullname' ).' ['.$edited_User->dget( 'login' ).']' );
}
$Form->hidden_ctrl();
$Form->hidden( 'user_ID', $edited_User->ID );

// "User permissions" section
$Form->begin_fieldset( T_('User permissions'), array( 'class' => 'fieldset clear' ) );
$edited_User->get_Group();

if( $edited_User->ID != 1 && $current_User->check_perm( 'users', 'edit' ) )
{
	// This is not Admin and we're not restricted: we're allowed to change the user group
	$chosengroup = ( $edited_User->Group === NULL ) ? $Settings->get( 'newusers_grp_ID' ) : $edited_User->Group->ID;
	$GroupCache = & get_Cache( 'GroupCache' );
	$Form->select_object( 'edited_user_grp_ID', $chosengroup, $GroupCache, T_('User group') );
}
else
{
	echo '<input type="hidden" name="edited_user_grp_ID" value="'.$edited_User->Group->ID.'" />';
	$Form->info( T_('User group'), $edited_User->Group->dget( 'name' ) );
}

$field_note = '[0 - 10] '.T_('(this feature has very little use and may one day go away)');
if( $action != 'view_user' && $current_User->check_perm( 'users', 'edit' ) )
{
	$Form->text_input( 'edited_user_level', $edited_User->get( 'level' ), 2, T_('User level'), $field_note, array( 'required' => true ) );
}
else
{
	$Form->info_field( T_('User level'), $edited_User->get( 'level' ), array( 'note' => $field_note ) );
}
$Form->end_fieldset();

// "Email communications" section
$Form->begin_fieldset( T_('Email communications') );
$email_fieldnote = '<a href="mailto:'.$edited_User->get( 'email' ).'">'.get_icon( 'email', 'imgtag', array( 'title' => T_('Send an email') ) ).'</a>';

if( $action != 'view_user' )
{
	// We can edit the values
	$Form->text_input( 'edited_user_email', $edited_User->email, 30, T_('Email'), $email_fieldnote, array( 'maxlength' => 100, 'required' => true ) );
	if( $current_User->check_perm( 'users', 'edit' ) )
	{
		// user has "edit users" perms
		$Form->checkbox( 'edited_user_validated', $edited_User->get( 'validated' ), T_('Validated email'), T_('Has this email address been validated (through confirmation email)?') );
	}
	else
	{
		// info only
		$Form->info( T_('Validated email'), ( $edited_User->get( 'validated' ) ? T_('yes') : T_('no') ), T_('Has this email address been validated (through confirmation email)?') );
	}
	$Form->checkbox( 'edited_user_allow_msgform', $edited_User->get( 'allow_msgform' ), T_('Message form'), T_('Check this to allow receiving emails through a message form.') );
	$Form->checkbox( 'edited_user_notify', $edited_User->get( 'notify' ), T_('Notifications'), T_('Check this to receive a notification whenever someone else comments on one of <strong>your</strong> posts.') );
}
else
{
	// display only
	$Form->info( T_('Email'), $edited_User->get( 'email' ), $email_fieldnote );
	$Form->info( T_('Validated email'), ( $edited_User->get( 'validated' ) ? T_('yes') : T_('no') ), T_('Has this email address been validated (through confirmation email)?') );
	$Form->info( T_('Message form'), ( $edited_User->get( 'allow_msgform' ) ? T_('yes') : T_('no') ) );
	$Form->info( T_('Notifications'), ( $edited_User->get( 'notify' ) ? T_('yes') : T_('no') ) );
}
$Form->end_fieldset();

// "Identity" section
$Form->begin_fieldset( T_('Identity') );
if( $action != 'view_user' )
{
	// We can edit the values
	if( $lock_id_and_nick && ( ! $current_User->ID === 1 ) )
	{
		// lock is set and user #1 is not out there ... no editing allowed
		$Form->info( T_('Login'), $edited_User->get( 'login' ) );
		$Form->hidden( 'edited_user_login', $edited_User->login );
	}
	else
	{
		$Form->text_input( 'edited_user_login', $edited_User->login, 20, T_('Login'), '', array( 'required' => true ) );
	}
	$Form->text_input( 'edited_user_firstname', $edited_User->firstname, 20, T_('First name'), '', array( 'maxlength' => 50 ) );
	$Form->text_input( 'edited_user_lastname', $edited_User->lastname, 20, T_('Last name'), '', array( 'maxlength' => 50 ) );
	if( $lock_id_and_nick && ( ! $current_User->ID === 1 ) )
	{
		// lock is set and user #1 is not out there ... no editing allowed
		$Form->info( T_('Nickname'), $edited_User->get( 'nickname' ) );
		$Form->hidden( 'edited_user_nickname', $edited_User->nickname );
	}
	else
	{
		$Form->text_input( 'edited_user_nickname', $edited_User->nickname, 20, T_('Nickname'), '', array( 'maxlength' => 50, 'required' => true ) );
	}
	$Form->select( 'edited_user_idmode', $edited_User->get( 'idmode' ), array( &$edited_User, 'callback_optionsForIdMode' ), T_('Identity shown') );
}
else
{
	// display only
	$Form->info( T_('Login'), $edited_User->get( 'login' ) );
	$Form->info( T_('First name'), $edited_User->get( 'firstname' ) );
	$Form->info( T_('Last name'), $edited_User->get( 'lastname' ) );
	$Form->info( T_('Nickname'), $edited_User->get( 'nickname' ) );
	$Form->info( T_('Identity shown'), $edited_User->get( 'preferredname' ) );
}

ob_start();
ob_start();
$Plugins->restart();
// $edited_User->ID is 0 if we are copying/creating, so the avatar and biography
// bits can't be added at this point. too lazy to really fix it is the thing ...
while( $loop_Plugin = & $Plugins->get_next() && $edited_User->ID )
{
	if( $loop_Plugin->UserSettings )
	{
		$plugin_user_settings = $loop_Plugin->GetDefaultUserSettings( $tmp_params = array( 'for_editing' => true ) );
		if( is_array( $plugin_user_settings ) )
		{
			foreach( $plugin_user_settings as $l_name => $l_meta )
			{
				// avatar field
				if( $l_name == 'use_avatar' )
				{
					// get avatar info
					$avatar_info = $edited_User->get_avatar( array(
							'size' => 'crop-64x64',
							'class' => 'floatleft'
						) );
					$avatar_tag = $avatar_info['tag']; // '' if no avatar
					$avatar_root = $avatar_info['root']; // '' if no avatar
					if( $avatar_root != '' )
					{
						$avatar_root = '&amp;root='.$avatar_root;
					}
					if( $current_User->check_perm( 'users', 'all' ) || ( $current_User->ID == $edited_User->ID ) )
					{
						if( ! empty( $avatar_tag ) && $action != 'view_user' )
						{
							$avatar_tag .= ' '.action_icon( T_( 'Remove' ), 'delete', '?ctrl=user&amp;user_ID='.$edited_User->ID.'&amp;action=remove_avatar', ' '.T_('Remove').' &raquo;', 5, 5 );
							if( $current_User->check_perm( 'files', 'view' ) )
							{
								$avatar_tag .= '<br />'.action_icon( T_( 'Change' ), 'link', '?ctrl=files'.$avatar_root.'&amp;user_ID='.$edited_User->ID, ' '.T_('Change').' &raquo;', 5, 5 );
							}
						}
						elseif( $current_User->check_perm( 'files', 'view' ) && $action != 'view_user' )
						{
							$avatar_tag .= ' '.action_icon( T_( 'Upload or choose an avatar' ), 'link', '?ctrl=files'.$avatar_root.'&amp;user_ID='.$edited_User->ID, ' '.T_('Upload/Select').' &raquo;', 5, 5 );
						}
					}
					$Form->info( T_( 'Avatar' ), $avatar_tag );
				}
				// biography field
				if( $l_name == 'use_biography' )
				{
					if( $current_User->check_perm( 'users', 'all' ) || ( $current_User->ID == $edited_User->ID ) )
					{
						$Form->textarea( 'edited_user_biography', $edited_User->get( 'biography' ), 5, T_('Biography'), T_('This is called for by a plugin so we do not know what it is used for.'), 50, 'large' );
					}
					else
					{
						$Form->info( T_('Biography'), $edited_User->get( 'biography' ), T_('This is called for by a plugin so we do not know what it is used for.') );
					}
				}
			}
		}
	}
}
$has_contents = strlen( ob_get_contents() );
//$Form->end_fieldset();
if( $has_contents )
{
	ob_end_flush();
	ob_end_flush();
}
else
{
	// No content, discard output buffers
	ob_end_clean();
	ob_end_clean();
}

if( $action != 'view_user' )
{
	// We can edit the values
	if( $lock_id_and_nick && ( ! $current_User->ID === 1 ) )
	$Form->checkbox( 'edited_user_showonline', $edited_User->get( 'showonline' ), T_('Show online'), T_('Check this to be displayed as online when visiting the site.') );
	$Form->checkbox( 'edited_user_set_login_multiple_sessions', $UserSettings->get( 'login_multiple_sessions', $edited_User->ID), T_('Multiple sessions'),
		T_('Check this if you want to log in from different computers/browsers at the same time. Otherwise, logging in from a new computer/browser will disconnect you on the previous one.') );
}
else
{
	// display only
	$Form->info( T_('Show online'), ( $edited_User->get( 'showonline' ) ) ? T_('yes') : T_('no') );
	$Form->info( T_('Multiple sessions'), ( $UserSettings->get( 'login_multiple_sessions', $edited_User->ID ) ? T_('Allowed') : T_('Forbidden') ) );
}
$Form->end_fieldset();

// "Password" section
if( $action != 'view_user' )
{
	// We can edit the values
	$Form->begin_fieldset( T_('Password') );
	$Form->password_input( 'edited_user_pass1', '', 20, T_('New password'),
		array( 'note' => ( ! empty( $edited_User->ID ) ? T_('Leave empty if you don\'t want to change the password.') : '' ), 'maxlength' => 50, 'required' => ( $edited_User->ID == 0 ), 'autocomplete' => 'off' ) );
	$Form->password_input( 'edited_user_pass2', '', 20, T_('Confirm new password'),
		array( 'note' => sprintf( T_('Minimum length: %d characters.'), $Settings->get( 'user_minpwdlen' ) ), 'maxlength' => 50, 'required' => ( $edited_User->ID == 0 ), 'autocomplete' => 'off' ) );
	$Form->end_fieldset();
}

// "Preferences" section
$Form->begin_fieldset( T_('Preferences') );
$value_admin_template = get_param('edited_user_admin_template');
if( ! $value_admin_template )
{
	// no value supplied through POST/GET
	$value_admin_template = $UserSettings->get( 'admin_template', $edited_User->ID );
}
if( ! $value_admin_template )
{
	// Nothing set yet for the user, use the default
	$value_admin_template = $Settings->get( 'admin_template' );
}
if( $action != 'view_user' )
{
	// We can edit the values
	$Form->select( 'edited_user_locale', $edited_User->get( 'locale' ), 'locale_options_return', T_('Preferred locale'), T_('Preferred locale for admin interface, notifications, etc.'));
	$Form->select_input_array( 'edited_user_admin_template', $value_admin_template, get_admin_templates(), T_('Admin template'), T_('The template defines how the backoffice appears to you.') );
	// To display or hide icon legend
	$Form->checkbox( 'edited_user_legend', $UserSettings->get( 'display_icon_legend', $edited_User->ID ), T_('Display icon legend'), T_('Display a legend at the bottom of every page including all action icons used on that page.') );
	// To activate or deactivate bozo validator
	$Form->checkbox( 'edited_user_bozo', $UserSettings->get( 'control_form_abortions', $edited_User->ID ), T_('Control form closing'), T_('This will alert you if you fill in data into a form and try to leave the form before submitting the data.') );
	// To activate focus on first form input text
	$Form->checkbox( 'edited_user_focusonfirst', $UserSettings->get( 'focus_on_first_input', $edited_User->ID ), T_('Focus on first field'), T_('The focus will automatically go to the first input text field.') );
	// Number of results per page
	$Form->text( 'edited_user_results_per_page', $UserSettings->get( 'results_per_page', $edited_User->ID ), 3, T_('Results per page'), T_('Number of rows displayed in results tables.') );
	// Number of blogs to display in the admin
	$Form->text( 'edited_user_num_admin_blogs', $UserSettings->get( 'num_admin_blogs', $edited_User->ID ), 2, T_('Blog menu length'), T_('Number of blogs displayed in the admin console before using a select box') );
}
else
{
	// display only
	$Form->info( T_('Preferred locale'), $edited_User->get( 'locale' ), T_('Preferred locale for admin interface, notifications, etc.') );
	$Form->info_field( T_('Admin template'), $value_admin_template, array( 'note' => T_('The template defines how the backoffice appears to you.') ) );
	// fp> TODO: a lot of things will not be displayed in view only mode. Do we want that?
	$Form->info_field( T_('Results per page'), $UserSettings->get( 'results_per_page', $edited_User->ID ), array( 'note' => T_('Number of rows displayed in results tables.') ) );
}
$Form->end_fieldset();

// "Plugin options" section
if( $action != 'view_user' )
{
	// We can edit the values
	// PluginUserSettings
	load_funcs( 'plugins/_plugin.funcs.php' );

	$Form->begin_fieldset( T_('Plugin options') );

	$Plugins->restart();
	while( $loop_Plugin = & $Plugins->get_next() )
	{
		if( ! $loop_Plugin->UserSettings && ! $Plugins->has_event( $loop_Plugin->ID, 'PluginSettingsEditDisplayAfter' ) )
		{
			continue;
		}

		// use output buffers to display the fieldset only if there is content (either from PluginUserSettings or PluginSettingsEditDisplayAfter).
		ob_start();
		$Form->begin_fieldset( $loop_Plugin->name );
		ob_start();
		// UserSettings:
		$plugin_user_settings = $loop_Plugin->GetDefaultUserSettings( $tmp_params = array( 'for_editing' => true ) );
		if( is_array( $plugin_user_settings ) )
		{
			foreach( $plugin_user_settings as $l_name => $l_meta )
			{
				// exclude extended identity fields
				if( $l_name == 'use_avatar' || $l_name == 'use_biography' )
				{
					continue;
				}
				// Display form field for this setting:
				autoform_display_field( $l_name, $l_meta, $Form, 'UserSettings', $loop_Plugin, $edited_User );
			}
		}

		// fp> what's a use case for this event? (I soooo want to nuke it...)
		$Plugins->call_method( $loop_Plugin->ID, 'PluginUserSettingsEditDisplayAfter',
			$tmp_params = array( 'Form' => & $Form, 'User' => $edited_User ) );

		$has_contents = strlen( ob_get_contents() );
		$Form->end_fieldset();

		if( $has_contents )
		{
			ob_end_flush();
			ob_end_flush();
		}
		else
		{
			// No content, discard output buffers
			ob_end_clean();
			ob_end_clean();
		}
	}

	$Form->end_fieldset();
}

// "Additional info" section
$Form->begin_fieldset( T_('Additional info') );
if( ! $creating )
{
	// We're NOT creating a new user
	$Form->info_field( T_('ID'), $edited_User->ID );
	$Form->info_field( T_('Posts'), $edited_User->get_num_posts() );
	$Form->info_field( T_('Created on'), $edited_User->dget( 'datecreated' ) );
	$Form->info_field( T_('From IP'), $edited_User->dget( 'ip' ) );
	$Form->info_field( T_('From Domain'), $edited_User->dget( 'domain' ) );
	$Form->info_field( T_('With Browser'), $edited_User->dget( 'browser' ) );
}

if( ( $url = $edited_User->get( 'url' ) ) != '' )
{
	if( !preg_match('#://#', $url) )
	{
		$url = 'http://'.$url;
	}
	$url_fieldnote = '<a href="'.$url.'" target="_blank">'.get_icon( 'www', 'imgtag', array( 'title' => T_('Visit the site (opens new window)') ) ).'</a>';
}
else
{
	$url_fieldnote = '';
}

if( $action != 'view_user' )
{
	// We can edit the values
	$Form->text_input( 'edited_user_url', $edited_User->url, 30, T_('URL'), $url_fieldnote, array( 'maxlength' => 100 ) );
}
else
{
	// display only
	$Form->info( T_('URL'), $edited_User->get( 'url' ), $url_fieldnote );
}
$Form->end_fieldset();

if( $action != 'view_user' )
{
	// Edit buttons
	$Form->buttons( array(
		array( '', 'actionArray[userupdate]', T_('Save !'), 'SaveButton' ),
		array( 'reset', '', T_('Reset'), 'ResetButton' ),
		// dh> TODO: Non-Javascript-confirm before trashing all settings with a misplaced click
		array( 'type' => 'submit', 'name' => 'actionArray[default_settings]', 'value' => T_('Restore defaults'), 'class' => 'ResetButton',
			'onclick' => "return confirm('".TS_('This will reset all your user settings.').'\n'.TS_('This cannot be undone.').'\n'.TS_('Are you sure?')."');" ),
	) );
}

$Form->end_form();

// End payload block
$this->disp_payload_end();

?>
