<?php
/**
 * This file implements the UI view for the user/group list for user/group editing
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package users
 */
if(!defined('QP_MAIN_INIT')) die('fail');

global $current_User;
global $DB;
global $Settings;
global $usedgroups;

// query which groups have users (to prevent deletion of groups which have users)
$usedgroups = $DB->get_col( 'SELECT grp_ID
	FROM T_groups
	INNER JOIN T_users ON user_grp_ID = grp_ID
	GROUP BY grp_ID' );

// Query user list
$keywords = param( 'keywords', 'string', '', true );

$where_clause = '';

if( ! empty( $keywords ) )
{
	$kw_array = split( ' ', $keywords );
	foreach( $kw_array as $kw )
	{
		// Note: we use CONCAT_WS (Concat With Separator) because CONCAT returns NULL if any arg is NULL
		$where_clause .= 'CONCAT_WS( " ", user_login, user_firstname, user_lastname, user_nickname, user_email) LIKE "%'.$DB->escape( $kw ).'%" AND ';
	}
}

$sql = "SELECT T_users.*, grp_ID, grp_name,
	COUNT(blog_ID) AS nb_blogs
	FROM T_users
	RIGHT JOIN T_groups ON user_grp_ID = grp_ID
	LEFT JOIN T_blogs on user_ID = blog_owner_user_ID
	WHERE $where_clause 1
	GROUP BY user_ID, grp_ID
	ORDER BY grp_name, *";

$count_sql = 'SELECT COUNT(*)
	FROM T_users
	WHERE '.$where_clause.' 1';

$Results = new Results( $sql, 'user_', '--A', NULL, $count_sql );
$Results->title = T_('Groups & Users');

// Table icons
if( $current_User->check_perm( 'users', 'edit', false ) )
{
	// create new user link
	$Results->global_icon( T_('Create a new user...'), 'new', '?ctrl=users&amp;action=new_user', T_('Add user').' &raquo;' );
	$Results->global_icon( T_('Create a new group...'), 'new', '?ctrl=users&amp;action=new_group', T_('Add group').' &raquo;' );
}

$Results->filter_area = array(
	'callback' => 'filter_userlist',
	'url_ignore' => 'results_user_page,keywords',
	'presets' => array( 'all' => array( T_('All users'), '?ctrl=users' ),
) );

// Grouping params
$Results->group_by = 'grp_ID';
$Results->ID_col = 'user_ID';

// Group columns
$Results->grp_cols[] = array(
	'td_class' => 'firstcol'.( $current_User->check_perm( 'users', 'edit', false ) ? '' : ' lastcol' ),
	'td_colspan' => -1, // nb_colds - 1
	'td' => '<a href="?ctrl=users&amp;grp_ID=$grp_ID$">$grp_name$</a>'
		.'conditional( (#grp_ID# == '.$Settings->get( 'newusers_grp_ID' ).'), \' <span class="notes">('.T_('default group for new users').')</span>\' )',
);

$Results->grp_cols[] = array(
	'td_class' => 'shrinkwrap',
	'td' => '%grp_actions( {row} )%',
);

// Data columns
$Results->cols[] = array(
	'th' => T_('ID'),
	'th_class' => 'shrinkwrap',
	'td_class' => 'shrinkwrap',
	'order' => 'user_ID',
	'td' => '$user_ID$',
);

$Results->cols[] = array(
	'th' => T_('Login'),
	'th_class' => 'shrinkwrap',
	'order' => 'user_login',
	'td' => '<a href="?ctrl=users&amp;user_ID=$user_ID$"><strong>$user_login$</strong></a>',
);

$Results->cols[] = array(
	'th' => T_('Nickname'),
	'th_class' => 'shrinkwrap',
	'order' => 'user_nickname',
	'td' => '$user_nickname$',
);

$Results->cols[] = array(
	'th' => T_('Name'),
	'order' => 'user_lastname, user_firstname',
	'td' => '$user_firstname$ $user_lastname$',
);

// User #1 can see all ...
if( $current_User->ID === '1' )
{
	$Results->cols[] = array(
		'th' => T_('Email'),
		'td_class' => 'shrinkwrap',
		'td' => '%user_mailto( #user_email# )%',
	);
}

$Results->cols[] = array(
	'th' => T_('Message'),
	'td_class' => 'shrinkwrap',
	'td' => '%user_message( #user_allow_msgform#, #user_email# , #user_ID#, #user_nickname# )%',
);

$Results->cols[] = array(
	'th' => T_('URL'),
	'td_class' => 'shrinkwrap',
	'td' => 'conditional( (#user_url# != \'http://\') && (#user_url# != \'\'), \'<a href="$user_url$" title="Website: $user_url$">'
		.get_icon( 'www', 'imgtag', array( 'class' => 'middle', 'title' => 'Website: $user_url$' ) ).'</a>\', \'&nbsp;\' )',
	);

$Results->cols[] = array(
	'th' => T_('Blogs'),
	'order' => 'nb_blogs',
	'th_class' => 'shrinkwrap',
	'td_class' => 'center',
	'td' => 'conditional( (#nb_blogs# > 0), #nb_blogs#, \'&nbsp;\' )',
);

if( ! $current_User->check_perm( 'users', 'edit', false ) )
{
	$Results->cols[] = array(
		'th' => T_('Level'),
		'th_class' => 'shrinkwrap',
		'td_class' => 'right',
		'order' => 'user_level',
		'default_dir' => 'D',
		'td' => '$user_level$',
	);
}
else
{
	$Results->cols[] = array(
		'th' => T_('Level'),
		'th_class' => 'shrinkwrap',
		'td_class' => 'shrinkwrap',
		'order' => 'user_level',
		'default_dir' => 'D',
		'td' => '%display_level( #user_level#, #user_ID# )%',
	);

	$Results->cols[] = array(
		'th' => T_('Actions'),
		'td_class' => 'shrinkwrap',
		'td' => action_icon( T_('Edit this user...'), 'edit', '%regenerate_url( \'action\', \'user_ID=$user_ID$\' )%' )
			.action_icon( T_('Duplicate this user...'), 'copy', '%regenerate_url( \'action\', \'action=new_user&amp;user_ID=$user_ID$\' )%' )
			.'conditional( (#user_ID# != 1) && (#nb_blogs# < 1) && (#user_ID# != '.$current_User->ID.'), \''
			.action_icon( T_('Delete this user!'), 'delete', '%regenerate_url( \'action\', \'action=delete_user&amp;user_ID=$user_ID$\' )%' ).'\', \''
			.get_icon( 'delete', 'noimg' ).'\' )'
	);
}

// Display result
$Results->display();

/**
 * dummy docblock
 */
function display_level( $user_level, $user_ID )
{
	$r = '';
	if( $user_level > 0)
	{
		$r .= action_icon( TS_('Decrease user level'), 'decrease',
			regenerate_url( 'action', 'action=promote&amp;prom=down&amp;user_ID='.$user_ID ) );
	}
	else
	{
		$r .= get_icon( 'decrease', 'noimg' );
	}
	$r .= sprintf( '<code>% 2d </code>', $user_level );
	if( $user_level < 10 )
	{
		$r.= action_icon( TS_('Increase user level'), 'increase',
			regenerate_url( 'action', 'action=promote&amp;prom=up&amp;user_ID='.$user_ID ) );
	}
	else
	{
		$r .= get_icon( 'increase', 'noimg' );
	}
	return $r;
}


/**
 * Callback to add filters on top of the result set
 *
 * @param Form
 */
function filter_userlist( & $Form )
{
	$Form->text( 'keywords', get_param('keywords'), 20, T_('Keywords'), T_('Separate with space'), 50 );
}


/**
 * dummy docblock
 */
function grp_actions( & $row )
{
	global $Settings;
	global $usedgroups;

	$r = action_icon( T_('Edit this group...'), 'edit', regenerate_url( 'action', 'grp_ID='.$row->grp_ID ) );
	$r .= action_icon( T_('Duplicate this group...'), 'copy', regenerate_url( 'action', 'action=new_group&amp;grp_ID='.$row->grp_ID ) );

	if( ( $row->grp_ID != 1 ) && ( $row->grp_ID != $Settings->get( 'newusers_grp_ID' ) ) && ! in_array( $row->grp_ID, $usedgroups ) )
	{
		// delete
		$r .= action_icon( T_('Delete this group!'), 'delete', regenerate_url( 'action', 'action=delete_group&amp;grp_ID='.$row->grp_ID ) );
	}
	else
	{
		$r .= get_icon( 'delete', 'noimg' );
	}
	return $r;
}


/**
 * dummy docblock
 */
function user_mailto( $email )
{
	if( empty( $email ) )
	{
		return '&nbsp;';
	}
	return action_icon( T_('Email').': '.$email, 'email', 'mailto:'.$email, T_('Email') );
}

/**
 * dummy docblock
 */
function user_message( $allow_form, $email, $ID, $nickname )
{
	if( $allow_form === '0' )
	{
		return '&nbsp;';
	}

	$form_url = url_add_param( '', 'recipient_id='.$ID.'&amp;redirect_to='.rawurlencode( url_rel_to_same_host( regenerate_url( '', '', '', '&' ), '' ) ) );

	return action_icon( sprintf( T_('Send email to %s'), $nickname ), 'email', $form_url );
}

?>
