<?php
/**
 * This file implements the "Blog List" widget
 *
 * To call this widget directly in your template (bypassing containers), use the
 * following code modified with any params you need to customize:
 * <code>
 * // display the Blog List widget
 * template_widget( array(
 *   'widget' => 'bloglist',
 *   'widget_title' => '', // default shown, customize as desired
 * ) );
 * </code>
 *
 * This widget has 1 param that you can set when with a direct widget call:
 * - 'blog_list_type' => default is 'public', Optionally set this to 'owner' to
 *   show a list of blogs owned by whoever is viewing the page.
 *
 * @see ComponentWidget::init_display()
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package widgets
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * bloglist_Widget class
 *
 * @package widgets
 */
class bloglist_Widget extends ComponentWidget
{
	/**
	 * Constructor
	 */
	function bloglist_Widget( $db_row = NULL )
	{
		// Call parent constructor
		parent::ComponentWidget( $db_row, 'widget', 'bloglist' );
		$this->widget_name = T_('Blog List');
		$this->widget_title = '';
	}


	/**
	 * Display the widget!
	 *
	 * @param array must contain at least the basic display params
	 */
	function display( $params )
	{
		global $app_baseurl;
		global $Blog;

		$this->init_display( $params );

		echo $this->disp_params['block_start'];
		$this->disp_title();

		$BlogCache = & get_Cache( 'BlogCache' );

		if( $this->disp_params['blog_list_type'] == 'owner' )
		{
			// Load blogs of same owner
			$blog_array = $BlogCache->load_owner_blogs( $Blog->owner_user_ID, 'ID' );
		}
		else
		{
			// Load all public blogs
			$blog_array = $BlogCache->load_public( 'ID' );
		}

		if( ! empty( $blog_array ) )
		{
			echo $this->disp_params['list_start'];
			foreach( $blog_array as $l_blog_ID )
			{
				// Loop through all blogs
				$l_Blog = & $BlogCache->get_by_ID( $l_blog_ID );
				if( $Blog && $l_blog_ID == $Blog->ID )
				{
					// This is the blog being displayed on this page
					echo $this->disp_params['item_selected_start'];
					$link_class = $this->disp_params['link_selected_class'];
				}
				else
				{
					echo $this->disp_params['item_start'];
					$link_class = $this->disp_params['link_default_class'];;
				}

				echo '<a href="'.$l_Blog->gen_blogurl().'" class="'.$link_class.'" title="'.$l_Blog->dget( 'name', 'htmlattr' ).'">';

				if( $Blog && $l_blog_ID == $Blog->ID )
				{
					// This is the blog being displayed on this page
					echo $this->disp_params['item_selected_text_start'];
					printf( $this->disp_params['item_selected_text'], $l_Blog->dget( 'shortname', 'htmlbody' ) );
					echo $this->disp_params['item_selected_text_end'];
					echo '</a>';
					echo $this->disp_params['item_selected_end'];
				}
				else
				{
					echo $this->disp_params['item_text_start'];
					printf( $this->disp_params['item_text'], $l_Blog->dget( 'shortname', 'htmlbody' ) );
					echo $this->disp_params['item_text_end'];
					echo '</a>';
					echo $this->disp_params['item_end'];
				}
			}
			echo $this->disp_params['list_end'];
		}
		echo $this->disp_params['block_end'];
	}


	/**
	 * Get widget's short description
	 */
	function get_desc()
	{
		return T_('Display chosen list of blogs.');
	}


	/**
	 * Get widget's title
	 *
	 * @todo (3069) EdB> rename this to get_title()
	 */
	function get_short_desc()
	{
		return format_to_output( $this->disp_params['widget_title'] );
	}


	/**
	 * Get definitions for editable params
	 *
	 * @see Plugin::GetDefaultSettings()
	 * @param local params like 'for_editing' => true
	 */
	function get_param_definitions( $params )
	{
		$r = array_merge( array(
			'blog_list_type' => array(
				'label' => T_('List'),
				'type' => 'select',
				'options' => array(
					'owner' => T_('Same owner'),
					'public' => T_('Public'),
				),
				'defaultvalue' => 'public',
				'note' => T_('Blogs to list'),
			),
		), parent::get_param_definitions( $params ) );
		return $r;
	}

}

?>
