<?php
/**
 * This file implements the "Category List" widget
 *
 * To call this widget directly in your template (bypassing containers), use
 * the following code modified with any params you need to customize:
 * <code>
 * // display a widget without a container
 * template_widget( array(
 * 	'widget' => 'category_list',
 * 	'widget_title' => 'Categories', // default shown, customize as desired
 * ) );
 * </code>
 *
 * This widget has 6 params that you can set when with a direct widget call:<ul>
 * <li>'option_all' => Default is 'All', this allows the visitor to reset the filter.
 * Leave blank if you want no such option.</li>
 * <li>'use_form' => Default is 0, set to 1 to add checkboxes to allow selection
 * of multiple categories.</li>
 * <li>'default_match' => Default is "or", options are "nor" or "and".</li>
 * <li>'disp_names_for_blog_list' => Default is 1, change to 0 if you do not want
 * blog names displayed on an aggregated blog.</li>
 * <li>'exclude_cats' => Default is {empty}, A comma-separated list of category
 * IDs that you want to exclude from the list.</li>
 * <li>'display_checkboxes' => Default is 0, change to 1 if you want check-boxes
 * displayed for each category name.</li></ul>
 *
 * @see ComponentWidget::init_display()
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package widgets
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * category_list_Widget class
 *
 * @package widgets
 */
class category_list_Widget extends ComponentWidget
{
	/**
	 * Constructor
	 */
	function category_list_Widget( $db_row = NULL )
	{
		// Call parent constructor
		parent::ComponentWidget( $db_row, 'widget', 'category_list' );
		$this->widget_name = T_('Category List');
		$this->widget_title = T_('Categories');
	}


	/**
	 * Display the widget!
	 *
	 * @param array MUST contain at least the basic display params
	 */
	function display( $params )
	{
		global $Blog;
		global $cat_modifier;

		$this->init_display( $params ); // just in case it hasn't been done before

		$CategoryCache = & get_Cache( 'CategoryCache' );

		$callbacks = array(
			'line' => array( $this, 'cat_line' ),
			'no_children' => array( $this, 'cat_no_children' ),
			'before_level' => array( $this, 'cat_before_level' ),
			'after_level' => array( $this, 'cat_after_level' )
		);

		// START DISPLAY
		echo $this->disp_params['block_start'];
		$this->disp_title();

		if( $this->disp_params['use_form'] )
		{
			// We want a complete form
			echo '<form method="get" action="'.$Blog->gen_blogurl().'">';
		}

		$aggregate_blog_IDs = $Blog->get_setting( 'aggregate_blog_IDs' );
		if( empty( $aggregate_blog_IDs ) )
		{
			// We want to display cats for ONE blog
			$tmp_disp = '';
			if( $this->disp_params['option_all'] )
			{
				// We want to display a link to all cats
				$tmp_disp .= $this->disp_params['item_start'].'<a href="';
				if( $this->disp_params['link_type'] == 'context' )
				{
					// We want to preserve current browsing context
					$tmp_disp .= regenerate_url( 'cats,catsel' );
				}
				else
				{
					$tmp_disp .= $Blog->gen_blogurl();
				}
				$tmp_disp .= '">'.$this->disp_params['option_all'].'</a>';
				$tmp_disp .= $this->disp_params['item_end'];
			}

			$r = $tmp_disp.$CategoryCache->recurse( $callbacks, $Blog->ID );

			if( ! empty( $r ) )
			{
				echo $this->disp_params['list_start'];
				echo $r;
				echo $this->disp_params['list_end'];
			}
		}
		else
		{
			// We want to display cats for SEVERAL blogs
			$BlogCache = & get_Cache( 'BlogCache' );

			// Make sure everything is loaded at once (vs multiple queries)
			$CategoryCache->load_all();

			echo $this->disp_params['collist_start'];
			if( $aggregate_blog_IDs == '*' )
			{
				$BlogCache->load_all();
				$blog_ID_array = $BlogCache->get_ID_array();
			}
			else
			{
				$blog_ID_array = explode( ',', $aggregate_blog_IDs );
			}
			foreach( $blog_ID_array as $curr_blog_ID )
			{
				// Get blog
				$loop_Blog = & $BlogCache->get_by_ID( $curr_blog_ID, false );
				if( empty( $loop_Blog ) )
				{
					// That one doesn't exist (any more?)
					continue;
				}

				// Display blog title, if requested
				if( $this->disp_params['disp_names_for_blog_list'] )
				{
					echo $this->disp_params['coll_start'];
					echo '<a href="';
					if( $this->disp_params['link_type'] == 'context' )
					{
						// We want to preserve current browsing context
						echo regenerate_url( 'blog,cats,catsel', 'blog='.$curr_blog_ID );
					}
					else
					{
						$loop_Blog->disp( 'url', 'raw' );
					}
					echo '">';
					$loop_Blog->disp( 'name' );
					echo '</a>';
					echo $this->disp_params['coll_end'];
				}

				$r = $CategoryCache->recurse( $callbacks, $curr_blog_ID );

				if( ! empty( $r ) )
				{
					echo $this->disp_params['list_start'];
					echo $r;
					echo $this->disp_params['list_end'];
				}
			}
		}

		if( $this->disp_params['use_form'] || $this->disp_params['display_checkboxes'] )
		{
			// We want to add form fields
			?>
			<div class="tile">
			<input type="radio" name="cat" value="" id="catANY" class="radio" <?php if( $cat_modifier != '-' && $cat_modifier != '*' ) echo 'checked="checked" '?> />
			<label for="catANY"><?php echo T_('ANY') ?></label>
			</div>
			<div class="tile">
			<input type="radio" name="cat" value="-" id="catANYBUT" class="radio" <?php if( $cat_modifier == '-' ) echo 'checked="checked" '?> />
			<label for="catANYBUT"><?php echo T_('ANY BUT') ?></label>
			</div>
			<div class="tile">
			<input type="radio" name="cat" value="*" id="catALL" class="radio" <?php if( $cat_modifier == '*' ) echo 'checked="checked" '?> />
			<label for="catALL"><?php echo T_('ALL') ?></label>
			</div>
			<?php
			if( $this->disp_params['use_form'] )
			{
				// We want a complete form
				?>
				<div class="tile">
				<input type="submit" value="<?php echo T_('Filter categories'); ?>" />
				</div>
				</form>
				<?php
			}
		}
		echo $this->disp_params['block_end'];

		return true;
	}


	/**
	 * Get widget's description
	 */
	function get_desc()
	{
		return T_('List of all categories; click filters blog on selected category.');
	}


	/**
	 * Get definitions for this widget's settings
	 *
	 * @see Plugin::GetDefaultSettings()
	 * @param local params like 'for_editing' => true
	 */
	function get_param_definitions( $params )
	{
		$r = array_merge( array(
			'option_all' => array(
				'label' => T_('Option "All"'),
				'type' => 'text',
				'maxlength' => 100,
				'defaultvalue' => T_('All'),
				'note' => T_('The "All categories" link allows to reset the filter. Leave blank if you want no such option.'),
			),
			'use_form' => array(
				'label' => T_('Use form'),
				'type' => 'checkbox',
				'defaultvalue' => 0,
				'note' => T_('Add checkboxes to allow selection of multiple categories.'),
			),
			'disp_names_for_blog_list' => array(
				'label' => T_('Display blog names'),
				'type' => 'checkbox',
				'defaultvalue' => 1, /* previous behaviour */
				'note' => T_('Display blog names, if this is an aggregated blog.'),
			),
			// Hidden, used by the item list sidebar in the backoffice
			'display_checkboxes' => array(
				'label' => 'Internal: Display checkboxes', // This key is required
				'defaultvalue' => 0,
				'no_edit' => true,
			),
		), parent::get_param_definitions( $params ) );
		return $r;
	}


	/**
	 * Get widget's title
	 *
	 * @todo (3069) EdB: rename this to get_title()
	 */
	function get_short_desc()
	{
		return format_to_output( $this->disp_params['widget_title'] );
	}

// function(s) unique to this widget ------------------------------------------

	/**
	 * Support function for this widget: (callback) Generate code when exiting from a level
	 *
	 * @param int level of the category in the recursive tree
	 * @return string HTML
	 */
	function cat_after_level( $level )
	{
		$r = '';
		if( $level > 0 )
		{
			// If this is not the root
			$r .= $this->disp_params['group_end'];
			// End current (parent) line
			$r .= $this->disp_params['item_end'];
		}
		return $r;
	}


	/**
	 * Support function for this widget: (callback) Generate code when entering a new level
	 *
	 * @param int level of the category in the recursive tree
	 * @return string HTML
	 */
	function cat_before_level( $level )
	{
		$r = '';
		if( $level > 0 )
		{
			// If this is not the root
			$r .= $this->disp_params['group_start'];
		}
		return $r;
	}


	/**
	 * Support function for this widget: (callback) Generate category line when it has children
	 *
	 * @param Category generic category we want to display
	 * @param int level of the category in the recursive tree
	 * @return string HTML
	 */
	function cat_line( $Category, $level )
	{
		global $cat_array;

		if( ! isset( $cat_array ) )
		{
			$cat_array = array();
		}

		if( in_array( $Category->ID, $cat_array ) )
		{
			// This category is in the current selection
			$r = $this->disp_params['item_selected_start'];
		}
		else
		{
			$r = $this->disp_params['item_start'];
		}

		if( $this->disp_params['use_form'] || $this->disp_params['display_checkboxes'] )
		{
			// We want to add form fields
			$r .= '<label><input type="checkbox" name="catsel[]" value="'.$Category->ID.'" class="checkbox"';
			if( in_array( $Category->ID, $cat_array ) )
			{
				// This category is in the current selection
				$r .= ' checked="checked"';
			}
			$r .= ' /> ';
		}

		$r .= '<a href="';

		if( $this->disp_params['link_type'] == 'context' )
		{
			// We want to preserve current browsing context
			$r .= regenerate_url( 'cats,catsel', 'cat='.$Category->ID );
		}
		else
		{
			$r .= $Category->get_permanent_url();
		}

		$r .= '">'.$Category->dget( 'name' ).'</a>';

		if( $this->disp_params['use_form'] || $this->disp_params['display_checkboxes'] )
		{
			// We want to add form fields
			$r .= '</label>';
		}

		// Do not end line here because we need to include children first!
		// $r .= $this->disp_params['item_end'];

		return $r;
	}


	/**
	 * Support function for this widget: (callback) Generate category line when it has no children
	 *
	 * @param Category generic category we want to display
	 * @param int level of the category in the recursive tree
	 * @return string HTML
	 */
	function cat_no_children( $Category, $level )
	{
		// End current line
		return $this->disp_params['item_end'];
	}

}

?>
