<?php
/**
 * This file implements the "Comment List" widget
 *
 * To call this widget directly in your template (bypassing containers), use
 * the following code modified with any params you need to customize:
 * <code>
 * // display a widget without a container
 * template_widget( array(
 * 	'widget' => 'comment_list',
 * 	'widget_title' => 'Recent comments', // default shown, customize as desired
 * ) );
 * </code>
 *
 * This widget has 5 params that you can set when with a direct widget call:<ul>
 * <li>'disp_order' => Default is 'DESC' for "Newest to oldest". Options are 'ASC'
 * for "Oldest to newest", or 'RAND' for "Random order".</li>
 * <li>'limit' => Default is 20, this is the maximum number of items to display.</li>
 * <li>'author_links' => Default is true, which links the author of a comment to
 * their URL if they provided one.</li>
 * <li>'hover_text' => Default is 'Read the full comment', this is the tool tip shown
 * when someone hovers over the link, limit is 40 characters.</li>
 * <li>'blog_ID' => Default is empty/null, which gives the current blog. Use a blog
 * ID (number) for comments on a specific blog.</li></ul>
 *
 * @see ComponentWidget::init_display()
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package widgets
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * comment_list_Widget class
 *
 * @package widgets
 */
class comment_list_Widget extends ComponentWidget
{
	/**
	 * Constructor
	 */
	function comment_list_Widget( $db_row = NULL )
	{
		// Call parent constructor
		parent::ComponentWidget( $db_row, 'widget', 'comment_list' );
		$this->widget_name = T_('Comment List');
		$this->widget_title = T_('Recent comments');
	}


	/**
	 * Display the widget!
	 *
	 * @param array MUST contain at least the basic display params
	 */
	function display( $params )
	{
		global $Blog;

		$this->init_display( $params ); // just in case it hasn't been done before

		$blogCache = get_Cache( 'BlogCache' );
		$listBlog = ( $this->disp_params['blog_ID'] ? $blogCache->get_by_ID( $this->disp_params['blog_ID'] ) : $Blog );

		// Create ItemList ... Note: we pass a widget specific prefix in
		// order to make sure to never interfere with the mainlist
		$limit = $this->disp_params['limit'];
		$order = $this->disp_params['disp_order'];

		$CommentList = new CommentList( $listBlog, "'comment','trackback'", array( 'published' ), '', '', $order, '', $limit );

		echo $this->disp_params['block_start'];
		$this->disp_title();
		echo $this->disp_params['list_start'];

		// loop through comments
		while( $Comment = & $CommentList->get_next() )
		{
			// load comment's Item object
			$Comment->get_Item();
			echo $this->disp_params['item_start'];
			// display comment author's name
			$Comment->author( array(
				'after_visitor' => ' ',
				'after_user' => ' ',
				'make_link' => $this->disp_params['author_links'],
			) );
			echo T_('on ');
			// display a permalink link to the comment
			$Comment->permanent_link( array(
				'text' => $Comment->Item->title,
				'title' => $this->disp_params['hover_text'],
			) );
			echo $this->disp_params['item_end'];
		}
		echo $this->disp_params['list_end'];
		echo $this->disp_params['block_end'];

		return true;
	}


	/**
	 * Get widget's description
	 */
	function get_desc()
	{
		return T_('List of comments; click goes to comment.');
	}


	/**
	 * Get definitions for this widget's settings
	 *
	 * @see Plugin::GetDefaultSettings()
	 * @param local params like 'for_editing' => true
	 */
	function get_param_definitions( $params )
	{
		$r = array_merge( array(
			'disp_order' => array(
				'label' => T_('Order'),
				'type' => 'select',
				'options' => array(
					'DESC' => T_('Newest to oldest'),
					'ASC' => T_('Oldest to newest'),
					'RAND' => T_('Random selection'),
				),
				'defaultvalue' => 'DESC',
				'note' => T_('Order to display items'),
			),
			'limit' => array(
				'label' => T_('Max items'),
				'size' => 4,
				'defaultvalue' => 20,
				'note' => T_('Maximum number of items to display.'),
			),
			'author_links' => array(
				'label' => T_('Link to author'),
				'type' => 'checkbox',
				'defaultvalue' => true,
				'note' => T_('Link the author to their url'),
			),
			'hover_text' => array(
				'label' => T_('Hover text'),
				'type' => 'text',
				'size' => 40,
				'defaultvalue' => T_('Read the full comment'),
				'note' => T_('Text to show when hovering over the link'),
			),
			'blog_ID' => array(
				'label' => T_('Blog'),
				'size' => 4,
				'note' => T_('ID of the blog to use, leave empty for the current blog.'),
			),
		), parent::get_param_definitions( $params ) );

		return $r;
	}


	/**
	 * Get widget's title
	 *
	 * @todo (3069) EdB: rename this to get_title()
	 */
	function get_short_desc()
	{
		return format_to_output( $this->disp_params['widget_title'] );
	}

}

?>
