<?php
/**
 * This file implements the "Common Navigation Links" widget
 *
 * To call this widget directly in your template (bypassing containers), use
 * the following code modified with any params you need to customize:
 * <code>
 * // display a widget without a container
 * template_widget( array(
 * 	'widget' => 'common_links',
 * 	'widget_title' => '', // default shown, customize as desired
 * ) );
 * </code>
 *
 * This widget has 11 params that you can set when with a direct widget call:<ul>
 * <li>'show_home' => default is '1' which will show a link to the blog's home page</li>
 * <li>'show_recently' => default is '1' which will show a link to "Recently"</li>
 * <li>'show_search' => default is '0' change to 1 to show a link to "Search"</li>
 * <li>'show_postidx' => default is '0' change to 1 to show a link to "Post index"</li>
 * <li>'show_archives' => default is '1' which will show a link to "Archives"</li>
 * <li>'show_categories' => default is '1' which will show a link to "Categories"</li>
 * <li>'show_mediaidx' => default is '0' change to 1 to show a link to "Photo index"</li>
 * <li>'show_latestcomments' => default is '1' which will show a link to "Latest comments"</li>
 * <li>'show_owneruserinfo' => default is '0' change to 1 to show a link to "Owner details"</li>
 * <li>'show_ownercontact' => default is '0' change to 1 to show a link to "Contact"</li>
 * <li>'show_sitemap' => default is '0' change to 1 to show a link to "Site map"</li></ul>
 *
 * @todo (0000) dh> why are "STRONG" tags hardcoded here? can this get
 * dropped/removed? should the style get adjusted to use font-weight:bold then?
 * @todo (4334) oldcore: don't display "show_archives" if archives plugin not installed
 * or depluginize archives (I'm not sure)
 * @todo (4334) oldcore: don't display "show_categories" if categories plugin not installed
 * or depluginize categories (I'm not sure)
 *
 * @see ComponentWidget::init_display()
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package widgets
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * common_links_Widget class
 *
 * @package widgets
 */
class common_links_Widget extends ComponentWidget
{
	/**
	 * Constructor
	 */
	function common_links_Widget( $db_row = NULL )
	{
		// Call parent constructor
		parent::ComponentWidget( $db_row, 'widget', 'common_links' );
		$this->widget_name = T_('Common Navigation Links');
		$this->widget_title = '';
	}


	/**
	 * Display the widget!
	 *
	 * @param array MUST contain at least the basic display params
	 */
	function display( $params )
	{
		global $Blog;

		$this->init_display( $params ); // just in case it hasn't been done before

		echo $this->disp_params['block_start'];
		$this->disp_title();
		echo $this->disp_params['list_start'];

		if( $this->disp_params['show_recently'] )
		{
			echo $this->disp_params['item_start'];
			echo '<strong><a href="'.$Blog->get( 'url' ).'" title="'.T_('visit this blog\'s home page').'">'.T_('Recently').'</a></strong>';
			echo $this->disp_params['item_end'];
		}

		if( $this->disp_params['show_archives'] )
		{
			echo $this->disp_params['item_start'];
			echo '<strong><a href="'.$Blog->get( 'arcdirurl' ).'" title="'.T_('visit this blog\'s archives').'">'.T_('Archives').'</a></strong>';
			echo $this->disp_params['item_end'];
		}

		if( $this->disp_params['show_categories'] )
		{
			echo $this->disp_params['item_start'];
			echo '<strong><a href="'.$Blog->get( 'catdirurl' ).'" title="'.T_('visit this blog\'s categories').'">'.T_('Categories').'</a></strong>';
			echo $this->disp_params['item_end'];
		}

		if( $this->disp_params['show_latestcomments'] )
		{
			echo $this->disp_params['item_start'];
			echo '<strong><a href="'.$Blog->get( 'lastcommentsurl' ).'" title="'.T_('see this blog\'s latest comments').'">'.T_('Latest comments').'</a></strong>';
			echo $this->disp_params['item_end'];
		}

		echo $this->disp_params['list_end'];
		echo $this->disp_params['block_end'];

		return true;
	}


	/**
	 * Get widget's description
	 */
	function get_desc()
	{
		return T_('Display these links: Recently, Archives, Categories, Latest Comments');
	}


	/**
	 * Get definitions for this widget's settings
	 *
	 * @see Plugin::GetDefaultSettings()
	 * @param local params like 'for_editing' => true
	 */
	function get_param_definitions( $params )
	{
		$r = array_merge( array(
			'show_recently' => array(
				'label' => T_('Show "Recently"'),
				'type' => 'checkbox',
				'defaultvalue' => '1',
				'note' => T_('Go to the most recent posts / the blog\'s home.'),
			),
			'show_archives' => array(
				'label' => T_('Show "Archives"'),
				'type' => 'checkbox',
				'defaultvalue' => '1',
				'note' => T_('Go to the monthly/weekly/daily archive list.'),
			),
			'show_categories' => array(
				'label' => T_('Show "Categories"'),
				'type' => 'checkbox',
				'defaultvalue' => '1',
				'note' => T_('Go to the category tree.'),
			),
			'show_latestcomments' => array(
				'label' => T_('Show "Latest comments"'),
				'type' => 'checkbox',
				'defaultvalue' => '1',
				'note' => T_('Go to the latest comments.'),
			),
		), parent::get_param_definitions( $params ) );

		return $r;

	}


	/**
	 * Get widget's title
	 *
	 * @todo (3069) EdB: rename this to get_title()
	 */
	function get_short_desc()
	{
		return format_to_output( $this->disp_params['widget_title'] );
	}

}

?>
