<?php
/**
 * This file implements the "Free HTML" widget
 *
 * To call this widget directly in your template (bypassing containers), use
 * the following code modified with any params you need to customize:
 * <code>
 * // display a widget without a container
 * template_widget( array(
 * 	'widget' => 'free_html',
 * 	'widget_title' => '', // default shown, customize as desired
 * ) );
 * </code>
 *
 * This widget has 1 param that you can set when with a direct widget call:<ul>
 * <li>'content' => default is empty/null. This field accepts HTML markup, useful
 * for adding 3rd party code like javascripts or the like.</li></ul>
 *
 * @see ComponentWidget::init_display()
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package widgets
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * free_html_Widget class
 *
 * @package widgets
 */
class free_html_Widget extends ComponentWidget
{
	/**
	 * Constructor
	 */
	function free_html_Widget( $db_row = NULL )
	{
		// Call parent constructor
		parent::ComponentWidget( $db_row, 'widget', 'free_html' );
		$this->widget_name = T_('Free HTML');
		$this->widget_title = '';
	}


	/**
	 * Display the widget!
	 *
	 * @param array MUST contain at least the basic display params
	 */
	function display( $params )
	{
		global $Blog;

		$this->init_display( $params ); // just in case it hasn't been done before

		// Blog common links
		echo $this->disp_params['block_start'];
		$this->disp_title();
		echo format_to_output( $this->disp_params['content'] );
		echo $this->disp_params['block_end'];

		return true;
	}


	/**
	 * Get widget's description
	 */
	function get_desc()
	{
		return T_('Custom text/HTML of your choice.');
	}


	/**
	 * Get definitions for this widget's settings
	 *
	 * @see Plugin::GetDefaultSettings()
	 * @param local params like 'for_editing' => true
	 */
	function get_param_definitions( $params )
	{
		$r = array_merge( array(
			'content' => array(
				'type' => 'html_textarea',
				'label' => T_('Block content'),
				'rows' => 10,
			),
		), parent::get_param_definitions( $params ) );

		return $r;
	}


	/**
	 * Get widget's title
	 *
	 * @todo (3069) EdB: rename this to get_title()
	 */
	function get_short_desc()
	{
		if( empty( $this->disp_params['widget_title'] ) )
		{
			return strmaxlen( htmlspecialchars( $this->disp_params['content'] ), 60 );
		}
		return format_to_output( $this->disp_params['widget_title'] );
	}

}

?>
