<?php
/**
 * This file implements the "Image / Logo" widget
 *
 * To call this widget directly in your template (bypassing containers), use
 * the following code modified with any params you need to customize:
 * <code>
 * // display a widget without a container
 * template_widget( array(
 * 	'widget' => 'logo',
 * 	'widget_title' => '', // default shown, customize as desired
 * ) );
 * </code>
 *
 * This widget has 4 params that you can set when with a direct widget call:<ul>
 * <li>'logo_file' => default is 'logo.png', the image/logo file must be uploaded
 * to the root of the blog's media directory.</li>
 * <li>'link_to' => default is 'home' which links to the blog's home. Options are
 * 'main' (links to the installation's root), or, 'other' (links to a URL you provide).</li>
 * <li>'other_url' => default is 'http://', if you set link_to to other then this
 * is where you provide your URL to link the image to.</li>
 * <li>'other_title' => default is '', if you set link_to to other then this is where
 * you provide a mouse-over tool tip for the link.</li></ul>
 *
 * @see ComponentWidget::init_display()
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package widgets
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * logo_Widget class
 *
 * @package widgets
 */
class logo_Widget extends ComponentWidget
{
	/**
	 * Constructor
	 */
	function logo_Widget( $db_row = NULL )
	{
		// Call parent constructor
		parent::ComponentWidget( $db_row, 'widget', 'logo' );
		$this->widget_name = T_('Image / Logo');
		$this->widget_title = '';

		$this->menu_link_options = array(
			'home' => T_('Blog home'),
			'main' => T_('Installation home'),
			'other' => T_('URL from text field')
		);
	}


	/**
	 * Display the widget!
	 *
	 * @param array MUST contain at least the basic display params
	 */
	function display( $params )
	{
		global $app_baseurl;
		global $Blog;

		$this->init_display( $params ); // just in case it hasn't been done before

		switch( $this->disp_params['link_to'] )
		{
			case 'home':
			$alt_text = $Blog->dget( 'name', 'htmlattr' );
			$link_value = '<a href="'.$Blog->get( 'url', 'raw' ).'" title="'.$alt_text.'">';
			break;

			case 'main':
			$alt_text = T_('visit this site\'s home page');
			$link_value = '<a href="'.$app_baseurl.'" title="'.$alt_text.'">';
			break;

			case 'other':
			$alt_text = $this->disp_params['other_title'];
			$link_value = '<a href="'.$this->disp_params['other_url'].'" title="'.$alt_text.'">';
			break;
		}

		echo $this->disp_params['block_start'];

		$this->disp_title();
		echo $link_value.'<img src="'.$Blog->get_media_url().$this->disp_params['logo_file'].'" alt="'.$alt_text.'" /></a>';

		echo $this->disp_params['block_end'];

		return true;

	}


	/**
	 * Get widget's description
	 */
	function get_desc()
	{
		return T_('Include an image/logo from the blog\'s file root.');
	}


	/**
	 * Get definitions for this widget's settings
	 *
	 * @see Plugin::GetDefaultSettings()
	 * @param local params like 'for_editing' => true
	 */
	function get_param_definitions( $params )
	{
		$r = array_merge( array(
			'logo_file' => array(
				'label' => T_('Image filename'),
				'valid_pattern' => array(
					'pattern' => '^[a-z0-9_\-][a-z0-9_.\-]*$i',
					'error' => T_('Invalid filename.'),
				),
				'defaultvalue' => 'logo.png',
				'note' => T_('The image/logo file must be uploaded to the root of the Blog\'s media dir'),
			),
			'link_to' => array(
				'label' => T_('Link to'),
				'type' => 'select',
				'options' => $this->menu_link_options,
				'defaultvalue' => 'home',
				'note' => T_('What do you want to link to?'),
			),
			'other_url' => array(
				'label' => T_('Other URL'),
				'note' => T_('for when you select "URL from text field" above'),
				'defaultvalue' => 'http://',
			),
			'other_title' => array(
				'label' => T_('Other URL title'),
				'note' => T_('for when you select "URL from text field" above'),
				'defaultvalue' => '',
			),
		), parent::get_param_definitions( $params ) );

		return $r;

	}


	/**
	 * Get widget's title
	 *
	 * @todo (3069) EdB: rename this to get_title()
	 */
	function get_short_desc()
	{
		$this->load_param_array();
		if( ! empty( $this->param_array['logo_file'] ) )
		{
			return $this->param_array['logo_file'];
		}
		else
		{
			return $this->get_name();
		}
	}

}

?>
