<?php
/**
 * This file implements the "Photo Index" widget
 *
 * To call this widget directly in your template (bypassing containers), use
 * the following code modified with any params you need to customize:
 * <code>
 * // display a widget without a container
 * template_widget( array(
 * 	'widget' => 'media_index',
 * 	'widget_title' => 'Photo index', // default shown, customize as desired
 * ) );
 * </code>
 *
 * This widget has 13 params that you can set when with a direct widget call:<ul>
 * <li>'thumb_size' => default is 'crop-80x80', see {@link get_available_thumb_sizes()}
 * for available options.</li>
 * <li>'thumb_layout' => default is 'grid' which assumes a table. Optionally you can
 * set this to 'list' which assumes a list.</li>
 * <li>'limit' => default is 3, this is the maximum number of items to display.</li>
 * <li>'order_by' => default is 'datestart' which orders your items by date. See
 * {@link get_available_sort_options()} for available options.</li>
 * <li>'order_dir' => default is 'DESC' which means descending order. Optionally you
 * can set this to 'ASC for ascending order.</li>
 * <li>'blog_ID' => default is empty/null which will give you an item list from the
 * current blog. Use a blog ID (number) for items from a specific blog , or separate
 * multiple blogs by commas.</li>
 * <li>'grid_nb_cols' => default is 2 which is the number of columns in grid mode,
 * and is only used in grid mode.</li>
 * <li>'grid_start' => default is '<table cellspacing="1" class="widget_grid">', this
 * accepts html and is only used in grid mode.</li>
 * <li>'grid_end' => default is '&lt;/table>', this accepts html and is only used in
 * grid mode.</li>
 * <li>'grid_colstart' => default is '&lt;tr>', this accepts html and is only used in
 * grid mode.</li>
 * <li>'grid_colend' => default is '&lt;/tr>', this accepts html and is only used in
 * grid mode.</li>
 * <li>'grid_cellstart' => default is '&lt;td>', this accepts html and is only used in
 * grid mode.</li>
 * <li>'grid_cellend' => default is '&lt;/td>', this accepts html and is only used in
 * grid mode.</li></ul>
 *
 * When selecting list mode you will also want to set some "list" parameters that
 * are not specific to the widget, or accept your installation's defaults:<ul>
 * <li>'list_start' => default is '&lt;ul>'</li>
 * <li>'list_end' => default is '&lt;/ul>'</li>
 * <li>'item_start' => default is '&lt;li>'</li>
 * <li>'item_end' => default is '&lt;/li>'</li></ul>
 *
 * @see ComponentWidget::init_display()
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package widgets
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * media_index_Widget class
 *
 * @package widgets
 */
class media_index_Widget extends ComponentWidget
{
	/**
	 * Constructor
	 */
	function media_index_Widget( $db_row = NULL )
	{
		// Call parent constructor
		parent::ComponentWidget( $db_row, 'widget', 'media_index' );
		$this->widget_name = T_('Photo Index');
		$this->widget_title = T_('Photo index');
	}


	/**
	 * Display the widget!
	 *
	 * @todo (4334) oldcore: because no way of getting images only, at "$ItemQuery->LIMIT(",
	 * we get 4 times more data than requested and hope that 25% at least will be images :/



	 * @param array MUST contain at least the basic display params
	 */
	function display( $params )
	{
		global $Blog;
		global $localtimenow;

		$this->init_display( $params ); // just in case it hasn't been done before

		$list_blogs = ( $this->disp_params['blog_ID'] ? $this->disp_params['blog_ID'] : $Blog->ID );

		echo $this->disp_params['block_start'];

		// Display title if requested
		$this->disp_title();

		// Display photos:

		$FileCache = & get_Cache( 'FileCache' );

		$FileList = new DataObjectList2( $FileCache );

		// Query list of files
		$SQL = new SQL();
		$SQL->SELECT( 'post_ID, post_datestart, post_datemodified, post_main_cat_ID, 
			post_urltitle, post_ptyp_ID, post_title, post_excerpt, post_url, file_ID, 
			file_title, file_root_type, file_root_ID, file_path, file_alt, file_desc' );
		$SQL->FROM( 'T_categories INNER JOIN T_item_cats ON cat_ID = postcat_cat_ID
			INNER JOIN T_items ON postcat_post_ID = post_ID
			INNER JOIN T_links ON post_ID = link_itm_ID
			INNER JOIN T_files ON link_file_ID = file_ID' );
		if( false ) // was $this->disp_params['cat_id'] but dunno why and it failed :(
		{
			$SQL->WHERE( 'cat_ID IN ('.$this->disp_params['cat_id'].')' );
		}
		else
		{
			$SQL->WHERE( 'cat_blog_ID IN ('.$list_blogs.')' ); // fp> TODO: want to restrict on images :]
		}
		$SQL->WHERE_and( 'post_status = "published"' );
		$SQL->WHERE_and( 'post_datestart <= \''.date2mysql( $localtimenow ).'\'' );
		$SQL->GROUP_BY( 'link_ID' );
		$SQL->LIMIT( $this->disp_params['limit'] );
		$SQL->ORDER_BY( gen_order_clause( $this->disp_params['order_by'], $this->disp_params['order_dir'],
			'post_', 'post_ID '.$this->disp_params['order_dir'].', link_ID' ) );

		$FileList->sql = $SQL->get();

		$FileList->query( false, false, false );

		$layout = $this->disp_params['thumb_layout'];
		if( $layout == 'grid' )
		{
			$r = $this->disp_params['grid_start'];
		}
		else
		{
			$r = $this->disp_params['list_start'];
		}

		$nb_cols = $this->disp_params['grid_nb_cols'];
		$count = 0;

		while( $File = & $FileList->get_next() )
		{
			if( $count >= $this->disp_params['limit'] )
			{
				// We have enough images already!
				break;
			}

			if( ! $File->is_image() )
			{
				// Skip anything that is not an image, Only images are selected or those files
				// where we don't know the file type yet. The file type will be set during the check.
				continue;
			}

			if( $layout == 'grid' )
			{
				if( $count % $nb_cols == 0 )
				{
					$r .= $this->disp_params['grid_colstart'];
				}
				$r .= $this->disp_params['grid_cellstart'];
			}
			else
			{
				$r .= $this->disp_params['item_start'];
			}

			// 1/ Hack a dirty permalink( will redirect to canonical)
			// $link = url_add_param( $Blog->get( 'url' ), 'p='.$post_ID );

			// 2/ Hack a link to the right "page" ... Very daring!!
			// $link = url_add_param( $Blog->get( 'url' ), 'paged='.$count );

			// 3/ Instantiate a light object in order to get permamnent url
			$ItemLight = new ItemLight( $FileList->get_row_by_idx( $FileList->current_idx - 1 ) ); // index had already been incremented
			$r .= '<a href="'.$ItemLight->get_permanent_url().'">';

			// Generate the IMG THUMBNAIL tag with all the alt, title and desc if available
			$r .= $File->get_thumb_imgtag( $this->disp_params['thumb_size'] );
			$r .= '</a>';
			if( $this->disp_params['disp_image_title'] )
			{
				// Dislay title of image or item
				$title = ( $File->get( 'title' ) ) ? $File->get( 'title' ) : $ItemLight->title;
				if( ! empty( $title ) )
				{
					$r .= '<span class="note">'.$title.'</span>';
				}
			}

			++$count;

			if( $layout == 'grid' )
			{
				$r .= $this->disp_params['grid_cellend'];
				if( $count % $nb_cols == 0 )
				{
					$r .= $this->disp_params['grid_colend'];
				}
			}
			else
			{
				$r .= $this->disp_params['item_end'];
			}
		}

		// Exit if no files found
		if( empty( $r ) ) return;

		echo $this->disp_params['block_start'];
		$this->disp_title();
		echo $r;

		if( $layout == 'grid' )
		{
			if( $count && ( $count % $nb_cols != 0 ) )
			{
				echo $this->disp_params['grid_colend'];
			}

			echo $this->disp_params['grid_end'];
		}
		else
		{
			echo $this->disp_params['list_end'];
		}

		echo $this->disp_params['block_end'];

		return true;
	}


	/**
	 * Get widget's description
	 */
	function get_desc()
	{
		return T_('Index of photos; click goes to original image post.');
	}


	/**
	 * Get definitions for this widget's settings
	 *
	 * @see Plugin::GetDefaultSettings()
	 * @param local params like 'for_editing' => true
	 */
	function get_param_definitions( $params )
	{
		load_funcs( 'files/model/_image.funcs.php' );

		$r = array_merge( array(
			'thumb_size' => array(
				'label' => T_('Thumbnail size'),
				'type' => 'select',
				'options' => get_available_thumb_sizes(),
				'defaultvalue' => 'crop-80x80',
				'note' => T_('Cropping and sizing of thumbnails'),
			),
			'thumb_layout' => array(
				'label' => T_('Layout'),
				'type' => 'select',
				'options' => array(
					'grid' => T_('Grid'),
					'list' => T_('List'),
				),
				'defaultvalue' => 'grid',
				'note' => T_('How to lay out the thumbnails'),
			),
			'limit' => array(
				'label' => T_('Max items'),
				'size' => 4,
				'defaultvalue' => 3,
				'note' => T_('Maximum number of items to display.'),
			),
			'order_by' => array(
				'label' => T_('Order by'),
				'type' => 'select',
				'options' => get_available_sort_options(),
				'defaultvalue' => 'datestart',
				'note' => T_('How to sort the items'),
			),
			'order_dir' => array(
				'label' => T_('Direction'),
				'type' => 'select',
				'options' => array(
					'ASC' => T_('Ascending'),
					'DESC' => T_('Descending'),
				),
				'defaultvalue' => 'DESC',
				'note' => T_('How to sort the items'),
			),
			'blog_ID' => array(
				'label' => T_('Blogs'),
				'size' => 4,
				'note' => T_('IDs of the blogs to use, leave empty for the current blog. Separate multiple blogs by commas.'),
			),
			'grid_sep' => array(
				'separator' => true,
			),
			'grid_nb_cols' => array(
				'label' => T_('Columns'),
				'size' => 4,
				'defaultvalue' => 2,
				'note' => T_('Number of columns in grid mode.'),
			),
			'grid_start' => array(
				'label' => T_('Grid start'),
				'type' => 'html_input',
				'size' => 48,
				'defaultvalue' => '<table cellspacing="1" class="widget_grid">',
			),
			'grid_end' => array(
				'label' => T_('Grid end'),
				'type' => 'html_input',
				'defaultvalue' => '</table>',
			),
			'grid_colstart' => array(
				'label' => T_('Column start'),
				'type' => 'html_input',
				'defaultvalue' => '<tr>',
			),
			'grid_colend' => array(
				'label' => T_('Column end'),
				'type' => 'html_input',
				'defaultvalue' => '</tr>',
			),
			'grid_cellstart' => array(
				'label' => T_('Cell start'),
				'type' => 'html_input',
				'size' => 48,
				'defaultvalue' => '<td>',
			),
			'grid_cellend' => array(
				'label' => T_('Cell end'),
				'type' => 'html_input',
				'defaultvalue' => '</td>',
			),
		), parent::get_param_definitions( $params ) );

		return $r;
	}


	/**
	 * Get widget's title
	 *
	 * @todo (3069) EdB: rename this to get_title()
	 */
	function get_short_desc()
	{
		return format_to_output( $this->disp_params['widget_title'] );
	}

}

?>
