<?php
/**
 * This file implements the "Menu Link" widget
 *
 * To call this widget directly in your template (bypassing containers), use the
 * following code modified with any params you need to customize:
 * <code>
 * // display a widget without a container
 * template_widget( array(
 * 	'widget' => 'menu_link',
 * 	'widget_title' => '', // default shown, customize as desired
 * ) );
 * </code>
 *
 * This widget has 1 param that you can set when with a direct widget call:<ul>
 * <li>'link_type' => default is 'home', which links to the blog's home page. Options
 * are 'main' (links to the installation home), 'arcdir' (links to the archive
 * directory), 'catdir' (links to the category directory), 'latestcomments' (links
 * to the latest comments), 'ownercontact' (links to the blog owner contact form),
 * 'login' => (links to the log in form), or, 'register' (links to the registration
 * form).</li></ul>
 *
 * @see ComponentWidget::init_display()
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package widgets
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * menu_link_Widget class
 *
 * @package widgets
 */
class menu_link_Widget extends ComponentWidget
{
	/**
	 * Link types
	 * @var array Array of menu names -> displayed text
	 */
	var $menu_link_widget_link_types = array();

	/**
	 * Constructor
	 */
	function menu_link_Widget( $db_row = NULL )
	{
		// Call parent constructor
		parent::ComponentWidget( $db_row, 'widget', 'menu_link' );
		$this->widget_name = T_('Menu Link');
		$this->widget_title = '';

		$this->menu_link_widget_link_types = array(
			'arcdir' => T_('Archive directory'),
			'home' => T_('Blog home'),
			'ownercontact' => T_('Blog owner contact form'),
			'catdir' => T_('Category directory'),
			'main' => T_('Installation home'),
			'latestcomments' => T_('Latest comments'),
			'login' => T_('Log in form'),
			'register' => T_('Registration form'),
		);
	}


	/**
	 * Display the widget!
	 *
	 * @param array MUST contain at least the basic display params
	 */
	function display( $params )
	{
		global $app_baseurl;
		global $Blog;
		global $disp;

		$this->init_display( $params ); // just in case it hasn't been done before


		switch( $this->disp_params['link_type'] )
		{
			case 'arcdir':
			$url = $Blog->get( 'arcdirurl' );
			$text = T_('Archives');
			$title = T_('visit this blog\'s archives');
			break;

			case 'catdir':
			$url = $Blog->get( 'catdirurl' );
			$text = T_('Categories');
			$title = T_('visit this blog\'s categories');
			break;

			case 'home':
			default:
			$url = $Blog->get( 'url' );
			$text = T_('Home');
			$title = T_('visit this blog\'s home page');
			break;

			case 'latestcomments':
			$url = $Blog->get( 'lastcommentsurl' );
			$text = T_('Latest comments');
			$title = T_('see this blog\'s latest comments');
			break;

			case 'login':
			if( is_logged_in() ) return false;

			$url = get_login_url();
			$text = T_('Log in');
			$title = T_('login (if you have an account)');
			break;

			case 'main':
			$url = $app_baseurl;
			$text = T_('Main');
			$title = T_('visit this site\'s home page');
			break;

			case 'ownercontact':
			$Blog->get_owner_User();
			if( ! $Blog->owner_User->allow_msgform )
			{
				// user does not allow contact form
				return;
			}
			$url = $Blog->get_contact_url( true );
			$text = T_('Contact');
			$title = T_('send a message to the owner of this blog');
			break;

			case 'register':
			global $Settings;

			if( is_logged_in() )
			{
				return false;
			}
			if( ! $Settings->get( 'newusers_canregister' ) )
			{
				return false;
			}
			$url = get_register_url();
			$text = T_('Register');
			$title = T_('register on this website');
			break;
		}

		echo $this->disp_params['block_start'];
		$this->disp_title();
		echo $this->disp_params['list_start'];
		echo $this->disp_params['item_start'];
		echo '<a href="'.$url.'" title="'.$title.'">'.$text.'</a>';
		echo $this->disp_params['item_end'];
		echo $this->disp_params['list_end'];
		echo $this->disp_params['block_end'];

		return true;
	}


	/**
	 * Get widget's short description
	 */
	function get_desc()
	{
		return T_('Display a configurable menu entry/link');
	}


	/**
	 * Get definitions for this widget's settings
	 *
	 * @see Plugin::GetDefaultSettings()
	 * @param local params like 'for_editing' => true
	 */
	function get_param_definitions( $params )
	{
		$r = array_merge( array(
			'link_type' => array(
				'label' => T_('Link Type'),
				'type' => 'select',
				'options' => $this->menu_link_widget_link_types,
				'defaultvalue' => 'home',
				'note' => T_('What do you want to link to?'),
			),
		), parent::get_param_definitions( $params ) );
		return $r;
	}


	/**
	 * Get widget's title
	 *
	 * @todo (3069) EdB: rename this to get_title()
	 */
	function get_short_desc()
	{
		$this->load_param_array();

		if( ! empty( $this->param_array['link_type'] ) )
		{
			// TRANS: %s is the link type, e. g. "Blog home" or "Log in form"
			return sprintf( T_('%s link'), $this->menu_link_widget_link_types[$this->param_array['link_type']] );
		}
		else
		{
			return $this->get_name();
		}
	}

}

?>
