<?php
/**
 * This file implements the "Page List" widget
 *
 * To call this widget directly in your template (bypassing containers), use the
 * following code modified with any params you need to customize:
 * <code>
 * // display a widget without a container
 * template_widget( array(
 * 	'widget' => 'page_list',
 * 	'widget_title' => 'Pages', // default shown, customize as desired
 * ) );
 * </code>
 *
 * This widget is an extension of the "Universal Item List" widget. Some of it's
 * params are set for you, 6 params are available via this widget call:<ul>
 * <li>'thumb_size' => default is 'crop-80x80', see {@link $thumbnail_sizes}</li>
 * <li>'item_group_by' => default is 'none' which means no grouping. Optionally you
 * can set this to 'category' for grouping by category.</li>
 * <li>'order_by' => default is 'datestart' which orders your items by date. See
 * {@link get_available_sort_options()} for available options.</li>
 * <li>'order_dir' => default is 'DESC' which means descending order. Optionally you
 * can set this to 'ASC for ascending order.</li>
 * <li>'limit' => default is 20, which means no more than 20 items will be shown.</li></ul>
 * <li>'disp_title' => default is true, which will give you titles displayed ... which
 * is pretty stupid because what else would you display in a freaking item list!</li>
 * <li>'attached_pics' => default is 'none' for "None", options are 'first' for
 * "Display first", and 'all' for "Display all".</li>
 * <li>'item_pic_link_type' default is 'auto' for "Automatic", options are 'permalink'
 * for "Item permalink", 'linkto_url' for "Item URL", and 'none'for "Nowhere".</li>
 *
 * Params that are not editable are as follows:<ul>
 * <li>'title_link' => false, the block title is not linked to the blog.</li>
 * <li>'item_type' => '1000', "Pages"</li>
 * <li>'follow_mainlist' => 'none', you will not restrict the list to contents
 * related to what is displayed in the main area.</li>
 * <li>'blog_ID' => empty/null, the item list is from the current blog.</li>
 * <li>'item_title_link_type' => 'auto', "link to URL if there is one, else permalink".</li>
 * <li>'disp_excerpt' => false, do not show an item's excerpt.</li>
 * <li>'disp_teaser' => false, do not show an item's teaser.</li>
 * <li>'disp_teaser_maxwords' => 20, but it doesn't matter because teasers aren't shown.</li>
 * <li>'widget_css_class' => default is I dunno what so this has to change :(</li>
 * <li>'widget_ID' => default is I dunno what so this has to change :(</li></ul>
 *
 * @see ComponentWidget::init_display()
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package widgets
 */
if(!defined('QP_MAIN_INIT')) die('fail');

$wiCache = get_Cache( 'WidgetCache' );
$wiCache->load_widget( 'item_list' );

/**
 * page_list_Widget class
 *
 * @package widgets
 */
class page_list_Widget extends item_list_Widget
{
	/**
	 * Constructor
	 */
	function page_list_Widget( $db_row = NULL )
	{
		// Call parent constructor
		parent::ComponentWidget( $db_row, 'widget', 'page_list' );
		$this->widget_name = T_('Page List');
		$this->widget_title = T_('Pages');
	}


	/**
	 * Display the widget!
	 *
	 * @param array must contain at least the basic display params
	 */
	function display( $params )
	{
		// Force some params (because this is a simplified widget)
		$params['item_type'] = '1000'; // Use item types 1000 (pages) only

		parent::display( $params );
		return true;
	}


	/**
	 * Get widget's description
	 */
	function get_desc()
	{
		return T_('Simplified Item list for listing pages.');
	}


	/**
	 * Get definitions for this widget's settings
	 *
	 * @see Plugin::GetDefaultSettings()
	 * @param local params like 'for_editing' => true
	 */
	function get_param_definitions( $params )
	{
		// This is derived from item_list_Widget, we do not add any params
		$r = parent::get_param_definitions( $params );
		// We only change the defaults and hide some params
		$r['title_link']['no_edit'] = true;
		$r['item_type']['no_edit'] = true;
		$r['follow_mainlist']['no_edit'] = true;
		$r['blog_ID']['no_edit'] = true;
		$r['item_title_link_type']['no_edit'] = true;
		$r['disp_excerpt']['no_edit'] = true;
		$r['disp_teaser']['no_edit'] = true;
		$r['disp_teaser_maxwords']['no_edit'] = true;
		$r['widget_css_class']['no_edit'] = true;
		$r['widget_ID']['no_edit'] = true;

		return $r;
	}


	/**
	 * Get widget's title
	 *
	 * @todo (3069) EdB: rename this to get_title()
	 */
	function get_short_desc()
	{
		return format_to_output( $this->disp_params['widget_title'] );
	}

}

?>
