<?php
/**
 * This file implements the "Search Form" widget
 *
 * To call this widget directly in your template (bypassing containers), use the
 * following code modified with any params you need to customize:
 * <code>
 * // display a widget without a container
 * template_widget( array(
 * 	'widget' => 'search_form',
 * 	'widget_title' => 'Search', // default shown, customize as desired
 * ) );
 * </code>
 *
 * This widget has 1 param that you can set when with a direct widget call:<ul>
 * <li>'disp_search_options' => default is true, displays radio buttons for "All Words",
 * "Some Word" and "Entire Phrase"</li></ul>
 *
 * @see ComponentWidget::init_display()
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package widgets
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * search_form_Widget class
 *
 * @package widgets
 */
class search_form_Widget extends ComponentWidget
{
	/**
	 * Constructor
	 */
	function search_form_Widget( $db_row = NULL )
	{
		// Call parent constructor
		parent::ComponentWidget( $db_row, 'widget', 'search_form' );
		$this->widget_name = T_('Search Form');
		$this->widget_title = T_('Search');
	}


	/**
	 * Display the widget!
	 *
	 * @param array MUST contain at least the basic display params
	 */
	function display( $params )
	{
		global $Blog;

		$this->init_display( $params ); // just in case it hasn't been done before

		echo $this->disp_params['block_start'];
		$this->disp_title();
		form_formstart( $Blog->gen_blogurl(), 'search', 'SearchForm' );
		echo '<div><p>';
		$s = get_param( 's' );
		echo '<input type="text" name="s" size="25" value="'.htmlspecialchars( $s ).'" class="SearchField" />';
		echo '</p>';
		if( $this->disp_params['disp_search_options'] )
		{
			$sentence = get_param( 'sentence' );
			echo '<div class="search_options">';
			echo '<div class="search_option"><input type="radio" name="sentence" value="AND" id="sentAND" '.( $sentence=='AND' ? 'checked="checked" ' : '' ).'/><label for="sentAND">'.T_('All words').'</label></div>';
			echo '<div class="search_option"><input type="radio" name="sentence" value="OR" id="sentOR" '.( $sentence=='OR' ? 'checked="checked" ' : '' ).'/><label for="sentOR">'.T_('Some word').'</label></div>';
			echo '<div class="search_option"><input type="radio" name="sentence" value="sentence" id="sentence" '.( $sentence=='sentence' ? 'checked="checked" ' : '' ).'/><label for="sentence">'.T_('Entire phrase').'</label></div>';
			echo '</div>';
		}
		echo '<input type="submit" name="submit" class="submit" value="'.T_('Search').'" />';
		echo '</div></form>';
		echo $this->disp_params['block_end'];

		return true;
	}


	/**
	 * Get widget's description
	 */
	function get_desc()
	{
		return T_('Display search form');
	}


	/**
	 * Get definitions for this widget's settings
	 *
	 * @see Plugin::GetDefaultSettings()
	 * @param local params like 'for_editing' => true
	 */
	function get_param_definitions( $params )
	{
		$r = array_merge( array(
			'disp_search_options' => array(
				'label' => T_('Search options'),
				'type' => 'checkbox',
				'defaultvalue' => true,
				'note' => T_('Display radio buttons for "All Words", "Some Word" and "Entire Phrase"'),
			),
		), parent::get_param_definitions( $params ) );

		return $r;
	}


	/**
	 * Get widget's title
	 *
	 * @todo (3069) EdB: rename this to get_title()
	 */
	function get_short_desc()
	{
		return format_to_output( $this->disp_params['widget_title'] );
	}

}

?>
