<?php
/**
 * This file implements the "Single Item / Post" widget
 *
 * To call this widget directly in your template (bypassing containers), use the
 * following code modified with any params you need to customize:
 * <code>
 * // display the Single Item / Post widget
 * template_widget( array(
 * 	'widget' => 'single_item',
 * 	'widget_title' => '', // default shown, customize as desired
 * ) );
 * </code>
 *
 * This widget has 6 params that you can set when with a direct widget call:<ul>
 * <li>'item_ID' => default is empty/null, this is the ID of the post/item you want
 * to link to.</li>
 * <li>'item_markup' => default is 'h3', optionally you can set this to 'p', 'h1',
 * 'h2', 'h4', 'h5', 'h6', or, 'none'.</li>
 * <li>'link_text' => default is '' which will give you the item's title. Change as
 * you see fit.</li>
 * <li>'link_title' => default is 'this is a link to an important article', this is
 * the mouse-over tool tip presented to visitors.</li>
 * <li>'show_excerpt' => default is 0, set to 1 if you want an excerpt displayed.</li>
 * <li>'excerpt_markup' => default is 'default' which I dunno what it gives you. You
 * can select 'p', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', or, 'none'.</li></ul>
 *
 * @see ComponentWidget::init_display()
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package widgets
 */
if( !defined('QP_MAIN_INIT') ) die( 'Please, do not access this page directly.' );

/**
 * single_item_Widget class
 *
 * @package widgets
 */
class single_item_Widget extends ComponentWidget
{
	/**
	 * Constructor
	 */
	function single_item_Widget( $db_row = NULL )
	{
		// Call parent constructor
		parent::ComponentWidget( $db_row, 'widget', 'single_item' );
		$this->widget_name = T_('Single Item / Post');
		$this->widget_title = '';
	}


	/**
	 * Display the widget!
	 *
	 * @param array MUST contain at least the basic display params
	 */
	function display( $params )
	{
		$this->init_display( $params ); // just in case it hasn't been done before

		if( $this->disp_params['item_ID'] == '' )
		{
			return;
		}

		$ItemCache = & get_Cache( 'ItemCache' );
		$Item = & $ItemCache->get_by_ID( $this->disp_params['item_ID'], false );

		// are we using a different text?
		$text = $Item->title;
		if( $this->disp_params['link_text'] != '' )
		{
			$text = $this->disp_params['link_text'];
		}
		// create a link even though it ain't done
		$linked_text = '<a href="'.$Item->get_permanent_url().'" title="'.$this->disp_params['link_title'].'">'.$text.'</a>';
		$pre_text = '';
		$post_text = '';
		if( $this->disp_params['item_markup'] != 'none' )
		{
			$pre_text = '<'.$this->disp_params['item_markup'].'>';
			$post_text = '</'.$this->disp_params['item_markup'].'>';
		}

		// start the output process...
		echo $this->disp_params['block_start'];
		// display a title if we have one
		$this->disp_title();
		// display the linked item
		echo $pre_text.$linked_text.$post_text;

		// now for the excerpt, if desired
		if( $this->disp_params['show_excerpt'] )
		{
			$pre_excerpt = '';
			$post_excerpt = '';
			if( $this->disp_params['excerpt_markup'] != 'none' )
			{
				$pre_excerpt = '<'.$this->disp_params['excerpt_markup'].'>';
				$post_excerpt = '</'.$this->disp_params['excerpt_markup'].'>';
			}
			echo $pre_excerpt.$Item->excerpt.$post_excerpt;
		}

		// close the block
		echo $this->disp_params['block_end'];
	}


	/**
	 * Get widget's description
	 */
	function get_desc()
	{
		return T_('Display a link to a single item');
	}


	/**
	 * Get definitions for this widget's settings
	 *
	 * @see Plugin::GetDefaultSettings()
	 * @param local params like 'for_editing' => true
	 */
	function get_param_definitions( $params )
	{
		$r = array_merge( array(
			'item_ID' => array(
				'label' => T_('Item'),
				'size' => 4,
				'note' => T_('ID of the post/item to link'),
			),
			'item_markup' => array(
				'label' => T_('Item markup'),
				'type' => 'select',
				'options' => array(
					'p' => 'P',
					'h1' => 'H1',
					'h2' => 'H2',
					'h3' => 'H3',
					'h4' => 'H4',
					'h5' => 'H5',
					'h6' => 'H6',
					'none' => T_('none'),
				),
				'defaultvalue' => 'h3',
				'note' => T_('HTML markup for the item title'),
			),
			'link_text' => array(
				'label' => T_('Link text'),
				'defaultvalue' => '',
				'note' => T_('leave blank for item\'s title'),
			),
			'link_title' => array(
				'label' => T_('Link title'),
				'defaultvalue' => T_('this is a link to an important article'),
			),
			'show_excerpt' => array(
				'type' => 'checkbox',
				'label' => T_('Show excerpt'),
				'defaultvalue' => 0,
				'note' => T_('check to display an excerpt of the item'),
			),
			'excerpt_markup' => array(
				'label' => T_('Excerpt markup'),
				'type' => 'select',
				'options' => array(
					'p' => 'P',
					'h1' => 'H1',
					'h2' => 'H2',
					'h3' => 'H3',
					'h4' => 'H4',
					'h5' => 'H5',
					'h6' => 'H6',
					'none' => T_('none'),
				),
				'defaultvalue' => 'default',
				'note' => T_('HTML markup for the item excerpt'),
			),
		), parent::get_param_definitions( $params ) );

		return $r;
	}


	/**
	 * Get widget's title
	 *
	 * @todo (3069) EdB: rename this to get_title()
	 */
	function get_short_desc()
	{
		return format_to_output( $this->disp_params['widget_title'] );
	}

}

?>
