<?php
/**
 * This file implements the "User Tools" widget
 *
 * To call this widget directly in your template (bypassing containers), use the
 * following code modified with any params you need to customize:
 * <code>
 * // display a widget without a container
 * template_widget( array(
 * 	'widget' => 'user_tools',
 * 	'widget_title' => 'User tools', // default shown, customize as desired
 * ) );
 * </code>
 *
 * This widget has 6 params that you can set when with a direct widget call:<ul>
 * <li>'user_login_link' => default is 'Login', this is the link text to display.</li>
 * <li>'user_logout_link' => default is 'Logout', this is the link text to display.</li>
 * <li>'user_profile_link' => default is 'Profile', this is the link text to display.</li>
 * <li>'user_subs_link' => default is 'Subscriptions', this is the link text to display.</li>
 * <li>'user_admin_link' => default is 'Admin', this is the link text to display.</li>
 * <li>'user_register_link' => default is 'Register', this is the link text to display.</li></ul>
 *
 * @see ComponentWidget::init_display()
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package widgets
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * user_tools_Widget class
 *
 * @package widgets
 */
class user_tools_Widget extends ComponentWidget
{
	/**
	 * Constructor
	 */
	function user_tools_Widget( $db_row = NULL )
	{
		// Call parent constructor
		parent::ComponentWidget( $db_row, 'widget', 'user_tools' );
		$this->widget_name = T_('User Tools');
		$this->widget_title = T_('User tools');
	}


	/**
	 * Display the widget!
	 *
	 * @param array MUST contain at least the basic display params
	 */
	function display( $params )
	{
		$this->init_display( $params ); // just in case it hasn't been done before

		echo $this->disp_params['block_start'];
		$this->disp_title();
		echo $this->disp_params['list_start'];
		user_login_link( $this->disp_params['item_start'], $this->disp_params['item_end'], $this->disp_params['user_login_link'] );
		user_register_link( $this->disp_params['item_start'], $this->disp_params['item_end'], $this->disp_params['user_register_link'] );
		user_admin_link( $this->disp_params['item_start'], $this->disp_params['item_end'], $this->disp_params['user_admin_link'] );
		user_profile_link( $this->disp_params['item_start'], $this->disp_params['item_end'], $this->disp_params['user_profile_link'] );
		user_subs_link( $this->disp_params['item_start'], $this->disp_params['item_end'], $this->disp_params['user_subs_link'] );
		user_logout_link( $this->disp_params['item_start'], $this->disp_params['item_end'], $this->disp_params['user_logout_link'] );
		echo $this->disp_params['list_end'];
		echo $this->disp_params['block_end'];
	}


	/**
	 * Get widget's description
	 */
	function get_desc()
	{
		return T_('Display user tools: Log in, Admin, Profile, Subscriptions, Log out');
	}


	/**
	 * Get definitions for this widget's settings
	 *
	 * @see Plugin::GetDefaultSettings()
	 * @param local params like 'for_editing' => true
	 */
	function get_param_definitions( $params )
	{
		$r = array_merge( array(
			'user_login_link' => array(
				'label' => T_('Login link'),
				'type' => 'text',
				'size' => 40,
				'defaultvalue' => T_('Login'),
				'note' => T_('Link text to display'),
			),
			'user_logout_link' => array(
				'label' => T_('Logout link'),
				'type' => 'text',
				'size' => 40,
				'defaultvalue' => T_('Logout'),
				'note' => T_('Link text to display'),
			),
			'user_profile_link' => array(
				'label' => T_('Profile link'),
				'type' => 'text',
				'size' => 40,
				'defaultvalue' => T_('Profile'),

				'note' => T_('Link text to display'),
			),
			'user_subs_link' => array(
				'label' => T_('Subscriptions link'),
				'type' => 'text',
				'size' => 40,
				'defaultvalue' => T_('Subscriptions'),
				'note' => T_('Link text to display'),
			),
			'user_admin_link' => array(
				'label' => T_('Admin link'),
				'type' => 'text',
				'size' => 40,
				'defaultvalue' => T_('Admin'),
				'note' => T_('Link text to display'),
			),
			'user_register_link' => array(
				'label' => T_('Register link'),
				'type' => 'text',
				'size' => 40,
				'defaultvalue' => T_('Register'),
				'note' => T_('Link text to display'),
			),
		), parent::get_param_definitions( $params ) );
		return $r;
	}


	/**
	 * Get widget's title
	 *
	 * @todo (3069) EdB: rename this to get_title()
	 */
	function get_short_desc()
	{
		return format_to_output( $this->disp_params['widget_title'] );
	}

}

?>
