<?php
/**
 * This file implements the "XML Feeds (RSS / Atom)" widget
 *
 * To call this widget directly in your template (bypassing containers), use the
 * following code modified with any params you need to customize:
 * <code>
 * // display a widget without a container
 * template_widget( array(
 * 	'widget' => 'xml_feeds',
 * 	'widget_title' => '$icon$'.'XML Feeds', // default shown, customize as desired
 * ) );
 * </code>
 *
 * This widget has 3 params that you can set when with a direct widget call:<ul>
 * <li>'disp_info_link' => default is 0, set to 1 to display "What is RSS?" link.</li>
 * <li>'info_link_text' => default is 'What is RSS?', this is the text displayed if
 * you have a link displayed.</li>
 * <li>'info_link' => default is 0, set to 1 to add target="_blank" to the "What is
 * RSS?" link if you have a link displayed.</li></ul>
 *
 * @see ComponentWidget::init_display()
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package widgets
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * xml_feeds_Widget class
 *
 * @package widgets
 */
class xml_feeds_Widget extends ComponentWidget
{
	/**
	 * Constructor
	 */
	function xml_feeds_Widget( $db_row = NULL )
	{
		// Call parent constructor
		parent::ComponentWidget( $db_row, 'widget', 'xml_feeds' );
		$this->widget_name = T_('XML Feeds (RSS / Atom)');
		$this->widget_title = '$icon$ '.T_('XML Feeds');
	}


	/**
	 * Display the widget!
	 *
	 * @param array MUST contain at least the basic display params
	 */
	function display( $params )
	{
		global $Blog;

		$this->init_display( $params ); // just in case it hasn't been done before

		echo $this->disp_params['block_start'];
		$this->disp_title( $this->get_title() );
		echo $this->disp_params['list_start'];

		$TemplateCache = & get_Cache( 'TemplateCache' );
		$TemplateCache->load_by_type( 'feed' );

		foreach( $TemplateCache->cache as $Template )
		{
			if( $Template->type != 'feed' )
			{
				// This template cannot be used here...
				continue;
			}

			echo $this->disp_params['item_start'];
			echo $Template->name.': ';
			echo '<a href="'.$Blog->get_item_feed_url( $Template->folder ).'" title="'.T_('subscribe to this blogs posts').'">'.T_('Posts').'</a>';
			if( $Blog->allowcomments != 'never' )
			{
				echo ', <a href="'.$Blog->get_comment_feed_url( $Template->folder ).'" title="'.T_('subscribe to this blogs comments').'">'.T_('Comments').'</a>';
			}

			echo $this->disp_params['item_end'];
		}

		echo $this->disp_params['list_end'];

		// Display "info" link, if activated
		if( $this->disp_params['disp_info_link'] )
		{
			$feedhelp = '<a href="http://wikipedia.org/wiki/RSS" title="from Wikipedia, the free encyclopedia"';
			if( $this->disp_params['info_link'] )
			{
				$feedhelp .= ' target="_blank"';
			}
			$feedhelp .= '>'.$this->disp_params['info_link_text'].'</a>';
			echo $feedhelp;
		}
		echo $this->disp_params['block_end'];

		return true;
	}


	/**
	 * Get widget's short description
	 */
	function get_desc()
	{
		return T_('List of all available XML feeds.');
	}


	/**
	 * Get widget's title
	 *
	 * @todo (3069) EdB> rename this to get_title()
	 */
	function get_short_desc()
	{
		return format_to_output( $this->get_title() );
	}


	/**
	 * Get definitions for editable params
	 *
	 * @see Plugin::GetDefaultSettings()
	 * @param local params like 'for_editing' => true
	 */
	function get_param_definitions( $params )
	{
		$r = array_merge( array(
			'disp_info_link' => array(
				'label' => T_('Help link'),
				'type' => 'checkbox',
				'defaultvalue' => 0,
				'note' => T_('Check this to display "What is RSS?" link'),
			),
			'info_link_text' => array(
				'label' => T_('Help text'),
				'type' => 'text',
				'defaultvalue' => T_('What is RSS?'),
				'size' => 40,
			),
			'info_link' => array(
				'label' => T_('New Window'),
				'type' => 'checkbox',
				'defaultvalue' => 0,
				'note' => T_('Check this to add target="_blank" to the "What is RSS?" link'),
			),
		), parent::get_param_definitions( $params ) );
		return $r;
	}


	/**
	 * support function for display()
	 */
	function get_title()
	{
		global $rsc_url;

		$title = str_replace( '$icon$', get_icon( 'feed' ).' ', $this->disp_params['widget_title'] );
		return $title;
	}

}

?>
