<?php
/**
 * This file implements the WidgetCache class
 *
 * @todo (1111) vars type 'foo' need useful/valid descriptions
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package widgets
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * WidgetCache class
 *
 * @package widgets
 */
class WidgetCache extends DataObjectCache
{
	/**
	 * Cache by container
	 * @var array of widget_blog_ID => array of container_name => array of Widget
	 */
	var $cache_container_Widget_array = array();
	/**
	 * Indicates whether to load enabled widgets only
	 * @var boolean
	 */
	var $load_enabled_only;

	/**
	 * Constructor
	 *
	 * @param boolean Load enabled widgets only?
	 */
	function WidgetCache( $enabled_only = false )
	{
		parent::DataObjectCache( 'ComponentWidget', false, 'T_widgets', 'wi_', 'wi_ID', NULL, NULL, NULL );
		$this->load_enabled_only = $enabled_only;
	}


	/**
	 * Get Widget by Blog ID
	 *
	 * @param integer blog ID
	 * @return array of widget_blog_ID => array of container_name => array of Widget
	 */
	function & get_by_blogID( $widget_blog_ID )
	{
		global $DB;

		if( ! isset( $this->cache_container_Widget_array[$widget_blog_ID] ) )
		{
			// Not in Cache yet
			$sql = 'SELECT * FROM T_widgets WHERE wi_widget_blog_ID = '.$widget_blog_ID;
			if( $this->load_enabled_only )
			{
				// We want to load enabled widgets only
				$sql .= ' AND wi_enabled = 1';
			}
			$sql .= ' ORDER BY wi_sco_name, wi_order';

			$widget_rs = $DB->get_results( $sql, OBJECT, 'Get list of widgets for blog' );

			$this->cache_container_Widget_array[$widget_blog_ID] = array();
			$count = count( $widget_rs );
			for( $i = 0; $i < $count; $i++ )
			{
				// fp> NOTE: object COPYing is weird here but it needs to be like
				// this in PHP4 or all objects from the loop will look the same
				if( $ComponentWidget = & $this->new_obj( $widget_rs[$i] ) )
				{
					// We were able to instantiate the widget, add to regular cache
					// (but not with $this->add() because we need a COPY!!)
					$this->cache[$ComponentWidget->ID] = $ComponentWidget; // COPY!!!! WEIRD BUT NECESSARY / PHP 4 (fp)
					// This is the cache we're interested in
					$this->cache_container_Widget_array[$ComponentWidget->widget_blog_ID][$ComponentWidget->sco_name][] = & $this->cache[$ComponentWidget->ID];
				}
				unset( $ComponentWidget );
			}
		}
		return $this->cache_container_Widget_array[$widget_blog_ID];
	}


	/**
	 * Instanciate a new object within this cache
	 *
	 * @todo (0000) [TODO_HERE] replace with dummy widget in order to give a chance to clean up
	 */
	function & new_obj( $row = NULL )
	{
		if( $row->wi_type == 'widget' )
		{
			if( ! $this->load_widget( $row->wi_code ) )
			{
				// [TODO_HERE]
				$r = NULL;
				return $r;
			}
			$objtype = $row->wi_code.'_Widget';
		}
		else
		{
			$objtype = 'ComponentWidget';
		}

		// Instantiate a custom object
		$obj = new $objtype( $row ); // COPY !!

		return $obj;
	}


	/**
	 * Loads appropriate widget class file
	 *
	 * @param string $wi_code widget to load
	 * @return boolean Whether the class file could be loaded. Note this method will
	 * return true if the file has been included before (there is no way to find out
	 * whether require_once() really included a file)
	 */
	function load_widget( $wi_code )
	{
		global $inc_path;

		// file_exists() should not have a big impact on IO performance since PHP caches the results
		if( file_exists( $inc_path.'widgets/_'.$wi_code.'.widget.php' ) )
		{
			// widget exists ... require_once makes sure each widget class is only loaded once
			require_once $inc_path.'widgets/_'.$wi_code.'.widget.php';
			return true;
		}
		return false;
	}


	/**
	 * Function without a Title
	 *
	 * @param integer blog ID
	 * @param string Container name
	 * @return array of Widget
	 */
	function & get_by_blog_container( $widget_blog_ID, $container )
	{
		// Make sure blog is loaded
		$this->get_by_blogID( $widget_blog_ID );
		return $this->cache_container_Widget_array[$widget_blog_ID][$container];
	}

}

?>
