<?php
/**
 * This file implements the UI view for the Available widgets.
 *
 * @todo (1111) Need to add plugin-generated widgets with the core widget list, based
 * on some sort of grouping, then alphabetize the list. Any plugin-generated widget
 * that doesn't land in a group gets left in a pile and alphabetized in that pile.
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package widgets
 */
if(!defined('QP_MAIN_INIT')) die('fail');

global $container;
global $Plugins;

echo '<h2><span class="right_icons">'.action_icon( T_('Cancel!'), 'close', regenerate_url( 'container' ) ).'</span>'.sprintf(T_('Widgets available for insertion into &laquo;%s&raquo;'), $container ).'</h2>';

$core_componentwidget_defs = array(
	'*'.T_('Blog Metadata'),
		'title',
		'tagline',
		'longdesc',
	'*'.T_('General Purpose'),
		'free_html',
		'logo',
		'single_item',
	'*'.T_('Item Lists'),
		'item_list', // Universal Item List
		'link_list', // Simple List
		'linkblog', // Simple List
		'post_list', // Simple List
		'page_list', // Simple List
		'related_post_list', // Simple List
		'comment_list',
		'media_index',
		'xml_feeds',
	'*'.T_('Navigation'),
		'bloglist',
		'category_list',
		'common_links',
		'menu_link',
		'search_form',
		'tag_cloud',
	'*'.T_('Other / Miscellaneous'),
		'user_tools',
);

$i = 0;
$wiCache = get_Cache( 'WidgetCache' );
foreach( $core_componentwidget_defs as $code )
{
	$i++;
	if( $code[0] == '*' )
	{
		// widget group
		if( $i > 1 )
		{
			echo '</ul>';
		}
		echo '<h3>'.substr( $code, 1 ).':</h3><ul class="widget_list">';
	}
	else
	{
		$wiCache->load_widget( $code );
		$classname = $code.'_Widget';
		$ComponentWidget = new $classname( NULL, 'widget', $code );

		echo '<li>';
		echo '<a href="'.regenerate_url( '', 'action=create&amp;type=widget&amp;code='.$ComponentWidget->code ).'" title="'.T_('Add this widget to the container').'">';
		echo get_icon( 'new' ).'<strong>'.$ComponentWidget->get_name().'</strong>';
		echo '</a> <span class="notes">'.$ComponentWidget->get_desc().'</span>';
		echo '</li>';
	}
}

// now we get the Plugins that implement a templatetag ...
$Plugin_array = $Plugins->get_list_by_event( 'TemplateTag' );
// remove the plugins with no code ... because this gets used to install them
foreach( $Plugin_array as $k => $v )
{
	if( empty( $v->code ) )
	{
		unset( $Plugin_array[$k] );
	}
}

// if we have plugins make a UL of them
if( ! empty( $Plugin_array ) )
{
	echo '</ul>';
	echo '<h3>'.T_('Plugins').':</h3><ul class="widget_list">';
	foreach( $Plugin_array as $ID => $Plugin )
	{
		echo '<li>';
		echo '<a href="'.regenerate_url( '', 'action=create&amp;type=plugin&amp;code='.$Plugin->code ).'" title="'.T_('Add this widget to the container').'">';
		echo get_icon( 'new' ).'<strong>'.$Plugin->name.'</strong>';
		echo '</a> <span class="notes">'.$Plugin->short_desc.'</span>';
		echo '</li>';
	}
}
echo '</ul>';

?>
