<?php
/**
 * This file implements creation of DB tables
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package install
 */
if(!defined('QP_MAIN_INIT')) die('fail');

load_funcs( 'categories/model/_category.funcs.php' );

/**
 * Used for fresh install
 */
function create_tables()
{
	global $inc_path;

	// Load DB schema from modules
	load_db_schema();
	load_funcs( '_core/model/db/_upgrade.funcs.php' );
	// Alter DB to match DB schema
	install_make_db_schema_current( false );
}


/**
 * Insert all default data:
 */
function create_default_data()
{
	global $conf_path;
	global $current_locale;
	global $DB;
	global $default_locale;
	global $locales;
	global $Plugins;
	global $timestamp;

	// Inserting sample data triggers events instead of checking if $Plugins is an object there, just use a fake one...
	$Plugins = new Plugins_admin_no_DB(); // COPY

	// Customized or canned user info?
	if( file_exists( $conf_path.'_create_users.php' ) )
	{
		/**
		 * Get customized user info
		 */
		require_once $conf_path.'_create_users.php';
	}
	else
	{
		/**
		 * Get canned user info
		 */
		require_once dirname(__FILE__).'/_create_users.php';
	}

	echo T_('Creating default Post Types').' ... ';
	$DB->query( "
		INSERT INTO T_item_types ( ptyp_ID, ptyp_name )
		VALUES
			( 1, 'Post' ),
			( 1000, 'Page' ),
			( 1500, 'Intro-Main' ),
			( 1520, 'Intro-Cat' ),
			( 1530, 'Intro-Tag' ),
			( 1570, 'Intro-Sub' ),
			( 2000, 'Sidebar link' ),
			( 3000, 'Reserved' ),
			( 4000, 'Reserved' ),
			( 5000, 'Reserved' ) " );
	task_end();

	echo T_('Creating default file types').' ... ';
	// Contribs: feel free to add more types here...
	// @todo (0000) dh> shouldn't they get localized to the app's default locale? fp> ftyp_name, yes
	$DB->query( "INSERT INTO T_filetypes ( ftyp_ID, ftyp_extensions, ftyp_name, ftyp_mimetype, ftyp_icon, ftyp_viewtype, ftyp_allowed )
		VALUES
			(1, 'gif', 'GIF image', 'image/gif', 'gif.png', 'image', 1),
			(2, 'png', 'PNG image', 'image/png', 'png.gif', 'image', 1),
			(3, 'jpg jpeg', 'JPEG image', 'image/jpeg', 'jpg.gif', 'image', 1),
			(4, 'txt', 'Text file', 'text/plain', 'text.png', 'text', 1),
			(5, 'htm html', 'HTML file', 'text/html', 'html.png', 'browser', 0),
			(6, 'pdf', 'PDF file', 'application/pdf', 'pdf.png', 'browser', 1),
			(7, 'doc', 'Microsoft Word file', 'application/msword', 'doc.gif', 'external', 1),
			(8, 'xls', 'Microsoft Excel file', 'application/vnd.ms-excel', 'xls.gif', 'external', 1),
			(9, 'ppt', 'Powerpoint', 'application/vnd.ms-powerpoint', 'ppt.gif', 'external', 1),
			(10, 'pps', 'Slideshow', 'pps', 'pps.gif', 'external', 1),
			(11, 'zip', 'ZIP archive', 'application/zip', 'zip.gif', 'external', 1),
			(12, 'php php3 php4 php5 php6', 'PHP script', 'application/x-httpd-php', 'php.png', 'text', 0),
			(13, 'css', 'Style sheet', 'text/css', 'css.png', 'text', 1),
			(14, 'mp3', 'MPEG audio file', 'audio/mpeg', 'mp3.png', 'browser', 1),
			(15, 'm4a', 'MPEG audio file', 'audio/x-m4a', 'm4a.png', 'browser', 1),
			(16, 'mp4', 'MPEG video', 'video/mp4', 'mp4.png', 'browser', 1),
			(17, 'mov', 'Quicktime video', 'video/quicktime', 'mov.png', 'browser', 1)
		" );
	task_end();

	if( ! empty( $current_locale ) )
	{
		// Make sure the user sees his new system localized.
		echo T_('Activating selected default locale').' ... ';
		$DB->query( 'INSERT INTO T_locales '
			.'( loc_locale, loc_charset, loc_datefmt, loc_timefmt, '
			.'loc_startofweek, loc_name, loc_messages, loc_priority, '
			.'loc_enabled ) '
			.'VALUES ( '.$DB->quote( $current_locale ).', '
			.$DB->quote( $locales[$current_locale]['charset'] ).', '
			.$DB->quote( $locales[$current_locale]['datefmt'] ).', '
			.$DB->quote( $locales[$current_locale]['timefmt'] ).', '
			.$DB->quote( $locales[$current_locale]['startofweek'] ).', '
			.$DB->quote( $locales[$current_locale]['name'] ).', '
			.$DB->quote( $locales[$current_locale]['messages'] ).', '
			.$DB->quote( $locales[$current_locale]['priority'] ).', '
			.' 1)' );
		task_end();
	}

	// Customized or canned installation settings?
	if( file_exists( $conf_path.'_create_system.php' ) )
	{
		/**
		 * Get customized installation settings
		 */
		require $conf_path.'_create_system.php';
	}
	else
	{
		$override = array();
	}
	create_default_settings( $override );

	install_templates();

	install_plugins();

	return true;
}


/**
 * Create a new a blog
 *
 * This funtion has to handle all needed DB dependencies!
 * @todo (0000) move this to Blog object (only half done here)
 */
function create_blog( $blogs, $settings, $categories )
{
	global $basepath;
	global $conf_path;
	global $default_locale;

	$Blog = new Blog( NULL );

	// set everything in T_blogs except access_type and siteurl
	unset( $blogs['template'] );
	foreach( $blogs as $key => $value )
	{
		if( $key == 'access_type' )
		{
			$access_type = $value;
		}
		elseif( $key == 'siteurl' )
		{
			$siteurl = $value;
		}
		else
		{
			$Blog->set( $key, $value );
		}
	}
	$Blog->dbinsert();

	// set access_type in T_blogs depending on blog ID
	$access_type = $blogs['access_type'];
	if( $access_type == '#' )
	{
		if( $Blog->ID === 1 )
		{
			$access_type = 'default';
		}
		else
		{
			$access_type = 'relative';
		}
	}
	$Blog->set( 'access_type', $access_type );

	// set siteurl in T_blogs if access_type is 'relative'
	$create_stub = false;
	if( $access_type == 'relative' )
	{
		$siteurl = $blogs['siteurl'];
		if( $siteurl == '#' )
		{
			$siteurl = 'blog'.$Blog->ID.'.php';
		}
		$Blog->set( 'siteurl', $siteurl );
		$create_stub = true;
	}
	$Blog->dbupdate();

	// set this blog's values in T_blog_settings table
	if( count( $settings ) )
	{
		foreach( $settings as $key => $value )
		{
			$Blog->set_setting( $key, $value );
		}
	}
	$Blog->dbupdate();

	// set this blog's values in T_categories table
	$cats_array = array();
	foreach( $categories as $key => $value )
	{
		// $cat_parent_ID is another category, get the id
		if( $value != 'NULL' )
		{
			foreach( $cats_array as $cat_array )
			{
				if( array_search( $value, $cat_array ) )
				{
					$value = array_search( $value, $cat_array );
				}
			}
		}
		$foo = cat_create( $value, $key, $Blog->ID );
		$cats_array[] = array( $foo => $key );
	}

	// create the stub file if one is called for
	if( $create_stub && ! is_file( $basepath.$siteurl ) )
	{
		$source_file = $basepath.'stubfile.template.php';
		$file_loaded = @file( $source_file );
		if( empty( $file_loaded ) )
		{
			// we don't have the stubfile template? Not good :(
			debug_die( sprintf( T_('Could not load stubfile template [%s]. Did you delete it?'), $source_file ) );
		}
		else
		{
			$new_file = $basepath.$siteurl;
			$f = @fopen( $new_file, 'w' );
			if( $f == false )
			{
				// We could not fopen a new filename? cheap host :(
				debug_die( sprintf( T_('Could not open a new stub file [%s]. Lame host?'), $new_file ) );
			}
			else
			{
				$file_imploded = implode( '', $file_loaded );
				$new_stub = str_replace( '$blog = NNN;', '$blog = '.$Blog->ID.';', $file_imploded );
				fwrite( $f, $new_stub );
				fclose( $f );
			}
		}
		if( ! is_file( $basepath.$siteurl ) )
		{
			// Is it possible to fopen but not fwrite? If so we end up here :(
			debug_die( sprintf( T_('Could not write the new stub file [%s]. Lame host?'), $new_stub ) );
		}
	}

	// Customized or canned widgets?
	if( file_exists( $conf_path.'_create_widgets.php' ) )
	{
		/**
		 * Get customized widgets
		 */
		require_once $conf_path.'_create_widgets.php';
	}
	else
	{
		/**
		 * Get canned blog widgets
		 */
		require_once dirname(__FILE__).'/_create_widgets.php';
		// kept here for historical purposes - I might need this one day ;)
		// $templateCache = get_Cache( 'TemplateCache');
		// $Template = $templateCache->get_by_ID( $Blog->template_ID );
		// $Template->CreateDefaultWidgets( $Blog );
	}

	return $cats_array;
}


/**
 * Spread the sample entries across the past
 *
 * This function modifies timestamp by a fixed amount of time +/- 2 hours, so
 * the "worst case" math has to have the latest entry in the past. If the "coin
 * toss" always comes up heads and the random addition is always 7200 we can now
 * calculate backwards to find what the minimum fixed amount of time needs to be.
 *
 * NOTE: using $timestamp inside if( $fixed_entry_delta === NULL ) ensures we
 * never get closer than a day to right now.
 *
 * @todo (1111) do this the other way. Create an array for timestamps with values
 * for each timestamped bit based on the most recent is "now minus 10 seconds"
 * and 24 +/-4 hours between items, then flip the array and work through it as
 * we insert posts. That gives us built-in variance between posts and the most
 * recent will always be right about now :)
 *
 * @return integer The new $timestamp value.
 */
function installer_timestamp()
{
	global $count_timestamped_items;
	global $timestamp;

	static $fixed_entry_delta = NULL;

	if( $fixed_entry_delta === NULL )
	{
		$backdate = ( $count_timestamped_items * 86400 );
		$max_per_item_spread = ( $backdate / ( $count_timestamped_items - 1 ) );
		$ave_per_item_spread = ( $max_per_item_spread - 7200 );
		$fixed_entry_delta = floor( $ave_per_item_spread );
		return $timestamp;
	}

	if( mt_rand( 2 ,3 ) === 2 )
	{
		// coin toss is heads, add time+
		$timestamp += $fixed_entry_delta + mt_rand( 0, 7200 );
	}
	else
	{
		// coin toss is tails, add time-
		$timestamp += $fixed_entry_delta - mt_rand( 0, 7200 );
	}
	return $timestamp;
}


/**
 * This is called only for fresh installs and fills the tables with
 * demo/tutorial things.
 */
function create_demo_contents()
{
	global $app_baseurl;
	global $app_db_version; // unused?
	global $app_name;
	global $app_version;
	global $blog_all_ID; // unused?
	global $conf_path;
	global $DB;
	global $default_locale;
	global $FileRootCache; // unused?
	global $installed_templates;
	global $Plugins; // unused?
	global $query; // why would we have this as a global?
	global $templates_path;
	global $timestamp; // unused?

	echo T_('Creating default blogs').' ... ';

	// Customized or canned blog settings?
	if( file_exists( $conf_path.'_create_blogs.php' ) )
	{
		/**
		 * Get customized blog settings
		 */
		require_once $conf_path.'_create_blogs.php';
	}
	else
	{
		/**
		 * Get canned blog settings
		 */
		require_once dirname(__FILE__).'/_create_blogs.php';
	}

	$cats_all_array = array();
	foreach( $blogs_array as $new_blog )
	{
		$blogs = $new_blog['blogs'];
		$settings = $new_blog['settings'];
		$categories = $new_blog['categories'];

		// check for minimum requirements
		if( ! ( isset( $blogs['name'] ) && isset( $blogs['shortname'] ) && isset( $blogs['urlname'] ) ) )
		{
			debug_die( 'your _create_blogs.php file is missing either name or shortname or urlname' );
		}

		// check for at least one category
		if( ! count( $categories ) )
		{
			debug_die( 'your _create_blogs.php file has no categories' );
		}

		// Make sure we are not missing anything we need in $blogs
		$blogs = array_merge( array(
			'tagline' => '',
			'longdesc' => '',
			'locale' => $default_locale,
			'access_type' => '#',
			'siteurl' => '#',
			'template_type' => 'xhtml',
		), $blogs );

		// Make sure we are not missing anything we need in $settings
		$settings = array_merge( array(
				'num_posts' => 5,
			), $settings );

		// get the template ID
		$template = $blogs['template'];
		if( isset( $installed_templates[$template] ) )
		{
			$template_ID = $installed_templates[$template];
		}
		else
		{
			debug_die( 'your _create_blogs.php file calls for a template that does not exist' );
		}

		// add template ID and type to $new_blog
		$blogs = array_merge( array(
				'template_ID' => $template_ID,
			), $blogs );

		// create the new blog, get an array of created cats back
		$foo = create_blog( $blogs, $settings, $categories );
		$cats_all_array = array_merge( $cats_all_array, $foo );
	}

	// needed for _create_items to work properly
	$GLOBALS['cats_all_array'] = $cats_all_array;

	task_end();

	if( $install_sample_posts )
	{
		// Customized or canned items?
		if( file_exists( $conf_path.'_create_items.php' ) )
		{
			/**
			 * Get customized items
			 */
			include $conf_path.'_create_items.php';
		}
		else
		{
			/**
			 * Get canned items
			 */
			include dirname(__FILE__).'/_create_items.php';
		}
	}

}

?>
