<?php
/**
 * This file implements upgrading of DB tables
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package install
 */
if(!defined('QP_MAIN_INIT')) die('fail');

load_funcs( '_core/_param.funcs.php' );

/**
 * Converts languages in a given table into according locales
 *
 * @param string name of the table
 * @param string name of the column where lang is stored
 * @param string name of the table's ID column
 */
function convert_lang_to_locale( $table, $columnlang, $columnID )
{
	global $DB;
	global $default_locale;
	global $locales;

	if( !preg_match('/[a-z]{2}-[A-Z]{2}(-.{1,14})?/', $default_locale) )
	{
		// we want a valid locale
		$default_locale = 'en-EU';
	}
	echo 'Converting langs to locales for '. $table. '...<br />';
	// query given languages in $table
	$query = "SELECT $columnID, $columnlang FROM $table";
	$languagestoconvert = array();
	foreach( $DB->get_results( $query, ARRAY_A ) as $row )
	{
		// remember the ID for that locale
		$languagestoconvert[$row[$columnlang]][] = $row[$columnID];
	}
	foreach( $languagestoconvert as $lkey => $lIDs)
	{
		// converting the languages we've found
		$converted = false;
		echo '&nbsp; Converting lang \''. $lkey. '\' '; // (with IDs: '. implode( ', ', $lIDs ). ').. ';

		if( preg_match('/[a-z]{2}-[A-Z]{2}(-.{1,14})?/', $lkey) )
		{
			// Already valid
			echo 'nothing to update, already valid!<br />';
			continue;
		}
		if( ( strlen( $lkey ) == 2 ) && ( substr( $default_locale, 0, 2 ) != $lkey ) )
		{
			// we have an old two letter lang code to convert and it doesn't match the default locale
			foreach( $locales as $newlkey => $v )
			{
				// loop given locales
				if( substr( $newlkey, 0, 2 ) == strtolower( $lkey ) ) # TODO: check if valid/suitable
				{
					// if language matches, update
					$converted = $DB->query( "UPDATE $table SET $columnlang = '$newlkey' WHERE $columnlang = '$lkey'" );
					echo 'to locale \''. $newlkey. '\'<br />';
					break;
				}
			}
		}
		if( ! $converted )
		{
			// we have nothing converted yet, setting default:
			$DB->query( "UPDATE $table SET $columnlang = '$default_locale' WHERE $columnlang = '$lkey'" );
			echo 'forced to default locale \''. $default_locale. '\'<br />';
		}
	}
	echo "\n";
}


/**
 * Alter table: add a column
 *
 * @param string Name of the table involved
 * @param string Name of the column you are adding
 * @param string Name of the column the new column follows
 * @param string Description of the new column
 * @return boolean True if the column has been added, False if not.
 */
function db_add_col( $table, $col_name, $add_after, $col_desc )
{
	global $DB;

	if( $DB->query( 'ALTER TABLE '.$table.' ADD COLUMN '.$col_name.' '.$col_desc ) === false )
	{
		return false;
	}
	return true;
}


/**
 * Alter table: change a column name
 *
 * @param string Name of the table involved
 * @param string Name of the column you are change
 * @param string New name of the column
 * @param string New (or existing!) description of the column
 * @return boolean True if the column has been added, False if not.
 */
function db_change_col( $table, $col_name, $new_name, $col_desc )
{
	global $DB;

	if( $DB->query( 'ALTER TABLE '.$table.' CHANGE '.$col_name.' '.$new_name.' '.$col_desc ) === false )
	{
		return false;
	}
	return true;
}


/**
 * Alter table: modify column definition
 *
 * @param string Name of the table involved
 * @param string Name of the column you are modifying
 * @param string New description of the column
 * @return boolean True if the column has been modified, False if not.
 */
function db_mod_col( $table, $col_name, $col_desc )
{
	global $DB;

	if( $DB->query( 'ALTER TABLE '.$table.' MODIFY COLUMN '.$col_name.' '.$col_desc ) === false )
	{
		return false;
	}
	return true;
}


/**
 * Alter table: add index
 *
 * @param string Name of the table involved
 * @param string Name of your new index
 * @param string Definition of your new index
 * @return boolean True if the index has been added, false if not.
 */
function db_add_index( $table, $name, $def )
{
	global $DB;

	if( $DB->query( 'ALTER TABLE '.$table.' ADD INDEX '.$name.' ('.$def.')' ) === false )
	{
		return false;
	}
	return true;
}


/**
 * Alter table: drop index
 *
 * @param string Name of the table involved
 * @param string Name of the index you are dropping
 * @return boolean True if the index has been dropped, false if not.
 */
function db_drop_index( $table, $name )
{
	global $DB;

	if( $DB->query( 'ALTER TABLE '.$table.' DROP INDEX '.$name ) === false )
	{
		return false;
	}
	return true;
}


/**
 * Does a column exist?
 *
 * @param string Table name
 * @param array Column name
 * @return boolean Does a given column name exist in DB?
 */
function db_cols_exist( $table, $col_name )
{
	global $DB;

	foreach( $DB->get_results('SHOW COLUMNS FROM '.$table) as $row )
	{
		if( $row->Field === $col_name )
		{
			return true;
		}
	}

	return false;
}


/**
 * Drops a column if it exists
 *
 * @param string Table name
 * @param array Column name
 * @return boolean true if the column existed and was dropped
 */
function db_drop_col( $table, $col_name )
{
	global $DB;

	if( ! db_cols_exist( $table, $col_name ) )
	{
		return false;
	}
	if( $DB->query( 'ALTER TABLE '.$table.' DROP COLUMN '.$col_name ) === false )
	{
		return false;
	}
	return true;
}


/**
 * Does an index exist?
 *
 * @param string Table name
 * @param string Index name
 * @return boolean true if the index key name exists
 */
function db_index_exists( $table, $index_name )
{
	global $DB;

	$index_name = strtolower( $index_name );
	foreach( $DB->get_results('SHOW INDEX FROM '.$table) as $row )
	{
		if( strtolower( $row->Key_name ) == $index_name )
		{
			return true;
		}
	}
	return false;
}


/**
 * Create a DB version checkpoint
 *
 * This increments table.db_upgrade during an upgrade cycle. It starts matching
 * table.db_version and end matching $app_db_version. If the upgrader crashes
 * this value lets us pick up exactly where we left off :)
 * @param string version of DB at checkpoint
 */
function set_upgrade_checkpoint( $version )
{
	global $DB;
	global $locale;
	global $script_start_time;

	$DB->query( "UPDATE T_settings SET set_value = '{$version}' WHERE set_name = 'db_upgrade'" );
	$max_exe_time = ini_get( 'max_execution_time' );
	$elapsed_time = time() - $script_start_time;
	if( $max_exe_time && $elapsed_time > $max_exe_time - 10 )
	{
		// Max exe time not disabled and we're reaching the end
		echo 'We are reaching the time limit for this script. Please click <a href="index.php?locale='.$locale.'&amp;action=upgradedb">continue</a>...';
		// Dirty temporary solution
		exit(0);
	}
	echo $version.' ... ';
	task_end();
}


/**
 * Upgrade database tables
 *
 * This is a process. Life is a process. Therefore this is life.
 */
function upgrade_dbase_tables()
{
	global $app_db_config;
	global $app_db_tableprefix;
	global $app_db_version;
	global $app_name;
	global $DB;
	global $db_storage_charset;
	global $old_db_version;
	global $schema_queries;

	// Load DB schema from modules
	load_db_schema();
	load_funcs( '_core/model/db/_upgrade.funcs.php' );

	// BEGIN installation status checks...
	echo '<p>'.T_('Checking DB schema version...').' ';

	// are we not installed?
	$old_db_version = get_db_version( 'version' );
	if( $old_db_version === NULL )
	{
		printf( T_( '<strong>OOPS! %s doesn\'t seem to be installed yet.</strong></p>' ), $app_name );
		return;
	}

	// are we too old to rock and roll or too young to die?
	echo $old_db_version, ' : ';
	if( $old_db_version < 0 )
	{
		debug_die( T_('This version is too old!') );
	}
	if( $old_db_version > $app_db_version )
	{
		debug_die( T_('This version is too recent! We cannot downgrade to the version you are trying to install...') );
	}

	// did we crash while upgrading?
	$cur_db_version = get_db_version( 'upgrade' );
	if( $cur_db_version === NULL )
	{
		// this is not a crash: first upgrade from Quam Plures v0.0.0
		$cur_db_version = $old_db_version;
		$DB->query( "INSERT INTO T_settings (set_name, set_value) VALUES ('db_upgrade', '{$cur_db_version}')" );
	}
	elseif( $cur_db_version != $old_db_version )
	{
		// this is a crash :(
		echo '<br /><strong>'.T_('It looks like we crashed while upgrading.').'</strong> '.T_('We will try to pick up where we left off').' ';
		echo sprintf( T_('You had version %s and crashed at %s on the way to %s.'), $old_db_version, $cur_db_version, $app_db_version ).' ... ';
		if( $cur_db_version === '0' )
		{
			// dammit! I can't for the life of me figure out how it is storing a zero :(
			debug_die( T_('SERIOUS crash problem: $cur_db_version is ZERO.') );
		}
	}

	echo "OK.<br />\n";

	// Try to obtain some serious time to do some serious processing (5 minutes and 57 seconds)
	@set_time_limit( 357 );

	// ---------------------------------------------------------------------------
	// BEGIN UPGRADES ------------------------------------------------------------
	// ---------------------------------------------------------------------------

	if( $cur_db_version < 1 )
	{
		$cur_db_version++;
		echo 'Modifying template_type in the templates__template table ... ';
		if( db_mod_col( $app_db_tableprefix."templates__template", "template_type", "ENUM('xhtml', 'normal', 'feed', 'other') NOT NULL DEFAULT 'xhtml'" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 2 )
	{
		$cur_db_version++;
		echo 'Updating template_type in the templates__template table (1 of 2) ... ';
		if( $DB->query( "UPDATE ".$app_db_tableprefix."templates__template SET template_type = REPLACE( template_type, 'normal', 'xhtml' )" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 3 )
	{
		$cur_db_version++;
		echo 'Updating template_type in the templates__template table (2 of 2) ... ';
		if( $DB->query( "UPDATE ".$app_db_tableprefix."templates__template SET template_type = 'other' WHERE template_name = 'Sitemap'" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 4 )
	{
		$cur_db_version++;
		echo 'Modifying template_type in the templates__template table ... ';
		if( db_mod_col( $app_db_tableprefix."templates__template", "template_type", "ENUM('xhtml', 'feed', 'other') NOT NULL DEFAULT 'xhtml'" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 5 )
	{
		$cur_db_version++;
		echo 'Modifying aspm_source in the antispam table ... ';
		if( db_mod_col( $app_db_tableprefix."antispam", "aspm_source", "ENUM('core','comment','entered','referer','shared','local','reported','central') NOT NULL DEFAULT 'core'" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 6 )
	{
		$cur_db_version++;
		echo 'Updating aspm_source in the antispam table (1 of 3) ... ';
		if( $DB->query( "UPDATE ".$app_db_tableprefix."antispam SET aspm_source = REPLACE( aspm_source, 'local', 'core' )" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 7 )
	{
		$cur_db_version++;
		echo 'Updating aspm_source in the antispam table (2 of 3) ... ';
		if( $DB->query( "UPDATE ".$app_db_tableprefix."antispam SET aspm_source = REPLACE( aspm_source, 'reported', 'core' )" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 8 )
	{
		$cur_db_version++;
		echo 'Updating aspm_source in the antispam table (3 of 3) ... ';
		if( $DB->query( "UPDATE ".$app_db_tableprefix."antispam SET aspm_source = REPLACE( aspm_source, 'central', 'core' )" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 9 )
	{
		$cur_db_version++;
		echo 'Modifying aspm_source in the antispam table ... ';
		if( db_mod_col( $app_db_tableprefix."antispam", "aspm_source", "ENUM('core','comment','entered','referer','shared') NOT NULL DEFAULT 'core'" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 10 )
	{
		$cur_db_version++;
		echo 'Adding aspm_shortened to the antispam table ... ';
		if( db_add_col( $app_db_tableprefix."antispam", "aspm_shortened", "aspm_source", "TINYINT(1) NOT NULL default 0" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 11 )
	{
		$cur_db_version++;
		echo 'Adding aspm_added_on to the antispam table ... ';
		if( db_add_col( $app_db_tableprefix."antispam", "aspm_added_on", "aspm_shortened", "DATETIME NOT NULL DEFAULT '2011-12-20 00:00:00'" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 12 )
	{
		$cur_db_version++;
		echo 'Adding aspm_lastseen to the antispam table ... ';
		if( db_add_col( $app_db_tableprefix."antispam", "aspm_lastseen", "aspm_added_on", "DATETIME NOT NULL DEFAULT '2011-12-20 00:00:00'" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 13 )
	{
		$cur_db_version++;
		echo 'Adding aspm_comments to the antispam table ... ';
		if( db_add_col( $app_db_tableprefix."antispam", "aspm_comments", "aspm_lastseen", "int(11) unsigned NOT NULL default '0'" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 14 )
	{
		$cur_db_version++;
		echo 'Adding aspm_messages to the antispam table ... ';
		if( db_add_col( $app_db_tableprefix."antispam", "aspm_messages", "aspm_comments", "int(11) unsigned NOT NULL default '0'" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 15 )
	{
		$cur_db_version++;
		echo 'Adding aspm_referers to the antispam table ... ';
		if( db_add_col( $app_db_tableprefix."antispam", "aspm_referers", "aspm_messages", "int(11) unsigned NOT NULL default '0'" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 16 )
	{
		$cur_db_version++;
		echo 'Adding user_avatar_ID to the users table ... ';
		if( db_add_col( $app_db_tableprefix."users", "user_avatar_ID", "user_validated", "int(10) NULL" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 17 )
	{
		$cur_db_version++;
		echo 'Adding user_biography to the users table ... ';
		if( db_add_col( $app_db_tableprefix."users", "user_biography", "user_avatar_ID", "TEXT NULL" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 18 )
	{
		$cur_db_version++;
		echo 'Modifying template_type in the templates__template table ... ';
		if( db_mod_col( $app_db_tableprefix."templates__template", "template_type", "ENUM('xhtml', 'html5', 'feed', 'other') NOT NULL DEFAULT 'xhtml'" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 19 )
	{
		$cur_db_version++;
		echo 'Adding blog_template_type to the blogs table ... ';
		if( db_add_col( $app_db_tableprefix."blogs", "blog_template_type", "blog_template_ID", "ENUM('xhtml','html5','feed','other') NOT NULL default 'xhtml'" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 20 )
	{
		$cur_db_version++;
		echo 'Adding sco_order to the templates__container table ... ';
		if( db_add_col( $app_db_tableprefix."templates__container", "sco_order", "sco_name", "TINYINT NOT NULL default 0" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 21 )
	{
		$cur_db_version++;
		echo 'Adding post_attachments to the items__item table ... ';
		if( db_add_col( $app_db_tableprefix."items__item", "post_attachments", "post_featured", "tinyint(1) NOT NULL DEFAULT 0" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 22 )
	{
		$cur_db_version++;
		echo 'Modifying cat_blog_ID in the categories table ... ';
		if( db_mod_col( $app_db_tableprefix."categories", "cat_blog_ID", "int(10) unsigned NOT NULL default 1" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 23 )
	{
		$cur_db_version++;
		echo 'Updating set_name in the settings table ... ';
		if( $DB->query( "UPDATE ".$app_db_tableprefix."settings SET set_name = 'allow_moving_cats' WHERE set_name = 'allow_moving_chapters'" )  !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 24 )
	{
		$cur_db_version++;
		echo 'Updating cset_value in the coll_settings table (1 of 2) ... ';
		if( $DB->query( "UPDATE ".$app_db_tableprefix."coll_settings SET cset_value = 'subcat' WHERE cset_value = 'subchap'" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 25 )
	{
		$cur_db_version++;
		echo 'Updating cset_value in the coll_settings table (2 of 2) ... ';
		if( $DB->query( "UPDATE ".$app_db_tableprefix."coll_settings SET cset_value = 'cats' WHERE cset_value = 'chapters'" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 26 )
	{
		$cur_db_version++;
		echo 'Updating set_name in the settings table ... ';
		if( $DB->query( "UPDATE ".$app_db_tableprefix."settings SET set_name = 'category_order' WHERE set_name = 'chapter_ordering'" )  !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 27 )
	{
		$cur_db_version++;
		echo 'Updating cset_name in the coll_settings table (1 of 5) ... ';
		if( $DB->query( "UPDATE ".$app_db_tableprefix."coll_settings SET cset_name = 'category_content' WHERE cset_name = 'chapter_content'" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 28 )
	{
		$cur_db_version++;
		echo 'Updating cset_name in the coll_settings table (2 of 5) ... ';
		if( $DB->query( "UPDATE ".$app_db_tableprefix."coll_settings SET cset_name = 'category_links' WHERE cset_name = 'chapter_links'" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 29 )
	{
		$cur_db_version++;
		echo 'Updating cset_name in the coll_settings table (3 of 5) ... ';
		if( $DB->query( "UPDATE ".$app_db_tableprefix."coll_settings SET cset_name = 'category_noindex' WHERE cset_name = 'chapter_noindex'" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 30 )
	{
		$cur_db_version++;
		echo 'Updating cset_name in the coll_settings table (4 of 5) ... ';
		if( $DB->query( "UPDATE ".$app_db_tableprefix."coll_settings SET cset_name = 'category_posts_per_page' WHERE cset_name = 'chapter_posts_per_page'" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 31 )
	{
		$cur_db_version++;
		echo 'Updating cset_name in the coll_settings table (5 of 5) ... ';
		if( $DB->query( "UPDATE ".$app_db_tableprefix."settings SET set_name = 'aggregate_blog_IDs' WHERE set_name = 'aggregate_coll_IDs'" )  !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 32 )
	{
		$cur_db_version++;
		echo 'Renaming cset_coll_ID in the coll_settings table ... ';
		if( db_change_col( $app_db_tableprefix."coll_settings", "cset_coll_ID", "cset_blog_ID", "INT(11) UNSIGNED NOT NULL" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 33 )
	{
		$cur_db_version++;
		echo 'Renaming sub_coll_ID in the subscriptions table ... ';
		if( db_change_col( $app_db_tableprefix."subscriptions", "sub_coll_ID", "sub_blog_ID", "INT(11) unsigned not null" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 34 )
	{
		$cur_db_version++;
		echo 'Renaming wi_coll_ID in the widget table ... ';
		if( db_change_col( $app_db_tableprefix."widget", "wi_coll_ID", "wi_widget_blog_ID", "INT(11) UNSIGNED NOT NULL" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 35 )
	{
		$cur_db_version++;
		echo 'Dropping column blog_links_blog_ID from the blogs table ... ';
		if( db_drop_col( $app_db_tableprefix."blogs", "blog_links_blog_ID" ) )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 36 )
	{
		$cur_db_version++;
		echo 'Updating ptyp_name in the items__type table (1 of 2) ... ';
		if( $DB->query( "UPDATE ".$app_db_tableprefix."items__type SET ptyp_name = 'Reserved' WHERE ptyp_name = 'Sidebar link'" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 37 )
	{
		$cur_db_version++;
		echo 'Updating ptyp_name in the items__type table (2 of 2) ... ';
		if( $DB->query( "UPDATE ".$app_db_tableprefix."items__type SET ptyp_name = 'Sidebar link' WHERE ptyp_name = 'Podcast'" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 38 )
	{
		$cur_db_version++;
		echo 'Updating post_ptyp_ID in the items__item table ... ';
		if( $DB->query( "UPDATE ".$app_db_tableprefix."items__item SET post_ptyp_ID = 1 WHERE post_ptyp_ID = 2000" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 39 )
	{
		$cur_db_version++;
		echo 'Creating collections table ... ';
		if( $DB->query( "CREATE TABLE ".$app_db_tableprefix."collections (
			coll_ID int(10) unsigned NOT NULL auto_increment,
			coll_parent_ID int(10) unsigned NULL,
			coll_name varchar(255) NOT NULL,
			coll_urlname varchar(255) NOT NULL,
			coll_blog_ID int(10) unsigned NOT NULL default 1,
			coll_description varchar(255) NULL DEFAULT NULL,
			coll_order int(11) NULL DEFAULT NULL,
			PRIMARY KEY coll_ID (coll_ID),
			UNIQUE coll_urlname( coll_urlname ),
			KEY coll_blog_ID (coll_blog_ID),
			KEY coll_parent_ID (coll_parent_ID),
			KEY coll_order (coll_order)
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 40 )
	{
		$cur_db_version++;
		echo 'Creating item_colls table ... ';
		if( $DB->query( "CREATE TABLE ".$app_db_tableprefix."item_colls (
			itemcoll_post_ID int(11) unsigned NOT NULL,
			itemcoll_coll_ID int(11) unsigned NOT NULL,
			PRIMARY KEY itemcoll_pk (itemcoll_post_ID,itemcoll_coll_ID),
			UNIQUE collitem ( itemcoll_coll_ID, itemcoll_post_ID )
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 41 )
	{
		$cur_db_version++;
		echo 'Creating user_cyberspace table ... ';
		if( $DB->query( "CREATE TABLE ".$app_db_tableprefix."user_cyberspace (
			user_ID int(11) unsigned NOT NULL DEFAULT 0,
			user_diggurl varchar(255) NULL,
			user_facebook varchar(255) NULL,
			user_flickr varchar(255) NULL,
			user_linkdin varchar(255) NULL,
			user_myspace varchar(255) NULL,
			user_stumble varchar(255) NULL,
			user_twitter varchar(255) NULL,
			user_youtube varchar(255) NULL,
			user_gplus varchar(255) NULL,
			user_aolaim varchar(255) NULL,
			user_icqid varchar(255) NULL,
			user_msnim varchar(255) NULL,
			user_skype varchar(255) NULL,
			user_yahooim varchar(255) NULL,
			PRIMARY KEY user_ID (user_ID)
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 42 )
	{
		$cur_db_version++;
		echo 'Creating user_realworld table ... ';
		if( $DB->query( "CREATE TABLE ".$app_db_tableprefix."user_realworld (
			user_ID int(11) unsigned NOT NULL DEFAULT 0,
			user_address_1 varchar(255) NULL,
			user_address_2 varchar(255) NULL,
			user_city varchar(255) NULL,
			user_county varchar(255) NULL,
			user_state varchar(255) NULL,
			user_province varchar(255) NULL,
			user_postal_code varchar(255) NULL,
			user_nation varchar(255) NULL,
			user_wallphone varchar(50) NULL,
			user_cellphone varchar(50) NULL,
			user_homefax varchar(50) NULL,
			user_workphone varchar(50) NULL,
			user_workfax varchar(50) NULL,
			user_birthday varchar(50) NULL,
			user_gender varchar(50) NULL,
			PRIMARY KEY user_ID (user_ID)
		) ENGINE = innodb DEFAULT CHARSET = $db_storage_charset" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 43 )
	{
		$cur_db_version++;
		echo 'Dropping the global cache__table ... ';
		if( $DB->query( "DROP TABLE IF EXISTS ".$app_db_tableprefix."global__cache" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 44 )
	{
		$cur_db_version++;
		echo 'Renaming the bloggroups table to blog_groups ... ';
		if( $DB->query( "ALTER TABLE ".$app_db_tableprefix."bloggroups RENAME TO ".$app_db_tableprefix."blog_groups" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 45 )
	{
		$cur_db_version++;
		echo 'Renaming the coll_settings table to blog_settings ... ';
		if( $DB->query( "ALTER TABLE ".$app_db_tableprefix."coll_settings RENAME TO ".$app_db_tableprefix."blog_settings" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 46 )
	{
		$cur_db_version++;
		echo 'Renaming the blogusers table to blog_users ... ';
		if( $DB->query( "ALTER TABLE ".$app_db_tableprefix."blogusers RENAME TO ".$app_db_tableprefix."blog_users" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 47 )
	{
		$cur_db_version++;
		echo 'Renaming the cron__log table to cron_log ... ';
		if( $DB->query( "ALTER TABLE ".$app_db_tableprefix."cron__log RENAME TO ".$app_db_tableprefix."cron_log" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 48 )
	{
		$cur_db_version++;
		echo 'Renaming the cron__task table to cron_task ... ';
		if( $DB->query( "ALTER TABLE ".$app_db_tableprefix."cron__task RENAME TO ".$app_db_tableprefix."cron_task" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 49 )
	{
		$cur_db_version++;
		echo 'Renaming the items__item table to items ... ';
		if( $DB->query( "ALTER TABLE ".$app_db_tableprefix."items__item RENAME TO ".$app_db_tableprefix."items" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 50 )
	{
		$cur_db_version++;
		echo 'Renaming the postcats table to item_cats ... ';
		if( $DB->query( "ALTER TABLE ".$app_db_tableprefix."postcats RENAME TO ".$app_db_tableprefix."item_cats" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 51 )
	{
		$cur_db_version++;
		echo 'Renaming the items__prerendering table to item_prerendering ... ';
		if( $DB->query( "ALTER TABLE ".$app_db_tableprefix."items__prerendering RENAME TO ".$app_db_tableprefix."item_prerendering" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 52 )
	{
		$cur_db_version++;
		echo 'Renaming the items__status table to item_status ... ';
		if( $DB->query( "ALTER TABLE ".$app_db_tableprefix."items__status RENAME TO ".$app_db_tableprefix."item_status" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 53 )
	{
		$cur_db_version++;
		echo 'Renaming the items__itemtag table to item_tags ... ';
		if( $DB->query( "ALTER TABLE ".$app_db_tableprefix."items__itemtag RENAME TO ".$app_db_tableprefix."item_tags" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 54 )
	{
		$cur_db_version++;
		echo 'Renaming the items__type table to item_types ... ';
		if( $DB->query( "ALTER TABLE ".$app_db_tableprefix."items__type RENAME TO ".$app_db_tableprefix."item_types" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 55 )
	{
		$cur_db_version++;
		echo 'Renaming the items__version table to item_versions ... ';
		if( $DB->query( "ALTER TABLE ".$app_db_tableprefix."items__version RENAME TO ".$app_db_tableprefix."item_versions" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 56 )
	{
		$cur_db_version++;
		echo 'Renaming the pluginevents table to plugin_events ... ';
		if( $DB->query( "ALTER TABLE ".$app_db_tableprefix."pluginevents RENAME TO ".$app_db_tableprefix."plugin_events" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 57 )
	{
		$cur_db_version++;
		echo 'Renaming the pluginsettings table to plugin_settings ... ';
		if( $DB->query( "ALTER TABLE ".$app_db_tableprefix."pluginsettings RENAME TO ".$app_db_tableprefix."plugin_settings" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 58 )
	{
		$cur_db_version++;
		echo 'Renaming the pluginusersettings table to plugin_usersettings ... ';
		if( $DB->query( "ALTER TABLE ".$app_db_tableprefix."pluginusersettings RENAME TO ".$app_db_tableprefix."plugin_usersettings" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 59 )
	{
		$cur_db_version++;
		echo 'Renaming the items__tag table to tags ... ';
		if( $DB->query( "ALTER TABLE ".$app_db_tableprefix."items__tag RENAME TO ".$app_db_tableprefix."tags" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 60 )
	{
		$cur_db_version++;
		echo 'Renaming the templates__template table to templates ... ';
		if( $DB->query( "ALTER TABLE ".$app_db_tableprefix."templates__template RENAME TO ".$app_db_tableprefix."templates" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 61 )
	{
		$cur_db_version++;
		echo 'Renaming the templates__container table to template_containers ... ';
		if( $DB->query( "ALTER TABLE ".$app_db_tableprefix."templates__container RENAME TO ".$app_db_tableprefix."template_containers" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 62 )
	{
		$cur_db_version++;
		echo 'Renaming the usersettings table to user_settings ... ';
		if( $DB->query( "ALTER TABLE ".$app_db_tableprefix."usersettings RENAME TO ".$app_db_tableprefix."user_settings" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 63 )
	{
		$cur_db_version++;
		echo 'Renaming the widget table to widgets ... ';
		if( $DB->query( "ALTER TABLE ".$app_db_tableprefix."widget RENAME TO ".$app_db_tableprefix."widgets" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 64 )
	{
		$cur_db_version++;
		echo 'Renaming the track__keyphrase table to keyphrases ... ';
		if( $DB->query( "ALTER TABLE ".$app_db_tableprefix."track__keyphrase RENAME TO ".$app_db_tableprefix."keyphrases" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 65 )
	{
		$cur_db_version++;
		echo 'Renaming the track__goal table to goals ... ';
		if( $DB->query( "ALTER TABLE ".$app_db_tableprefix."track__goal RENAME TO ".$app_db_tableprefix."goals" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 66 )
	{
		$cur_db_version++;
		echo 'Renaming the track__goalhit table to goal_hits ... ';
		if( $DB->query( "ALTER TABLE ".$app_db_tableprefix."track__goalhit RENAME TO ".$app_db_tableprefix."goal_hits" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 67 )
	{
		$cur_db_version++;
		echo 'Removing ptyp_ID = 1600 from item_types table ... ';
		if( $DB->query( "DELETE FROM ".$app_db_tableprefix."item_types WHERE ptyp_ID = 1600" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 68 )
	{
		$cur_db_version++;
		echo 'Modifying file_root_type in the files table ... ';
		if( db_mod_col( $app_db_tableprefix."files", "file_root_type", "ENUM('absolute','user','blog','shared','templates','collection') NOT NULL DEFAULT 'absolute'" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 69 )
	{
		$cur_db_version++;
		echo 'Updating file_root_type in the files table ... ';
		if( $DB->query( "UPDATE ".$app_db_tableprefix."files SET template_type = REPLACE( file_root_type, 'collection', 'blog' )" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

	if( $cur_db_version < 70 )
	{
		$cur_db_version++;
		echo 'Modifying file_root_type in the files table ... ';
		if( db_mod_col( $app_db_tableprefix."files", "file_root_type", "ENUM('absolute','user','blog','shared','templates') NOT NULL DEFAULT 'absolute'" ) !== false )
		{
			set_upgrade_checkpoint( $cur_db_version );
		}
		else debug_die( T_('Upgrader crashed doing step# ').$cur_db_version );
	}

// -----------------------------------------------------------------------------------------
// Remember changing the DB schema, too! Check out these files:
// - qp_inc/_core/model/__core.install.php
// - qp_inc/sessions/model/_sessions.install.php
// -----------------------------------------------------------------------------------------
// Remember to update $app_db_version in _application.php
// -----------------------------------------------------------------------------------------

	// get our last upgrade version number
	$end_of_upgrade = get_db_version( 'upgrade' );
	// double-check our upgrade against our target
	if( $end_of_upgrade != $app_db_version )
	{
		// This is a serious problem that should never reach production!
		// It means that our branch-work wasn't done well, so fix it :)
		debug_die( 'The end of the upgrade process did not get us where we needed to be! '.$end_of_upgrade.' vs '.$app_db_version );
	}
	// set our db_version to match upgrade number (and therefore target)
	$DB->query( "UPDATE T_settings SET set_value = '$end_of_upgrade' WHERE set_name = 'db_version'" );

	// Check to make sure the DB schema is up to date:
	$upgrade_db_deltas = array(); // This holds changes to make, if any (just all queries)
	foreach( $schema_queries as $table => $query_info )
	{
		// For each table in the schema, check diffs...
		foreach( db_delta( $query_info[1], array('drop_column', 'drop_index') ) as $table => $queries )
		{
			foreach( $queries as $qinfo )
			{
				foreach( $qinfo['queries'] as $query )
				{
					// subqueries for this query (usually one, but may include required other queries)
					$upgrade_db_deltas[] = $query;
				}
			}
		}
	}

	if( empty( $upgrade_db_deltas ) )
	{
		echo '<p>'.T_('The database schema is up to date.').'</p>';
	}
	else
	{
		// delta queries have to be confirmed or executed now..
		$confirmed_db_upgrade = param('confirmed', 'integer', 0); // force confirmation
		$upgrade_db_deltas_confirm_md5 = param( 'upgrade_db_deltas_confirm_md5', 'string', '' );

		if( ! $confirmed_db_upgrade )
		{
			if( ! empty( $upgrade_db_deltas_confirm_md5 ) )
			{
				// received confirmation from form
				if( $upgrade_db_deltas_confirm_md5 != md5( implode('', $upgrade_db_deltas) ) )
				{
					// unlikely to happen
					echo '<p class="error">'
						.T_('The DB schema has been changed since confirmation.')
						.'</p>';
				}
				else
				{
					$confirmed_db_upgrade = true;
				}
			}
		}

		if( ! $confirmed_db_upgrade )
		{
			global $action, $locale;
			$Form = new Form( NULL, '', 'post' );
			$Form->begin_form( 'fform', T_('Upgrade database') );
			$Form->begin_fieldset();
			$Form->hidden( 'upgrade_db_deltas_confirm_md5', md5(implode( '', $upgrade_db_deltas )) );
			$Form->hidden( 'action', $action );
			$Form->hidden( 'locale', $locale );
			echo '<p>'.T_('The version number is correct, but we have detected changes in the database schema. This can happen with <abbr title="Version Control System">VCS</abbr> versions...').'</p>';
			echo '<p>'.T_('The following database changes will be carried out. If you are not sure what this means, it will probably be alright.').'</p>';
			echo '<ul>';
			foreach( $upgrade_db_deltas as $l_delta )
			{
				#echo '<li><code>'.nl2br( $l_delta ).'</code></li>';
				echo '<li><pre>'.str_replace( "\t", '  ', $l_delta ).'</pre></li>';
			}
			echo '</ul>';
			$Form->submit( array( '', T_('Upgrade database!'), 'ActionButton' ) );
			$Form->end_form();

			return false;
		}

		// Alter DB to match DB schema:
		install_make_db_schema_current( true );
	}

	return true;
}

?>
