<?php
/**
 * This file implements support functions for the installer
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package install
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * Open a block
 */
function block_open()
{
	global $block_status;
	if( isset( $block_status ) && $block_status == 'open' )
	{
		return;
	}
	$block_status = 'open';
	echo '<div class="block1">';
}

/**
 * Close a block
 */
function block_close()
{
	global $block_status;
	if( ! isset( $block_status ) || $block_status == 'closed' )
	{
		return;
	}
	$block_status = 'closed';
	echo '</div>';
}

/**
 * Create a new category
 *
 * @param string category parent ID ('NULL' as string for root)
 * @param string category name
 * @param integer|NULL Blog ID (will be taken from parent cat, if empty)
 */
function cat_create( $cat_parent_ID, $cat_name, $cat_blog_ID = NULL )
{
	global $DB;

	if( $cat_blog_ID == NULL )
	{
		if( empty( $cat_parent_ID ) ) debug_die ( 'cat_create(-) missing parameters!' );
		$cat_blog_ID = get_catblog( $cat_parent_ID );
	}

	// Dirty temporary fix:
	// $cat_urlname = preg_replace( '/[^a-z0-9]/', '-', strtolower( $cat_name ) );

	$cat_urlname = urltitle_validate( $cat_name, $cat_name, 0, false, 'cat_urlname', 'foo', 'T_categories', true );

	$sql = "INSERT INTO T_categories( cat_parent_ID, cat_name, cat_blog_ID, cat_urlname )
		VALUES ( $cat_parent_ID, ".$DB->quote( $cat_name ).", $cat_blog_ID, ".$DB->quote( $cat_urlname )." )";
	if( ! $DB->query( $sql ) )
	{
		return 0;
	}

	return $DB->insert_id;
}


/**
 * Language selector
 */
function display_locale_selector()
{
	global $locales, $default_locale, $action;

	static $selector_already_displayed = false;

	if( $selector_already_displayed )
	{
		return;
	}
	$selector_already_displayed = true;

	echo '<ul class="locales"><li>';
	locale_flag( $default_locale, 'w16px', 'flag', '', true, /* Do not rely on $app_baseurl/$rsc_url here: */ '../qp_rsc/flags' );
	echo T_('Change language' ).' : </li>'."\n";
	foreach( $locales as $lkey => $lvalue )
	{
		if( $default_locale == $lkey ) continue;
		echo '<li><a href="index.php?locale='.$lkey.'" title=" '.T_( $lvalue['name'] ).' ">';
		locale_flag( $lkey, 'w16px', 'flag', '', true, /* Do not rely on $app_baseurl/$rsc_url here: */ '../qp_rsc/flags' );
		echo '</a></li>'."\n";
	}
	echo '</ul>';
}


/**
 * Base config recap
 */
function display_base_config_recap()
{
	global $default_locale, $conf_db_user, $conf_db_password, $conf_db_name, $conf_db_host, $app_db_config, $app_db_tableprefix, $app_baseurl;

	static $base_config_recap_already_displayed = false;

	if( $base_config_recap_already_displayed )
	{
		return;
	}
	$base_config_recap_already_displayed = true;

	block_open();
	?>
	<h2><?php echo T_('Base config recap...')?></h2>

	<p><?php printf( T_('If you don\'t see correct settings here, STOP before going any further, and <a %s>update your base configuration</a>.'), 'href="index.php?action=start&amp;locale='.$default_locale.'"' ) ?></p>

	<?php
	if( ! isset( $conf_db_user ) ) $conf_db_user = $app_db_config['user'];
	if( ! isset( $conf_db_password ) ) $conf_db_password = $app_db_config['password'];
	if( ! isset( $conf_db_name ) ) $conf_db_name = $app_db_config['name'];
	if( ! isset( $conf_db_host ) ) $conf_db_host = $app_db_config['host'];

	echo '<pre>',
	T_('MySQL Username').': '.$conf_db_user."\n".
	T_('MySQL Password').': '.( ( $conf_db_password != 'demopass' ? T_('(Set, but not shown for security reasons)') : 'demopass' ) )."\n".
	T_('MySQL Database name').': '.$conf_db_name."\n".
	T_('MySQL Host/Server').': '.$conf_db_host."\n".
	T_('MySQL tables prefix').': '.$app_db_tableprefix."\n\n".
	T_('Base URL').': '.$app_baseurl.
	'</pre>';
	block_close();
}


/**
 * Install new DB
 */
function install_newdb()
{
	global $admin_url;
	global $app_db_version;
	global $app_name;

	/**
	 * Get the installation creation functions
	 */
	require_once dirname(__FILE__).'/_functions_create.php';

	if( ( $old_db_version = get_db_version( 'version' ) ) !== NULL )
	{
		echo '<p><strong>';
		printf( T_('OOPS! It seems %s is already installed!'), $app_name );
		echo '</strong></p>';
		echo '<p>';
		echo T_('You can <a href="../index.php">view your blog</a> ');
		printf( T_('or log in to the <a %s>admin</a>'), 'href="'.$admin_url.'"' );
		echo '</p>';

		if( $old_db_version < $app_db_version )
		{
			echo '<p>'.sprintf( T_('Would you like to <a %s>upgrade your existing installation now</a>?'), 'href="?action=dbupgrade"' ).'</p>';
		}

		return;
	}

	$installer_version = param( 'installer_version', 'integer', 0 );
	if( $installer_version >= 10 )
	{
		$create_sample_contents = param( 'create_sample_contents', 'integer', 0 );
		$create_htaccess = param( 'create_htaccess', 'integer', 0 );
	}
	else
	{
		// Probably an automated script calling?
		// force the sample contents since they haven't been explicitely disabled
		$create_sample_contents = 1;
		// force the .htaccess since it hasn't been explicitely disabled
		$create_htaccess = 1;
	}

	echo '<h2>'.sprintf( T_('Installing %s ...'), $app_name ).'</h2>';
	flush();

	create_tables();
	flush();

	create_default_data();
	flush();

	if( $create_sample_contents )
	{
		global $Settings;

		// We're gonna need some environment in order to create the demo contents...
		$Settings = new GeneralSettings();

		$UserCache = & get_Cache( 'UserCache' );
		// Create $current_User object.
		// (Assigning by reference does not work with "global" keyword (PHP 5.2.8))
		$GLOBALS['current_User'] = & $UserCache->get_by_ID( 1 );

		create_demo_contents();
		flush();
	}

	global $basepath;

	if( $create_htaccess && ! is_file( $basepath.'.htaccess' ) )
	{
		$source_file = $basepath.'sample.htaccess';
		$file_loaded = @file( $source_file );
		if( empty( $file_loaded ) )
		{
			// we don't have the sample.htaccess file? Not good :(
			debug_die( T_('Could not load sample.htaccess file. Did you delete it?') );
		}
		else
		{
			$new_file = $basepath.'.htaccess';
			$f = @fopen( $new_file, 'w' );
			if( $f == false )
			{
				// We could not fopen a new filename? cheap host :(
				debug_die( T_('Could not create a new .htaccess file. Lame host?') );
			}
			else
			{
				$file_imploded = implode( '', $file_loaded );
				// this line won't exist but is a good starting idea for doing the anti-hotlink stuff
				// $new_htaccess = str_replace( '$blog = NNN;', '$blog = '.$Blog->ID.';', $file_imploded );
				$new_htaccess = $file_imploded;
				fwrite( $f, $new_htaccess );
				fclose( $f );
			}
		}
		if( ! is_file( $basepath.'.htaccess' ) )
		{
			// Is it possible to fopen but not fwrite? If so we end up here :(
			debug_die( T_('Could not write the new .htaccess file. Lame host?') );
		}
		flush();
	}

	echo '<h2>'.T_('Installation successful!').'</h2>';
	echo '<p><strong>';
	echo T_('You can now <a href="../index.php">view your blog</a>').' ';
	printf( T_('or <a %s>log in</a> with the following credentials:'), 'href="'.$admin_url.'"' );
	echo '</strong></p>';
	echo '<table>';
	echo '<tr><td>', T_( 'Login' ), ': &nbsp;</td><td><strong>'.param('conf_admin_login', 'string').'</strong></td></tr>';
	printf( '<tr><td>%s: &nbsp;</td><td><strong>%s</strong></td></tr>', T_( 'Password' ), param('conf_admin_pass', 'string') );
	echo '</table>';
}


/**
 * End install task
 *
 * This will offer other display methods in the future
 */
function task_end()
{
	echo /* TRANS: "okay", or "good" */ T_('OK.')."<br />\n";
	flush();
}


/**
 * Get DB version
 *
 * returns either the database version or the version during upgrading
 */
function get_db_version( $which )
{
	global $DB;

	$DB->save_error_state();
	$DB->halt_on_error = false;
	$DB->show_errors = false;

	$r = NULL;

	if( $which == 'version' )
	{
		// actual value (after successful install or upgrade)
		$r = $DB->get_var( 'SELECT set_value FROM T_settings WHERE set_name = "db_version"' );
	}
	if( $which == 'upgrade' )
	{
		// value while upgrading (matches db_version when done)
		$r = $DB->get_var( 'SELECT set_value FROM T_settings WHERE set_name = "db_upgrade"' );
	}

	$DB->restore_error_state();

	return $r;
}


/**
 * @return boolean Does a given column name exist in DB?
 */
function db_col_exists( $table, $col_name )
{
	global $DB;

	$col_name = strtolower( $col_name );

	$r = false;
	$DB->save_error_state();
	foreach( $DB->get_results('SHOW COLUMNS FROM '.$table) as $row )
	{
		if( strtolower( $row->Field ) == $col_name )
		{
			$r = true;
			break;
		}
	}
	$DB->restore_error_state();

	return $r;
}


/**
 * Clean up extra quotes in comments
 */
function cleanup_comment_quotes()
{
  global $DB;

	echo T_('Checking for extra quote escaping in comments').' ... ';
	$query = "SELECT comment_ID, comment_content FROM T_comments
		WHERE comment_content LIKE '%\\\\\\\\\'%'
		OR comment_content LIKE '%\\\\\\\\\"%' ";
	/* FP: the above looks overkill, but MySQL is really full of surprises... tested on 4.0.14-nt */
	$rows = $DB->get_results( $query, ARRAY_A );
	if( $DB->num_rows )
	{
		echo sprintf( T_('Updating %d comments ... '), $DB->num_rows );
		foreach( $rows as $row )
		{
			$query = "UPDATE T_comments
				SET comment_content = ".$DB->quote( stripslashes( $row['comment_content'] ) )."
				WHERE comment_ID = ".$row['comment_ID'];
			$DB->query( $query );
		}
	}
	task_end();

}


/**
 * Validate install requirements
 *
 * @return array List of errors, empty array if ok.
 */
function install_validate_requirements()
{
	$errors = array();

	return $errors;
}


/**
 * Insert default settings into T_settings
 *
 * It only writes those to DB, that get overridden (passed as array), or have no
 * default in {@link _generalsettings.class.php} / {@link GeneralSettings::default}.
 *
 * @param array associative array (settings name => value to use), allows overriding of defaults
 */
function create_default_settings( $override = array() )
{
	global $app_db_version;
	global $DB;
	global $default_group_ID;
	global $default_locale;

	$defaults = array(
		'db_version' => $app_db_version,
		'default_locale' => $default_locale,
		'newusers_grp_ID' => $default_group_ID,
		'default_blog_ID' => 1,
	);

	$settings = array_merge( array_keys( $defaults ), array_keys( $override ) );
	$settings = array_unique( $settings );
	$insertvalues = array();
	foreach( $settings as $name )
	{
		if( isset( $override[$name] ) )
		{
			$insertvalues[] = '('.$DB->quote( $name ).', '.$DB->quote( $override[$name] ).')';
		}
		else
		{
			$insertvalues[] = '('.$DB->quote( $name ).', '.$DB->quote( $defaults[$name] ).')';
		}
	}
	echo T_('Creating default settings').( count( $override ) ? ' '.sprintf( T_('(with %d existing values)'), count( $override ) ) : '' ).' ... ';
	$DB->query(
		"INSERT INTO T_settings (set_name, set_value)
		VALUES ".implode( ', ', $insertvalues ) );
	task_end();
}


/**
 * Install all templates
 */
function install_templates()
{
	global $templates_path;

	// thanks to http://www.the-art-of-web.com/php/dirlist/ for the basis of the following code
	$template_names = array();
	$d = dir( $templates_path ) or die( sprintf( T_('Failed opening directory %s for reading'), $templates_path ) );
	while( false !== ( $entry = $d->read() ) )
	{
		// skip hidden folders
		if( $entry[0] == '.' )
		{
			continue;
		}
		// save directory names
		$templates_path_entry = $templates_path.$entry;
		if( is_dir( $templates_path_entry ) )
		{
			$template_names[] = $entry;
		}
	}
	$d->close();

	// install all templates alphabetically
	echo T_('Installing templates ... ');
	sort( $template_names );
	load_funcs( 'templates/_template.funcs.php' );
	$template_id = 1;
	$installed_templates = array();
	foreach( $template_names as $a_template )
	{
		template_install( $a_template );
		$installed_templates[$a_template] = $template_id;
		$template_id++;
	}
	// need this when we install sample blogs
	$GLOBALS['installed_templates'] = $installed_templates;
	task_end();
}


/**
 * Install all plugins
 *
 * This gets called separately on fresh installs and installs all plugins found
 * in the qp_plugins folder EXCEPT 'test_plugin' and 'generic_ping_plugin'.
 *
 * @param integer Old DB version, so that only new plugins gets installed
 */
function install_plugins( $old_db_version = 0 )
{
	global $Plugins_admin;
	global $plugins_path;

	$Plugins_admin = & get_Cache('Plugins_admin');

	// Create global $Plugins instance, which is required during installation of
	// basic plugins, not only for the ones getting installed, but also during e.g.
	// count_regs(), which instantiates each plugin (which may then use (User)Settings
	// in PluginInit (through Plugin::__get)).
	$GLOBALS['Plugins'] = & $Plugins_admin;

	// Tblue> 0001 is octal notation and _could_ lead to unwanted side effects in the future (if changed)
	if( $old_db_version < 1 )
	{
		// thanks to http://www.the-art-of-web.com/php/dirlist/ for the basis of this code
		// array to hold return value
		$plugins = array();
		// open pointer to directory and read list of folders/files
		$d = dir( $plugins_path ) or die( sprintf( T_('Failed opening directory %s for reading'), $plugins_path ) );
		while( false !== ( $entry = $d->read() ) )
		{
			// skip hidden folders
			if( $entry[0] == '.' )
			{
				continue;
			}
			// save directory names
			$plugins_path_entry = $plugins_path.$entry;
			if( is_dir( $plugins_path_entry ) )
			{
				if( substr( $entry, -7 ) == '_plugin' && $entry != 'generic_ping_plugin' && $entry != 'test_plugin' )
				{
					$plugins[] = $entry;
				}
			}
		}
		$d->close();

		// install all plugins alphabetically
		echo T_('Installing plugins ... ');
		sort( $plugins );
		foreach( $plugins as $a_plugin )
		{
			install_plugin( $a_plugin );
		}
		task_end();
	}

	// perform AfterInstall() tasks
	$Plugins_admin->restart();
	while( $a_plugin = & $Plugins_admin->get_next() )
	{
		$Plugins_admin->call_method( $a_plugin->ID, 'AfterInstall', $params = array() );
	}

}


/**
 * @return true on success
 */
function uninstall_plugin( $plugin )
{
	global $Plugins_admin;

	if( $edit_Plugin = & $Plugins_admin->get_by_classname( $plugin ) )
	{
		// plugin is installed
		echo T_('Uninstalling plugin: ').$plugin.' ... ';
		$Plugins_admin->uninstall( $edit_Plugin->ID );
	}
}


/**
 * @return true on success
 */
function install_plugin( $plugin )
{
	global $Plugins_admin;
	global $DB;

	$edit_Plugin = & $Plugins_admin->install( $plugin, 'broken' ); // "broken" by default, gets adjusted later
	if( ! is_a( $edit_Plugin, 'Plugin' ) )
	{
		return false;
	}

	// perform GetDbLayout()
	if( $db_layout = $edit_Plugin->GetDbLayout() )
	{
		foreach( $db_layout as $a_table )
		{
			$DB->query( $a_table );
		}
	}

	// Try to enable plugin
	$enable_return = $edit_Plugin->BeforeEnable();
	if( $enable_return !== true )
	{
		$Plugins_admin->set_Plugin_status( $edit_Plugin, 'disabled' ); // does not unregister it
		return false;
	}

	$Plugins_admin->set_Plugin_status( $edit_Plugin, 'enabled' );
	return true;

}


/**
 * Create relations
 *
 * @todo NOT UP TO DATE AT ALL :( -- update field names before activating this
 */
function create_relations()
{
	global $DB;

	echo T_('Creating relations').' ... ';

	$DB->query( 'alter table T_blog_users
		add constraint FK_bloguser_blog_ID
		foreign key (bloguser_blog_ID)
		references T_blogs (blog_ID)
		on delete restrict
		on update restrict' );

	$DB->query( 'alter table T_blog_users
		add constraint FK_bloguser_user_ID
		foreign key (bloguser_user_ID)
		references T_users (user_ID)
		on delete restrict
		on update restrict' );

	$DB->query( 'alter table T_categories
		add constraint FK_cat_blog_ID
		foreign key (cat_blog_ID)
		references T_blogs (blog_ID)
		on delete restrict
		on update restrict,
		add constraint FK_cat_parent_ID
		foreign key (cat_parent_ID)
		references T_categories (cat_ID)
		on delete restrict
		on update restrict' );

	$DB->query( 'alter table T_comments
		add constraint FK_comment_post_ID
		foreign key (comment_post_ID)
		references T_items (post_ID)
		on delete restrict
		on update restrict' );

	$DB->query( 'alter table T_item_cats
		add constraint FK_postcat_cat_ID
		foreign key (postcat_cat_ID)
		references T_categories (cat_ID)
		on delete restrict
		on update restrict,
		add constraint FK_postcat_post_ID
		foreign key (postcat_post_ID)
		references T_items (post_ID)
		on delete restrict
		on update restrict' );

	$DB->query( 'alter table T_items
		add constraint FK_post_assigned_user_ID
		foreign key (post_assigned_user_ID)
		references T_users (user_ID)
		on delete restrict
		on update restrict,
		add constraint FK_post_lastedit_user_ID
		foreign key (post_lastedit_user_ID)
		references T_users (user_ID)
		on delete restrict
		on update restrict,
		add constraint FK_post_creator_user_ID
		foreign key (post_creator_user_ID)
		references T_users (user_ID)
		on delete restrict
		on update restrict,
		add constraint FK_post_main_cat_ID
		foreign key (post_main_cat_ID)
		references T_categories (cat_ID)
		on delete restrict
		on update restrict,
		add constraint FK_post_parent_ID
		foreign key (post_parent_ID)
		references T_items (post_ID)
		on delete restrict
		on update restrict,
		add constraint FK_post_pst_ID
		foreign key (post_pst_ID)
		references T_item_status (pst_ID)
		on delete restrict
		on update restrict,
		add constraint FK_post_ptyp_ID
		foreign key (post_ptyp_ID)
		references T_item_types (ptyp_ID)
		on delete restrict
		on update restrict' );

	$DB->query( 'alter table T_links
		add constraint FK_link_creator_user_ID
		foreign key (link_creator_user_ID)
		references T_users (user_ID)
		on delete restrict
		on update restrict' );
	$DB->query( 'alter table T_links
		add constraint FK_link_lastedit_user_ID
		foreign key (link_lastedit_user_ID)
		references T_users (user_ID)
		on delete restrict
		on update restrict' );
	$DB->query( 'alter table T_links
		add constraint FK_link_dest_itm_ID
		foreign key (link_dest_itm_ID)
		references T_items (post_ID)
		on delete restrict
		on update restrict' );
	$DB->query( 'alter table T_links
		add constraint FK_link_file_ID
		foreign key (link_file_ID)
		references T_files (file_ID)
		on delete restrict
		on update restrict' );
	$DB->query( 'alter table T_links
		add constraint FK_link_itm_ID
		foreign key (link_itm_ID)
		references T_items (post_ID)
		on delete restrict
		on update restrict' );

	$DB->query( 'alter table T_plugin_settings
		add constraint FK_pset_plug_ID
		foreign key (pset_plug_ID)
		references T_plugins (plug_ID)
		on delete restrict
		on update restrict' );

	$DB->query( 'alter table T_plugin_usersettings
		add constraint FK_puset_plug_ID
		foreign key (puset_plug_ID)
		references T_plugins (plug_ID)
		on delete restrict
		on update restrict' );

	$DB->query( 'alter table T_plugin_events
		add constraint FK_pevt_plug_ID
		foreign key (pevt_plug_ID)
		references T_plugins (plug_ID)
		on delete restrict
		on update restrict' );

	$DB->query( 'alter table T_users
		add constraint FK_user_grp_ID
		foreign key (user_grp_ID)
		references T_groups (grp_ID)
		on delete restrict
		on update restrict' );

	$DB->query( 'alter table T_user_settings
		add constraint FK_uset_user_ID
		foreign key (uset_user_ID)
		references T_users (user_ID)
		on delete restrict
		on update restrict' );

	$DB->query( 'alter table T_subscriptions
		add constraint FK_sub_blog_ID
		foreign key (sub_blog_ID)
		references T_blogs (blog_ID)
		on delete restrict
		on update restrict' );

	$DB->query( 'alter table T_subscriptions
		add constraint FK_sub_user_ID
		foreign key (sub_user_ID)
		references T_users (user_ID)
		on delete restrict
		on update restrict' );

	task_end();
}


/**
 * Loads the QP database scheme
 *
 * This gets updated through {@link db_delta()} which generates the queries needed
 * to get to this scheme. Please see {@link db_delta()} for things to take care of.
 */
function load_db_schema()
{
	global $inc_path;
	global $modules;
	global $schema_queries;

	global $db_storage_charset, $DB;
	if( empty( $db_storage_charset ) )
	{
		// If no specific charset has been requested for datstorage, use the one of
		// the current connection (optimize for speed - no conversions)
		$db_storage_charset = $DB->connection_charset;
	}

	// Load modules
	echo T_('Loading modules').' ... ';
	foreach( $modules as $module )
	{
		require_once $inc_path.$module.'/model/_'.$module.'.install.php';
	}
	task_end();

}

?>
