<?php
/**
 * Net_DNS: A resolver library for PHP
 *
 * @author {@link http://wonderwinds.com/ Ed Bennett}
 * @author Eric Kilfoil <eric@ypass.net>
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.php.net/license/3_01.txt The PHP License
 * @package external
 * @subpackage Net_DNS
 */

/**
 * @package external
 * @subpackage Net_DNS
 */
class Net_DNS_Header
{
	var $id;
	var $qr;
	var $opcode;
	var $aa;
	var $tc;
	var $rd;
	var $ra;
	var $rcode;
	var $qdcount;
	var $ancount;
	var $nscount;
	var $arcount;

	/**
	 * dummy docblock
	 */
	function Net_DNS_Header($data = '')
	{
		if (empty($data)) {
			$this->id = Net_DNS_Resolver::nextid();
			$this->qr = 0;
			$this->opcode = 0;
			$this->aa = 0;
			$this->tc = 0;
			$this->rd = 1;
			$this->ra = 0;
			$this->rcode = 0;
			$this->qdcount = 1;
			$this->ancount = 0;
			$this->nscount = 0;
			$this->arcount = 0;
		} else {
			/*
			 * The header MUST be at least 12 bytes.
			 * Passing the full datagram to this constructor
			 * will examine only the header section of the DNS packet
			 */
			if (strlen($data) < 12) {
				return false;
			}

			$a = unpack('nid/C2flags/n4counts', $data);
			$this->id = $a['id'];
			$this->qr = ($a['flags1'] >> 7) & 0x1;
			$this->opcode = ($a['flags1'] >> 3) & 0xf;
			$this->aa = ($a['flags1'] >> 2) & 0x1;
			$this->tc = ($a['flags1'] >> 1) & 0x1;
			$this->rd = $a['flags1'] & 0x1;
			$this->ra = ($a['flags2'] >> 7) & 0x1;
			$this->rcode = $a['flags2'] & 0xf;
			$this->qdcount = $a['counts1'];
			$this->ancount = $a['counts2'];
			$this->nscount = $a['counts3'];
			$this->arcount = $a['counts4'];
		}

		$dns = new Net_DNS();
		if ($dns->opcodesbyval($this->opcode)) {
			$this->opcode = $dns->opcodesbyval($this->opcode);
		}
		if ($dns->rcodesbyval($this->rcode)) {
			$this->rcode = $dns->rcodesbyval($this->rcode);
		}
	}


	/**
	 * dummy docblock
	 */
	function display()
	{
		echo $this->string();
	}


	/**
	 * dummy docblock
	 */
	function string()
	{
		$retval = ';; id = ' . $this->id . "\n";
		if ($this->opcode == 'UPDATE') {
			$retval .= ';; qr = ' . $this->qr . '    ' .
				'opcode = ' . $this->opcode . '    ' .
				'rcode = ' . $this->rcode . "\n";
			$retval .= ';; zocount = ' . $this->qdcount . '  ' .
				'prcount = ' . $this->ancount . '  '           .
				'upcount = ' . $this->nscount . '  '           .
				'adcount = ' . $this->arcount . "\n";
			} else {
				$retval .= ';; qr = ' . $this->qr . '    ' .
					'opcode = ' . $this->opcode . '    ' .
					'aa = ' . $this->aa . '    ' .
					'tc = ' . $this->tc . '    ' .
					'rd = ' . $this->rd . "\n";

				$retval .= ';; ra = ' . $this->ra . '    ' .
					'rcode  = ' . $this->rcode . "\n";

				$retval .= ';; qdcount = ' . $this->qdcount . '  ' .
					'ancount = ' . $this->ancount . '  ' .
					'nscount = ' . $this->nscount . '  ' .
					'arcount = ' . $this->arcount . "\n";
			}
		return $retval;
	}


	/**
	 * dummy docblock
	 */
	function data()
	{
		$dns = new Net_DNS();
		$opcode = $dns->opcodesbyname($this->opcode);
		$rcode = $dns->rcodesbyname($this->rcode);

		$byte2 = ($this->qr << 7)
			| ($opcode << 3)
			| ($this->aa << 2)
			| ($this->tc << 1)
			| ($this->rd);

		$byte3 = ($this->ra << 7) | $rcode;

		return pack('nC2n4', $this->id,
			$byte2,
			$byte3,
			$this->qdcount,
			$this->ancount,
			$this->nscount,
			$this->arcount);
	}

}

?>
