<?php
/**
 * Net_DNS: A resolver library for PHP
 *
 * @author {@link http://wonderwinds.com/ Ed Bennett}
 * @author Eric Kilfoil <eric@ypass.net>
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.php.net/license/3_01.txt The PHP License
 * @package external
 * @subpackage Net_DNS
 */

/**
 * @package external
 * @subpackage Net_DNS
 */
class Net_DNS_Packet
{
	var $debug;
	var $header;
	var $compnames;
	var $answerfrom;
	var $answersize;
	var $question;
	var $answer;
	var $authority;
	var $additional;

	const INT32SZ = 4;
	const INT16SZ = 2;

	/**
	 * dummy docblock
	 */
	function Net_DNS_Packet($debug = false)
	{
		$this->debug = $debug;
		$this->compnames = array();
	}


	/**
	 * dummy docblock
	 */
	function buildQuestion($name, $type = 'A', $class = 'IN')
	{
		$this->header = new Net_DNS_Header();
		$this->header->qdcount = 1;
		$this->question[0] = new Net_DNS_Question($name, $type, $class);
		$this->answer = null;
		$this->authority = null;
		$this->additional = null;
	}


	/**
	 * dummy docblock
	 */
	function parse($data)
	{
		if ($this->debug) {
			echo ';; HEADER SECTION' . "\n";
		}

		$this->header = new Net_DNS_Header($data);
		$header = $this->header;

		if ($this->debug) {
			$header->display();
		}

		/*
		 * Print and parse the QUESTION section of the packet
		 */
		if ($this->debug) {
			echo "\n";
			$section = ($header->opcode == 'UPDATE') ? 'ZONE' : 'QUESTION';
			echo ";; $section SECTION (" . $header->qdcount . ' record' .
				($header->qdcount == 1 ? '' : 's') . ")\n";
		}

		$offset = 12;

		$this->question = array();
		for ($ctr = 0; $ctr < $header->qdcount; $ctr++) {
			list($qobj, $offset) = $this->parse_question($data, $offset);
			if (is_null($qobj)) {
				return null;
			}

			$this->question[count($this->question)] = $qobj;
			if ($this->debug) {
				echo ";;\n;";
				$qobj->display();
			}
		}

		/*
		 * Print and parse the PREREQUISITE or ANSWER section of the packet
		 */
		if ($this->debug) {
			echo "\n";
			$section = ($header->opcode == 'UPDATE') ? 'PREREQUISITE' :'ANSWER';
			echo ";; $section SECTION (" .
				$header->ancount . ' record' .
				(($header->ancount == 1) ? '' : 's') .
				")\n";
		}

		$this->answer = array();
		for ($ctr = 0; $ctr < $header->ancount; $ctr++) {
			list($rrobj, $offset) = $this->parse_rr($data, $offset);

			if (is_null($rrobj)) {
				return null;
			}
			array_push($this->answer, $rrobj);
			if ($this->debug) {
				$rrobj->display();
			}
		}

		/*
		 * Print and parse the UPDATE or AUTHORITY section of the packet
		 */
		if ($this->debug) {
			echo "\n";
			$section = ($header->opcode == 'UPDATE') ? 'UPDATE' : 'AUTHORITY';
			echo ";; $section SECTION (" .
				$header->nscount . ' record' .
				(($header->nscount == 1) ? '' : 's') .
				")\n";
		}

		$this->authority = array();
		for ($ctr = 0; $ctr < $header->nscount; $ctr++) {
			list($rrobj, $offset) = $this->parse_rr($data, $offset);

			if (is_null($rrobj)) {
				$this->header = $header;
				return null;
			}
			array_push($this->authority, $rrobj);
			if ($this->debug) {
				$rrobj->display();
			}
		}

		/*
		 * Print and parse the ADDITIONAL section of the packet
		 */
		if ($this->debug) {
			echo "\n";
			echo ';; ADDITIONAL SECTION (' .
				$header->arcount . ' record' .
				(($header->arcount == 1) ? '' : 's') .
				")\n";
		}

		$this->additional = array();
		for ($ctr = 0; $ctr < $header->arcount; $ctr++) {
			list($rrobj, $offset) = $this->parse_rr($data, $offset);

			if (is_null($rrobj)) {
				$this->header = $header;
				return null;
			}
			array_push($this->additional, $rrobj);
			if ($this->debug) {
				$rrobj->display();
			}
		}

		$this->header = $header;
		return true;
	}


	/**
	 * dummy docblock
	 */
	function data()
	{
		$data = $this->header->data();

		$qdcount = $this->header->qdcount;
		$ancount = $this->header->ancount;
		$nscount = $this->header->nscount;
		$arcount = $this->header->arcount;
		for ($ctr = 0; $ctr < $qdcount; $ctr++) {
			$data .= $this->question[$ctr]->data($this, strlen($data));
		}

		for ($ctr = 0; $ctr < $ancount; $ctr++) {
			$data .= $this->answer[$ctr]->data($this, strlen($data));
		}

		for ($ctr = 0; $ctr < $nscount; $ctr++) {
			$data .= $this->authority[$ctr]->data($this, strlen($data));
		}

		for ($ctr = 0; $ctr < $arcount; $ctr++) {
			$data .= $this->additional[$ctr]->data($this, strlen($data));
		}

		return $data;
	}


	/**
	 * dummy docblock
	 */
	function dn_comp($name, $offset)
	{
		$names = explode('.', $name);
		$compname = '';
		while (count($names)) {
			$dname = join('.', $names);
			if (isset($this->compnames[$dname])) {
				$compname .= pack('n', 0xc000 | $this->compnames[$dname]);
				break;
			}

			$this->compnames[$dname] = $offset;
			$first = array_shift($names);
			$length = strlen($first);
			$compname .= pack('Ca*', $length, $first);
			$offset += $length + 1;
		}
		if (!count($names)) {
			$compname .= pack('C', 0);
		}
		return $compname;
	}


	/**
	 * dummy docblock
	 */
	function dn_expand($packet, $offset)
	{
		$packetlen = strlen($packet);
		$name = '';
		while (1) {
			if ($packetlen < ($offset + 1)) {
				return array(null, null);
			}

			$a = unpack("@$offset/Cchar", $packet);
			$len = (int)$a['char'];

			if ($len == 0) {
				$offset++;
				break;
			} else if (($len & 0xc0) == 0xc0) {
				if ($packetlen < ($offset + Net_DNS_Packet::INT16SZ)) {
					return array(null, null);
				}
				$ptr = unpack("@$offset/ni", $packet);
				$ptr = $ptr['i'];
				$ptr = $ptr & 0x3fff;
				$name2 = Net_DNS_Packet::dn_expand($packet, $ptr);

				if (is_null($name2[0])) {
					return array(null, null);
				}
				$name .= $name2[0];
				$offset += Net_DNS_Packet::INT16SZ;
				break;
			} else {
				$offset++;

				if ($packetlen < ($offset + $len)) {
					return array(null, null);
				}

				$elem = substr($packet, $offset, $len);
				$name .= $elem . '.';
				$offset += $len;
			}
		}
		$name = preg_replace('/\.$/', '', $name);
		return array($name, $offset);
	}


	/**
	 * dummy docblock
	 */
	function label_extract($packet, $offset)
	{
		$packetlen = strlen($packet);
		$name = '';
		if ($packetlen < ($offset + 1)) {
			return array(null, null);
		}

		$a = unpack("@$offset/Cchar", $packet);
		$len = $a['char'];
		$offset++;

		if ($len + $offset > $packetlen) {
			$name = substr($packet, $offset);
			$offset = $packetlen;
		} else {
			$name = substr($packet, $offset, $len);
			$offset += $len;
		}
		return array($name, $offset);
	}


	/**
	 * dummy docblock
	 */
	function parse_question($data, $offset)
	{
		list($qname, $offset) = $this->dn_expand($data, $offset);
		if (is_null($qname)) {
			return array(null, null);
		}

		if (strlen($data) < ($offset + 4)) {
			return array(null, null);
		}

		$q = unpack("@$offset/n2int", $data);
		$qtype = $q['int1'];
		$qclass = $q['int2'];
		$offset += 4;

		$qtype = Net_DNS::typesbyval($qtype);
		$qclass = Net_DNS::classesbyval($qclass);

		$q = new Net_DNS_Question($qname, $qtype, $qclass);
		return array($q, $offset);
	}


	/**
	 * dummy docblock
	 */
	function parse_rr($data, $offset)
	{
		list($name, $offset) = $this->dn_expand($data, $offset);
		if ($name === null) {
			return array(null, null);
		}

		if (strlen($data) < ($offset + 10)) {
			return array(null, null);
		}

		$a = unpack("@$offset/n2tc/Nttl/nrdlength", $data);
		$type = $a['tc1'];
		$class = $a['tc2'];
		$ttl = $a['ttl'];
		$rdlength = $a['rdlength'];

		$type = Net_DNS::typesbyval($type);
		$class = Net_DNS::classesbyval($class);

		$offset += 10;
		if (strlen($data) < ($offset + $rdlength)) {
			return array(null, null);
		}

		$rrobj = &Net_DNS_RR::factory(array($name,
					$type,
					$class,
					$ttl,
					$rdlength,
					$data,
					$offset));

		if (is_null($rrobj)) {
			return array(null, null);
		}

		$offset += $rdlength;

		return array($rrobj, $offset);
	}


	/**
	 * dummy docblock
	 */
	function display()
	{
		echo $this->string();
	}



	/**
	 * dummy docblock
	 */
	function string()
	{
		$retval = '';
		if ($this->answerfrom) {
			$retval .= ';; Answer received from ' . $this->answerfrom . '(' .
				$this->answersize . " bytes)\n;;\n";
		}

		$retval .= ";; HEADER SECTION\n";
		$retval .= $this->header->string();
		$retval .= "\n";

		$section = ($this->header->opcode == 'UPDATE') ? 'ZONE' : 'QUESTION';
		$retval .= ";; $section SECTION (" . $this->header->qdcount .
			' record' . ($this->header->qdcount == 1 ? '' : 's') .
			")\n";

		foreach ($this->question as $qr) {
			$retval .= ';; ' . $qr->string() . "\n";
		}

		$section = ($this->header->opcode == 'UPDATE') ? 'PREREQUISITE' : 'ANSWER';
		$retval .= "\n;; $section SECTION (" . $this->header->ancount .
			' record' . ($this->header->ancount == 1 ? '' : 's') .
			")\n";

		if (is_array($this->answer)) {
			foreach ($this->answer as $ans) {
				$retval .= ';; ' . $ans->string() . "\n";
			}
		}

		$section = ($this->header->opcode == 'UPDATE') ? 'UPDATE' : 'AUTHORITY';
		$retval .= "\n;; $section SECTION (" . $this->header->nscount .
			' record' . ($this->header->nscount == 1 ? '' : 's') .
			")\n";

		if (is_array($this->authority)) {
			foreach ($this->authority as $auth) {
				$retval .= ';; ' . $auth->string() . "\n";
			}
		}

		$retval .= "\n;; ADDITIONAL SECTION (" . $this->header->arcount .
			' record' . ($this->header->arcount == 1 ? '' : 's') .
			")\n";

		if (is_array($this->additional)) {
			foreach ($this->additional as $addl) {
				$retval .= ';; ' . $addl->string() . "\n";
			}
		}

		$retval .= "\n\n";
		return $retval;
	}

}

?>
