<?php
/**
 * Net_DNS: A resolver library for PHP
 *
 * @author {@link http://wonderwinds.com/ Ed Bennett}
 * @author Eric Kilfoil <eric@ypass.net>
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.php.net/license/3_01.txt The PHP License
 * @package external
 * @subpackage Net_DNS
 */

/**
 * @package external
 * @subpackage Net_DNS
 */
class Net_DNS_RR_LOC extends Net_DNS_RR
{
	var $reference_alt = 10000000;
	var $reference_latlon = 2147483648;
	var $conv_sec = 1000;
	var $conv_min = 60000;
	var $conv_deg = 3600000;
	var $default_min = 0;
	var $default_sec = 0;
	var $default_size = 1;
	var $default_horiz_pre = 10000;
	var $default_vert_pre = 10;
	var $data;
	var $offset;
	var $raw_latitude;
	var $raw_longitude;
	var $raw_alt;
	var $size;
	var $hp;
	var $vp;
	var $altitude;
	var $degree_latitude;
	var $degree_longitude;
	var $min_latitude;
	var $min_longitude;
	var $sec_latitude;
	var $sec_longitude;
	var $ns_hem;
	var $ew_hem;
	var $pretty_print_string;
	var $parsed;
	var $version;

	/**
	 * dummy docblock
	 */
	function Net_DNS_RR_LOC($rro, $data, $offset = 0)
	{
		// Keep all of the common fields.
		$this->name = $rro->name;
		$this->type = $rro->type;
		$this->class = $rro->class;
		$this->ttl = $rro->ttl;
		$this->rdlength = $rro->rdlength;
		$this->rdata = $rro->rdata;
		
		// And keep the actual data.
		$this->data = $data;
		$this->offset = $offset;
	}


	/**
	 * dummy docblock
	 */
	function parse() {
		if (isset($this->offset) && isset($this->data) && !($this->parsed)) {
			if ($this->rdlength > 0) {
				$off = $this->offset;
				
				$a = unpack(
					"@$off/Cversion/Csize/Choriz_pre/Cvert_pre/Nlat/Nlong/Nalt", 
					$this->data
				);

				$this->version = $a['version'];
				$this->size = $this->precsize_ntoval($a['size']);
				$this->hp = $this->precsize_ntoval($a['horiz_pre']);
				$this->vp = $this->precsize_ntoval($a['vert_pre']);

				// If these are all 0, use the defaults.
				if (!$this->size) {
					$this->size = $this->default_size;
				}
					
				if (!$this->hp) {
					$this->hp = $this->default_horiz_pre;
				}
				 
				if (!$this->vp) {
					$this->vp = $this->default_vert_pre;
				}

				$this->raw_latitude = $a['lat'];
				$this->raw_longitude = $a['long'];
				$this->raw_alt = $a['alt'];
				$this->altitude = ($this->raw_alt - $this->reference_alt) / 100;
				
				$this->pretty_print_string = 
					$this->latlon2dms($this->raw_latitude, "NS", true) . ' ' .
					$this->latlon2dms($this->raw_longitude, "EW", false) . ' ' .
					$this->altitude . 'm ' .
					$this->size . 'm ' .
					$this->hp . 'm ' .
					$this->vp . 'm';
				
				$this->parsed = true;
			}
		}
	}


	/**
	 * dummy docblock
	 */
	function rdatastr()
	{
		if (!$this->parsed) {
			$this->parse_data();
		}
		
		if ($this->pretty_print_string) {
			return $this->pretty_print_string;
		}
		
		return '; no data';
	}


	/**
	 * dummy docblock
	 */
	function rr_rdata($packet, $offset)
	{
		if (!$this->parsed) {
			$this->parse_data();
		}
 
		$rdata = "";
		if (isset($this->version)) {
			$rdata .= pack("C", $this->version);
			if ($this->version == 0) {
				$rdata .= pack(
					"C3", $this->precsize_valton($this->size),
					$this->precsize_valton($this->hp),
					$this->precsize_valton($this->vp)
				);
				$rdata .= pack(
					"N3", 
					$this->raw_latitude,
					$this->raw_longitude,
					$this->raw_alt
				);
			} else {
				// We don't know how to handle other versions.
			}
		}
		return $rdata;
	}


	/**
	 * dummy docblock
	 */
	function precsize_ntoval($prec) 
	{
		$mantissa = (($prec >> 4) & 0x0f) % 10;
		$exponent = ($prec & 0x0f) % 10;
		return $mantissa * $poweroften[$exponent];
	}


	/**
	 * dummy docblock
	 */
	function precsize_valton($val) 
	{
		$exponent = 0;
		while ($val >= 10) {
			$val /= 10;
			++$exponent;
		}
		return (intval($val) << 4) | ($exponent & 0x0f);
	}


	/**
	 * dummy docblock
	 */
	function latlon2dms($rawmsec, $hems, $is_lat = false) 
	{
		// Adjust for hemisphere problems (E and N can have negative values, 
		// which need to be corrected for).
		$flipped = false;
		if ($rawmsec < 0) {
			$rawmsec = -1 * $rawmsec;
			$flipped = true;
		}
	 
		$abs = abs($rawmsec - $this->reference_latlon);
		$deg = intval($abs / $this->conv_deg);
		$abs -= $deg * $this->conv_deg;
		$min = intval($abs / $this->conv_min); 
		$abs -= $min * $this->conv_min;
		$sec = intval($abs / $this->conv_sec);
		$abs -= $sec * $this->conv_sec;
		$msec = $abs;
		$hem = substr($hems, (($rawmsec >= $this->reference_latlon) ? 0 : 1), 1);
		if ($flipped) {
			$hem = substr($hems, (($rawmsec >= $this->reference_latlon) ? 1 : 0), 1);
		}

		// Save the results.
		if ($is_lat) {
			$this->degree_latitude = $deg;
			$this->min_latitude = $min;
			$this->sec_latitude = $sec;
			$this->ns_hem = $hem;
		} else {
			$this->degree_longitude = $deg;
			$this->min_longitude = $min;
			$this->sec_longitude = $sec;
			$this->ew_hem = $hem;
		}
		
		return sprintf("%d %02d %02d.%03d %s", $deg, $min, $sec, $msec, $hem);
	}

}

?>
