<?php 
/**
 * This file implements the TuringTest plugin
 *
 * @author {@link http://wonderwinds.com/ Ed Bennett}
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://creativecommons.org/licenses/by-sa/3.0/us/ Creative Commons (Attribution, Share Alike)
 * @package plugins
 * @subpackage TuringTest
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * @package plugins
 *
 * @subpackage TuringTest
 */
class turingtest_plugin extends Plugin
{
	var $author = 'EdB';
	var $author_url = 'http://wonderwinds.com/';
	var $code = 'turingtest';
	var $group = 'antispam';
	var $help_url = 'http://1912webworks.com/plugins.php/turing-test';
	var $number_of_installs = 1;
	var $priority = 10;
	var $version = '1.0';

	/**
	 * @see Plugin::PluginInit()
	 */
	function PluginInit( & $params )
	{
		$this->name = $this->T_('Turing Test');
		$this->short_desc = $this->T_('Stop spam-bots with one simple question');
		$this->long_desc = $this->T_('Stops spam-bots with a question that you provide the answer to, is configurable, and can be applied to comments and/or the registration form and/or the message form.');
	}


	/**
	 * @see Plugin::BeforeEnable()
	 */
	function BeforeEnable()
	{
		if( $error = $this->check_settings() )
		{
			return $error;
		}
		return true;
	}


	/**
	 * @see Plugin::GetDefaultSettings()
	 */
	function GetDefaultSettings( & $params )
	{
		$default_settings = array(
			'primary_start' => array(
				'label' => $this->T_('Comment Settings'),
				'type' => 'layout',
				'layout' => 'begin_fieldset',
			),
			'comment_question' => array(
				'label' => $this->T_('Question'),
				'type' => 'text',
				'defaultvalue' => '',
				'size' => 80,
				'maxlength' => 120,
			),
			'comment_answer' => array(
				'label' => $this->T_('Correct answer'),
				'type' => 'text',
				'defaultvalue' => '',
				'size' => 32,
				'maxlength' => 32,
			),
			'comment_reject' => array(
				'label' => $this->T_('Reply if wrong'),
				'type' => 'text',
				'defaultvalue' => '',
				'size' => 80,
				'maxlength' => 120,
			),
			'comment_label' => array(
				'label' => $this->T_('Answer label'),
				'type' => 'text',
				'defaultvalue' => $this->T_('Answer'),
				'size' => 16,
				'maxlength' => 16,
			),
			'comment_note' => array(
				'label' => $this->T_('note'),
				'type' => 'text',
				'defaultvalue' => $this->T_('antispam test'),
				'size' => 32,
				'maxlength' => 64,
			),
			'pass_comments' => array(
				'label' => $this->T_('Free pass'),
				'type' => 'select',
				'options' => array( 
					'noone' => $this->T_('No one'), 
					'registered' => $this->T_('Registered Members'), 
					'commenters' => $this->T_('Previous Commenters') 
				),
				'defaultvalue' => 'commenters',
			),
			'primary_end' => array( 'layout' => 'end_fieldset' ),
			'secondary_start' => array(
				'label' => $this->T_('Message Form Settings'),
				'type' => 'layout',
				'layout' => 'begin_fieldset',
			),
			'messageform_question' => array(
				'label' => $this->T_('Question'),
				'type' => 'text',
				'defaultvalue' => '',
				'size' => 80,
				'maxlength' => 120,
			),
			'messageform_answer' => array(
				'label' => $this->T_('Correct answer'),
				'type' => 'text',
				'defaultvalue' => '',
				'size' => 32,
				'maxlength' => 32,
			),
			'messageform_reject' => array(
				'label' => $this->T_('Reply if wrong'),
				'type' => 'text',
				'defaultvalue' => '',
				'size' => 80,
				'maxlength' => 120,
			),
			'messageform_label' => array(
				'label' => $this->T_('Answer label'),
				'type' => 'text',
				'defaultvalue' => $this->T_('Answer'),
				'size' => 16,
				'maxlength' => 16,
			),
			'messageform_note' => array(
				'label' => $this->T_('note'),
				'type' => 'text',
				'defaultvalue' => $this->T_('antispam test'),
				'size' => 32,
				'maxlength' => 64,
			),
			'pass_messages' => array(
				'label' => $this->T_('Free pass'),
				'type' => 'select',
				'options' => array( 
					'noone' => $this->T_('No one'), 
					'registered' => $this->T_('Registered Members'), 
					'commenters' => $this->T_('Previous Commenters') 
				),
				'defaultvalue' => 'commenters',
			),
			'secondary_end' => array( 'layout' => 'end_fieldset' ),
			'tertiary_start' => array(
				'label' => $this->T_('Registration Settings'),
				'type' => 'layout',
				'layout' => 'begin_fieldset',
			),
			'registration_question' => array(
				'label' => $this->T_('Question'),
				'type' => 'text',
				'defaultvalue' => '',
				'size' => 80,
				'maxlength' => 120,
			),
			'registration_answer' => array(
				'label' => $this->T_('Correct answer'),
				'type' => 'text',
				'defaultvalue' => '',
				'size' => 32,
				'maxlength' => 32,
			),
			'registration_reject' => array(
				'label' => $this->T_('Reply if wrong'),
				'type' => 'text',
				'defaultvalue' => '',
				'size' => 80,
				'maxlength' => 120,
			),
			'registration_label' => array(
				'label' => $this->T_('Answer label'),
				'type' => 'text',
				'defaultvalue' => $this->T_('Answer'),
				'size' => 16,
				'maxlength' => 16,
			),
			'registration_note' => array(
				'label' => $this->T_('note'),
				'type' => 'text',
				'defaultvalue' => $this->T_('antispam test'),
				'size' => 32,
				'maxlength' => 64,
			),
			'tertiary_end' => array( 'layout' => 'end_fieldset' ),
		);
		return $default_settings;
	}


	/**
	 * @see Plugin::DisplayCommentFormFieldset()
	 */
	function DisplayCommentFormFieldset( & $params )
	{
		// get out if the admin hasn't provided a question, answer, and reject notice
		if( $this->Settings->get( 'comment_question' ) == ''
			|| $this->Settings->get( 'comment_answer' ) == ''
			|| $this->Settings->get( 'comment_reject' ) == '' )
		{
			return;
		}

		$turingtest_answer = '';

		switch( $this->Settings->get( 'pass_comments' ) )
		{
			case 'registered':
			if( is_logged_in() ) {
				$turingtest_answer = $this->Settings->get( 'comment_answer' );
			}
			break;
			case 'commenters':
  		global $cookie_name;
			$commenter_cookies_exists = isset( $_COOKIE[$cookie_name] ) ? true : false;
			if( (is_logged_in()) || ( $commenter_cookies_exists ) )
			{
				$turingtest_answer = $this->Settings->get( 'comment_answer' );
			}
			break;
		}

		$Form = & $params['Form'];
		if( ! isset( $params['form_use_fieldset'] ) || $params['form_use_fieldset'] )
		{
			$Form->begin_fieldset();
		}
		echo '<strong>'.$this->Settings->get( 'comment_question' ).'</strong>';
		$Form->text_input( 'answer_turing_test', $turingtest_answer, 32, $this->Settings->get( 'comment_label' ), $this->Settings->get( 'comment_note' ), array( 'maxlength' => 32, 'class' => 'bComment' ) );
		if( ! isset( $params['form_use_fieldset'] ) || $params['form_use_fieldset'] )
		{
			$Form->end_fieldset();
		}
	}


	/**
	 * @see Plugin::AdminDispCommentFormField()
	 */
	function AdminDispCommentFormField( & $params )
	{
		$this->DisplayCommentFormFieldset( $params );
	}


	/**
	 * @see Plugin::BeforeCommentFormInsert()
	 */
	function BeforeCommentFormInsert( & $params )
	{
		if( ! empty( $params['is_preview'] ) )
		{
			return;
		}

		// get out if the admin hasn't provided a question, answer, and reject notice
		if( $this->Settings->get( 'comment_question' ) == ''
			|| $this->Settings->get( 'comment_answer' ) == ''
			|| $this->Settings->get( 'comment_reject' ) == '' )
		{
			return;
		}

		$submitted_answer = isset( $_POST['answer_turing_test'] ) ? $_POST['answer_turing_test'] : '';
		if( $submitted_answer != $this->Settings->get( 'comment_answer' ) )
		{
			$this->msg( $this->Settings->get( 'comment_reject' ), 'error' );
		}

	}


	/**
	 * @see Plugin::DisplayMessageFormFieldset()
	 */
	function DisplayMessageFormFieldset( & $params )
	{
		// get out if the admin hasn't provided a question, answer, and reject notice
		if( $this->Settings->get( 'messageform_question' ) == ''
			|| $this->Settings->get( 'messageform_answer' ) == ''
			|| $this->Settings->get( 'messageform_reject' ) == '' )
		{
			return;
		}

		$turingtest_answer = '';

		switch( $this->Settings->get( 'pass_messages' ) )
		{
			case 'registered':
			if( is_logged_in() )
			{
				$turingtest_answer = $this->Settings->get( 'messageform_answer' );
			}
			break;
			case 'commenters':
  		global $cookie_name;
			$commenter_cookies_exists = isset( $_COOKIE[$cookie_name] ) ? true : false;
			if( (is_logged_in()) || ( $commenter_cookies_exists ) )
			{
				$turingtest_answer = $this->Settings->get( 'messageform_answer' );
			}
			break;
		}

		$Form = & $params['Form'];
		if( ! isset( $params['form_use_fieldset'] ) || $params['form_use_fieldset'] )
		{
			$Form->begin_fieldset();
		}
		echo '<strong>'.$this->Settings->get( 'messageform_question' ).'</strong>';
		$Form->text_input( 'answer_tt_M', $turingtest_answer, 32, $this->Settings->get( 'messageform_label' ), $this->Settings->get( 'messageform_note' ), array( 'maxlength' => 32, 'class' => 'bComment' ) );
		if( ! isset( $params['form_use_fieldset'] ) || $params['form_use_fieldset'] )
		{
			$Form->end_fieldset();
		}

	}


	/**
	 * @see Plugin::BeforeSendMessage()
	 */
	function BeforeSendMessage( & $params )
	{
		if( ! empty( $params['is_preview'] ) )
		{
			return;
		}

		// get out if the admin hasn't provided a question, answer, and reject notice
		if( $this->Settings->get( 'messageform_question' ) == ''
			|| $this->Settings->get( 'messageform_answer' ) == ''
			|| $this->Settings->get( 'messageform_reject' ) == '' )
		{
			return;
		}

		$submitted_answer = isset( $_POST['answer_tt_M'] ) ? $_POST['answer_tt_M'] : '';
		if( $submitted_answer != $this->Settings->get( 'messageform_answer' ) )
		{
			$this->msg( $this->Settings->get( 'messageform_reject' ), 'error' );
		}

	}


	/**
	 * @see Plugin::DisplayRegisterFormFieldset()
	 */
	function DisplayRegisterFormFieldset( & $params )
	{
		// get out if the admin hasn't provided a question, answer, and reject notice
		if( $this->Settings->get( 'registration_question' ) == ''
			|| $this->Settings->get( 'registration_answer' ) == ''
			|| $this->Settings->get( 'registration_reject' ) == '' )
		{
			return;
		}

		$turingtest_answer = '';

		$Form = & $params['Form'];
		if( ! isset( $params['form_use_fieldset'] ) || $params['form_use_fieldset'] )
		{
			$Form->begin_fieldset();
		}
		echo '<strong>'.$this->Settings->get( 'registration_question' ).'</strong>';
		$Form->text_input( 'answer_tt_R', $turingtest_answer, 32, $this->Settings->get( 'registration_label' ), $this->Settings->get( 'registration_note' ), array( 'maxlength' => 32, 'class' => 'bComment' ) );
		if( ! isset( $params['form_use_fieldset'] ) || $params['form_use_fieldset'] )
		{
			$Form->end_fieldset();
		}

	}


	/**
	 * @see Plugin::RegisterFormSent()
	 */
	function RegisterFormSent( & $params )
	{
		// get out if the admin hasn't provided a question, answer, and reject notice
		if( $this->Settings->get( 'registration_question' ) == ''
			|| $this->Settings->get( 'registration_answer' ) == ''
			|| $this->Settings->get( 'registration_reject' ) == '' )
		{
			return;
		}

		$submitted_answer = isset( $_POST['answer_tt_R'] ) ? $_POST['answer_tt_R'] : '';
		if( $submitted_answer != $this->Settings->get( 'registration_answer' ) )
		{
			$this->msg( $this->Settings->get( 'registration_reject' ), 'error' );
		}
	}


	/**
	 * Check if we have settings for comments or messageform or registration before enabling
	 */
	function check_settings()
	{
		// get out if the admin hasn't provided a question, answer, and reject notice
		if( ( $this->Settings->get( 'comment_question' ) == ''
				|| $this->Settings->get( 'comment_answer' ) == ''
				|| $this->Settings->get( 'comment_reject' ) == '' )
			&& ( $this->Settings->get( 'messageform_question' ) == ''
				|| $this->Settings->get( 'messageform_answer' ) == ''
				|| $this->Settings->get( 'messageform_reject' ) == '' )
			&& ( $this->Settings->get( 'registration_question' ) == ''
				|| $this->Settings->get( 'registration_answer' ) == ''
				|| $this->Settings->get( 'registration_reject' ) == '' ) )
		{
			$error = $this->T_('You must provide a Question, Answer, and Rejection Notice for comments or messages or registration before Turing Test will enable.');
			$this->debug_log( $error );
			return $error;
		}
	}

}

?>
