<?php
/**
 * This file implements the Twitter plugin
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package plugins
 * @subpackage Twitter
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * twitter_plugin class
 * @package plugins
 * @subpackage Twitter
 */
class twitter_plugin extends Plugin
{
	var $author = 'EdB';
	var $author_url = 'http://wonderwinds.com/';
	var $code = 'twitter';
	var $group = 'pingers';
	var $help_url = 'http://1912webworks.com/plugins.php/twitter-plugin';
	var $number_of_installs = 1;
	var $priority = 50;
	var $version = '1.0';

	/**
	 * @see Plugin::PluginInit()
	 */
	function PluginInit( & $params )
	{
		$this->name = $this->T_('Twitter');
		$this->short_desc = $this->T_('Tweet when you post :)');
		$this->long_desc = $this->T_('User ID#1 must first register your installation as an application. After that each user will be able to trust the application on their profile page or the write page. When a post is made it will tweet in your name whatever you set up for the message parameter. The default is $twitcerpt$, which will create an excerpt and link that total 140 characters.');

		$this->ping_service_name = 'twitter.com';
		$this->ping_service_note = $this->T_('Update the blogger\'s twitter account with each new post.');
	}


	/**
	 * @see Plugin::BeforeEnable()
	 */
	function BeforeEnable()
	{

		if( !extension_loaded( 'curl') )
		{
			return $this->T_('The twitter plugin requires curl extension.');
		}

		return true;
	}


	/**
	 * @see Plugin::GetDbLayout()
	 */
	function GetDbLayout()
	{
		return array(
			"CREATE TABLE IF NOT EXISTS ".$this->get_sql_table( 'twitter' )." (
				t_key VARCHAR(32) NOT NULL default '',
				t_secret VARCHAR(52) NOT NULL default '',
				t_number VARCHAR(10) NOT NULL default '',
				PRIMARY KEY( t_key )
			)"
		);
	}


	/**
	 * @see Plugin::AdminAfterMenuInit()
	 */
	function AdminAfterMenuInit()
	{
		$this->register_menu_entry( $this->name );
	}


	/**
	 * @see Plugin::AdminAfterNavbarInit()
	 */
	function AdminAfterNavbarInit()
	{
		global $topleft_Menu;

		$topleft_Menu->add_menu_entries( 'tools', array(
				'twitter_sep' => array( 'separator' => true ),
				'twitter_link' => array(
					'text' => $this->name.'&hellip;',
					'href' => $this->get_tools_tab_url(),
					'title' => format_to_output( $this->short_desc, 'htmlattr' ),
				),
			)
		);
	}


	/**
	 * @see Plugin::AdminTabAction()
	 */
	function AdminTabAction( $params )
	{

		global $DB;
		global $current_User;

		// this is adding keys
		if( param( $this->get_class_id( 'what' ) ) == 'add' )
		{
			$key = param( $this->get_class_id('key') );
			$secret = param( $this->get_class_id('secret') );
			$number = param( $this->get_class_id('number') );

			if( $key == '' || $secret == '' )
			{
				$this->msg( $this->T_('The "key" and "secret" fields are required!'), 'error' );
				return;
			}

			$query = "INSERT INTO ".$this->get_sql_table('twitter')." SET t_key = '".$DB->escape( $key )."', t_secret = '".$DB->escape( $secret )."', t_number = '".$DB->escape( $number )."'";
			if( $DB->query( $query ) )
			{
				$this->msg( $this->T_('Your application keys have been added to your database!'), 'success' );
			}
			else
			{
				$this->msg( $this->T_('This is bad: we tried to add the keys to your database but failed :('), 'error' );
			}
		}

		// this is dropping keys
		if( param( $this->get_class_id( 'what' ) ) == 'reset' )
		{
			$DB->query( "TRUNCATE TABLE ".$this->get_sql_table('twitter') );
		}

		// if the keys are stored we're okay
		$this->keys_check = false;
		$query = "SELECT * FROM ".$this->get_sql_table('twitter');
		if( $results = $DB->get_results( $query ) )
		{
			$this->keys_check = true;
		}

	}


	/**
	 * @see Plugin::AdminTabPayload()
	 */
	function AdminTabPayload( $params )
	{

		global $DB;
		global $current_User;

		if( $this->keys_check )
		{
			// we have registration keys
			$query = "SELECT * FROM ".$this->get_sql_table('twitter');
			if( $app_data = $DB->get_row( $query, ARRAY_A ) )
			{
				// get the user's twitter authentication status
				$user_id = $current_User->ID;
				$status = $this->get_twitter_link( $user_id, 'status' );
				$Form = & new Form( NULL, 'twitter_keys_a', 'post', 'compact' );
				$Form->begin_form( 'fform', $this->T_('Have you authorized this app to tweet in your name, or do you want/need to?') );
				$Form->hidden_ctrl();
				$Form->hiddens_by_key( get_memorized() );
				$Form->begin_fieldset( '' );
				$Form->info_field( 'Your twitter status', $status );
				$Form->end_fieldset( 'foo' );
				$Form->end_form();

				if( $current_User->ID == '1' )
				{
					// user ID#1 can drop the keys if desired, and gets a link to the app details page
					$Form = & new Form( NULL, 'twitter_keys_b', 'post', 'compact' );
					$Form->begin_form( 'fform', $this->T_('Application Info') );
					$Form->hidden_ctrl();
					$Form->hiddens_by_key( get_memorized() );
					$Form->begin_fieldset( '' );
					$Form->info_field( 'opens new window', '<a href="http://dev.twitter.com/apps/'.$app_data['t_number'].'" target="_blank">'.$this->T_('Application Info (link launches new window)').'</a>' );
					$Form->end_fieldset( 'foo' );
					$Form->end_form();

					$Form = & new Form( NULL, 'twitter_keys_c', 'post', 'compact' );
					$Form->begin_form( 'fform', $this->T_('Drop Application Keys') );
					$Form->hidden_ctrl();
					$Form->hiddens_by_key( get_memorized() );
					$Form->hidden( $this->get_class_id().'_what', 'reset' );
					$Form->info_field( 'Warning', 'If you do this all of your bloggers will have to authenticate again.' );
					$Form->button_input( array(
						'value' => $this->T_('Wipe out my keys so I can do it again (WARNING: THIS CAN NOT BE UNDONE!)'),
					) );
					$Form->end_form();
				}
			}
		}
		elseif( $current_User->ID == '1' )
		{
			// no keys so walk through adding them
			global $basedomain;
			global $basesubpath;
			global $app_baseurl;
			global $Settings;
			global $BlogCache;

			$default_blog_on_index = $Settings->get( 'default_blog_ID' );
			$default_Blog = & $BlogCache->get_by_ID( $default_blog_on_index, false );
			$callback = $this->get_plugin_url( true ).'_callback.php';

			echo '<p>'.$this->T_('Twitter uses the OAuth protocol to authenticate and access Twitter accounts.').' '
				.$this->T_('That means you have to register your blog as an application with Twitter.').' '
				.$this->T_('Why? <strong>Because twitter says so</strong>. The link below will open a new window, but first...').'</p>';

			echo '<h3>'.$this->T_('...some things to know before you register:').'</h3>';
			echo '<ol style="list-style:decimal; margin: 0 1ex 1ex 2ex;line-height:1.3em;">';
			echo '<li>'.$this->T_('<strong>Application Icon</strong> is not required and will be shown to your bloggers when they decide to trust your app or not.').'</li>';
			echo '<li>'.$this->T_('<strong>Application Name</strong> (<em class="red">required</em>) must be "unique to twitter". Maybe your base URL minus the http part and trailing slash?');
			echo '<br /><code style="font-size:1.2em;background-color:white;margin-left:4ex;padding:2px;">'.substr( $basedomain.$basesubpath, 0, -1 ).'</code></li>';
			echo '<li>'.$this->T_('<strong>Application Description</strong> (<em class="red">required</em>). Maybe the tagline or short description from your primary blog?');
			echo '<ul style="margin: 0 1ex 1ex 2ex;line-height:1.3em;">';
			echo '<li><strong>'.$this->T_('TAGLINE').':</strong><code style="font-size:1.2em;background-color:white;margin-left:1ex;padding:2px;">'.$default_Blog->get( 'tagline' ).'</code></li>';
			echo '<li><strong>'.$this->T_('SHORT DESCRIPTION').':</strong><code style="font-size:1.2em;background-color:white;margin-left:1ex;padding:2px;">'.$default_Blog->get( 'description' ).'</code></li>';
			echo '</ul></li>';
			echo '<li>'.$this->T_('<strong>Application Website</strong> (<em class="red">required</em>) should be the same as your $app_baseurl value:');
			echo '<br /><code style="font-size:1.2em;background-color:white;margin-left:4ex;padding:2px;">'.$app_baseurl.'</code></li>';
			echo '<li>'.$this->T_('<strong>Organization</strong> is not required but should be because they assume it exists. Your bloggers will be asked to trust "application name by organization". Maybe your preferred name? Or a business name?</li>');
			echo '<li>'.$this->T_('<strong>Website</strong> is not required and hey didn\'t we just do this in step 4?');
			echo '<br /><code style="font-size:1.2em;background-color:white;margin-left:4ex;padding:2px;">'.$app_baseurl.'</code></li>';
			echo '<li>'.$this->T_('<strong>Application Type</strong> should be and needs to be <strong>Browser.</strong></li>');
			echo '<li>'.$this->T_('<strong>Callback URL</strong> is where twitter will come back to after one of your bloggers posts (and tweets) something.');
			echo '<br /><code style="font-size:1.2em;background-color:white;margin-left:4ex;padding:2px;">'.$callback.'</code></li>';
			echo '<li>'.$this->T_('<strong>Default Access type</strong> needs to be <strong>Read &amp; Write.</strong> but won\'t be so make sure you set it correctly.</li>');
			echo '<li>'.$this->T_('Do their stupid captcha thing, press the "Save" button, then you can copy/paste some info back to this form.</li>');
			echo '</ol>';

			echo '<h4 style="text-align:center"><a href="http://twitter.com/oauth_clients/new" target="_blank">[_Register your installation as a new Application_]</a></h4>';

			$Form = & new Form( NULL, 'twitter_keys', 'post', 'compact' );
			$Form->begin_form( 'fform', $this->T_('Add twitter keys') );
			$Form->hidden_ctrl();
			$Form->hiddens_by_key( get_memorized() );
			$Form->hidden( $this->get_class_id().'_what', 'add' );
			$Form->text_input( $this->get_class_id().'_key', '', '32', $this->T_('Consumer key'), $this->T_('<em class="red">required</em>: copy/paste from twitter'), array( 'maxlength' => '32' ) );
			$Form->text_input( $this->get_class_id().'_secret', '', '52', $this->T_('Consumer secret'), $this->T_('<em class="red">required</em>: copy/paste from twitter'), array( 'maxlength' => '52' ) );
			$Form->text_input( $this->get_class_id().'_number', '', '52', $this->T_('Application ID#'), $this->T_('<em class="green">optional</em>: find this in the URL if registration is successful'), array( 'maxlength' => '52' ) );
			$Form->button_input( array(
				'value' => $this->T_('Roll them bones'),
			) );
			$Form->end_form();
		}
		else
		{
			// other users get nothing
			return;
		}

	}


	/**
	 * @see Plugin::GetDefaultSettings()
	 */
	function GetDefaultSettings( & $params )
	{
		global $DB;
		// if the keys are stored we don't need settings
		$query = "SELECT * FROM ".$this->get_sql_table('twitter');
		if( $results = $DB->get_results( $query ) )
		{
			return;
		}

		$default_settings = array(
			'twitter_contact' => array(
				'label' => $this->T_('Instructions'),
				'type' => 'info',
				'info' => $this->T_('1) Hit the "Tools -> Twitter Keys" page first to register your installation.<br />2) Each blogger will have to approve this application on their profile page.'),
			),
		);
		return $default_settings;
	}


	/**
	 * @see Plugin::GetDefaultUserSettings()
	 */
	function GetDefaultUserSettings( & $params )
	{
		global $current_User;

		$user_id = 0;
		$info = NULL;
		if( is_logged_in() )
		{
			$user_id = $current_User->ID;
		}
		if( $user_id )
		{
			// initialize info only once, when needs to display the link (user_id is set)
			$info = $this->get_twitter_link( $user_id );
			$user_settings = array(
				'twitter_contact' => array(
					'label' => $this->T_('Twitter account status'),
					'type' => 'info',
					'info' => $info,
				),
				'twitter_msg_format' => array(
					'label' => $this->T_('Message format'),
					'type' => 'text',
					'defaultvalue' => $this->T_('$twitcerpt$'),
					'maxlength' => 140,
					'size' => 30,
					'note' => $this->T_('$twitcerpt$ is an excerpt + URL that totals 140 characters'),
				),
			);
			return $user_settings;
		}
	}


	/**
	 * @see Plugin::ItemSendPing()
	 */
	function ItemSendPing( & $params )
	{
		global $DB;
		// if the keys are not stored we get out
		$query = "SELECT * FROM ".$this->get_sql_table('twitter');
		if( ! $results = $DB->get_results( $query ) )
		{
			return false;
		}

		// get the keys
		$query = "SELECT * FROM ".$this->get_sql_table('twitter');
		if( $app_data = $DB->get_row( $query, ARRAY_A ) )
		{
			$consumer_key = $app_data['t_key'];
			$consumer_secret = $app_data['t_secret'];
		}

		// get twitter account info
		$oauth_token = $this->UserSettings->get( 'twitter_token' );
		$oauth_token_secret = $this->UserSettings->get( 'twitter_secret' );
		if( empty( $oauth_token ) || empty( $oauth_token_secret ) )
		{
			// no twitter account :(
			$params['xmlrpcresp'] = $this->T_('You must configure a twitter username/password before you can post to twitter.');
			return false;
		}
		else
		{
			// get the message format
			$msg = $this->UserSettings->get( 'twitter_msg_format' );
		}

		// converitate the $message$ bits
		$title =  $params['Item']->dget( 'title', 'xml' );
		$excerpt =  $params['Item']->dget( 'excerpt', 'xml' );
		$url = $params['Item']->get_permanent_url();
		$short_url = $params['Item']->Blog->gen_blogurl().'?p='.$params['Item']->ID;
		$clip_to = 140 - ( app_strlen( $short_url ) + 4 );
		if( $excerpt != '' )
		{
			// if we have an excerpt then use it
			$twitcerpt = trim( app_substr( $excerpt, 0, $clip_to ) ).'... '.$short_url;
		}
		else
		{
			// no excerpt? use the title
			$twitcerpt = trim( app_substr( $title, 0, $clip_to ) ).'... '.$short_url;
		}

		$msg = str_replace( '$title$', $title, $msg );
		$msg = str_replace( '$excerpt$', $excerpt, $msg );
		$msg = str_replace( '$url$', $url, $msg );
		$msg = str_replace( '$twitcerpt$', $twitcerpt, $msg );

		require_once 'twitteroauth/twitteroauth.php';
		$connection = new TwitterOAuth( $consumer_key, $consumer_secret, $oauth_token, $oauth_token_secret );

		$result = $connection->post('statuses/update', array( 'status' => $msg ));

		if( empty( $result ) )
		{
			return false;
		}

		$account = $connection->get( 'account/verify_credentials' );

		$params['xmlrpcresp'] = $this->T_('Posted to account: @').$account->screen_name;

		return true;
	}


	/**
	 * @see Plugin::AdminDisplayFieldsetSidebar()
	 */
	function AdminDisplayFieldsetSidebar( & $params )
	{
		// use cat_ID to get blog_ID to get pingers...
		global $DB;
		$main_cat_ID = $params['Item']->main_cat_ID;
		$query = "SELECT cat_blog_ID FROM T_categories WHERE cat_ID = '{$main_cat_ID}'";
		$this_blog_id = $DB->get_var( $query );
		$query = "SELECT cset_value
			FROM T_blog_settings
			WHERE cset_blog_ID ='{$this_blog_id}'
			AND cset_name = 'ping_plugins'";
		$ping_string = $DB->get_var( $query );
		$ping_plugins = array_unique( explode( ',', $ping_string ) );

		// get out if this blog doesn't have this pinger
		if( ! in_array( $this->code, $ping_plugins ) )
		{
			return;
		}

		// get the user's twitter authentication status
		global $current_User;
		$user_id = $current_User->ID;
		$status = $this->get_twitter_link( $user_id, 'status' );

		$params['Form']->begin_fieldset( $this->T_('twitter status') );
		$params['Form']->info_field( '', $status );
		$params['Form']->end_fieldset( 'foo' );

	}


	/**
	 * Get link to twitter oAuth
	 */
	function get_twitter_link( $user_id, $give = 'info' )
	{
		global $DB;
		// if the keys are not stored we get out
		$query = "SELECT * FROM ".$this->get_sql_table('twitter');
		if( ! $results = $DB->get_results( $query ) )
		{
			if( $give == 'info' )
			{
				return $this->T_('This installation has not registered as an application with twitter yet.<br />You will not be able to tweet when you post until this installation is registered.');
			}
			elseif( $give == 'status' )
			{
				return $this->T_('This installation is not registered!');
			}
		}

		// get the keys
		$query = "SELECT * FROM ".$this->get_sql_table('twitter');
		if( $app_data = $DB->get_row( $query, ARRAY_A ) )
		{
			$consumer_key = $app_data['t_key'];
			$consumer_secret = $app_data['t_secret'];
		}

		require_once 'twitteroauth/twitteroauth.php';
		global $BlogCache;

		// get and set UserSettings
		if( empty( $this->UserSettings ) )
		{
			if( $give == 'info' )
			{
				return NULL;
			}
		}

		$oauth_token = $this->UserSettings->get( 'twitter_token', $user_id );
		if( ! empty( $oauth_token ) )
		{
			// user has already approved app, get token secret
			$oauth_token_secret = $this->UserSettings->get( 'twitter_secret', $user_id );
			$connection = new TwitterOAuth( $consumer_key, $consumer_secret, $oauth_token, $oauth_token_secret );
			// get linked user account
			$account = $connection->get( 'account/verify_credentials' );

			// if they turn off approval via twitter forget everything
			if( isset( $account->error ) )
			{
				global $Session;
				$Session->delete( 'oauth_token' );
				$Session->delete( 'oauth_token_secret' );
				$Session->dbsave();
			}
			else
			{
				$result = $this->T_('Linked to').': @'.$account->screen_name;
			}

		}

		// create new connection
		$connection = new TwitterOAuth( $consumer_key, $consumer_secret );

		// set callback params
		$callback = $this->get_plugin_url( true ).'_callback.php';

		$req_token = $connection->getRequestToken( $callback );

		$token = $req_token['oauth_token'];

		// Save temporary credentials to session
		global $Session;
		$Session->delete( 'oauth_token' );
		$Session->delete( 'oauth_token_secret' );
		$Session->set( 'user_id', $user_id );
		$Session->set( 'plugin_id', $this->ID );
		$Session->set( 'oauth_token', $req_token['oauth_token'] );
		$Session->set( 'oauth_token_secret', $req_token['oauth_token_secret'] );
		$Session->set( 'consumer_key', $consumer_key );
		$Session->set( 'consumer_secret', $consumer_secret );

		// info so that we can return to the write page
		$redir = 'foo';
		if( $give == 'status' )
		{
			$this_action = isset( $_GET['action'] ) ? $_GET['action'] : '';
			if( $this_action == 'edit' )
			{
				$redir = 'ctrl=items&action=edit&p='.$_GET['p'];
			}
			elseif( $this_action == 'new' )
			{
				$redir = 'ctrl=items&action=new&blog='.$_GET['blog'];
			}
		}

		$Session->set( 'redir', $redir );
		$Session->dbsave();

		if( empty( $result ) )
		{
			// wasn't linked to twitter
			$result = '<a href="'.$connection->getAuthorizeURL( $req_token, false ).'" title="'.$this->T_('sign in with twitter').'"><img src="'.$this->get_plugin_url( true ).'twitter_button.png" width="151" height="24" alt="'.T_('sign in with twitter').'" /></a>';
		}
		else
		{
			// was linked so show it with an option to switch on user profile page
			if( $give == 'info' )
			{
				$result = $result.' <a href='.$connection->getAuthorizeURL( $req_token, false ).'>'.$this->T_('Link to another account').'</a>';
			}
		}

		return $result;
	}

}

?>
