<?php
/**
 * This file implements the File view (including resizing of images)
 *
 * @todo (0000) dh> Add support for ETag / If-Modified-Since. Maybe send "Expires",
 * too? (to "force" caching). fp> for more efficient caching (like creating a thumbnail
 * on view 1 then displaying the thumbnail again on view 2), this should probably
 * redirect to the static file right after creating it (when $public_access_to_media=true
 * OF COURSE). dh> this would add another redirect/HTTP request and no cache handling,
 * assuming that the server is not configured for smart caching. Additionally, it
 * does not help for non-public access, which is the meat of this file. I've added
 * "Expires: in ten years" now, but not for thumbs (see comment there).
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package files
 */

/**
 * Load the configuration minimum
 */
require_once dirname(__FILE__).'/../qp_config/_config.php';

/**
 * Main initialization
 *
 * Not _blog_main.inc.php because we are not trying to initialize any particular blog
 */
require_once $inc_path.'/_main.inc.php';

// Check permission:
if( ! $public_access_to_media )
{
	if( ! isset( $current_User ) )
	{
		debug_die( 'No permission to get file (not logged in)!', array( 'status' => '403 Forbidden' ) );
	}
	$current_User->check_perm( 'files', 'view', true );
	// fp> TODO: check specific READ perm for requested fileroot
}

// load params
param( 'root', 'string', true ); // root directory from the dropdown box (user_X or blog_X; X is ID - 'user' for current user (default))
param( 'path', 'string', true );
param( 'size', 'string', NULL ); // Can be used for images.
param( 'mtime', 'integer', 0 ); // used for unique URLs (that never expire).

if( false !== strpos( urldecode( $path ), '../' ) )
{
	debug_die( 'Relative pathnames not allowed!' );
}

// load FileRoot info
$FileRootCache = & get_Cache( 'FileRootCache' );
$FileRoot = & $FileRootCache->get_by_ID( $root );

// load File object (not the file content)
$File = new File( $FileRoot->type, $FileRoot->in_type_ID, $path );

if( ! empty( $size ) && $File->is_image() )
{
	// We want a thumbnail, this will do all the magic
	$File->thumbnail( $size );
	// fp> TODO: for more efficient caching, this should probably redirect to the static
	// file right after creating it (when $public_access_to_media=true OF COURSE)
	// @todo (0000) dh> it would be nice, if the above would not do all the magic, but
	// return image data and error code instead, allowing us to send "Expires" for given
	// "mtime" like below. We do not want to send it for error images after all.
}
else
{
	// We want the regular file, headers to display the file directly in the browser
	if( ! is_readable( $File->get_full_path() ) )
	{
		debug_die( sprintf( 'File "%s" is not readable!', rel_path_to_base( $File->get_full_path() ) ) );
	}

	$Filetype = & $File->get_Filetype();
	if( ! empty( $Filetype ) )
	{
		header( 'Content-type: '.$Filetype->mimetype );
		if( $Filetype->viewtype == 'download' )
		{
			header( 'Content-disposition: attachment; filename="'
				.addcslashes( $File->get_name(), '\\"' ).'"' ); // escape quotes and slashes, according to RFC
		}
	}
	$file_path = $File->get_full_path();
	header( 'Content-Length: '.filesize( $file_path ) );

	// The URL refers to this specific file, therefore we can tell the browser that it does not expire anytime soon
	if( $mtime && $mtime == $File->get_lastmod_ts() )
	{
		header('Expires: '.date( 'r', time()+31536000 ) ); // 86400*365 (1 year)
	}

	// Display the content of the file
	readfile( $file_path );
}

?>
