<?php
/**
 * This file registers a new user
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package admin
 */

/**
 * Load the configuration minimum
 */
require_once dirname(__FILE__).'/../qp_config/_config.php';

/**
 * Main initialization, without a specific blog identified
 */
require_once $inc_path.'_main.inc.php';

// Login is not required on the register page
$login_required = false;

param( 'action', 'string', '' );
param( 'login', 'string', '' );
param( 'email', 'string', '' );
// do not default to $admin_url; "empty" gets handled better in the end (uses $blogurl, if no admin perms)
param( 'redirect_to', 'string', '' );

if( ! $Settings->get( 'newusers_canregister' ) )
{
	$action = 'disabled';
}

switch( $action )
{
	case 'register':
	// Do the registration
	param( 'pass1', 'string', '' );
	param( 'pass2', 'string', '' );

	// Call plugin event to allow catching input in general and validating own things from
	// DisplayRegisterFormFieldset event
	$Plugins->trigger_event( 'RegisterFormSent', array(
		'login' => & $login,
		'email' => & $email,
		'locale' => & $locale,
		'pass1' => & $pass1,
		'pass2' => & $pass2,
	) );

	if( $Messages->count( 'error' ) )
	{
		// a Plugin has added an error
		break;
	}

	// Check profile params
	profile_check_params( array(
		'login' => $login,
		'pass1' => $pass1,
		'pass2' => $pass2,
		'email' => $email,
		'pass_required' => true,
	) );

	// all logins to be lowercase to guarantee uniqueness regardless of the database case handling for UNIQUE indexes
	$login_raw = $login;
	$login = strtolower( $login );

	$UserCache = & get_Cache( 'UserCache' );
	if( $UserCache->get_by_login( $login ) )
	{
		// The login is already registered
		param_error( 'login', sprintf( T_('The login &laquo;%s&raquo; is already registered, please choose another one.'), $login ) );
	}

	if( $Messages->count( 'error' ) )
	{
		break;
	}

	$DB->begin();

	$new_User = new User();
	$new_User->set( 'login', $login );
	$new_User->set( 'pass', md5( $pass1 ) ); // encrypted
	$new_User->set( 'nickname', $login_raw );
	$new_User->set_email( $email );
	$new_User->set( 'ip', $Hit->IP );
	$new_User->set( 'domain', $Hit->get_remote_host( true ) );
	$new_User->set( 'browser', substr( $Hit->get_user_agent(), 0, 200 ) );
	$new_User->set_datecreated( $localtimenow );
	$new_User->set( 'locale', $locale );
	$newusers_grp_ID = $Settings->get( 'newusers_grp_ID' );
	$GroupCache = & get_Cache( 'GroupCache' );
	$new_user_Group = & $GroupCache->get_by_ID( $newusers_grp_ID );
	$new_User->set_Group( $new_user_Group );

	// Determine if the user must validate before using the system
	$new_User->set( 'validated', ! $Settings->get( 'newusers_mustvalidate' ) );

	$new_User->dbinsert();

	// we need this to "rollback" user creation if there's no DB transaction support
	$new_user_ID = $new_User->ID;

	// Actions to be appended to the user registration transaction
	if( $Plugins->trigger_event_first_false( 'AppendUserRegistrTransact', array( 'User' => & $new_User ) ) )
	{
		$DB->rollback();

		// Delete, in case there's no transaction support
		$new_User->dbdelete( $Debuglog );

		$Messages->add( T_('No user account has been created!'), 'error' );
		break; // break out to _reg_form.php
	}

	// User created
	$DB->commit();

	$UserCache->add( $new_User );

	// Send email to admin (using his locale)
	$AdminUser = & $UserCache->get_by_ID( 1 );
	locale_temp_switch( $AdminUser->get( 'locale' ) );

	$message  = T_('New user registration on your blog').":\n"
		."\n"
		.T_('Login:')." $login\n"
		.T_('Email').": $email\n"
		."\n"
		.T_('Edit user').': '.$admin_url.'?ctrl=users&user_ID='.$new_User->ID."\n";

	send_mail( $AdminUser->get( 'email' ), NULL, T_('New user registration on your blog'), $message, $notify_from ); // ok, if this may fail..
	locale_restore_previous(); // restores previous locale
	$Plugins->trigger_event( 'AfterUserRegistration', array( 'User' => & $new_User ) );

	if( $Settings->get( 'newusers_mustvalidate' ) )
	{
		// We want that the user validates his email address
		if( $new_User->send_validate_email( $redirect_to ) )
		{
			$Messages->add( T_('An email has been sent to your email address. Please click on the link therein to validate your account.'), 'success' );
		}
		else
		{
			$Messages->add( T_('Sorry, the email with the link to validate and activate your password could not be sent.')
				.'<br />'.T_('Possible reason: the PHP mail() function may have been disabled on the server.'), 'error' );
			// @todo (0000) fp> allow to enter a different email address (just in case it's that kind of problem)
		}
	}

	// Autologin the user. This is more comfortable for the user and avoids
	// extra confusion when account validation is required.
	$Session->set_User( $new_User );

	/**
	 * Confirmation screen
	 */
	require $admintemplates_path.'login/_reg_complete.main.php';

	exit(0);
	break;

	case 'disabled':

	/**
	 * Registration disabled
	 */
	require $admintemplates_path.'login/_reg_disabled.main.php';

	exit(0);
}

/**
 * Registration form
 */
require $admintemplates_path.'login/_reg_form.main.php';

?>
