<?php
/**
 * XML-RPC APIs
 *
 * This file implements the XML-RPC handler, to be called by remote clients.
 *
 * @uses XMLRPC_for_PHP
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package xml
 */

// Disable Cookies
$_COOKIE = array();

if( ! isset($HTTP_RAW_POST_DATA) )
{
	$HTTP_RAW_POST_DATA = implode("\r\n", file('php://input'));
}
// Trim requests (used by XML-RPC library); fix for mozBlog and other cases where
// '<?xml' isn't on the very first line
$HTTP_RAW_POST_DATA = trim( $HTTP_RAW_POST_DATA );

/**
 * Load the configuration minimum
 */
require_once dirname(__FILE__).'/../qp_config/_config.php';

/**
 * Main initialization, without a specific blog identified
 */
require_once $inc_path.'_main.inc.php';

load_funcs( 'xmlrpc/model/_xmlrpc.funcs.php' );

if( CANUSEXMLRPC !== TRUE )
{
	// We cannot use XML-RPC: send an error response ( "17 Internal server error" ).
    //this should be structured as an xml response
	$errResponse = new xmlrpcresp( 0, 17, 'Cannot use XML-RPC. Probably the server is missing the XML extension. Error: '.CANUSEXMLRPC );
	die( $errResponse->serialize() );
}

// We can't display standard error messages. We must return XMLRPC responses
$DB->halt_on_error = false;
$DB->show_errors = false;

$post_default_title = ''; // posts submitted via the xmlrpc interface get that title

/**
 * Array defining the available Remote Procedure Calls
 */
$xmlrpc_procs = array();

// Load APIs:
include_once $inc_path.'xmlrpc/apis/_blogger.api.php';
include_once $inc_path.'xmlrpc/apis/_b2.api.php';
include_once $inc_path.'xmlrpc/apis/_metaweblog.api.php';
include_once $inc_path.'xmlrpc/apis/_mt.api.php';

// fp> xmlrpc.php should actually only load the function/api/plugin to execute once
// it has been identified. Maybe it would make sense to register xmlrpc apis/functions
// in a DB table (before making plugins). It would probably make sense to have *all*
// xmlrpc methods implemented as plugins (maybe 1 plugin per API; it should be possible
// to add a single func to an API with an additional plugin)

load_funcs( 'xmlrpc/model/_xmlrpcs.funcs.php' ); // This will add generic remote calls

// Set up the XML-RPC server
$s = new xmlrpc_server( $xmlrpc_procs, false );
// Use the request encoding for the response
$s->response_charset_encoding = 'auto';
// DO THE SERVING
$s->service();

?>
