<?php
/**
 * Blog Items feed, Atom format
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package xml
 * @subpackage atom
 */
if(!defined('QP_MAIN_INIT')) die('fail');

// EXTERNAL FEED PROVIDER?
$atom_redirect = $Blog->get_setting( 'atom_redirect' );
if( ! empty( $atom_redirect ) && empty( $Category ) && $redir == 'yes' )
{
	header_redirect( $atom_redirect, 301 );
	exit( 0 );
}

// Note: even if we request the same post as $Item earlier, the following will do more restrictions (dates, etc.)
// Init the MainList object:
init_MainList( $Blog->get_setting('posts_per_feed') );

// What level of detail do we want?
$feed_content = $Blog->get_setting('feed_content');
if( $feed_content == 'none' )
{
	// We don't want to provide this feed! This will normaly have been detected earlier but just for security
	debug_die( 'Feeds are disabled.');
}

if( $debug )
{
	header_content_type( 'application/xml' ); // sets charset
}
else
{
	header_content_type( 'application/atom+xml' ); // sets charset
}

echo '<?xml version="1.0" encoding="'.$io_charset.'"?'.'>';
?>
<feed xml:lang="<?php $Blog->disp( 'locale', 'xml' ) ?>" xmlns="http://www.w3.org/2005/Atom">
<title><?php
$Blog->disp( 'name', 'xml' );
// title for the current request
request_title( array(
	'title_before' => ' - ',
	'format' => 'xml',
) ); ?></title>
<link rel="alternate" type="text/html" href="<?php $Blog->disp( 'url', 'xml' ) ?>" />
<link rel="self" type="application/atom+xml" href="<?php $Blog->disp( 'atom_url', 'xmlattr' ) ?>" />
<id><?php $Blog->disp( 'atom_url', 'xmlattr' ); ?></id>
<subtitle><?php $Blog->disp( 'shortdesc', 'xml' ) ?></subtitle>
<generator uri="http://quamplures.net/" version="<?php echo $app_version; ?>"><?php echo $app_name ?></generator>
<updated><?php echo gmdate('Y-m-d\TH:i:s\Z'); ?></updated>
<?php
// loop through as long as we have an Item...
while( $Item = & mainlist_get_item() )
{
	// Load Item's creator User:
	$Item->get_creator_User();
	?>
	<entry>
	<title type="text"><?php $Item->title( array(
		'format' => 'xml',
		'link_type' => 'none',
	) ); ?></title>
	<link rel="alternate" type="text/html" href="<?php $Item->permanent_url( 'single' ) ?>" />
	<author>
		<name><?php $Item->creator_User->preferred_name( 'xml' ) ?></name>
		<?php $Item->creator_User->url( '<uri>', "</uri>\n", 'xml' ) ?>
	</author>
	<?php
	$Categories = $Item->get_Categories();
	foreach ( $Categories as $Category )
	{
		// walter> if adding categories in the feed for all categories is expensive,
		// we can add it just for the main category
		//  $Category = $Item->get_main_Category();
		//  $cat_name = $Category->dget( 'name', 'xmlattr' );
		$cat_name = $Category->dget( 'name', 'xmlattr' );
		?>
		<category term="<?php echo $cat_name; ?>" />
		<?php
	}
	?>
	<id><?php $Item->permanent_url( 'single' ) ?></id>
	<?php
	// display issue date/time of item
	$Item->issue_date( array(
		'before' => '<published>',
		'after' => '</published>',
		'date_format' => 'isoZ',
		'use_GMT' => true,
	) );
	?>
	<updated><?php $Item->mod_date( 'isoZ', true ) ?></updated>
	<?php
	if( $feed_content == 'excerpt' )
	{
		// EXCERPTS
		?>
		<content type="html"><![CDATA[<?php
			$content = $Item->get_excerpt( 'entityencoded' );
			echo make_rel_links_abs( $content );
			// Display Item footer text (text can be edited in Blog Settings):
			$Item->footer( array( 'mode' => 'xml' ) );
		?>]]></content>
		<?php
		}
	elseif( $feed_content == 'normal' || $feed_content == 'full' )
	{
		// POST CONTENTS
		?>
		<content type="html"><![CDATA[<?php
			// URL link, if the post has one
			$Item->url_link( array(
				'before' => '<p>',
				'after' => '</p>',
			) );
			// Display images that are linked to this post:
			$content = $Item->get_images( array(
				'before_image_legend' => '<div><i>',
				'after_image_legend' => '</i></div>',
				'image_size' => 'fit-320'
			), 'htmlbody' );
			$content .= $Item->get_content_teaser( 1, false );
			if( $feed_content == 'normal' )
			{
				// Teasers only
				$content .= $Item->get_more_link( array(
					'before' => '',
					'after' => '',
					'disppage' => 1,
				) );
			}
			else
			{
				// Full contents
				$content .= $Item->get_content_extension( 1, true );
			}
			echo make_rel_links_abs( $content );
			// display item's footer
			$Item->footer( array( 'mode' => 'xml' ) );
		?>]]></content>
		<?php
	}
	?>
	</entry>
	<?php
	}
?>
</feed>
<?php $Hit->log(); // log the hit on this page ?>
