<?php
/**
 * template when $disp == 'credits'
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package templates
 */
if(!defined('QP_MAIN_INIT')) die('fail');

if( $Blog->get_setting( 'credits_page' ) != 'credits' )
{
	return; // credits page not wanted
}
?>

<div id="credits">
<dl>
<?php
$credits_array = explode( ',', $Blog->get_setting( 'credit_sections' ) );

// First: template credits from the template.class file
if( isset( $Template ) && ( $credits = $Template->get_template_credits() ) && ( in_array( 'template', $credits_array ) ) )
{
	echo '<dt>'.T_('Template Credits').':</dt>'."\n";
	echo '<dd><dl>'."\n";

	foreach( $credits as $linkback )
	{
		echo '<dt>'.format_to_output( $linkback['pre'], 'htmlbody' ).'</dt>';
		echo '<dd><a href="'.format_to_output( $linkback['url'], 'htmlattr' ).'" title="';
		echo format_to_output( $linkback['title'], 'htmlattr' ).'">';
		echo format_to_output( $linkback['text'], 'text' ).'</a></dd><dd>';
		echo format_to_output( $linkback['post'], 'htmlbody' )."</dd>\n";
	}

	echo '</dl></dd>'."\n";
}

// Second: plugin credits excluding 'core' plugins
global $Plugins;

$credits = array();
$Plugins->restart();

while( ( $a_plugin = & $Plugins->get_next() ) && ( in_array( 'plugins', $credits_array ) ) )
{
	// Skip our own plugins
	if( empty( $a_plugin->name ) || $a_plugin->author == 'Dracones Incompertus' || $a_plugin->author == 'The b2evo Group' )
	{
		continue;
	}

	// plugin $name linked to $help_url (if set)
	$p_name = format_to_output( $a_plugin->name, 'htmlbody' );
	if( $a_plugin->help_url != '' )
	{
		$p_name = '<a href="'.format_to_output( $a_plugin->help_url, 'htmlattr' ).'" title="'.format_to_output( T_('visit this plugins web page'), 'htmlattr' ).'">'.$p_name.'</a>';
	}

	// plugin $author linked to $author_url (if set)
	$a_name = format_to_output( $a_plugin->author, 'htmlbody' );
	if( $a_plugin->author_url != '' )
	{
		$a_name = '<a href="'.format_to_output( $a_plugin->author_url, 'htmlattr' ).'" title="'.format_to_output( T_('visit the authors website'), 'htmlattr' ).'">'.$a_name.'</a>';
	}

	// plugin $long_desc used as description
	$credits[ strtolower( $a_plugin->name ) ] = array(
		'p_name' => $p_name,
		'a_name' => $a_name,
		'desc' => format_to_output( $a_plugin->long_desc, 'htmlbody' ),
		'extras' => $a_plugin->get_plugin_credits(),
	);
}

if( ! empty( $credits ) )
{
	ksort( $credits );
	echo '<dt>'.T_('Plugin Credits').':</dt>'."\n";
	echo '<dd><dl>'."\n";
	foreach( $credits as $credit )
	{
		echo '<dt>'.$credit['p_name'].' '.T_('by').' '.$credit['a_name'].'</dt><dd>'.$credit['desc']."</dd>\n";
		if( $credit['extras'] )
		{
			foreach( $credit['extras'] as $extra )
			{
				echo '<dd><strong><a href="'.$extra['link'].'" title="hyperlink">'.$extra['title'].'</a>:</strong> '.$extra['text']."</dd>\n";
			}
		}
	}
	echo '</dl></dd>'."\n";
}

// Third: core contributor credits
if( in_array( 'contribs', $credits_array ) )
{
	global $templates_path;

	/**
	 * Load the list of contributors for this version
	 */
	require_once $templates_path.'/_contributors.php';

	uasort( $_hic_sunt_dracones, create_function( '$a, $b', 'return strnatcasecmp( $a[\'name\'], $b[\'name\'] );' ) );

	echo '<dt>'.T_('Core Contributors').':</dt>'."\n";
	echo '<dd><dl>'."\n";
	foreach( $_hic_sunt_dracones as $dracone )
	{
		echo '<dt><a href="'.format_to_output( $dracone['homepage'], 'htmlattr' ).'" title="';
		echo format_to_output( $dracone['homepage'], 'htmlattr' ).'">';
		echo format_to_output( $dracone['name'], 'text' ).'</a></dt><dd>';
		locale_flag( $dracone['locale'], 'h10px', 'leftmargin' );
		echo format_to_output( $dracone['spiel'], 'htmlbody')."</dd>\n";
	}
	echo '</dl></dd>'."\n";
}

// Finally: QP's only linkback
if( isset( $Template ) )
{
	echo '<dt>'.format_to_output( $Template->poweredby_linkback['pre'], 'htmlbody' ).':</dt>'."\n";
	echo '<dd><dl>', "\n";
	echo '<dt><a href="'.format_to_output( $Template->poweredby_linkback['url'], 'htmlattr' ).'" title="';
	echo format_to_output( $Template->poweredby_linkback['title'], 'htmlattr' ), '">';
	echo format_to_output( $Template->poweredby_linkback['text'], 'text' ), '</a></dt><dd>';
	echo format_to_output( $Template->poweredby_linkback['post'], 'htmlbody'), "</dd>\n";
	echo '</dl></dd>', "\n";
}
?>
</dl>
</div>
