<?php
/**
 * item comment form template (sub-template)
 *
 * @todo (1111) funky form names need to go away in favor of field names unique
 * to each installation.
 * @author {@link http://wonderwinds.com/ Ed Bennett}
 * @author {@link http://astonishme.co.uk/ Yabba (Paul Jones, rest in peace)}
 * @copyright (c) 2013 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package templates
 * @subpackage pictogram
 */
if(!defined('QP_MAIN_INIT')) die('fail');

global $comment_allow_msgform;
global $comment_allowed_tags;
global $comment_cookies;
global $comments_use_autobr;
global $cookie_email;
global $cookie_name;
global $cookie_url;
global $PageCache;

// set some default params...
$params = array_merge( array(
	'disp_comment_form' => true,
	'form_title_start' => '<h3>',
	'form_title_end' => '</h3>',
	'policy_text' => '',
	'textarea_lines' => 10,
	'default_text' => '',
	'preview_start' => '<div class="bComment" id="comment_preview">',
	'comment_template' => '_item_comment.inc.php',
	'preview_end' => '</div>',
), $params );

// Comment form
if( $params['disp_comment_form'] && $Item->can_comment() )
{
	// INIT/PREVIEW
	if( $Comment = $Session->get( 'core.preview_Comment' ) )
	{
		// We have a comment to preview
		if( $Comment->item_ID == $Item->ID )
		{
			// We do not want the current rendered page to be cached!!
			$PageCache->abort_collect();

			// --- --- --- --- --- --- --- --- --- --- ---
			// --- A SINGLE COMMENT PREVIEWED HERE ---
			template_include( $params['comment_template'], array(
				'Comment' => & $Comment,
				'comment_start' => $params['preview_start'],
				'comment_end' => $params['preview_end'],
			) );

			// Form fields
			$comment_content = $Comment->original_content;
			// for visitors
			$comment_author = $Comment->author;
			$comment_author_email = $Comment->author_email;
			$comment_author_url = $Comment->author_url;
		}

		// delete any preview comment from session data
		$Session->delete( 'core.preview_Comment' );
	}
	else
	{
		// New comment
		$Comment = new Comment();
		if( $PageCache->is_collecting )
		{
			// This page is going into the cache, we don't want personal data cached!!!
			// fp> These fields should be filled out locally with Javascript tapping directly into the cookies. Anyone JS savvy enough to do that?
			$comment_author = '';
			$comment_author_email = '';
			$comment_author_url = '';
		}
		else
		{
			$comment_author = isset( $_COOKIE[$cookie_name] ) ? trim( $_COOKIE[$cookie_name] ) : '';
			$comment_author_email = isset( $_COOKIE[$cookie_email] ) ? trim( $_COOKIE[$cookie_email] ) : '';
			$comment_author_url = isset( $_COOKIE[$cookie_url] ) ? trim( $_COOKIE[$cookie_url] ) : '';
		}
		if( empty( $comment_author_url ) )
		{
			$comment_author_url = 'http://';
		}
		$comment_content =  $params['default_text'];
	}

	if( $PageCache->is_collecting )
	{
		// This page is going into the cache, we don't want personal data cached!!!
		// fp> These fields should be filled out locally with Javascript tapping directly into the cookies. Anyone JS savvy enough to do that?
	}
	else
	{
		// Get values that may have been passed through after a preview
		param( 'comment_cookies', 'integer', NULL );
		param( 'comment_allow_msgform', 'integer', NULL ); // checkbox

		if( is_null( $comment_cookies ) )
		{
			// "Remember me" checked, if remembered before
			$comment_cookies = isset( $_COOKIE[$cookie_name] ) || isset( $_COOKIE[$cookie_email] ) || isset( $_COOKIE[$cookie_url] );
		}
	}

	echo '<div class="blockwrap cmt_form">'."\n";

	$Form = new Form( $srvc_url.'comment_post.php', 'comment_form_id_'.$Item->ID, 'post' );
	$Form->begin_form( 'comment', '', array( 'target' => '_self' ) );

	// @todo (0000) dh> a plugin hook would be useful here to add something to the top of the Form.
	// Actually, the best would be, if the $Form object could be changed by a plugin before display!

	$Form->hidden( 'comment_post_ID', $Item->ID );
	$Form->hidden( 'redirect_to',
		// Make sure we get back to the right page (on the right domain)
		// @todo (0000) fp> check if we can use the permalink instead but we must check that
		// application wide, that is to say: check with the comments in a pop-up etc...
		// url_rel_to_same_host(regenerate_url( '', '', $Blog->get( 'blogurl' ), '&' ), $srvc_url)
		// fp> what we need is a regenerate_url that will work in permalinks
		// fp> below is a simpler approach:
		$Item->get_feedback_url( $disp == 'feedback-popup', '&' )
	);

	if( is_logged_in() )
	{
		echo $params['form_title_start'];
		echo T_('Leave a comment');
		echo ' '.$current_User->get_preferred_name();
		if( $Item->can_rate() )
		{
			// Comment rating
			echo '<span class="cmt_vote user-vote">'."\n";
			echo T_('Rate : ')."\n";
			// display options to rate a comment
			$Comment->rating_input( array('label_low' => 1, 'label_high' => 5) );
			echo '</span>';
		}
		echo $params['form_title_end'];
	}
	else
	{
		// User is not logged in
		// Note: we use funky field names in order to defeat the most basic guestbook spam bots
		echo '<fieldset class="visitor-details"><h3 class="blocknice">'.T_('About you').'</h3>'."\n";
		echo '<p>'.T_('None of the details below are required').'</p>'."\n";
		$Form->text( 'u', $comment_author, 40, T_('Name'), '', 100, 'cmt_input' );
		$Form->text( 'i', $comment_author_email, 40, T_('Email'), '<br />'.T_('Your email address will <strong>not</strong> be revealed on this site.'), 100, 'cmt_input' );
		$Form->text( 'o', $comment_author_url, 40, T_('Website'), '<br />'.T_('Your URL might be displayed. Spam URLs will be binned'), 100, 'cmt_input' );

		if( ! empty( $params['policy_text'] ) )
		{
			// We have a policy text to display
			$Form->info_field( '', $params['policy_text'] );
		}

		$comment_options = array();
		$comment_options[] = '<label><input type="checkbox" class="checkbox" name="comment_cookies" tabindex="7"'
			.( $comment_cookies ? ' checked="checked"' : '' ).' value="1" /> '.T_('Remember you').'</label>'
			.' <span class="note">('.T_('Name, email &amp; website').')</span>';

		if( ! empty( $comment_options ) )
		{
			echo $Form->begin_field( NULL, '', true );
			echo T_('Option :').implode( '<br />', $comment_options );
			echo $Form->end_field();
		}

		echo '</fieldset><fieldset class="visitor-comment"><h3 class="blocknice">'.T_('Your say')."\n";
		if( $Item->can_rate() )
		{
			// Comment rating
			echo '<span class="cmt_vote visitor-vote">'."\n";
			echo T_('Rate : ')."\n";
			// display options to rate a comment
			$Comment->rating_input( array('label_low' => 1, 'label_high' => 5) );
			echo '</span>';
		}
		echo '</h3>'."\n";
	}

	echo '<div class="comment_toolbars">';
	// CALL PLUGINS NOW
	$Plugins->trigger_event( 'DisplayCommentToolbar', array() );
	echo '</div>';

	// Message field
	$note = '';
	// $note = T_('Allowed XHTML tags').': '.htmlspecialchars(str_replace( '><',', ', $comment_allowed_tags));
	$Form->textarea( 'p', $comment_content, $params['textarea_lines'], '', $note, 40, 'cmt_spiel' );

	// set app_Canvas for plugins
	echo '<script type="text/javascript">var app_Canvas = document.getElementById( "p" );</script>';

	if( substr( $comments_use_autobr, 0, 4 ) == 'opt-' )
	{
		$comment_options[] = '<label><input type="checkbox" class="checkbox" name="comment_autobr" tabindex="6"'
			.( ( $comments_use_autobr == 'opt-out' ) ? ' checked="checked"' : '' )
			.' value="1" /> '.T_('Auto-BR').'</label>'
			.' <span class="note">('.T_('Line breaks become &lt;br /&gt;').')</span>';
	}

	if( ! is_logged_in() )
	{
		echo '</fieldset>'."\n";
	}

	echo '<div class="clear"></div>'."\n";

	$Plugins->trigger_event( 'DisplayCommentFormFieldset', array(
		'Form' => & $Form,
		'Item' => & $Item,
	) );

	$Form->begin_fieldset();
	echo '<div class="input input-buttons">';
	$Form->button_input( array(
		'name' => 'submit_comment_post_'.$Item->ID.'[save]',
		'class' => 'submit',
		'value' => T_('Send comment'),
		'tabindex' => 10,
	) );

	$Form->button_input( array(
		'name' => 'submit_comment_post_'.$Item->ID.'[preview]',
		'class' => 'preview',
		'value' => T_('Preview'),
		'tabindex' => 9,
	) );

	$Plugins->trigger_event( 'DisplayCommentFormButton', array(
		'Form' => & $Form,
		'Item' => & $Item,
	) );

	echo '</div>';
	$Form->end_fieldset();
	?>

	<div class="clear"></div>

	<?php
	$Form->end_form();
}

?>
</div>
