<?php
/**
 * item feedback template (sub-template)
 *
 * @author {@link http://wonderwinds.com/ Ed Bennett}
 * @author {@link http://astonishme.co.uk/ Yabba (Paul Jones, rest in peace)}
 * @copyright (c) 2013 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package templates
 * @subpackage pictogram
 */
if(!defined('QP_MAIN_INIT')) die('fail');

global $c;
global $tb;

// set some default params...
$params = array_merge( array(
	'disp_comments' => true,
	'disp_comment_form' => true,
	'disp_trackbacks' => true,
	'disp_trackback_url' => true,
	'before_section_title' => '<h3>',
	'after_section_title' => '</h3>',
	'comment_list_start' => "\n\n",
	'comment_list_end' => "\n\n",
	'comment_start' => '<div class="bComment">',
	'comment_end' => '</div>',
	'preview_start' => '<div class="bComment" id="comment_preview">',
	'preview_end' => '</div>',
	'comment_template' => '_item_comment.inc.php',
	'form_title_start' => '<h3>',
	'form_title_end' => '</h3>',
	), $params );

if( ! $Item->can_see_comments() )
{
	// Comments are disabled for this post
	return;
}

if( empty( $c ) )
{
	// Comments not requested
	$params['disp_comments'] = false;
	$params['disp_comment_form'] = false;
}

if( empty( $tb ) || ! $Blog->get( 'allowtrackbacks' ) )
{
	// Trackback not requested or not allowed
	$params['disp_trackbacks'] = false;
	$params['disp_trackback_url'] = false;
}

if( ! ( $params['disp_comments'] || $params['disp_comment_form'] || $params['disp_trackbacks'] || $params['disp_trackback_url'] ) )
{
	// Nothing more to do....
	return false;
}

echo '<a id="feedbacks"></a>';

$type_list = array();
$disp_title = array();

// We requested to display comments?
if( $params['disp_comments'] )
{
	if( $Item->can_see_comments() )
	{
		$type_list[] = "'comment'";
		if( $title = $Item->get_feedback_title( 'comments' ) )
		{
			$disp_title[] = $title;
		}
	}
	else
	{
		$params['disp_comments'] = false;
	}
	echo '<a id="comments"></a>';
}

// We requested to display trackbacks?
if( $params['disp_trackbacks'] )
{
	$type_list[] = "'trackback'";
	if( $title = $Item->get_feedback_title( 'trackbacks' ) )
	{
		$disp_title[] = $title;
	}
	echo '<a id="trackbacks"></a>';
}

// We want to display the trackback URL?
if( $params['disp_trackback_url'] )
{
	echo $params['before_section_title'];
	echo T_('Trackback address for this post');
	echo $params['after_section_title'];

	// Trigger plugin event ...
	if( ! $Plugins->trigger_event_first_true( 'DisplayTrackbackAddr', array(
		'Item' => & $Item,
		'template' => '<code>%url%</code>' ) )
	)
	{
		// if no plugin displayed a payload we just display the default
		echo '<p class="trackback_url"><a href="'.$Item->get_trackback_url().'">'.T_('Trackback URL (right click and copy shortcut/link location)').'</a></p>';
	}
}

// if we have something to display ...
if( $params['disp_comments'] || $params['disp_trackbacks'] )
{
	// ... first we make a title ...
	if( empty( $disp_title ) )
	{
		if( $title = $Item->get_feedback_title( 'feedbacks', '', T_('Feedback awaiting moderation'), T_('Feedback awaiting moderation'), 'draft' ) )
		{
			$disp_title[] = $title;
		}
	}
	// still no title?
	if( empty( $disp_title ) )
	{
		$disp_title[] = T_('No feedback yet');
	}

	// show the title :)
	echo $params['before_section_title'];
	// display link for comments feed?
	if( $Blog->get_setting( 'feed_content' ) != 'none' )
	{
		$Item->feedback_feed_link( '_rss2', '<span class="cmt_feed">', '</span> ', get_icon( 'feed', 'imgtag') );
	}
	echo implode( ', ', $disp_title);
	echo $params['after_section_title'];

	// spark off a comment list...
	$CommentList = new CommentList( NULL, implode(',', $type_list), array('published'), $Item->ID, '', 'ASC' );

	// loop through comments...
	echo $params['comment_list_start'];
	while( $Comment = & $CommentList->get_next() )
	{
		// --- --- --- --- --- --- --- --- --- --- ---
		// --- A SINGLE COMMENT INCLUDED HERE ---
		template_include( $params['comment_template'], array(
			'Comment' => & $Comment,
			'comment_start' => $params['comment_start'],
			'comment_end' => $params['comment_end'],
			'creator_user_ID' => $params['creator_user_ID'],
		) );
	}
	echo $params['comment_list_end'];

	// display count of comments to be moderated
	$Item->feedback_moderation( 'feedbacks', '<div class="moderation_msg"><p>', '</p></div>', '',
		T_('This post has 1 feedback awaiting moderation... %s'),
		T_('This post has %d feedbacks awaiting moderation... %s') );
}

// --- --- --- --- --- --- --- --- --- --- ---
// --- COMMENT FORM INCLUDED HERE ---
template_include( '_item_comment_form.inc.php', $params );
?>
