<?php
/**
 * This file implements the UI controller for System configuration and analysis
 *
 * @todo (0000) dh> output_buffering (recommend off)
 * @todo (0000) dh> session.auto_start (recommend off)
 * @todo (0000) dh> How to change ini settings in .htaccess (for mod_php), link to manual
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package tools
 */
if(!defined('QP_MAIN_INIT')) die('fail');

global $app_allow_dbase_reset;
global $app_name;
global $app_version;
global $app_date;
global $app_homepage;
global $basepath;
global $install_subdir;
global $localtimenow;
global $media_path;
global $servertimenow;

$facilitate_exploits = '<p>'.T_('When enabled, this feature is known to facilitate hacking exploits in any PHP application.')."</p>\n<p>"
	.sprintf( T_('%s includes additional measures in order not to be affected by this.'."\n".'However, for maximum security, we still recommend disabling this PHP feature.'), $app_name )."</p>\n";
$change_ini = '<p>'.T_('If possible, change this setting to <code>%s</code> in your php.ini or ask your hosting provider about it.').'</p>';

echo '<h2>'.T_('About this system').'</h2>';
$block_item_Widget = new Widget( 'block_item' );

// CORE SOFTWARE BLOCK
$block_item_Widget->title = $app_name;
$block_item_Widget->disp_template_replaced( 'block_start' );

// Version check
$app_timestamp = mysql2timestamp( $app_date );
$app_date = date_i18n( locale_datefmt(), $app_timestamp );
$ver_val = sprintf( T_('%s released on %s'), $app_version, $app_date );
init_system_check( sprintf( T_('%s version'), $app_name ), $ver_val );

// age check
$app_age = ( $localtimenow - $app_timestamp ) / 3600 / 24 / 30; // approx age in months
if( $app_age > 24 )
{
	disp_system_check( 'error', T_('This version is old. You should check for newer releases on').' <a href="'.$app_homepage.'">'.$app_name.'</a>.' );
}
elseif( $app_age > 12 )
{
	disp_system_check( 'warning', sprintf( T_('This version is aging. You may want to check for newer releases on').' <a href="'.$app_homepage.'">'.$app_name.'</a>.' ) );
}
else
{
	disp_system_check( 'ok' );
}

// Media folder writable?
list( $mediadir_status, $mediadir_msg ) = system_check_media_dir();
$mediadir_long = '';
if( $mediadir_status == 'error' )
{
	$mediadir_long = '<p>'.sprintf( T_('You will not be able to upload files/images and %s will not be able to generate thumbnails.'), $app_name )."</p>\n"
	.'<p>'.T_('Your host requires that you set special file permissions on your media directory.')."</p>\n";
}
init_system_check( T_('Media directory'), $mediadir_msg.' - '.$media_path );
disp_system_check( $mediadir_status, $mediadir_long );

// /qp_install/index.php file deleted?
$install_removed = system_check_install_removed();
init_system_check( T_('qp_install/index.php file'), $install_removed ? T_('Deleted') : T_('Not deleted').' - '.$basepath.$install_subdir );
if( ! $install_removed )
{
	disp_system_check( 'warning', T_('For maximum security, it is recommended that you delete your /qp_install/index.php file, or the entire /qp_install/ folder once you are done with install or upgrade.') );
	// Database reset allowed?
	init_system_check( T_('Database reset'), $app_allow_dbase_reset ? T_('Allowed!') : T_('Forbidden') );
	if( $app_allow_dbase_reset )
	{
		disp_system_check( 'error', '<p>'.sprintf( T_('Currently, anyone who accesses your install folder could entirely reset your %s database.'), $app_name )."</p>\n"
			.'<p>'.T_('ALL YOUR DATA WOULD BE LOST!')."</p>\n"
			.'<p>'.T_('As soon as possible, change the setting <code>$app_allow_dbase_reset = 0;</code> in your /qp_config/_basic.config.php.').'</p>' );
	}
	else
	{
		disp_system_check( 'ok' );
	}
}
else
{
	disp_system_check( 'ok' );
}

$block_item_Widget->disp_template_raw( 'block_end' );

// TIME BLOCK
$block_item_Widget->title = T_('Time');
$block_item_Widget->disp_template_replaced( 'block_start' );
init_system_check( T_('Server time'), date_i18n( locale_datetimefmt( ' - ' ), $servertimenow ) );
disp_system_check( 'note' );
init_system_check( T_('GMT / UTC time'), gmdate( locale_datetimefmt( ' - ' ), $servertimenow ) );
disp_system_check( 'note' );
init_system_check( sprintf( T_('%s time'), $app_name ), date_i18n( locale_datetimefmt( ' - ' ), $localtimenow ) );
disp_system_check( 'note' );
$block_item_Widget->disp_template_raw( 'block_end' );

// MYSQL BLOCK
$block_item_Widget->title = T_('MySQL');
$block_item_Widget->disp_template_replaced( 'block_start' );
// MySQL version
$mysql_version = $DB->get_version();
init_system_check( T_('MySQL version'), $DB->version_long );
if( version_compare( $mysql_version, '5.0' ) < 0 )
{
	disp_system_check( 'error', T_('This version is too old. The minimum recommended MySQL version is 5.0.') );
}
else
{
	disp_system_check( 'ok' );
}
// UTF8 support?
$ok = system_check_db_utf8();
init_system_check( T_('MySQL UTF-8 support'), $ok ? T_('Yes') : T_('No') );
if( ! $ok )
{
	disp_system_check( 'warning', T_('UTF-8 is not supported by your MySQL server.') ); // fp> TODO: explain why this is bad. Better yet: try to detect if we really need it, base don other conf variables.
}
else
{
	disp_system_check( 'ok' );
}
$block_item_Widget->disp_template_raw( 'block_end' );

// PHP BLOCK
$block_item_Widget->title = T_('PHP');
$block_item_Widget->disp_template_replaced( 'block_start' );
// User ID
list( $uid, $uname, $running_as ) = system_check_process_user();
init_system_check( T_('PHP running as USER:'), $running_as );
disp_system_check( 'note' );
// Group ID
list( $gid, $gname, $running_as ) = system_check_process_group();
init_system_check( T_('PHP running as GROUP:'), $running_as );
disp_system_check( 'note' );
// Version
init_system_check( T_('PHP version'), PHP_VERSION );
if( version_compare( PHP_VERSION, '5.2.0', '<' ) )
{
	disp_system_check( 'error', sprintf( T_('This version is too old. %1$s will not run correctly. You must ask your host to upgrade PHP to version 5.0.0 or higher before you can run %1$s.'), $app_name ) );
}
else
{
	disp_system_check( 'ok' );
}
// register_globals?
init_system_check( 'PHP register_globals', ini_get( 'register_globals' ) ? T_('On') : T_('Off') );
if( ini_get( 'register_globals' ) )
{
	disp_system_check( 'warning', $facilitate_exploits.' '.sprintf( $change_ini, 'register_globals = Off' ) );
}
else
{
	disp_system_check( 'ok' );
}
// allow_url_include? (since 5.2, supercedes allow_url_fopen for require()/include()
if( version_compare(PHP_VERSION, '5.2', '>=') )
{
	init_system_check( 'PHP allow_url_include', ini_get( 'allow_url_include' ) ? T_('On') : T_('Off') );
	if( ini_get( 'allow_url_include' ) )
	{
		disp_system_check( 'warning', $facilitate_exploits.' '.sprintf( $change_ini, 'allow_url_include = Off' ) );
	}
	else
	{
		disp_system_check( 'ok' );
	}
}
// allow_url_fopen
init_system_check( 'PHP allow_url_fopen', ini_get( 'allow_url_fopen' ) ? T_('On') : T_('Off') );
if( ini_get( 'allow_url_fopen' ) )
{
	disp_system_check( 'warning', $facilitate_exploits.' '.sprintf( $change_ini, 'allow_url_fopen = Off' ) );
}
else
{
	disp_system_check( 'ok' );
}
// Magic quotes
if( !strcasecmp( ini_get( 'magic_quotes_sybase' ), 'on' ) )
{
	$magic_quotes = T_('On').' (magic_quotes_sybase)';
	$message = 'magic_quotes_sybase = Off';
}
elseif( get_magic_quotes_gpc() )
{
	$magic_quotes = T_('On').' (magic_quotes_gpc)';
	$message = 'magic_quotes_gpc = Off';
}
else
{
	$magic_quotes = T_('Off');
	$message = '';
}
init_system_check( 'PHP Magic Quotes', $magic_quotes );
if( ! empty( $message ) )
{
	disp_system_check( 'warning', T_('PHP is adding extra quotes to all inputs. This leads to unnecessary extra processing.')
		.' '.sprintf( $change_ini, $message ) );
}
else
{
	disp_system_check( 'ok' );
}
// Max upload size
$upload_max_filesize = system_check_upload_max_filesize();
init_system_check( 'PHP upload_max_filesize', ini_get( 'upload_max_filesize' ) );
disp_system_check( 'ok' );
// Max post size
$post_max_size = system_check_post_max_size();
init_system_check( 'PHP post_max_size', ini_get( 'post_max_size' ) );
if( $post_max_size > $upload_max_filesize )
{
	disp_system_check( 'ok' );
}
else
{
	disp_system_check( 'error', T_('post_max_size should be larger than upload_max_filesize') );
}
// Memory limit
$memory_limit = system_check_memory_limit();
if( empty( $memory_limit ) )
{
	init_system_check( 'PHP memory_limit', T_('n.a.') );
	disp_system_check( 'note' );
}
else
{
	init_system_check( 'PHP memory_limit', ini_get( 'memory_limit' ) );
	if( $memory_limit < 8096 )
	{
		disp_system_check( 'error', sprintf( T_('The memory_limit is very low. Some features of %s will fail to work.'), $app_name ) );
	}
	elseif( $memory_limit < 12288 )
	{
		disp_system_check( 'warning', sprintf( T_('The memory_limit is low. Some features of %s may fail to work.'), $app_name ) );
	}
	else
	{
		disp_system_check( 'ok' );
	}
}
// mbstring extension
init_system_check( T_('PHP mbstring extension'), extension_loaded( 'mbstring' ) ? T_('Loaded') : T_('Not loaded') );
if( ! extension_loaded( 'mbstring' ) )
{
	disp_system_check( 'warning', sprintf( T_('%s will not be able to convert character sets and special characters/languages may not be displayed correctly. Enable the mbstring extension in your php.ini file or ask your hosting provider about it.'), $app_name ) );
}
else
{
	disp_system_check( 'ok' );
}
// XML extension
init_system_check( T_('PHP XML extension'), extension_loaded('xml') ? T_('Loaded') : T_('Not loaded') );
if( ! extension_loaded('xml' ) )
{
	disp_system_check( 'warning', T_('The XML extension is not loaded.') ); // fp> This message only repeats the exact same info that is already displayed. Not helpful.
}
else
{
	disp_system_check( 'ok' );
}
// IMAP extension
$imap_loaded = extension_loaded( 'imap' );
init_system_check( T_('PHP IMAP extension'), $imap_loaded ? T_('Loaded') : T_('Not loaded') );
if( ! $imap_loaded )
{
	disp_system_check( 'warning', sprintf( T_('You will not be able to use the Blog by email feature of %s. Enable the IMAP extension in your php.ini file or ask your hosting provider about it.'), $app_name ) );
}
else
{
	disp_system_check( 'ok' );
}
$block_item_Widget->disp_template_raw( 'block_end' );

// GD LIBRARY BLOCK
$block_item_Widget->title = T_('GD Library (image handling)');
$block_item_Widget->disp_template_replaced( 'block_start' );
// GD version
$gd_version = system_check_gd_version();
init_system_check( T_('GD Library version'), isset( $gd_version ) ? $gd_version : T_('Not installed') );
if( ! isset( $gd_version ) )
{
	disp_system_check( 'warning', T_('You will not be able to automatically generate thumbnails for images. Enable the gd2 extension in your php.ini file or ask your hosting provider about it.') );
}
else
{
	disp_system_check( 'ok' );
	$gd_info = gd_info();
	// JPG (Note: "JPG Support" was renamed to "JPEG Support" in PHP 5.3)
	init_system_check( T_('GD JPG Support'), ( ! empty( $gd_info['JPG Support'] ) || ! empty( $gd_info['JPEG Support'] ) ) ? T_('Read/Write') : T_('No') );
	if( empty( $gd_info['JPG Support'] ) && empty( $gd_info['JPEG Support'] ) )
	{
		disp_system_check( 'warning', T_('You will not be able to automatically generate thumbnails for JPG images.') );
	}
	else
	{
		disp_system_check( 'ok' );
	}
	// PNG
	init_system_check( T_('GD PNG Support'), ! empty( $gd_info['PNG Support'] ) ? T_('Read/Write') : T_('No') );
	if( empty( $gd_info['PNG Support'] ) )
	{
		disp_system_check( 'warning', T_('You will not be able to automatically generate thumbnails for PNG images.') );
	}
	else
	{
		disp_system_check( 'ok' );
	}
	// GIF
	if( ! empty( $gd_info['GIF Create Support'] ) )
	{
		$gif_support = T_('Read/Write');
	}
	elseif( ! empty( $gd_info['GIF Read Support'] ) )
	{
		$gif_support = T_('Read');
	}
	else
	{
		$gif_support = T_('No');
	}
	init_system_check( T_('GD GIF Support'), $gif_support );
	if( $gif_support == T_('No') )
	{
		disp_system_check( 'warning', T_('You will not be able to automatically generate thumbnails for GIF images.') );
	}
	elseif( $gif_support == T_('Read') )
	{
		disp_system_check( 'warning', T_('Thumbnails for GIF images will be generated as PNG or JPG.') );
	}
	else
	{
		disp_system_check( 'ok' );
	}
}
$block_item_Widget->disp_template_raw( 'block_end' );

// phpinfo BLOCK
$block_item_Widget->title = T_('phpinfo()');
$block_item_Widget->disp_template_replaced( 'block_start' );

$subacts = array(
	T_('Configuration') => INFO_GENERAL,
	T_('Environment') => INFO_ENVIRONMENT,
	T_('General') => INFO_GENERAL,
	T_('Modules') => INFO_MODULES,
	T_('Variables') => INFO_VARIABLES,
	T_('ALL') => INFO_ALL,
);

foreach( $subacts as $key => $value )
{
	echo '<div class="system_check"><div class="system_check_name"><a href="'.regenerate_url( '', 'action=info&amp;subact='.$value ).'">'.sprintf( T_('Your server\'s phpinfo() "%s" information'), $key ).'</a></div><div class="system_check_value_ok">&nbsp;</div></div>';
}

$block_item_Widget->disp_template_raw( 'block_end' );

/**
 * dummy docblock
 */
function disp_system_check( $condition, $message = '' )
{
	global $syscheck_name;
	global $syscheck_value;

	echo '<div class="system_check"><div class="system_check_name">';
	echo $syscheck_name;
	echo '</div><div class="system_check_value_'.$condition.'">';
	echo $syscheck_value;
	echo '&nbsp;</div>';
	if( ! empty( $message ) )
	{
		echo '<div class="system_check_message_'.$condition.'">';
		echo $message;
		echo '</div>';
	}
	echo '</div>';
}


/**
 * dummy docblock
 */
function init_system_check( $name, $value )
{
	global $syscheck_name;
	global $syscheck_value;

	$syscheck_name = $name;
	$syscheck_value = $value;
}

?>
